@@ -1,7 +1,7 @@
 /* GStreamer
  *
  * Copyright (C) 2007 Rene Stadler <mail@renestadler.de>
- * Copyright (C) 2007 Sebastian Dröge <slomo@circular-chaos.org>
+ * Copyright (C) 2007-2009 Sebastian Dröge <sebastian.droege@collabora.co.uk>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,6 +25,8 @@
 
 #include "gstgiobasesrc.h"
 
+#include <gst/base/gsttypefindhelper.h>
+
 GST_DEBUG_CATEGORY_STATIC (gst_gio_base_src_debug);
 #define GST_CAT_DEFAULT gst_gio_base_src_debug
 
@@ -33,8 +35,8 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-GST_BOILERPLATE (GstGioBaseSrc, gst_gio_base_src, GstBaseSrc,
-    GST_TYPE_BASE_SRC);
+#define gst_gio_base_src_parent_class parent_class
+G_DEFINE_TYPE (GstGioBaseSrc, gst_gio_base_src, GST_TYPE_BASE_SRC);
 
 static void gst_gio_base_src_finalize (GObject * object);
 
@@ -45,39 +47,26 @@ static gboolean gst_gio_base_src_get_size (GstBaseSrc * base_src,
 static gboolean gst_gio_base_src_is_seekable (GstBaseSrc * base_src);
 static gboolean gst_gio_base_src_unlock (GstBaseSrc * base_src);
 static gboolean gst_gio_base_src_unlock_stop (GstBaseSrc * base_src);
-static gboolean gst_gio_base_src_check_get_range (GstBaseSrc * base_src);
 static GstFlowReturn gst_gio_base_src_create (GstBaseSrc * base_src,
     guint64 offset, guint size, GstBuffer ** buf);
 static gboolean gst_gio_base_src_query (GstBaseSrc * base_src,
     GstQuery * query);
 
 static void
-gst_gio_base_src_base_init (gpointer gclass)
+gst_gio_base_src_class_init (GstGioBaseSrcClass * klass)
 {
-  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
+  GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
+  GstBaseSrcClass *gstbasesrc_class = (GstBaseSrcClass *) klass;
 
   GST_DEBUG_CATEGORY_INIT (gst_gio_base_src_debug, "gio_base_src", 0,
       "GIO base source");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-}
-
-static void
-gst_gio_base_src_class_init (GstGioBaseSrcClass * klass)
-{
-  GObjectClass *gobject_class;
-
-  GstElementClass *gstelement_class;
-
-  GstBaseSrcClass *gstbasesrc_class;
-
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
-  gstbasesrc_class = (GstBaseSrcClass *) klass;
-
   gobject_class->finalize = gst_gio_base_src_finalize;
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_factory));
+
   gstbasesrc_class->start = GST_DEBUG_FUNCPTR (gst_gio_base_src_start);
   gstbasesrc_class->stop = GST_DEBUG_FUNCPTR (gst_gio_base_src_stop);
   gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_gio_base_src_get_size);
@@ -86,14 +75,12 @@ gst_gio_base_src_class_init (GstGioBaseSrcClass * klass)
   gstbasesrc_class->unlock = GST_DEBUG_FUNCPTR (gst_gio_base_src_unlock);
   gstbasesrc_class->unlock_stop =
       GST_DEBUG_FUNCPTR (gst_gio_base_src_unlock_stop);
-  gstbasesrc_class->check_get_range =
-      GST_DEBUG_FUNCPTR (gst_gio_base_src_check_get_range);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_gio_base_src_create);
   gstbasesrc_class->query = GST_DEBUG_FUNCPTR (gst_gio_base_src_query);
 }
 
 static void
-gst_gio_base_src_init (GstGioBaseSrc * src, GstGioBaseSrcClass * gclass)
+gst_gio_base_src_init (GstGioBaseSrc * src)
 {
   src->cancel = g_cancellable_new ();
 }
@@ -125,16 +112,26 @@ static gboolean
 gst_gio_base_src_start (GstBaseSrc * base_src)
 {
   GstGioBaseSrc *src = GST_GIO_BASE_SRC (base_src);
-
-  if (!G_IS_INPUT_STREAM (src->stream)) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-        ("No stream given yet"));
-    return FALSE;
-  }
+  GstGioBaseSrcClass *gbsrc_class = GST_GIO_BASE_SRC_GET_CLASS (src);
 
   src->position = 0;
 
-  GST_DEBUG_OBJECT (src, "started stream");
+  /* FIXME: This will likely block */
+  src->stream = gbsrc_class->get_stream (src);
+  if (G_UNLIKELY (!G_IS_INPUT_STREAM (src->stream))) {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+        ("No input stream provided by subclass"));
+    return FALSE;
+  } else if (G_UNLIKELY (g_input_stream_is_closed (src->stream))) {
+    GST_ELEMENT_ERROR (src, LIBRARY, FAILED, (NULL),
+        ("Input stream is already closed"));
+    return FALSE;
+  }
+
+  if (G_IS_SEEKABLE (src->stream))
+    src->position = g_seekable_tell (G_SEEKABLE (src->stream));
+
+  GST_DEBUG_OBJECT (src, "started source");
 
   return TRUE;
 }
@@ -143,12 +140,11 @@ static gboolean
 gst_gio_base_src_stop (GstBaseSrc * base_src)
 {
   GstGioBaseSrc *src = GST_GIO_BASE_SRC (base_src);
-
+  GstGioBaseSrcClass *klass = GST_GIO_BASE_SRC_GET_CLASS (src);
   gboolean success;
-
   GError *err = NULL;
 
-  if (G_IS_INPUT_STREAM (src->stream)) {
+  if (klass->close_on_stop && G_IS_INPUT_STREAM (src->stream)) {
     GST_DEBUG_OBJECT (src, "closing stream");
 
     /* FIXME: can block but unfortunately we can't use async operations
@@ -166,6 +162,9 @@ gst_gio_base_src_stop (GstBaseSrc * base_src)
       GST_DEBUG_OBJECT (src, "g_input_stream_close succeeded");
     }
 
+    g_object_unref (src->stream);
+    src->stream = NULL;
+  } else {
     g_object_unref (src->stream);
     src->stream = NULL;
   }
@@ -180,7 +179,6 @@ gst_gio_base_src_get_size (GstBaseSrc * base_src, guint64 * size)
 
   if (G_IS_FILE_INPUT_STREAM (src->stream)) {
     GFileInfo *info;
-
     GError *err = NULL;
 
     info = g_file_input_stream_query_info (G_FILE_INPUT_STREAM (src->stream),
@@ -207,13 +205,9 @@ gst_gio_base_src_get_size (GstBaseSrc * base_src, guint64 * size)
 
   if (GST_GIO_STREAM_IS_SEEKABLE (src->stream)) {
     goffset old;
-
     goffset stream_size;
-
     gboolean ret;
-
     GSeekable *seekable = G_SEEKABLE (src->stream);
-
     GError *err = NULL;
 
     old = g_seekable_tell (seekable);
@@ -231,7 +225,6 @@ gst_gio_base_src_get_size (GstBaseSrc * base_src, guint64 * size)
       } else {
         GST_WARNING_OBJECT (src, "Seeking to end of stream failed");
       }
-
       return FALSE;
     }
 
@@ -249,7 +242,6 @@ gst_gio_base_src_get_size (GstBaseSrc * base_src, guint64 * size)
       } else {
         GST_ERROR_OBJECT (src, "Seeking to the old position faile");
       }
-
       return FALSE;
     }
 
@@ -266,7 +258,6 @@ static gboolean
 gst_gio_base_src_is_seekable (GstBaseSrc * base_src)
 {
   GstGioBaseSrc *src = GST_GIO_BASE_SRC (base_src);
-
   gboolean seekable;
 
   seekable = GST_GIO_STREAM_IS_SEEKABLE (src->stream);
@@ -300,15 +291,6 @@ gst_gio_base_src_unlock_stop (GstBaseSrc * base_src)
   return TRUE;
 }
 
-static gboolean
-gst_gio_base_src_check_get_range (GstBaseSrc * base_src)
-{
-  /* FIXME: Implement dry-run variant using guesswork like gnomevfssrc? */
-
-  return GST_CALL_PARENT_WITH_DEFAULT (GST_BASE_SRC_CLASS,
-      check_get_range, (base_src), FALSE);
-}
-
 static GstFlowReturn
 gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
     GstBuffer ** buf_return)
@@ -327,23 +309,19 @@ gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
    * over DBus if our backend is GVfs and this is painfully slow. */
   if (src->cache && offset >= GST_BUFFER_OFFSET (src->cache) &&
       offset + size <= GST_BUFFER_OFFSET_END (src->cache)) {
-
     GST_DEBUG_OBJECT (src, "Creating subbuffer from cached buffer: offset %"
         G_GUINT64_FORMAT " length %u", offset, size);
 
-    buf = gst_buffer_create_sub (src->cache,
+    buf = gst_buffer_copy_region (src->cache, GST_BUFFER_COPY_ALL,
         offset - GST_BUFFER_OFFSET (src->cache), size);
 
     GST_BUFFER_OFFSET (buf) = offset;
     GST_BUFFER_OFFSET_END (buf) = offset + size;
-    GST_BUFFER_SIZE (buf) = size;
   } else {
     guint cachesize = MAX (4096, size);
-
+    guint8 *bdata;
     gssize read, res;
-
     gboolean success, eos;
-
     GError *err = NULL;
 
     if (src->cache) {
@@ -364,7 +342,7 @@ gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
         return ret;
     }
 
-    src->cache = gst_buffer_try_new_and_alloc (cachesize);
+    src->cache = gst_buffer_new_and_alloc (cachesize);
     if (G_UNLIKELY (src->cache == NULL)) {
       GST_ERROR_OBJECT (src, "Failed to allocate %u bytes", cachesize);
       return GST_FLOW_ERROR;
@@ -378,13 +356,14 @@ gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
      * supports reads up to 64k. So we loop here until we get at
      * at least the requested amount of bytes or a read returns
      * nothing. */
+    bdata = gst_buffer_map (src->cache, NULL, NULL, GST_MAP_WRITE);
     read = 0;
     while (size - read > 0 && (res =
             g_input_stream_read (G_INPUT_STREAM (src->stream),
-                GST_BUFFER_DATA (src->cache) + read, cachesize - read,
-                src->cancel, &err)) > 0) {
+                bdata + read, cachesize - read, src->cancel, &err)) > 0) {
       read += res;
     }
+    gst_buffer_unmap (src->cache, bdata, read);
 
     success = (read >= 0);
     eos = (cachesize > 0 && read == 0);
@@ -397,7 +376,6 @@ gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
 
     if (success && !eos) {
       src->position += read;
-      GST_BUFFER_SIZE (src->cache) = read;
 
       GST_BUFFER_OFFSET (src->cache) = offset;
       GST_BUFFER_OFFSET_END (src->cache) = offset + read;
@@ -407,11 +385,12 @@ gst_gio_base_src_create (GstBaseSrc * base_src, guint64 offset, guint size,
           "cached buffer: offset %" G_GUINT64_FORMAT " length %u", offset,
           size);
 
-      buf = gst_buffer_create_sub (src->cache, 0, MIN (size, read));
+      buf =
+          gst_buffer_copy_region (src->cache, GST_BUFFER_COPY_ALL, 0, MIN (size,
+              read));
 
       GST_BUFFER_OFFSET (buf) = offset;
       GST_BUFFER_OFFSET_END (buf) = offset + MIN (size, read);
-      GST_BUFFER_SIZE (buf) = MIN (size, read);
     } else {
       GST_DEBUG_OBJECT (src, "Read not successful");
       gst_buffer_unref (src->cache);
@@ -452,39 +431,3 @@ gst_gio_base_src_query (GstBaseSrc * base_src, GstQuery * query)
 
   return ret;
 }
-
-void
-gst_gio_base_src_set_stream (GstGioBaseSrc * src, GInputStream * stream)
-{
-  gboolean success;
-
-  GError *err = NULL;
-
-  g_return_if_fail (G_IS_INPUT_STREAM (stream));
-  g_return_if_fail ((GST_STATE (src) != GST_STATE_PLAYING &&
-          GST_STATE (src) != GST_STATE_PAUSED));
-
-  if (G_IS_INPUT_STREAM (src->stream)) {
-    GST_DEBUG_OBJECT (src, "closing old stream");
-
-    /* FIXME: can block but unfortunately we can't use async operations
-     * here because they require a running main loop */
-    success = g_input_stream_close (src->stream, src->cancel, &err);
-
-    if (!success && !gst_gio_error (src, "g_input_stream_close", &err, NULL)) {
-      GST_ELEMENT_WARNING (src, RESOURCE, CLOSE, (NULL),
-          ("g_input_stream_close failed: %s", err->message));
-      g_clear_error (&err);
-    } else if (!success) {
-      GST_ELEMENT_WARNING (src, RESOURCE, CLOSE, (NULL),
-          ("g_input_stream_close failed"));
-    } else {
-      GST_DEBUG_OBJECT (src, "g_input_stream_close succeeded");
-    }
-
-    g_object_unref (src->stream);
-    src->stream = NULL;
-  }
-
-  src->stream = stream;
-}
