@@ -1,7 +1,7 @@
 /* GStreamer
  *
  * Copyright (C) 2007 Rene Stadler <mail@renestadler.de>
- * Copyright (C) 2007 Sebastian Dröge <slomo@circular-chaos.org>
+ * Copyright (C) 2007-2009 Sebastian Dröge <sebastian.droege@collabora.co.uk>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -27,6 +27,23 @@
  * by an URI. This location can be specified using any protocol supported by
  * the GIO library or it's VFS backends. Common protocols are 'file', 'ftp',
  * or 'smb'.
+ *
+ * If the URI or #GFile already exists giosink will post a message of
+ * type %GST_MESSAGE_ELEMENT with name "file-exists" on the bus. The message
+ * also contains the #GFile and the corresponding URI.
+ * Applications can use the "file-exists" message to notify the user about
+ * the problem and to set a different target location or to remove the
+ * existing file. Note that right after the "file-exists" message a normal
+ * error message is posted on the bus which should be ignored if "file-exists"
+ * is handled by the application, for example by calling
+ * gst_bus_set_flushing(bus, TRUE) after the "file-exists" message was
+ * received and gst_bus_set_flushing(bus, FALSE) after the problem is
+ * resolved.
+ *
+ * Similar to the "file-exist" message a "not-mounted" message is posted
+ * on the bus if the target location is not mounted yet and needs to be
+ * mounted. This message can be used by application to mount the location
+ * and retry after the location was mounted successfully.
  * 
  * <refsect2>
  * <title>Example pipelines</title>
@@ -70,51 +87,36 @@ enum
 
 enum
 {
-  ARG_0,
-  ARG_LOCATION,
-  ARG_FILE
+  PROP_0,
+  PROP_LOCATION,
+  PROP_FILE
 };
 
-GST_BOILERPLATE_FULL (GstGioSink, gst_gio_sink, GstGioBaseSink,
-    GST_TYPE_GIO_BASE_SINK, gst_gio_uri_handler_do_init);
+#define gst_gio_sink_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstGioSink, gst_gio_sink, GST_TYPE_GIO_BASE_SINK,
+    gst_gio_uri_handler_do_init (g_define_type_id));
 
 static void gst_gio_sink_finalize (GObject * object);
 static void gst_gio_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_gio_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
-static gboolean gst_gio_sink_start (GstBaseSink * base_sink);
-
-static void
-gst_gio_sink_base_init (gpointer gclass)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
-
-  GST_DEBUG_CATEGORY_INIT (gst_gio_sink_debug, "gio_sink", 0, "GIO sink");
-
-  gst_element_class_set_details_simple (element_class, "GIO sink",
-      "Sink/File",
-      "Write to any GIO-supported location",
-      "Ren\xc3\xa9 Stadler <mail@renestadler.de>, "
-      "Sebastian Dröge <slomo@circular-chaos.org>");
-}
+static GOutputStream *gst_gio_sink_get_stream (GstGioBaseSink * base_sink);
 
 static void
 gst_gio_sink_class_init (GstGioSinkClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
-  GstBaseSinkClass *gstbasesink_class;
+  GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *gstelement_class = (GstElementClass *) klass;
+  GstGioBaseSinkClass *gstgiobasesink_class = (GstGioBaseSinkClass *) klass;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
-  gstbasesink_class = (GstBaseSinkClass *) klass;
+  GST_DEBUG_CATEGORY_INIT (gst_gio_sink_debug, "gio_sink", 0, "GIO sink");
 
   gobject_class->finalize = gst_gio_sink_finalize;
   gobject_class->set_property = gst_gio_sink_set_property;
   gobject_class->get_property = gst_gio_sink_get_property;
 
-  g_object_class_install_property (gobject_class, ARG_LOCATION,
+  g_object_class_install_property (gobject_class, PROP_LOCATION,
       g_param_spec_string ("location", "Location", "URI location to write to",
           NULL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
@@ -125,15 +127,23 @@ gst_gio_sink_class_init (GstGioSinkClass * klass)
    * 
    * Since: 0.10.20
    **/
-  g_object_class_install_property (gobject_class, ARG_FILE,
+  g_object_class_install_property (gobject_class, PROP_FILE,
       g_param_spec_object ("file", "File", "GFile to write to",
           G_TYPE_FILE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_gio_sink_start);
+  gst_element_class_set_details_simple (gstelement_class, "GIO sink",
+      "Sink/File",
+      "Write to any GIO-supported location",
+      "Ren\xc3\xa9 Stadler <mail@renestadler.de>, "
+      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
+
+  gstgiobasesink_class->get_stream =
+      GST_DEBUG_FUNCPTR (gst_gio_sink_get_stream);
+  gstgiobasesink_class->close_on_stop = TRUE;
 }
 
 static void
-gst_gio_sink_init (GstGioSink * sink, GstGioSinkClass * gclass)
+gst_gio_sink_init (GstGioSink * sink)
 {
 }
 
@@ -157,7 +167,7 @@ gst_gio_sink_set_property (GObject * object, guint prop_id,
   GstGioSink *sink = GST_GIO_SINK (object);
 
   switch (prop_id) {
-    case ARG_LOCATION:{
+    case PROP_LOCATION:{
       const gchar *uri = NULL;
 
       if (GST_STATE (sink) == GST_STATE_PLAYING ||
@@ -185,7 +195,7 @@ gst_gio_sink_set_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (GST_OBJECT (sink));
       break;
     }
-    case ARG_FILE:
+    case PROP_FILE:
       if (GST_STATE (sink) == GST_STATE_PLAYING ||
           GST_STATE (sink) == GST_STATE_PAUSED) {
         GST_WARNING
@@ -214,7 +224,7 @@ gst_gio_sink_get_property (GObject * object, guint prop_id,
   GstGioSink *sink = GST_GIO_SINK (object);
 
   switch (prop_id) {
-    case ARG_LOCATION:{
+    case PROP_LOCATION:{
       gchar *uri;
 
       GST_OBJECT_LOCK (GST_OBJECT (sink));
@@ -228,7 +238,7 @@ gst_gio_sink_get_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (GST_OBJECT (sink));
       break;
     }
-    case ARG_FILE:
+    case PROP_FILE:
       GST_OBJECT_LOCK (GST_OBJECT (sink));
       g_value_set_object (value, sink->file);
       GST_OBJECT_UNLOCK (GST_OBJECT (sink));
@@ -239,10 +249,10 @@ gst_gio_sink_get_property (GObject * object, guint prop_id,
   }
 }
 
-static gboolean
-gst_gio_sink_start (GstBaseSink * base_sink)
+static GOutputStream *
+gst_gio_sink_get_stream (GstGioBaseSink * bsink)
 {
-  GstGioSink *sink = GST_GIO_SINK (base_sink);
+  GstGioSink *sink = GST_GIO_SINK (bsink);
   GOutputStream *stream;
   GCancellable *cancel = GST_GIO_BASE_SINK (sink)->cancel;
   GError *err = NULL;
@@ -251,7 +261,7 @@ gst_gio_sink_start (GstBaseSink * base_sink)
   if (sink->file == NULL) {
     GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
         ("No location or GFile given"));
-    return FALSE;
+    return NULL;
   }
 
   uri = g_file_get_uri (sink->file);
@@ -267,24 +277,39 @@ gst_gio_sink_start (GstBaseSink * base_sink)
       /*if (GST_GIO_ERROR_MATCHES (err, EXISTS)) */
       /* FIXME: Retry with replace if overwrite == TRUE! */
 
-      if (GST_GIO_ERROR_MATCHES (err, NOT_FOUND))
+      if (GST_GIO_ERROR_MATCHES (err, NOT_FOUND)) {
         GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND, (NULL),
             ("Could not open location %s for writing: %s", uri, err->message));
-      else
-        GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_READ, (NULL),
+      } else if (GST_GIO_ERROR_MATCHES (err, EXISTS)) {
+        gst_element_post_message (GST_ELEMENT_CAST (sink),
+            gst_message_new_element (GST_OBJECT_CAST (sink),
+                gst_structure_new ("file-exists", "file", G_TYPE_FILE,
+                    sink->file, "uri", G_TYPE_STRING, uri, NULL)));
+
+        GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
+            ("Location %s already exists: %s", uri, err->message));
+      } else if (GST_GIO_ERROR_MATCHES (err, NOT_MOUNTED)) {
+        gst_element_post_message (GST_ELEMENT_CAST (sink),
+            gst_message_new_element (GST_OBJECT_CAST (sink),
+                gst_structure_new ("not-mounted", "file", G_TYPE_FILE,
+                    sink->file, "uri", G_TYPE_STRING, uri, NULL)));
+
+        GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
+            ("Location %s not mounted: %s", uri, err->message));
+      } else {
+        GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
             ("Could not open location %s for writing: %s", uri, err->message));
+      }
 
       g_clear_error (&err);
     }
     g_free (uri);
-    return FALSE;
+    return NULL;
   }
 
   GST_DEBUG_OBJECT (sink, "opened location %s", uri);
 
   g_free (uri);
 
-  gst_gio_base_sink_set_stream (GST_GIO_BASE_SINK (sink), stream);
-
-  return GST_BASE_SINK_CLASS (parent_class)->start (base_sink);
+  return stream;
 }
