@@ -63,9 +63,7 @@ struct _GstVisual
   VisActor *actor;
 
   /* audio/video state */
-  gint channels;
-  gint rate;                    /* Input samplerate */
-  gint bps;
+  GstAudioInfo info;
 
   /* framerate numerator & denominator */
   gint fps_n;
@@ -74,6 +72,7 @@ struct _GstVisual
   gint height;
   GstClockTime duration;
   guint outsize;
+  GstBufferPool *pool;
 
   /* samples per frame based on caps */
   guint spf;
@@ -100,23 +99,21 @@ GType gst_visual_get_type (void);
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_xRGB_HOST_ENDIAN "; "
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (" { "
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-        GST_VIDEO_CAPS_RGB "; "
+            "\"xRGB\", " "\"RGB\", "
 #else
-        GST_VIDEO_CAPS_BGR "; "
+            "\"BGRx\", " "\"BGR\", "
 #endif
-        GST_VIDEO_CAPS_RGB_16)
+            "\"RGB16\" } "))
     );
 
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int, "
-        "width = (int) 16, "
-        "depth = (int) 16, "
-        "endianness = (int) BYTE_ORDER, "
-        "signed = (boolean) TRUE, " "channels = (int) { 1, 2 }, "
+    GST_STATIC_CAPS ("audio/x-raw, "
+        "format = (string) " GST_AUDIO_NE (S16) ", "
+        "channels = (int) { 1, 2 }, "
 #if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
         "rate = (int) { 8000, 11250, 22500, 32000, 44100, 48000, 96000 }"
 #else
@@ -128,7 +125,7 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
 
 static void gst_visual_class_init (gpointer g_class, gpointer class_data);
 static void gst_visual_init (GstVisual * visual);
-static void gst_visual_dispose (GObject * object);
+static void gst_visual_finalize (GObject * object);
 
 static GstStateChangeReturn gst_visual_change_state (GstElement * element,
     GstStateChange transition);
@@ -139,8 +136,7 @@ static gboolean gst_visual_src_event (GstPad * pad, GstEvent * event);
 static gboolean gst_visual_src_query (GstPad * pad, GstQuery * query);
 
 static gboolean gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps);
-static gboolean gst_visual_src_setcaps (GstPad * pad, GstCaps * caps);
-static GstCaps *gst_visual_getcaps (GstPad * pad);
+static GstCaps *gst_visual_getcaps (GstPad * pad, GstCaps * filter);
 static void libvisual_log_handler (const char *message, const char *funcname,
     void *priv);
 
@@ -198,6 +194,7 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
         gst_static_pad_template_get (&src_template));
     gst_element_class_add_pad_template (element,
         gst_static_pad_template_get (&sink_template));
+
     gst_element_class_set_details_simple (element,
         longname, "Visualization",
         klass->plugin->info->about, "Benjamin Otte <otte@gnome.org>");
@@ -205,7 +202,7 @@ gst_visual_class_init (gpointer g_class, gpointer class_data)
     g_free (longname);
   }
 
-  object->dispose = gst_visual_dispose;
+  object->finalize = gst_visual_finalize;
 }
 
 static void
@@ -213,13 +210,11 @@ gst_visual_init (GstVisual * visual)
 {
   /* create the sink and src pads */
   visual->sinkpad = gst_pad_new_from_static_template (&sink_template, "sink");
-  gst_pad_set_setcaps_function (visual->sinkpad, gst_visual_sink_setcaps);
   gst_pad_set_chain_function (visual->sinkpad, gst_visual_chain);
   gst_pad_set_event_function (visual->sinkpad, gst_visual_sink_event);
   gst_element_add_pad (GST_ELEMENT (visual), visual->sinkpad);
 
   visual->srcpad = gst_pad_new_from_static_template (&src_template, "src");
-  gst_pad_set_setcaps_function (visual->srcpad, gst_visual_src_setcaps);
   gst_pad_set_getcaps_function (visual->srcpad, gst_visual_getcaps);
   gst_pad_set_event_function (visual->srcpad, gst_visual_src_event);
   gst_pad_set_query_function (visual->srcpad, gst_visual_src_query);
@@ -246,17 +241,16 @@ gst_visual_clear_actors (GstVisual * visual)
 }
 
 static void
-gst_visual_dispose (GObject * object)
+gst_visual_finalize (GObject * object)
 {
   GstVisual *visual = GST_VISUAL (object);
 
-  if (visual->adapter) {
-    g_object_unref (visual->adapter);
-    visual->adapter = NULL;
-  }
+  g_object_unref (visual->adapter);
+  if (visual->pool)
+    gst_object_unref (visual->pool);
   gst_visual_clear_actors (visual);
 
-  GST_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
+  GST_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
@@ -272,14 +266,14 @@ gst_visual_reset (GstVisual * visual)
 }
 
 static GstCaps *
-gst_visual_getcaps (GstPad * pad)
+gst_visual_getcaps (GstPad * pad, GstCaps * filter)
 {
   GstCaps *ret;
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   int depths;
 
   if (!visual->actor) {
-    ret = gst_caps_copy (gst_pad_get_pad_template_caps (visual->srcpad));
+    ret = gst_pad_get_pad_template_caps (visual->srcpad);
     goto beach;
   }
 
@@ -297,32 +291,46 @@ gst_visual_getcaps (GstPad * pad)
   GST_DEBUG_OBJECT (visual, "libvisual plugin supports depths %u (0x%04x)",
       depths, depths);
   /* if (depths & VISUAL_VIDEO_DEPTH_32BIT) Always supports 32bit output */
-  gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_xRGB_HOST_ENDIAN));
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+  gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("xRGB")));
+#else
+  gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("BGRx")));
+#endif
 
   if (depths & VISUAL_VIDEO_DEPTH_24BIT) {
 #if G_BYTE_ORDER == G_BIG_ENDIAN
-    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_RGB));
+    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("RGB")));
 #else
-    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_BGR));
+    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("BGR")));
 #endif
   }
   if (depths & VISUAL_VIDEO_DEPTH_16BIT) {
-    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_RGB_16));
+    gst_caps_append (ret, gst_caps_from_string (GST_VIDEO_CAPS_MAKE ("RGB16")));
   }
 
 beach:
 
+  if (filter) {
+    GstCaps *intersection;
+
+    intersection =
+        gst_caps_intersect_full (filter, ret, GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (ret);
+    ret = intersection;
+  }
+
   GST_DEBUG_OBJECT (visual, "returning caps %" GST_PTR_FORMAT, ret);
   gst_object_unref (visual);
   return ret;
 }
 
 static gboolean
-gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
+gst_visual_src_setcaps (GstVisual * visual, GstCaps * caps)
 {
-  GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
+  gboolean res;
   GstStructure *structure;
-  gint depth, pitch;
+  gint depth, pitch, rate;
+  const gchar *fmt;
 
   structure = gst_caps_get_structure (caps, 0);
 
@@ -332,12 +340,19 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
     goto error;
   if (!gst_structure_get_int (structure, "height", &visual->height))
     goto error;
-  if (!gst_structure_get_int (structure, "bpp", &depth))
+  if (!(fmt = gst_structure_get_string (structure, "format")))
     goto error;
   if (!gst_structure_get_fraction (structure, "framerate", &visual->fps_n,
           &visual->fps_d))
     goto error;
 
+  if (!strcmp (fmt, "BGR") || !strcmp (fmt, "RGB"))
+    depth = 24;
+  else if (!strcmp (fmt, "BGRx") || !strcmp (fmt, "xRGB"))
+    depth = 32;
+  else
+    depth = 16;
+
   visual_video_set_depth (visual->video,
       visual_video_depth_enum_from_value (depth));
   visual_video_set_dimension (visual->video, visual->width, visual->height);
@@ -345,21 +360,22 @@ gst_visual_src_setcaps (GstPad * pad, GstCaps * caps)
   visual_video_set_pitch (visual->video, pitch);
   visual_actor_video_negotiate (visual->actor, 0, FALSE, FALSE);
 
+  rate = GST_AUDIO_INFO_RATE (&visual->info);
+
   /* precalc some values */
   visual->outsize = visual->video->height * pitch;
-  visual->spf =
-      gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
+  visual->spf = gst_util_uint64_scale_int (rate, visual->fps_d, visual->fps_n);
   visual->duration =
       gst_util_uint64_scale_int (GST_SECOND, visual->fps_d, visual->fps_n);
 
-  gst_object_unref (visual);
-  return TRUE;
+  res = gst_pad_push_event (visual->srcpad, gst_event_new_caps (caps));
+
+  return res;
 
   /* ERRORS */
 error:
   {
     GST_DEBUG_OBJECT (visual, "error parsing caps");
-    gst_object_unref (visual);
     return FALSE;
   }
 }
@@ -368,23 +384,34 @@ static gboolean
 gst_visual_sink_setcaps (GstPad * pad, GstCaps * caps)
 {
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
-  GstStructure *structure;
+  GstAudioInfo info;
+  gint rate;
 
-  structure = gst_caps_get_structure (caps, 0);
+  if (!gst_audio_info_from_caps (&info, caps))
+    goto invalid_caps;
 
-  gst_structure_get_int (structure, "channels", &visual->channels);
-  gst_structure_get_int (structure, "rate", &visual->rate);
+  visual->info = info;
+
+  rate = GST_AUDIO_INFO_RATE (&info);
 
   /* this is how many samples we need to fill one frame at the requested
    * framerate. */
   if (visual->fps_n != 0) {
     visual->spf =
-        gst_util_uint64_scale_int (visual->rate, visual->fps_d, visual->fps_n);
+        gst_util_uint64_scale_int (rate, visual->fps_d, visual->fps_n);
   }
-  visual->bps = visual->channels * sizeof (gint16);
 
   gst_object_unref (visual);
+
   return TRUE;
+
+  /* ERRORS */
+invalid_caps:
+  {
+    GST_ERROR_OBJECT (visual, "invalid caps received");
+    gst_object_unref (visual);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -393,25 +420,29 @@ gst_vis_src_negotiate (GstVisual * visual)
   GstCaps *othercaps, *target;
   GstStructure *structure;
   GstCaps *caps;
+  GstQuery *query;
+  GstBufferPool *pool = NULL;
+  guint size, min, max, prefix, alignment;
 
-  caps = gst_pad_get_caps (visual->srcpad);
+  caps = gst_pad_get_caps (visual->srcpad, NULL);
 
   /* see what the peer can do */
-  othercaps = gst_pad_peer_get_caps (visual->srcpad);
+  othercaps = gst_pad_peer_get_caps (visual->srcpad, caps);
   if (othercaps) {
-    target = gst_caps_intersect (othercaps, caps);
-    gst_caps_unref (othercaps);
+    target = othercaps;
     gst_caps_unref (caps);
 
     if (gst_caps_is_empty (target))
       goto no_format;
 
+    target = gst_caps_make_writable (target);
     gst_caps_truncate (target);
   } else {
     /* need a copy, we'll be modifying it when fixating */
     target = gst_caps_copy (caps);
     gst_caps_unref (caps);
   }
+  GST_DEBUG_OBJECT (visual, "before fixate caps %" GST_PTR_FORMAT, target);
 
   /* fixate in case something is not fixed. This does nothing if the value is
    * already fixed. For video we always try to fixate to something like
@@ -422,7 +453,46 @@ gst_vis_src_negotiate (GstVisual * visual)
   gst_structure_fixate_field_nearest_fraction (structure, "framerate",
       DEFAULT_FPS_N, DEFAULT_FPS_D);
 
-  gst_pad_set_caps (visual->srcpad, target);
+  gst_pad_fixate_caps (visual->srcpad, target);
+
+  GST_DEBUG_OBJECT (visual, "after fixate caps %" GST_PTR_FORMAT, target);
+
+  gst_visual_src_setcaps (visual, target);
+
+  /* try to get a bufferpool now */
+  /* find a pool for the negotiated caps now */
+  query = gst_query_new_allocation (target, TRUE);
+
+  if (gst_pad_peer_query (visual->srcpad, query)) {
+    /* we got configuration from our peer, parse them */
+    gst_query_parse_allocation_params (query, &size, &min, &max, &prefix,
+        &alignment, &pool);
+  } else {
+    size = visual->outsize;
+    min = max = 0;
+    prefix = 0;
+    alignment = 0;
+  }
+
+  if (pool == NULL) {
+    GstStructure *config;
+
+    /* we did not get a pool, make one ourselves then */
+    pool = gst_buffer_pool_new ();
+
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_set (config, target, size, min, max, prefix,
+        alignment);
+    gst_buffer_pool_set_config (pool, config);
+  }
+
+  if (visual->pool)
+    gst_object_unref (visual->pool);
+  visual->pool = pool;
+
+  /* and activate */
+  gst_buffer_pool_set_active (pool, TRUE);
+
   gst_caps_unref (target);
 
   return TRUE;
@@ -454,29 +524,28 @@ gst_visual_sink_event (GstPad * pad, GstEvent * event)
       gst_visual_reset (visual);
       res = gst_pad_push_event (visual->srcpad, event);
       break;
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_CAPS:
     {
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
-      gboolean update;
+      GstCaps *caps;
 
+      gst_event_parse_caps (event, &caps);
+      res = gst_visual_sink_setcaps (pad, caps);
+      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_SEGMENT:
+    {
       /* the newsegment values are used to clip the input samples
        * and to convert the incomming timestamps to running time so
        * we can do QoS */
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
-
-      /* now configure the values */
-      gst_segment_set_newsegment_full (&visual->segment, update,
-          rate, arate, format, start, stop, time);
+      gst_event_copy_segment (event, &visual->segment);
 
       /* and forward */
       res = gst_pad_push_event (visual->srcpad, event);
       break;
     }
     default:
-      res = gst_pad_push_event (visual->srcpad, event);
+      res = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -499,7 +568,7 @@ gst_visual_src_event (GstPad * pad, GstEvent * event)
       GstClockTimeDiff diff;
       GstClockTime timestamp;
 
-      gst_event_parse_qos (event, &proportion, &diff, &timestamp);
+      gst_event_parse_qos (event, NULL, &proportion, &diff, &timestamp);
 
       /* save stuff for the _chain function */
       GST_OBJECT_LOCK (visual);
@@ -516,8 +585,13 @@ gst_visual_src_event (GstPad * pad, GstEvent * event)
       res = gst_pad_push_event (visual->sinkpad, event);
       break;
     }
+    case GST_EVENT_RECONFIGURE:
+      /* dont't forward */
+      gst_event_unref (event);
+      res = TRUE;
+      break;
     default:
-      res = gst_pad_push_event (visual->sinkpad, event);
+      res = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -553,7 +627,8 @@ gst_visual_src_query (GstPad * pad, GstQuery * query)
         /* the max samples we must buffer buffer */
         max_samples = MAX (VISUAL_SAMPLES, visual->spf);
         our_latency =
-            gst_util_uint64_scale_int (max_samples, GST_SECOND, visual->rate);
+            gst_util_uint64_scale_int (max_samples, GST_SECOND,
+            GST_AUDIO_INFO_RATE (&visual->info));
 
         GST_DEBUG_OBJECT (visual, "Our latency: %" GST_TIME_FORMAT,
             GST_TIME_ARGS (our_latency));
@@ -582,35 +657,22 @@ gst_visual_src_query (GstPad * pad, GstQuery * query)
   return res;
 }
 
-/* allocate and output buffer, if no format was negotiated, this
- * function will negotiate one. After calling this function, a
- * reverse negotiation could have happened. */
+/* Make sure we are negotiated */
 static GstFlowReturn
-get_buffer (GstVisual * visual, GstBuffer ** outbuf)
+ensure_negotiated (GstVisual * visual)
 {
-  GstFlowReturn ret;
+  gboolean reconfigure;
+
+  GST_OBJECT_LOCK (visual->srcpad);
+  reconfigure = GST_PAD_NEEDS_RECONFIGURE (visual->srcpad);
+  GST_OBJECT_FLAG_UNSET (visual->srcpad, GST_PAD_NEED_RECONFIGURE);
+  GST_OBJECT_UNLOCK (visual->srcpad);
 
   /* we don't know an output format yet, pick one */
-  if (GST_PAD_CAPS (visual->srcpad) == NULL) {
+  if (reconfigure || !gst_pad_has_current_caps (visual->srcpad)) {
     if (!gst_vis_src_negotiate (visual))
       return GST_FLOW_NOT_NEGOTIATED;
   }
-
-  GST_DEBUG_OBJECT (visual, "allocating output buffer with caps %"
-      GST_PTR_FORMAT, GST_PAD_CAPS (visual->srcpad));
-
-  /* now allocate a buffer with the last negotiated format. 
-   * Downstream could renegotiate a new format, which will trigger
-   * our setcaps function on the source pad. */
-  ret =
-      gst_pad_alloc_buffer_and_set_caps (visual->srcpad,
-      GST_BUFFER_OFFSET_NONE, visual->outsize,
-      GST_PAD_CAPS (visual->srcpad), outbuf);
-
-  /* no buffer allocated, we don't care why. */
-  if (ret != GST_FLOW_OK)
-    return ret;
-
   return GST_FLOW_OK;
 }
 
@@ -622,17 +684,15 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
   GstVisual *visual = GST_VISUAL (gst_pad_get_parent (pad));
   GstFlowReturn ret = GST_FLOW_OK;
   guint avail;
+  gint bpf, rate, channels;
 
   GST_DEBUG_OBJECT (visual, "chain function called");
 
-  /* If we don't have an output format yet, preallocate a buffer to try and
-   * set one */
-  if (GST_PAD_CAPS (visual->srcpad) == NULL) {
-    ret = get_buffer (visual, &outbuf);
-    if (ret != GST_FLOW_OK) {
-      gst_buffer_unref (buffer);
-      goto beach;
-    }
+  /* Make sure have an output format */
+  ret = ensure_negotiated (visual);
+  if (ret != GST_FLOW_OK) {
+    gst_buffer_unref (buffer);
+    goto beach;
   }
 
   /* resync on DISCONT */
@@ -640,9 +700,13 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     gst_adapter_clear (visual->adapter);
   }
 
+  rate = GST_AUDIO_INFO_RATE (&visual->info);
+  bpf = GST_AUDIO_INFO_BPF (&visual->info);
+  channels = GST_AUDIO_INFO_CHANNELS (&visual->info);
+
   GST_DEBUG_OBJECT (visual,
       "Input buffer has %d samples, time=%" G_GUINT64_FORMAT,
-      GST_BUFFER_SIZE (buffer) / visual->bps, GST_BUFFER_TIMESTAMP (buffer));
+      gst_buffer_get_size (buffer) / bpf, GST_BUFFER_TIMESTAMP (buffer));
 
   gst_adapter_push (visual->adapter, buffer);
 
@@ -650,6 +714,8 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     gboolean need_skip;
     const guint16 *data;
     guint64 dist, timestamp;
+    guint8 *outdata;
+    gsize outsize;
 
     GST_DEBUG_OBJECT (visual, "processing buffer");
 
@@ -657,19 +723,19 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     GST_DEBUG_OBJECT (visual, "avail now %u", avail);
 
     /* we need at least VISUAL_SAMPLES samples */
-    if (avail < VISUAL_SAMPLES * visual->bps)
+    if (avail < VISUAL_SAMPLES * bpf)
       break;
 
     /* we need at least enough samples to make one frame */
-    if (avail < visual->spf * visual->bps)
+    if (avail < visual->spf * bpf)
       break;
 
     /* get timestamp of the current adapter byte */
     timestamp = gst_adapter_prev_timestamp (visual->adapter, &dist);
     if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
       /* convert bytes to time */
-      dist /= visual->bps;
-      timestamp += gst_util_uint64_scale_int (dist, GST_SECOND, visual->rate);
+      dist /= bpf;
+      timestamp += gst_util_uint64_scale_int (dist, GST_SECOND, rate);
     }
 
     if (timestamp != -1) {
@@ -696,19 +762,19 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
 
     /* Read VISUAL_SAMPLES samples per channel */
     data =
-        (const guint16 *) gst_adapter_peek (visual->adapter,
-        VISUAL_SAMPLES * visual->bps);
+        (const guint16 *) gst_adapter_map (visual->adapter,
+        VISUAL_SAMPLES * bpf);
 
 #if defined(VISUAL_API_VERSION) && VISUAL_API_VERSION >= 4000 && VISUAL_API_VERSION < 5000
     {
       VisBuffer *lbuf, *rbuf;
       guint16 ldata[VISUAL_SAMPLES], rdata[VISUAL_SAMPLES];
-      VisAudioSampleRateType rate;
+      VisAudioSampleRateType vrate;
 
       lbuf = visual_buffer_new_with_buffer (ldata, sizeof (ldata), NULL);
       rbuf = visual_buffer_new_with_buffer (rdata, sizeof (rdata), NULL);
 
-      if (visual->channels == 2) {
+      if (channels == 2) {
         for (i = 0; i < VISUAL_SAMPLES; i++) {
           ldata[i] = *data++;
           rdata[i] = *data++;
@@ -720,32 +786,32 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
         }
       }
 
-      switch (visual->rate) {
+      switch (rate) {
         case 8000:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_8000;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_8000;
           break;
         case 11250:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_11250;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_11250;
           break;
         case 22500:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_22500;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_22500;
           break;
         case 32000:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_32000;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_32000;
           break;
         case 44100:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_44100;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_44100;
           break;
         case 48000:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_48000;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_48000;
           break;
         case 96000:
-          rate = VISUAL_AUDIO_SAMPLE_RATE_96000;
+          vrate = VISUAL_AUDIO_SAMPLE_RATE_96000;
           break;
         default:
           visual_object_unref (VISUAL_OBJECT (lbuf));
           visual_object_unref (VISUAL_OBJECT (rbuf));
-          GST_ERROR_OBJECT (visual, "unsupported rate %d", visual->rate);
+          GST_ERROR_OBJECT (visual, "unsupported rate %d", rate);
           ret = GST_FLOW_ERROR;
           goto beach;
           break;
@@ -753,10 +819,10 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
 
       visual_audio_samplepool_input_channel (visual->audio->samplepool,
           lbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          vrate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
           (char *) VISUAL_AUDIO_CHANNEL_LEFT);
       visual_audio_samplepool_input_channel (visual->audio->samplepool, rbuf,
-          rate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
+          vrate, VISUAL_AUDIO_SAMPLE_FORMAT_S16,
           (char *) VISUAL_AUDIO_CHANNEL_RIGHT);
 
       visual_object_unref (VISUAL_OBJECT (lbuf));
@@ -780,17 +846,23 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
     /* alloc a buffer if we don't have one yet, this happens
      * when we pushed a buffer in this while loop before */
     if (outbuf == NULL) {
-      ret = get_buffer (visual, &outbuf);
+      GST_DEBUG_OBJECT (visual, "allocating output buffer");
+      ret = gst_buffer_pool_acquire_buffer (visual->pool, &outbuf, NULL);
       if (ret != GST_FLOW_OK) {
+        gst_adapter_unmap (visual->adapter, 0);
         goto beach;
       }
     }
-    visual_video_set_buffer (visual->video, GST_BUFFER_DATA (outbuf));
+    outdata = gst_buffer_map (outbuf, &outsize, NULL, GST_MAP_WRITE);
+    visual_video_set_buffer (visual->video, outdata);
     visual_audio_analyze (visual->audio);
     visual_actor_run (visual->actor, visual->audio);
     visual_video_set_buffer (visual->video, NULL);
+    gst_buffer_unmap (outbuf, outdata, outsize);
     GST_DEBUG_OBJECT (visual, "rendered one frame");
 
+    gst_adapter_unmap (visual->adapter, 0);
+
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = visual->duration;
 
@@ -802,7 +874,7 @@ gst_visual_chain (GstPad * pad, GstBuffer * buffer)
         visual->spf);
 
     /* Flush out the number of samples per frame */
-    gst_adapter_flush (visual->adapter, visual->spf * visual->bps);
+    gst_adapter_flush (visual->adapter, visual->spf * bpf);
 
     /* quit the loop if something was wrong */
     if (ret != GST_FLOW_OK)
@@ -856,6 +928,11 @@ gst_visual_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      if (visual->pool) {
+        gst_buffer_pool_set_active (visual->pool, FALSE);
+        gst_object_unref (visual->pool);
+        visual->pool = NULL;
+      }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       gst_visual_clear_actors (visual);
