@@ -103,16 +103,20 @@ struct _GstOggPad
   gint64 first_granule;         /* the granulepos of first page == first sample in next page */
   GstClockTime first_time;      /* the timestamp of the second page or granuletime of first page */
 
-  gboolean     is_sparse;       /* TRUE if this is a subtitle pad or some other sparse stream */
-  GstClockTime last_stop;       /* last_stop when last push occured; used to detect when we
+  GstClockTime position;        /* position when last push occured; used to detect when we
                                  * need to send a newsegment update event for sparse streams */
 
   GList *continued;
 
   gboolean discont;
   GstFlowReturn last_ret;       /* last return of _pad_push() */
+  gboolean is_eos;
 
   gboolean added;
+
+  /* push mode seeking */
+  GstClockTime push_kf_time;
+  GstClockTime push_sync_time;
 };
 
 struct _GstOggPadClass
@@ -139,28 +143,51 @@ struct _GstOggDemux
   gboolean running;
 
   gboolean need_chains;
+  gboolean resync;
 
   /* state */
   GMutex *chain_lock;           /* we need the lock to protect the chains */
   GArray *chains;               /* list of chains we know */
   GstClockTime total_time;
+  gint bitrate;                 /* bitrate of the current chain */
 
   GstOggChain *current_chain;
   GstOggChain *building_chain;
 
   /* playback start/stop positions */
   GstSegment segment;
-  gboolean segment_running;
   guint32  seqnum;
 
   GstEvent *event;
   GstEvent *newsegment;         /* pending newsegment to be sent from _loop */
 
   /* annodex stuff */
-  gboolean have_fishead;
   gint64 basetime;
   gint64 prestime;
 
+  /* push mode seeking support */
+  GMutex *push_lock; /* we need the lock to protect the push mode variables */
+  gint64 push_byte_offset; /* where were are at in the stream, in bytes */
+  gint64 push_byte_length; /* length in bytes of the stream, -1 if unknown */
+  GstClockTime push_time_length; /* length in time of the stream */
+  GstClockTime push_start_time; /* start time of the stream */
+  GstClockTime push_time_offset; /* where were are at in the stream, in time */
+  enum { PUSH_PLAYING, PUSH_DURATION, PUSH_BISECT1, PUSH_LINEAR1, PUSH_BISECT2, PUSH_LINEAR2 } push_state;
+
+  GstClockTime push_seek_time_original_target;
+  GstClockTime push_seek_time_target;
+  gint64 push_last_seek_offset;
+  GstClockTime push_last_seek_time;
+  gint64 push_offset0, push_offset1; /* bisection search offset bounds */
+  GstClockTime push_time0, push_time1; /* bisection search time bounds */
+
+  double push_seek_rate;
+  GstSeekFlags push_seek_flags;
+  GstEvent *push_mode_seek_delayed_event;
+  gboolean push_disable_seeking;
+
+  gint push_bisection_steps[2];
+
   /* ogg stuff */
   ogg_sync_state sync;
 };
@@ -170,6 +197,7 @@ struct _GstOggDemuxClass
   GstElementClass parent_class;
 };
 
+gboolean gst_ogg_demux_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
 
