@@ -55,6 +55,8 @@ typedef gboolean (*GstOggMapIsHeaderPacketFunc) (GstOggStream * pad,
     ogg_packet * packet);
 typedef gint64 (*GstOggMapPacketDurationFunc) (GstOggStream * pad,
     ogg_packet * packet);
+typedef void (*GstOggMapExtractTagsFunc) (GstOggStream * pad,
+    ogg_packet * packet);
 
 typedef gint64 (*GstOggMapGranuleposToKeyGranuleFunc) (GstOggStream * pad,
     gint64 granulepos);
@@ -76,6 +78,7 @@ struct _GstOggMap
   GstOggMapIsHeaderPacketFunc is_header_func;
   GstOggMapPacketDurationFunc packet_duration_func;
   GstOggMapGranuleposToKeyGranuleFunc granulepos_to_key_granule_func;
+  GstOggMapExtractTagsFunc extract_tags_func;
 };
 
 extern const GstOggMap mappers[];
@@ -136,7 +139,8 @@ gst_ogg_stream_granulepos_to_granule (GstOggStream * pad, gint64 granulepos)
   }
 
   if (mappers[pad->map].granulepos_to_granule_func == NULL) {
-    GST_WARNING ("Failed to convert granulepos to granule");
+    GST_WARNING ("Failed to convert %s granulepos to granule",
+        gst_ogg_stream_get_media_type (pad));
     return -1;
   }
 
@@ -165,7 +169,8 @@ gst_ogg_stream_granule_to_granulepos (GstOggStream * pad, gint64 granule,
   }
 
   if (mappers[pad->map].granule_to_granulepos_func == NULL) {
-    GST_WARNING ("Failed to convert granule to granulepos");
+    GST_WARNING ("Failed to convert %s granule to granulepos",
+        gst_ogg_stream_get_media_type (pad));
     return -1;
   }
 
@@ -173,29 +178,28 @@ gst_ogg_stream_granule_to_granulepos (GstOggStream * pad, gint64 granule,
       keyframe_granule);
 }
 
-#if 0
 gboolean
-gst_ogg_stream_packet_granulepos_is_key_frame (GstOggStream * pad,
-    gint64 granulepos)
+gst_ogg_stream_granulepos_is_key_frame (GstOggStream * pad, gint64 granulepos)
 {
   if (granulepos == -1) {
     return FALSE;
   }
 
   if (mappers[pad->map].is_key_frame_func == NULL) {
-    GST_WARNING ("Failed to determine key frame");
+    GST_WARNING ("Failed to determine keyframeness for %s granulepos",
+        gst_ogg_stream_get_media_type (pad));
     return FALSE;
   }
 
   return mappers[pad->map].is_key_frame_func (pad, granulepos);
 }
-#endif
 
 gboolean
 gst_ogg_stream_packet_is_header (GstOggStream * pad, ogg_packet * packet)
 {
   if (mappers[pad->map].is_header_func == NULL) {
-    GST_WARNING ("Failed to determine header");
+    GST_WARNING ("Failed to determine headerness of %s packet",
+        gst_ogg_stream_get_media_type (pad));
     return FALSE;
   }
 
@@ -206,13 +210,39 @@ gint64
 gst_ogg_stream_get_packet_duration (GstOggStream * pad, ogg_packet * packet)
 {
   if (mappers[pad->map].packet_duration_func == NULL) {
-    GST_WARNING ("Failed to determine packet duration");
+    GST_WARNING ("Failed to determine %s packet duration",
+        gst_ogg_stream_get_media_type (pad));
     return -1;
   }
 
   return mappers[pad->map].packet_duration_func (pad, packet);
 }
 
+
+void
+gst_ogg_stream_extract_tags (GstOggStream * pad, ogg_packet * packet)
+{
+  if (mappers[pad->map].extract_tags_func == NULL) {
+    GST_DEBUG ("No tag extraction");
+    return;
+  }
+
+  mappers[pad->map].extract_tags_func (pad, packet);
+}
+
+const char *
+gst_ogg_stream_get_media_type (GstOggStream * pad)
+{
+  const GstCaps *caps = pad->caps;
+  const GstStructure *structure;
+  if (!caps)
+    return NULL;
+  structure = gst_caps_get_structure (caps, 0);
+  if (!structure)
+    return NULL;
+  return gst_structure_get_name (structure);
+}
+
 /* some generic functions */
 
 static gboolean
@@ -243,6 +273,11 @@ granule_to_granulepos_default (GstOggStream * pad, gint64 granule,
   gint64 keyoffset;
 
   if (pad->granuleshift != 0) {
+    /* If we don't know where the previous keyframe is yet, assume it is
+       at 0 or 1, depending on bitstream version. If nothing else, this
+       avoids getting negative granpos back. */
+    if (keyframe_granule < 0)
+      keyframe_granule = pad->theora_has_zero_keyoffset ? 0 : 1;
     keyoffset = granule - keyframe_granule;
     return (keyframe_granule << pad->granuleshift) | keyoffset;
   } else {
@@ -280,6 +315,42 @@ packet_duration_constant (GstOggStream * pad, ogg_packet * packet)
   return pad->frame_size;
 }
 
+/* helper: extracts tags from vorbis comment ogg packet.
+ * Returns result in *tags after free'ing existing *tags (if any) */
+static gboolean
+tag_list_from_vorbiscomment_packet (ogg_packet * packet,
+    const guint8 * id_data, const guint id_data_length, GstTagList ** tags)
+{
+  gchar *encoder = NULL;
+  GstTagList *list;
+  gboolean ret = TRUE;
+
+  g_return_val_if_fail (tags != NULL, FALSE);
+
+  list = gst_tag_list_from_vorbiscomment (packet->packet, packet->bytes,
+      id_data, id_data_length, &encoder);
+
+  if (!list) {
+    GST_WARNING ("failed to decode vorbis comments");
+    ret = FALSE;
+    goto exit;
+  }
+
+  if (encoder) {
+    if (encoder[0])
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_ENCODER, encoder,
+          NULL);
+    g_free (encoder);
+  }
+
+exit:
+  if (*tags)
+    gst_tag_list_free (*tags);
+  *tags = list;
+
+  return ret;
+}
+
 /* theora */
 
 static gboolean
@@ -287,9 +358,14 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
 {
   guint8 *data = packet->packet;
   guint w, h, par_d, par_n;
+  guint8 vmaj, vmin, vrev;
 
-  w = GST_READ_UINT24_BE (data + 14) & 0xFFFFF0;
-  h = GST_READ_UINT24_BE (data + 17) & 0xFFFFF0;
+  vmaj = data[7];
+  vmin = data[8];
+  vrev = data[9];
+
+  w = GST_READ_UINT24_BE (data + 14) & 0xFFFFFF;
+  h = GST_READ_UINT24_BE (data + 17) & 0xFFFFFF;
 
   pad->granulerate_n = GST_READ_UINT32_BE (data + 22);
   pad->granulerate_d = GST_READ_UINT32_BE (data + 26);
@@ -304,6 +380,7 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->granuleshift = ((GST_READ_UINT8 (data + 40) & 0x03) << 3) +
       (GST_READ_UINT8 (data + 41) >> 5);
 
+  pad->is_video = TRUE;
   pad->n_header_packets = 3;
   pad->frame_size = 1;
 
@@ -315,6 +392,12 @@ setup_theora_mapper (GstOggStream * pad, ogg_packet * packet)
     return FALSE;
   }
 
+  /* The interpretation of the granule position has changed with 3.2.1.
+     The granule is now made from the number of frames encoded, rather than
+     the index of the frame being encoded - so there is a difference of 1. */
+  pad->theora_has_zero_keyoffset =
+      ((vmaj << 16) | (vmin << 8) | vrev) < 0x030201;
+
   pad->caps = gst_caps_new_simple ("video/x-theora", NULL);
 
   if (w > 0 && h > 0) {
@@ -342,9 +425,6 @@ granulepos_to_granule_theora (GstOggStream * pad, gint64 granulepos)
   if (pad->granuleshift != 0) {
     keyindex = granulepos >> pad->granuleshift;
     keyoffset = granulepos - (keyindex << pad->granuleshift);
-    if (keyoffset == 0) {
-      pad->theora_has_zero_keyoffset = TRUE;
-    }
     if (pad->theora_has_zero_keyoffset) {
       keyoffset++;
     }
@@ -362,7 +442,7 @@ is_keyframe_theora (GstOggStream * pad, gint64 granulepos)
   if (granulepos == (gint64) - 1)
     return FALSE;
 
-  frame_mask = (1 << (pad->granuleshift + 1)) - 1;
+  frame_mask = (1 << pad->granuleshift) - 1;
 
   return ((granulepos & frame_mask) == 0);
 }
@@ -373,6 +453,22 @@ is_header_theora (GstOggStream * pad, ogg_packet * packet)
   return (packet->bytes > 0 && (packet->packet[0] & 0x80) == 0x80);
 }
 
+static void
+extract_tags_theora (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes > 0 && packet->packet[0] == 0x81) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\201theora", 7, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 /* dirac */
 
 static gboolean
@@ -388,6 +484,8 @@ setup_dirac_mapper (GstOggStream * pad, ogg_packet * packet)
     return FALSE;
   }
 
+  pad->is_video = TRUE;
+  pad->always_flush_page = TRUE;
   pad->granulerate_n = header.frame_rate_numerator * 2;
   pad->granulerate_d = header.frame_rate_denominator;
   pad->granuleshift = 22;
@@ -415,22 +513,16 @@ setup_dirac_mapper (GstOggStream * pad, ogg_packet * packet)
 static gboolean
 is_keyframe_dirac (GstOggStream * pad, gint64 granulepos)
 {
-  gint64 pt;
   int dist_h;
   int dist_l;
   int dist;
-  int delay;
-  gint64 dt;
 
   if (granulepos == -1)
     return -1;
 
-  pt = ((granulepos >> 22) + (granulepos & OGG_DIRAC_GRANULE_LOW_MASK)) >> 9;
   dist_h = (granulepos >> 22) & 0xff;
   dist_l = granulepos & 0xff;
   dist = (dist_h << 8) | dist_l;
-  delay = (granulepos >> 9) & 0x1fff;
-  dt = pt - delay;
 
   return (dist == 0);
 }
@@ -439,16 +531,10 @@ static gint64
 granulepos_to_granule_dirac (GstOggStream * pad, gint64 gp)
 {
   gint64 pt;
-  int dist_h;
-  int dist_l;
-  int dist;
   int delay;
   gint64 dt;
 
   pt = ((gp >> 22) + (gp & OGG_DIRAC_GRANULE_LOW_MASK)) >> 9;
-  dist_h = (gp >> 22) & 0xff;
-  dist_l = gp & 0xff;
-  dist = (dist_h << 8) | dist_l;
   delay = (gp >> 9) & 0x1fff;
   dt = pt - delay;
 
@@ -508,6 +594,7 @@ setup_vp8_mapper (GstOggStream * pad, ogg_packet * packet)
   fps_n = GST_READ_UINT32_BE (packet->packet + 18);
   fps_d = GST_READ_UINT32_BE (packet->packet + 22);
 
+  pad->is_video = TRUE;
   pad->is_vp8 = TRUE;
   pad->granulerate_n = fps_n;
   pad->granulerate_d = fps_d;
@@ -603,6 +690,15 @@ is_header_vp8 (GstOggStream * pad, ogg_packet * packet)
   return FALSE;
 }
 
+static void
+extract_tags_vp8 (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes >= 7 && memcmp (packet->packet, "OVP80\2 ", 7) == 0) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "OVP80\2 ", 7, &pad->taglist);
+  }
+}
+
 /* vorbis */
 
 static gboolean
@@ -611,17 +707,34 @@ setup_vorbis_mapper (GstOggStream * pad, ogg_packet * packet)
   guint8 *data = packet->packet;
   guint chans;
 
-  data += 1 + 6 + 4;
+  data += 1 + 6;
+  pad->version = GST_READ_UINT32_LE (data);
+  data += 4;
   chans = GST_READ_UINT8 (data);
   data += 1;
   pad->granulerate_n = GST_READ_UINT32_LE (data);
   pad->granulerate_d = 1;
   pad->granuleshift = 0;
+  pad->preroll = 2;
   pad->last_size = 0;
   GST_LOG ("sample rate: %d", pad->granulerate_n);
 
-  data += 8;
-  pad->bitrate = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate_upper = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate_nominal = GST_READ_UINT32_LE (data);
+  data += 4;
+  pad->bitrate_lower = GST_READ_UINT32_LE (data);
+
+  if (pad->bitrate_nominal > 0)
+    pad->bitrate = pad->bitrate_nominal;
+
+  if (pad->bitrate_upper > 0 && !pad->bitrate)
+    pad->bitrate = pad->bitrate_upper;
+
+  if (pad->bitrate_lower > 0 && !pad->bitrate)
+    pad->bitrate = pad->bitrate_lower;
+
   GST_LOG ("bit rate: %d", pad->bitrate);
 
   pad->n_header_packets = 3;
@@ -651,6 +764,40 @@ is_header_vorbis (GstOggStream * pad, ogg_packet * packet)
   return TRUE;
 }
 
+static void
+extract_tags_vorbis (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes == 0 || (packet->packet[0] & 0x01) == 0)
+    return;
+
+  if (((guint8 *) (packet->packet))[0] == 0x03) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\003vorbis", 7, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+        GST_TAG_ENCODER_VERSION, pad->version, NULL);
+
+    if (pad->bitrate_nominal > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_NOMINAL_BITRATE, (guint) pad->bitrate_nominal, NULL);
+
+    if (pad->bitrate_upper > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_MAXIMUM_BITRATE, (guint) pad->bitrate_upper, NULL);
+
+    if (pad->bitrate_lower > 0)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_MINIMUM_BITRATE, (guint) pad->bitrate_lower, NULL);
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 static gint64
 packet_duration_vorbis (GstOggStream * pad, ogg_packet * packet)
 {
@@ -658,7 +805,7 @@ packet_duration_vorbis (GstOggStream * pad, ogg_packet * packet)
   int size;
   int duration;
 
-  if (packet->packet[0] & 1)
+  if (packet->bytes == 0 || packet->packet[0] & 1)
     return 0;
 
   mode = (packet->packet[0] >> 1) & ((1 << pad->vorbis_log2_num_modes) - 1);
@@ -711,6 +858,22 @@ setup_speex_mapper (GstOggStream * pad, ogg_packet * packet)
   return TRUE;
 }
 
+static void
+extract_tags_count (GstOggStream * pad, ogg_packet * packet)
+{
+  /* packet 2 must be comment packet */
+  if (packet->bytes > 0 && pad->n_header_packets_seen == 1) {
+    tag_list_from_vorbiscomment_packet (packet, NULL, 0, &pad->taglist);
+
+    if (!pad->taglist)
+      pad->taglist = gst_tag_list_new ();
+
+    if (pad->bitrate)
+      gst_tag_list_add (pad->taglist, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, (guint) pad->bitrate, NULL);
+  }
+}
+
 
 /* flac */
 
@@ -820,6 +983,15 @@ packet_duration_flac (GstOggStream * pad, ogg_packet * packet)
   return -1;
 }
 
+static void
+extract_tags_flac (GstOggStream * pad, ogg_packet * packet)
+{
+  if (packet->bytes > 4 && ((packet->packet[0] & 0x7F) == 0x4)) {
+    tag_list_from_vorbiscomment_packet (packet,
+        packet->packet, 4, &pad->taglist);
+  }
+}
+
 /* fishead */
 
 static gboolean
@@ -920,7 +1092,7 @@ setup_fishead_mapper (GstOggStream * pad, ogg_packet * packet)
   pad->is_skeleton = TRUE;
   pad->is_sparse = TRUE;
 
-  pad->caps = gst_caps_new_simple ("none/none", NULL);
+  pad->caps = gst_caps_new_simple ("application/x-ogg-skeleton", NULL);
 
   return TRUE;
 }
@@ -932,12 +1104,15 @@ gst_ogg_map_parse_fisbone (GstOggStream * pad, const guint8 * data, guint size,
   GstOggSkeleton stype;
   guint serial_offset;
 
-  if (size < SKELETON_FISBONE_MIN_SIZE) {
+  if (size != 0 && size < SKELETON_FISBONE_MIN_SIZE) {
     GST_WARNING ("small fisbone packet of size %d, ignoring", size);
     return FALSE;
   }
 
-  if (memcmp (data, "fisbone\0", 8) == 0) {
+  if (size == 0) {
+    /* Skeleton EOS packet is zero bytes */
+    return FALSE;
+  } else if (memcmp (data, "fisbone\0", 8) == 0) {
     GST_INFO ("got fisbone packet");
     stype = GST_OGG_SKELETON_FISBONE;
     serial_offset = 12;
@@ -945,8 +1120,10 @@ gst_ogg_map_parse_fisbone (GstOggStream * pad, const guint8 * data, guint size,
     GST_INFO ("got index packet");
     stype = GST_OGG_SKELETON_INDEX;
     serial_offset = 6;
+  } else if (memcmp (data, "fishead\0", 8) == 0) {
+    return FALSE;
   } else {
-    GST_WARNING ("unknown skeleton packet %10.10s", data);
+    GST_WARNING ("unknown skeleton packet \"%10.10s\"", data);
     return FALSE;
   }
 
@@ -976,13 +1153,23 @@ gst_ogg_map_add_fisbone (GstOggStream * pad, GstOggStream * skel_pad,
 
   pad->have_fisbone = TRUE;
 
-  /* we just overwrite whatever was set before by the format-specific setup */
-  pad->granulerate_n = GST_READ_UINT64_LE (data);
-  pad->granulerate_d = GST_READ_UINT64_LE (data + 8);
+  /* We don't overwrite whatever was set before by the format-specific
+     setup: skeleton contains wrong information sometimes, and the codec
+     headers are authoritative.
+     So we only gather information that was not already filled out by
+     the mapper setup. This should hopefully allow handling unknown
+     streams a bit better, while not trashing correct setup from bad
+     skeleton data. */
+  if (pad->granulerate_n == 0 || pad->granulerate_d == 0) {
+    pad->granulerate_n = GST_READ_UINT64_LE (data);
+    pad->granulerate_d = GST_READ_UINT64_LE (data + 8);
+  }
+  if (pad->granuleshift < 0) {
+    pad->granuleshift = GST_READ_UINT8 (data + 28);
+  }
 
   start_granule = GST_READ_UINT64_LE (data + 16);
   pad->preroll = GST_READ_UINT32_LE (data + 24);
-  pad->granuleshift = GST_READ_UINT8 (data + 28);
 
   start_time = granulepos_to_granule_default (pad, start_granule);
 
@@ -1208,6 +1395,15 @@ is_header_ogm (GstOggStream * pad, ogg_packet * packet)
   return FALSE;
 }
 
+static void
+extract_tags_ogm (GstOggStream * pad, ogg_packet * packet)
+{
+  if (!(packet->packet[0] & 1) && (packet->packet[0] & 3 && pad->is_ogm_text)) {
+    tag_list_from_vorbiscomment_packet (packet,
+        (const guint8 *) "\003vorbis", 7, &pad->taglist);
+  }
+}
+
 static gint64
 packet_duration_ogm (GstOggStream * pad, ogg_packet * packet)
 {
@@ -1237,12 +1433,14 @@ setup_ogmaudio_mapper (GstOggStream * pad, ogg_packet * packet)
 {
   guint8 *data = packet->packet;
   guint32 fourcc;
+  gchar *fstr;
 
   pad->granulerate_n = GST_READ_UINT64_LE (data + 25);
   pad->granulerate_d = 1;
 
   fourcc = GST_READ_UINT32_LE (data + 9);
-  GST_DEBUG ("fourcc: %" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
+  fstr = g_strdup_printf ("%" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
+  GST_DEBUG ("fourcc: %s", fstr);
 
   pad->caps = gst_riff_create_audio_caps (fourcc, NULL, NULL, NULL, NULL, NULL);
 
@@ -1255,9 +1453,10 @@ setup_ogmaudio_mapper (GstOggStream * pad, ogg_packet * packet)
         "rate", G_TYPE_INT, pad->granulerate_n, NULL);
   } else {
     pad->caps = gst_caps_new_simple ("audio/x-ogm-unknown",
-        "fourcc", GST_TYPE_FOURCC, fourcc,
+        "fourcc", G_TYPE_STRING, fstr,
         "rate", G_TYPE_INT, pad->granulerate_n, NULL);
   }
+  g_free (fstr);
 
   pad->n_header_packets = 1;
   pad->is_ogm = TRUE;
@@ -1272,10 +1471,12 @@ setup_ogmvideo_mapper (GstOggStream * pad, ogg_packet * packet)
   guint32 fourcc;
   int width, height;
   gint64 time_unit;
+  gchar *fstr;
 
   GST_DEBUG ("time unit %d", GST_READ_UINT32_LE (data + 16));
   GST_DEBUG ("samples per unit %d", GST_READ_UINT32_LE (data + 24));
 
+  pad->is_video = TRUE;
   pad->granulerate_n = 10000000;
   time_unit = GST_READ_UINT64_LE (data + 17);
   if (time_unit > G_MAXINT || time_unit < G_MININT) {
@@ -1290,13 +1491,14 @@ setup_ogmvideo_mapper (GstOggStream * pad, ogg_packet * packet)
   fourcc = GST_READ_UINT32_LE (data + 9);
   width = GST_READ_UINT32_LE (data + 45);
   height = GST_READ_UINT32_LE (data + 49);
-  GST_DEBUG ("fourcc: %" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
+  fstr = g_strdup_printf ("%" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
+  GST_DEBUG ("fourcc: %s", fstr);
 
   pad->caps = gst_riff_create_video_caps (fourcc, NULL, NULL, NULL, NULL, NULL);
 
   if (pad->caps == NULL) {
     pad->caps = gst_caps_new_simple ("video/x-ogm-unknown",
-        "fourcc", GST_TYPE_FOURCC, fourcc,
+        "fourcc", G_TYPE_STRING, fstr,
         "framerate", GST_TYPE_FRACTION, pad->granulerate_n,
         pad->granulerate_d, NULL);
   } else {
@@ -1306,6 +1508,7 @@ setup_ogmvideo_mapper (GstOggStream * pad, ogg_packet * packet)
         "width", G_TYPE_INT, width, "height", G_TYPE_INT, height, NULL);
   }
   GST_DEBUG ("caps: %" GST_PTR_FORMAT, pad->caps);
+  g_free (fstr);
 
   pad->n_header_packets = 1;
   pad->frame_size = 1;
@@ -1386,56 +1589,36 @@ setup_pcm_mapper (GstOggStream * pad, ogg_packet * packet)
 
   switch (format) {
     case OGGPCM_FMT_S8:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 8,
-          "width", G_TYPE_INT, 8, "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S8", NULL);
       break;
     case OGGPCM_FMT_U8:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 8,
-          "width", G_TYPE_INT, 8, "signed", G_TYPE_BOOLEAN, FALSE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "U8", NULL);
       break;
     case OGGPCM_FMT_S16_LE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 16,
-          "width", G_TYPE_INT, 16,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S16LE", NULL);
       break;
     case OGGPCM_FMT_S16_BE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 16,
-          "width", G_TYPE_INT, 16,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S16BE", NULL);
       break;
     case OGGPCM_FMT_S24_LE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 24,
-          "width", G_TYPE_INT, 24,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S24LE", NULL);
       break;
     case OGGPCM_FMT_S24_BE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 24,
-          "width", G_TYPE_INT, 24,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S24BE", NULL);
       break;
     case OGGPCM_FMT_S32_LE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 32,
-          "width", G_TYPE_INT, 32,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S32LE", NULL);
       break;
     case OGGPCM_FMT_S32_BE:
-      caps = gst_caps_new_simple ("audio/x-raw-int",
-          "depth", G_TYPE_INT, 32,
-          "width", G_TYPE_INT, 32,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-          "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "S32BE", NULL);
       break;
     case OGGPCM_FMT_ULAW:
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
@@ -1444,30 +1627,26 @@ setup_pcm_mapper (GstOggStream * pad, ogg_packet * packet)
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
       break;
     case OGGPCM_FMT_FLT32_LE:
-      caps = gst_caps_new_simple ("audio/x-raw-float",
-          "width", G_TYPE_INT, 32,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "F32LE", NULL);
       break;
     case OGGPCM_FMT_FLT32_BE:
-      caps = gst_caps_new_simple ("audio/x-raw-float",
-          "width", G_TYPE_INT, 32,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "F32BE", NULL);
       break;
     case OGGPCM_FMT_FLT64_LE:
-      caps = gst_caps_new_simple ("audio/x-raw-float",
-          "width", G_TYPE_INT, 64,
-          "endianness", G_TYPE_INT, G_LITTLE_ENDIAN, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "F64LE", NULL);
       break;
     case OGGPCM_FMT_FLT64_BE:
-      caps = gst_caps_new_simple ("audio/x-raw-float",
-          "width", G_TYPE_INT, 64,
-          "endianness", G_TYPE_INT, G_BIG_ENDIAN, NULL);
+      caps = gst_caps_new_simple ("audio/x-raw",
+          "format", G_TYPE_STRING, "F64BE", NULL);
       break;
     default:
       return FALSE;
   }
 
-  gst_caps_set_simple (caps, "audio/x-raw-int",
+  gst_caps_set_simple (caps,
       "rate", G_TYPE_INT, pad->granulerate_n,
       "channels", G_TYPE_INT, channels, NULL);
   pad->caps = caps;
@@ -1497,6 +1676,7 @@ setup_cmml_mapper (GstOggStream * pad, ogg_packet * packet)
   GST_DEBUG ("blocksize1: %u", 1 << (data[0] & 0x0F));
 
   pad->caps = gst_caps_new_simple ("text/x-cmml", NULL);
+  pad->always_flush_page = TRUE;
   pad->is_sparse = TRUE;
 
   return TRUE;
@@ -1558,10 +1738,88 @@ setup_kate_mapper (GstOggStream * pad, ogg_packet * packet)
   }
 
   pad->is_sparse = TRUE;
+  pad->always_flush_page = TRUE;
 
   return TRUE;
 }
 
+static gint64
+packet_duration_kate (GstOggStream * pad, ogg_packet * packet)
+{
+  gint64 duration;
+
+  if (packet->bytes < 1)
+    return 0;
+
+  switch (packet->packet[0]) {
+    case 0x00:                 /* text data */
+      if (packet->bytes < 1 + 8 * 2) {
+        duration = 0;
+      } else {
+        duration = GST_READ_UINT64_LE (packet->packet + 1 + 8);
+        if (duration < 0)
+          duration = 0;
+      }
+      break;
+    default:
+      duration = GST_CLOCK_TIME_NONE;
+      break;
+  }
+
+  return duration;
+}
+
+static void
+extract_tags_kate (GstOggStream * pad, ogg_packet * packet)
+{
+  GstTagList *list = NULL;
+
+  if (packet->bytes <= 0)
+    return;
+
+  switch (packet->packet[0]) {
+    case 0x80:{
+      const gchar *canonical;
+      char language[16];
+
+      if (packet->bytes < 64) {
+        GST_WARNING ("Kate ID header packet is less than 64 bytes, ignored");
+        break;
+      }
+
+      /* the language tag is 16 bytes at offset 32, ensure NUL terminator */
+      memcpy (language, packet->packet + 32, 16);
+      language[15] = 0;
+
+      /* language is an ISO 639-1 code or RFC 3066 language code, we
+       * truncate to ISO 639-1 */
+      g_strdelimit (language, NULL, '\0');
+      canonical = gst_tag_get_language_code_iso_639_1 (language);
+      if (canonical) {
+        list = gst_tag_list_new_full (GST_TAG_LANGUAGE_CODE, canonical, NULL);
+      } else {
+        GST_WARNING ("Unknown or invalid language code %s, ignored", language);
+      }
+      break;
+    }
+    case 0x81:
+      tag_list_from_vorbiscomment_packet (packet,
+          (const guint8 *) "\201kate\0\0\0\0", 9, &list);
+      break;
+    default:
+      break;
+  }
+
+  if (list) {
+    if (pad->taglist) {
+      /* ensure the comment packet cannot override the category/language
+         from the identification header */
+      gst_tag_list_insert (pad->taglist, list, GST_TAG_MERGE_KEEP_ALL);
+    } else
+      pad->taglist = list;
+  }
+}
+
 
 /* *INDENT-OFF* */
 /* indent hates our freedoms */
@@ -1575,7 +1833,8 @@ const GstOggMap mappers[] = {
     is_keyframe_theora,
     is_header_theora,
     packet_duration_constant,
-    NULL
+    NULL,
+    extract_tags_theora
   },
   {
     "\001vorbis", 7, 22,
@@ -1586,7 +1845,8 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_vorbis,
     packet_duration_vorbis,
-    NULL
+    NULL,
+    extract_tags_vorbis
   },
   {
     "Speex", 5, 80,
@@ -1597,17 +1857,19 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_count,
     packet_duration_constant,
-    NULL
+    NULL,
+    extract_tags_count
   },
   {
     "PCM     ", 8, 0,
-    "audio/x-raw-int",
+    "audio/x-raw",
     setup_pcm_mapper,
     NULL,
     NULL,
     NULL,
     is_header_count,
     NULL,
+    NULL,
     NULL
   },
   {
@@ -1619,6 +1881,7 @@ const GstOggMap mappers[] = {
     NULL,
     is_header_count,
     NULL,
+    NULL,
     NULL
   },
   {
@@ -1630,6 +1893,7 @@ const GstOggMap mappers[] = {
     NULL,
     is_header_count,
     NULL,
+    NULL,
     NULL
   },
   {
@@ -1641,6 +1905,7 @@ const GstOggMap mappers[] = {
     NULL,
     is_header_true,
     NULL,
+    NULL,
     NULL
   },
   {
@@ -1652,6 +1917,7 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_fLaC,
     packet_duration_flac,
+    NULL,
     NULL
   },
   {
@@ -1663,7 +1929,8 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_flac,
     packet_duration_flac,
-    NULL
+    NULL,
+    extract_tags_flac
   },
   {
     "AnxData", 7, 0,
@@ -1673,6 +1940,7 @@ const GstOggMap mappers[] = {
     NULL,
     NULL,
     NULL,
+    NULL,
     NULL
   },
   {
@@ -1684,7 +1952,8 @@ const GstOggMap mappers[] = {
     NULL,
     is_header_count,
     packet_duration_constant,
-    NULL
+    NULL,
+    extract_tags_count
   },
   {
     "\200kate\0\0\0", 8, 0,
@@ -1694,8 +1963,9 @@ const GstOggMap mappers[] = {
     granule_to_granulepos_default,
     NULL,
     is_header_count,
+    packet_duration_kate,
     NULL,
-    NULL
+    extract_tags_kate
   },
   {
     "BBCD\0", 5, 13,
@@ -1706,7 +1976,8 @@ const GstOggMap mappers[] = {
     is_keyframe_dirac,
     is_header_count,
     packet_duration_constant,
-    granulepos_to_key_granule_dirac
+    granulepos_to_key_granule_dirac,
+    NULL
   },
   {
     "OVP80\1\1", 7, 4,
@@ -1717,7 +1988,8 @@ const GstOggMap mappers[] = {
     is_keyframe_vp8,
     is_header_vp8,
     packet_duration_vp8,
-    granulepos_to_key_granule_vp8
+    granulepos_to_key_granule_vp8,
+    extract_tags_vp8
   },
   {
     "\001audio\0\0\0", 9, 53,
@@ -1728,6 +2000,7 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_ogm,
     packet_duration_ogm,
+    NULL,
     NULL
   },
   {
@@ -1739,6 +2012,7 @@ const GstOggMap mappers[] = {
     NULL,
     is_header_ogm,
     packet_duration_constant,
+    NULL,
     NULL
   },
   {
@@ -1750,7 +2024,8 @@ const GstOggMap mappers[] = {
     is_keyframe_true,
     is_header_ogm,
     packet_duration_ogm,
-    NULL
+    NULL,
+    extract_tags_ogm
   }
 };
 /* *INDENT-ON* */
@@ -1786,3 +2061,71 @@ gst_ogg_stream_setup_map (GstOggStream * pad, ogg_packet * packet)
 
   return FALSE;
 }
+
+gboolean
+gst_ogg_stream_setup_map_from_caps_headers (GstOggStream * pad,
+    const GstCaps * caps)
+{
+  GstBuffer *buf;
+  const GstStructure *structure;
+  const GValue *streamheader;
+  const GValue *first_element;
+  ogg_packet packet;
+  guint8 *data;
+  gsize size;
+  gboolean ret;
+
+  GST_INFO ("Checking streamheader on caps %" GST_PTR_FORMAT, caps);
+
+  if (caps == NULL)
+    return FALSE;
+
+  structure = gst_caps_get_structure (caps, 0);
+  streamheader = gst_structure_get_value (structure, "streamheader");
+
+  if (streamheader == NULL) {
+    GST_LOG ("no streamheader field in caps %" GST_PTR_FORMAT, caps);
+    return FALSE;
+  }
+
+  if (!GST_VALUE_HOLDS_ARRAY (streamheader)) {
+    GST_ERROR ("streamheader field not an array, caps: %" GST_PTR_FORMAT, caps);
+    return FALSE;
+  }
+
+  if (gst_value_array_get_size (streamheader) == 0) {
+    GST_ERROR ("empty streamheader field in caps %" GST_PTR_FORMAT, caps);
+    return FALSE;
+  }
+
+  first_element = gst_value_array_get_value (streamheader, 0);
+
+  if (!GST_VALUE_HOLDS_BUFFER (first_element)) {
+    GST_ERROR ("first streamheader not a buffer, caps: %" GST_PTR_FORMAT, caps);
+    return FALSE;
+  }
+
+  buf = gst_value_get_buffer (first_element);
+  if (buf == NULL) {
+    GST_ERROR ("no first streamheader buffer");
+    return FALSE;
+  }
+
+  data = gst_buffer_map (buf, &size, 0, GST_MAP_READ);
+  if (data == NULL || size == 0) {
+    GST_ERROR ("invalid first streamheader buffer");
+    return FALSE;
+  }
+
+  GST_MEMDUMP ("streamheader", data, size);
+
+  packet.packet = data;
+  packet.bytes = size;
+
+  GST_INFO ("Found headers on caps, using those to determine type");
+  ret = gst_ogg_stream_setup_map (pad, &packet);
+
+  gst_buffer_unmap (buf, data, size);
+
+  return ret;
+}
