@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <string.h>
 
 #include <gst/gst.h>
@@ -29,6 +30,8 @@
 #include <gst/riff/riff-media.h>
 #include <gst/riff/riff-read.h>
 
+#include "gstogg.h"
+
 GST_DEBUG_CATEGORY_STATIC (gst_ogm_parse_debug);
 #define GST_CAT_DEFAULT gst_ogm_parse_debug
 
@@ -56,24 +59,6 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogm_parse_debug);
 #define GST_OGM_PARSE_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_OGM_PARSE, GstOgmParseClass))
 
-static const GstElementDetails gst_ogm_audio_parse_details =
-GST_ELEMENT_DETAILS ("OGM audio stream parser",
-    "Codec/Decoder/Audio",
-    "parse an OGM audio header and stream",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-
-static const GstElementDetails gst_ogm_video_parse_details =
-GST_ELEMENT_DETAILS ("OGM video stream parser",
-    "Codec/Decoder/Video",
-    "parse an OGM video header and stream",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-
-static const GstElementDetails gst_ogm_text_parse_details =
-GST_ELEMENT_DETAILS ("OGM text stream parser",
-    "Codec/Decoder/Subtitle",
-    "parse an OGM text header and stream",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-
 typedef struct _stream_header_video
 {
   gint32 width;
@@ -289,13 +274,17 @@ gst_ogm_audio_parse_base_init (GstOgmParseClass * klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstCaps *caps = gst_riff_create_audio_template_caps ();
 
-  gst_element_class_set_details (element_class, &gst_ogm_audio_parse_details);
+  gst_element_class_set_details_simple (element_class,
+      "OGM audio stream parser", "Codec/Decoder/Audio",
+      "parse an OGM audio header and stream",
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory_audio));
   audio_src_templ = gst_pad_template_new ("src",
       GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, audio_src_templ);
+  gst_caps_unref (caps);
 }
 
 static void
@@ -304,13 +293,17 @@ gst_ogm_video_parse_base_init (GstOgmParseClass * klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstCaps *caps = gst_riff_create_video_template_caps ();
 
-  gst_element_class_set_details (element_class, &gst_ogm_video_parse_details);
+  gst_element_class_set_details_simple (element_class,
+      "OGM video stream parser", "Codec/Decoder/Video",
+      "parse an OGM video header and stream",
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory_video));
   video_src_templ = gst_pad_template_new ("src",
       GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, video_src_templ);
+  gst_caps_unref (caps);
 }
 
 static void
@@ -319,13 +312,17 @@ gst_ogm_text_parse_base_init (GstOgmParseClass * klass)
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstCaps *caps = gst_caps_new_simple ("text/plain", NULL, NULL);
 
-  gst_element_class_set_details (element_class, &gst_ogm_text_parse_details);
+  gst_element_class_set_details_simple (element_class,
+      "OGM text stream parser", "Codec/Decoder/Subtitle",
+      "parse an OGM text header and stream",
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory_text));
   text_src_templ = gst_pad_template_new ("src",
       GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
   gst_element_class_add_pad_template (element_class, text_src_templ);
+  gst_caps_unref (caps);
 }
 
 static void
@@ -577,6 +574,7 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
     }
     case 'v':{
       guint32 fourcc;
+      gint time_unit;
 
       fourcc = GST_MAKE_FOURCC (ogm->hdr.subtype[0],
           ogm->hdr.subtype[1], ogm->hdr.subtype[2], ogm->hdr.subtype[3]);
@@ -587,7 +585,7 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
         GST_WARNING_OBJECT (ogm, "could not find video caps for fourcc %"
             GST_FOURCC_FORMAT, GST_FOURCC_ARGS (fourcc));
         caps = gst_caps_new_simple ("video/x-ogm-unknown", "fourcc",
-            GST_TYPE_FOURCC, fourcc, NULL);
+            G_TYPE_STRING, ogm->hdr.subtype, NULL);
         break;
       }
 
@@ -601,10 +599,15 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
           ogm->hdr.samples_per_unit, ogm->hdr.default_len,
           ogm->hdr.buffersize, ogm->hdr.bits_per_sample, caps);
 
+      /* GST_TYPE_FRACTION contains gint */
+      if (ogm->hdr.time_unit > G_MAXINT || ogm->hdr.time_unit < G_MININT)
+        GST_WARNING_OBJECT (ogm, "timeunit is out of range");
+
+      time_unit = (gint) CLAMP (ogm->hdr.time_unit, G_MININT, G_MAXINT);
       gst_caps_set_simple (caps,
           "width", G_TYPE_INT, ogm->hdr.s.video.width,
           "height", G_TYPE_INT, ogm->hdr.s.video.height,
-          "framerate", GST_TYPE_FRACTION, 10000000, ogm->hdr.time_unit, NULL);
+          "framerate", GST_TYPE_FRACTION, 10000000, time_unit, NULL);
       break;
     }
     case 't':{
@@ -622,16 +625,19 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
     goto cannot_decode;
 
   if (ogm->srcpad) {
-    GstCaps *current_caps = GST_PAD_CAPS (ogm->srcpad);
+    GstCaps *current_caps = gst_pad_get_current_caps (ogm->srcpad);
 
-    if (current_caps && caps && !gst_caps_is_equal (current_caps, caps)) {
-      GST_WARNING_OBJECT (ogm, "Already an existing pad %s:%s",
-          GST_DEBUG_PAD_NAME (ogm->srcpad));
-      gst_pad_set_active (ogm->srcpad, FALSE);
-      gst_element_remove_pad (GST_ELEMENT (ogm), ogm->srcpad);
-      ogm->srcpad = NULL;
-    } else {
-      GST_DEBUG_OBJECT (ogm, "Existing pad has the same caps, do nothing");
+    if (current_caps) {
+      if (caps && !gst_caps_is_equal (current_caps, caps)) {
+        GST_WARNING_OBJECT (ogm, "Already an existing pad %s:%s",
+            GST_DEBUG_PAD_NAME (ogm->srcpad));
+        gst_pad_set_active (ogm->srcpad, FALSE);
+        gst_element_remove_pad (GST_ELEMENT (ogm), ogm->srcpad);
+        ogm->srcpad = NULL;
+      } else {
+        GST_DEBUG_OBJECT (ogm, "Existing pad has the same caps, do nothing");
+      }
+      gst_caps_unref (current_caps);
     }
   }
 
@@ -640,8 +646,8 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
 
     ogm->srcpad = gst_pad_new_from_template (ogm->srcpadtempl, "src");
     gst_pad_use_fixed_caps (ogm->srcpad);
-    gst_pad_set_caps (ogm->srcpad, caps);
     gst_pad_set_active (ogm->srcpad, TRUE);
+    gst_pad_set_caps (ogm->srcpad, caps);
     gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
     GST_INFO_OBJECT (ogm, "Added pad %s:%s with caps %" GST_PTR_FORMAT,
         GST_DEBUG_PAD_NAME (ogm->srcpad), caps);
@@ -658,6 +664,15 @@ gst_ogm_parse_stream_header (GstOgmParse * ogm, const guint8 * data, guint size)
       gst_pad_push_event (ogm->srcpad, event);
     }
     g_list_free (cached_events);
+
+    {
+      GstTagList *tags;
+
+      tags = gst_tag_list_new ();
+      gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_SUBTITLE_CODEC,
+          "Ogm", NULL);
+      gst_element_found_tags_for_pad (GST_ELEMENT (ogm), ogm->srcpad, tags);
+    }
   }
 
   gst_caps_unref (caps);
@@ -705,8 +720,6 @@ gst_ogm_parse_comment_packet (GstOgmParse * ogm, GstBuffer * buf)
     /* do not push packet downstream, just let parent unref it */
     ret = GST_FLOW_OK;
   } else {
-    buf = gst_buffer_copy (buf);
-    gst_buffer_set_caps (buf, GST_PAD_CAPS (ogm->srcpad));
     ret = gst_pad_push (ogm->srcpad, buf);
   }
 
@@ -716,32 +729,27 @@ gst_ogm_parse_comment_packet (GstOgmParse * ogm, GstBuffer * buf)
 static void
 gst_ogm_text_parse_strip_trailing_zeroes (GstOgmParse * ogm, GstBuffer * buf)
 {
-  const guint8 *data;
-  guint size;
+  guint8 *data;
+  gsize size;
 
-  g_assert (gst_buffer_is_metadata_writable (buf));
+  g_assert (gst_buffer_is_writable (buf));
 
   /* zeroes are not valid UTF-8 characters, so strip them from output */
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
+  data = gst_buffer_map (buf, &size, NULL, GST_MAP_WRITE);
   while (size > 0 && data[size - 1] == '\0') {
     --size;
   }
-
-  GST_BUFFER_SIZE (buf) = size;
+  gst_buffer_unmap (buf, data, size);
 }
 
 static GstFlowReturn
-gst_ogm_parse_data_packet (GstOgmParse * ogm, GstBuffer * buf)
+gst_ogm_parse_data_packet (GstOgmParse * ogm, GstBuffer * buf,
+    const guint8 * data, gsize size)
 {
   GstFlowReturn ret;
-  const guint8 *data;
   GstBuffer *sbuf;
   gboolean keyframe;
-  guint size, len, n, xsize = 0;
-
-  data = GST_BUFFER_DATA (buf);
-  size = GST_BUFFER_SIZE (buf);
+  guint len, n, xsize = 0;
 
   if ((data[0] & 0x01) != 0)
     goto invalid_startcode;
@@ -757,10 +765,12 @@ gst_ogm_parse_data_packet (GstOgmParse * ogm, GstBuffer * buf)
     xsize = (xsize << 8) | data[n];
   }
 
-  GST_LOG_OBJECT (ogm, "[0x%02x] samples: %d, hdrbytes: %d, datasize: %d",
-      data[0], xsize, len, size - len - 1);
+  GST_LOG_OBJECT (ogm, "[0x%02x] samples: %d, hdrbytes: %d, datasize: %"
+      G_GSIZE_FORMAT, data[0], xsize, len, size - len - 1);
 
-  sbuf = gst_buffer_create_sub (buf, len + 1, size - len - 1);
+  sbuf =
+      gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, len + 1,
+      size - len - 1);
 
   if (GST_BUFFER_OFFSET_END_IS_VALID (buf))
     ogm->next_granulepos = GST_BUFFER_OFFSET_END (buf);
@@ -814,7 +824,6 @@ gst_ogm_parse_data_packet (GstOgmParse * ogm, GstBuffer * buf)
   }
 
   if (ogm->srcpad) {
-    gst_buffer_set_caps (sbuf, GST_PAD_CAPS (ogm->srcpad));
     GST_LOG_OBJECT (ogm, "Pushing buffer with ts=%" GST_TIME_FORMAT,
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (sbuf)));
     ret = gst_pad_push (ogm->srcpad, sbuf);
@@ -838,7 +847,7 @@ invalid_startcode:
 buffer_too_small:
   {
     GST_ELEMENT_ERROR (ogm, STREAM, DECODE, (NULL),
-        ("buffer too small, len+1=%u, size=%u", len + 1, size));
+        ("buffer too small, len+1=%u, size=%" G_GSIZE_FORMAT, len + 1, size));
     return GST_FLOW_ERROR;
   }
 }
@@ -848,9 +857,10 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buf)
 {
   GstFlowReturn ret = GST_FLOW_OK;
   GstOgmParse *ogm = GST_OGM_PARSE (GST_PAD_PARENT (pad));
-  guint8 *data = GST_BUFFER_DATA (buf);
-  guint size = GST_BUFFER_SIZE (buf);
+  guint8 *data;
+  gsize size;
 
+  data = gst_buffer_map (buf, &size, NULL, GST_MAP_READ);
   if (size < 1)
     goto buffer_too_small;
 
@@ -866,11 +876,12 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buf)
       break;
     }
     default:{
-      ret = gst_ogm_parse_data_packet (ogm, buf);
+      ret = gst_ogm_parse_data_packet (ogm, buf, data, size);
       break;
     }
   }
 
+  gst_buffer_unmap (buf, data, size);
   gst_buffer_unref (buf);
 
   if (ret != GST_FLOW_OK) {
@@ -883,6 +894,7 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buf)
 buffer_too_small:
   {
     GST_ELEMENT_ERROR (ogm, STREAM, DECODE, (NULL), ("buffer too small"));
+    gst_buffer_unmap (buf, data, size);
     gst_buffer_unref (buf);
     return GST_FLOW_ERROR;
   }
