@@ -1,6 +1,7 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  * Copyright (C) <2003> David Schleef <ds@schleef.org>
+ * Copyright (C) <2009> Young-Ho Cha <ganadist@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -46,16 +47,24 @@
 #include <gst/video/video.h>
 
 #include "gsttextrender.h"
+#include <string.h>
+
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+# define CAIRO_ARGB_A 3
+# define CAIRO_ARGB_R 2
+# define CAIRO_ARGB_G 1
+# define CAIRO_ARGB_B 0
+#else
+# define CAIRO_ARGB_A 0
+# define CAIRO_ARGB_R 1
+# define CAIRO_ARGB_G 2
+# define CAIRO_ARGB_B 3
+#endif
 
 GST_DEBUG_CATEGORY_EXTERN (pango_debug);
 #define GST_CAT_DEFAULT pango_debug
 
-static const GstElementDetails text_render_details =
-GST_ELEMENT_DETAILS ("Text renderer",
-    "Filter/Editor/Video",
-    "Renders a text string to an image bitmap",
-    "David Schleef <ds@schleef.org>, "
-    "Ronald S. Bultje <rbultje@ronald.bitfreak.net>");
+#define MINIMUM_OUTLINE_OFFSET 1.0
 
 #define DEFAULT_PROP_VALIGNMENT GST_TEXT_RENDER_VALIGN_BASELINE
 #define DEFAULT_PROP_HALIGNMENT GST_TEXT_RENDER_HALIGN_CENTER
@@ -68,21 +77,22 @@ GST_ELEMENT_DETAILS ("Text renderer",
 
 enum
 {
-  ARG_0,
+  PROP_0,
   PROP_HALIGNMENT,
   PROP_VALIGNMENT,
   PROP_LINE_ALIGNMENT,
   PROP_XPAD,
   PROP_YPAD,
-  ARG_FONT_DESC
+  PROP_FONT_DESC
 };
 
+#define VIDEO_FORMATS "{ AYUV, ARGB } "
 
 static GstStaticPadTemplate src_template_factory =
-    GST_STATIC_PAD_TEMPLATE ("src",
+GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") ";" GST_VIDEO_CAPS_ARGB)
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE (VIDEO_FORMATS))
     );
 
 static GstStaticPadTemplate sink_template_factory =
@@ -150,7 +160,11 @@ gst_text_render_line_align_get_type (void)
   return text_render_line_align_type;
 }
 
-GST_BOILERPLATE (GstTextRender, gst_text_render, GstElement, GST_TYPE_ELEMENT);
+static void gst_text_render_adjust_values_with_fontdesc (GstTextRender *
+    render, PangoFontDescription * desc);
+
+#define gst_text_render_parent_class parent_class
+G_DEFINE_TYPE (GstTextRender, gst_text_render, GST_TYPE_ELEMENT);
 
 static void gst_text_render_finalize (GObject * object);
 static void gst_text_render_set_property (GObject * object,
@@ -158,24 +172,12 @@ static void gst_text_render_set_property (GObject * object,
 static void gst_text_render_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
-static void
-gst_text_render_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template_factory));
-
-  gst_element_class_set_details (element_class, &text_render_details);
-}
-
 static void
 gst_text_render_class_init (GstTextRenderClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
+  PangoFontMap *fontmap;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -186,8 +188,21 @@ gst_text_render_class_init (GstTextRenderClass * klass)
   gobject_class->set_property = gst_text_render_set_property;
   gobject_class->get_property = gst_text_render_get_property;
 
-  klass->pango_context = pango_ft2_get_context (72, 72);
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FONT_DESC,
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_template_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template_factory));
+
+  gst_element_class_set_details_simple (gstelement_class, "Text renderer",
+      "Filter/Editor/Video",
+      "Renders a text string to an image bitmap",
+      "David Schleef <ds@schleef.org>, "
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
+
+  fontmap = pango_cairo_font_map_get_default ();
+  klass->pango_context =
+      pango_cairo_font_map_create_context (PANGO_CAIRO_FONT_MAP (fontmap));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FONT_DESC,
       g_param_spec_string ("font-desc", "font description",
           "Pango font description of font "
           "to be used for rendering. "
@@ -217,47 +232,85 @@ gst_text_render_class_init (GstTextRenderClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
-
 static void
-resize_bitmap (GstTextRender * render, gint width, gint height)
+gst_text_render_adjust_values_with_fontdesc (GstTextRender * render,
+    PangoFontDescription * desc)
 {
-  FT_Bitmap *bitmap = &render->bitmap;
-  gint pitch = (width | 3) + 1;
-  gint size = pitch * height;
+  gint font_size = pango_font_description_get_size (desc) / PANGO_SCALE;
 
-  /* no need to keep reallocating; just keep the maximum size so far */
-  if (size <= render->bitmap_buffer_size) {
-    bitmap->rows = height;
-    bitmap->width = width;
-    bitmap->pitch = pitch;
-    memset (bitmap->buffer, 0, render->bitmap_buffer_size);
-    return;
-  }
-  if (!bitmap->buffer) {
-    /* initialize */
-    bitmap->pixel_mode = ft_pixel_mode_grays;
-    bitmap->num_grays = 256;
-  }
-  if (bitmap->buffer)
-    bitmap->buffer = g_realloc (bitmap->buffer, size);
-  else
-    bitmap->buffer = g_malloc (size);
-  bitmap->rows = height;
-  bitmap->width = width;
-  bitmap->pitch = pitch;
-  memset (bitmap->buffer, 0, size);
-  render->bitmap_buffer_size = size;
+  render->shadow_offset = (double) (font_size) / 13.0;
+  render->outline_offset = (double) (font_size) / 15.0;
+  if (render->outline_offset < MINIMUM_OUTLINE_OFFSET)
+    render->outline_offset = MINIMUM_OUTLINE_OFFSET;
 }
 
 static void
-gst_text_render_render_text (GstTextRender * render)
+gst_text_render_render_pangocairo (GstTextRender * render)
 {
+  cairo_t *cr;
+  cairo_surface_t *surface;
+  cairo_t *cr_shadow;
+  cairo_surface_t *surface_shadow;
   PangoRectangle ink_rect, logical_rect;
+  gint width, height;
 
   pango_layout_get_pixel_extents (render->layout, &ink_rect, &logical_rect);
-  resize_bitmap (render, ink_rect.width, ink_rect.height + ink_rect.y);
-  pango_ft2_render_layout (&render->bitmap, render->layout, -ink_rect.x, 0);
-  render->baseline_y = ink_rect.y;
+
+  width = logical_rect.width + render->shadow_offset;
+  height = logical_rect.height + logical_rect.y + render->shadow_offset;
+
+  surface_shadow = cairo_image_surface_create (CAIRO_FORMAT_A8, width, height);
+  cr_shadow = cairo_create (surface_shadow);
+
+  /* clear shadow surface */
+  cairo_set_operator (cr_shadow, CAIRO_OPERATOR_CLEAR);
+  cairo_paint (cr_shadow);
+  cairo_set_operator (cr_shadow, CAIRO_OPERATOR_OVER);
+
+  /* draw shadow text */
+  cairo_save (cr_shadow);
+  cairo_set_source_rgba (cr_shadow, 0.0, 0.0, 0.0, 0.5);
+  cairo_translate (cr_shadow, render->shadow_offset, render->shadow_offset);
+  pango_cairo_show_layout (cr_shadow, render->layout);
+  cairo_restore (cr_shadow);
+
+  /* draw outline text */
+  cairo_save (cr_shadow);
+  cairo_set_source_rgb (cr_shadow, 0.0, 0.0, 0.0);
+  cairo_set_line_width (cr_shadow, render->outline_offset);
+  pango_cairo_layout_path (cr_shadow, render->layout);
+  cairo_stroke (cr_shadow);
+  cairo_restore (cr_shadow);
+
+  cairo_destroy (cr_shadow);
+
+  render->text_image = g_realloc (render->text_image, 4 * width * height);
+
+  surface = cairo_image_surface_create_for_data (render->text_image,
+      CAIRO_FORMAT_ARGB32, width, height, width * 4);
+  cr = cairo_create (surface);
+  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
+  cairo_paint (cr);
+  cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+
+  /* set default color */
+  cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+
+  cairo_save (cr);
+  /* draw text */
+  pango_cairo_show_layout (cr, render->layout);
+  cairo_restore (cr);
+
+  /* composite shadow with offset */
+  cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OVER);
+  cairo_set_source_surface (cr, surface_shadow, 0.0, 0.0);
+  cairo_paint (cr);
+
+  cairo_destroy (cr);
+  cairo_surface_destroy (surface_shadow);
+  cairo_surface_destroy (surface);
+  render->image_width = width;
+  render->image_height = height;
 }
 
 static void
@@ -274,47 +327,52 @@ gst_text_render_check_argb (GstTextRender * render)
 
     /* Check if AYUV or ARGB is first */
     for (i = 0; i < n; i++) {
-      GstStructure *s = gst_caps_get_structure (peer_caps, i);
-      if (gst_structure_has_name (s, "video/x-raw-rgb") &&
-          gst_structure_has_field (s, "alpha_mask")) {
-        render->use_ARGB = TRUE;
-        break;
-      } else if (gst_structure_has_name (s, "video/x-raw-yuv")) {
-        guint fourcc;
-        if (gst_structure_get_fourcc (s, "format", &fourcc) &&
-            fourcc == GST_MAKE_FOURCC ('A', 'Y', 'U', 'V')) {
-          render->use_ARGB = FALSE;
-          break;
-        }
-      }
+      GstStructure *s;
+      GstVideoFormat vformat;
+      const GstVideoFormatInfo *info;
+      const gchar *fmt;
+
+      s = gst_caps_get_structure (peer_caps, i);
+      if (!gst_structure_has_name (s, "video/x-raw"))
+        continue;
+
+      fmt = gst_structure_get_string (s, "format");
+      if (fmt == NULL)
+        continue;
+
+      vformat = gst_video_format_from_string (fmt);
+      info = gst_video_format_get_info (vformat);
+      if (info == NULL)
+        continue;
+
+      render->use_ARGB = GST_VIDEO_FORMAT_INFO_HAS_ALPHA (info);
     }
     gst_caps_unref (peer_caps);
   }
 }
 
 static gboolean
-gst_text_render_setcaps (GstPad * pad, GstCaps * caps)
+gst_text_render_src_setcaps (GstTextRender * render, GstCaps * caps)
 {
-  GstTextRender *render = GST_TEXT_RENDER (gst_pad_get_parent (pad));
   GstStructure *structure;
-  gboolean ret = FALSE;
+  gboolean ret;
   gint width = 0, height = 0;
 
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_get_int (structure, "width", &width);
   gst_structure_get_int (structure, "height", &height);
 
-  GST_DEBUG ("Got caps %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (render, "Got caps %" GST_PTR_FORMAT, caps);
 
-  if (width >= render->bitmap.width && height >= render->bitmap.rows) {
+  if (width >= render->image_width && height >= render->image_height) {
     render->width = width;
     render->height = height;
-    ret = TRUE;
   }
 
   gst_text_render_check_argb (render);
 
-  gst_object_unref (render);
+  ret = gst_pad_push_event (render->srcpad, gst_event_new_caps (caps));
+
   return ret;
 }
 
@@ -325,92 +383,84 @@ gst_text_render_fixate_caps (GstPad * pad, GstCaps * caps)
   GstStructure *s = gst_caps_get_structure (caps, 0);
 
   GST_DEBUG ("Fixating caps %" GST_PTR_FORMAT, caps);
-  gst_structure_fixate_field_nearest_int (s, "width", render->width);
-  gst_structure_fixate_field_nearest_int (s, "height", render->height);
+  gst_structure_fixate_field_nearest_int (s, "width", MAX (render->image_width,
+          DEFAULT_RENDER_WIDTH));
+  gst_structure_fixate_field_nearest_int (s, "height",
+      MAX (render->image_height + render->ypad, DEFAULT_RENDER_HEIGHT));
   GST_DEBUG ("Fixated to    %" GST_PTR_FORMAT, caps);
 
   gst_object_unref (render);
 }
 
+#define CAIRO_UNPREMULTIPLY(a,r,g,b) G_STMT_START { \
+  b = (a > 0) ? MIN ((b * 255 + a / 2) / a, 255) : 0; \
+  g = (a > 0) ? MIN ((g * 255 + a / 2) / a, 255) : 0; \
+  r = (a > 0) ? MIN ((r * 255 + a / 2) / a, 255) : 0; \
+} G_STMT_END
+
 static void
-gst_text_renderer_bitmap_to_ayuv (GstTextRender * render, FT_Bitmap * bitmap,
-    guchar * pixbuf, gint x0, gint x1, gint y0, gint y1)
+gst_text_renderer_image_to_ayuv (GstTextRender * render, guchar * pixbuf,
+    int xpos, int ypos, int stride)
 {
   int y;                        /* text bitmap coordinates */
-  int rowinc, bit_rowinc;
   guchar *p, *bitp;
-  guchar v;
+  guchar a, r, g, b;
+  int width, height;
 
-  x0 = CLAMP (x0, 0, render->width);
-  x1 = CLAMP (x1, 0, render->width);
+  width = render->image_width;
+  height = render->image_height;
 
-  y0 = CLAMP (y0, 0, render->height);
-  y1 = CLAMP (y1, 0, render->height);
-
-
-  rowinc = render->width - bitmap->width;
-  bit_rowinc = bitmap->pitch - bitmap->width;
-
-  bitp = bitmap->buffer;
-  p = pixbuf + ((x0 + (render->width * y0)) * 4);
-
-  for (y = y0; y < y1; y++) {
+  for (y = 0; y < height && ypos + y < render->height; y++) {
     int n;
+    p = pixbuf + (ypos + y) * stride + xpos * 4;
+    bitp = render->text_image + y * width * 4;
+    for (n = 0; n < width && n < render->width; n++) {
+      b = bitp[CAIRO_ARGB_B];
+      g = bitp[CAIRO_ARGB_G];
+      r = bitp[CAIRO_ARGB_R];
+      a = bitp[CAIRO_ARGB_A];
+      bitp += 4;
 
-    for (n = x0; n < x1; n++) {
-      v = *bitp;
-      if (v) {
-        p[0] = v;
-        p[1] = 255;
-        p[2] = 0x80;
-        p[3] = 0x80;
-      }
-      p += 4;
-      bitp++;
+      /* Cairo uses pre-multiplied ARGB, unpremultiply it */
+      CAIRO_UNPREMULTIPLY (a, r, g, b);
+
+      *p++ = a;
+      *p++ = CLAMP ((int) (((19595 * r) >> 16) + ((38470 * g) >> 16) +
+              ((7471 * b) >> 16)), 0, 255);
+      *p++ = CLAMP ((int) (-((11059 * r) >> 16) - ((21709 * g) >> 16) +
+              ((32768 * b) >> 16) + 128), 0, 255);
+      *p++ = CLAMP ((int) (((32768 * r) >> 16) - ((27439 * g) >> 16) -
+              ((5329 * b) >> 16) + 128), 0, 255);
     }
-    p += rowinc * 4;
-    bitp += bit_rowinc;
   }
 }
 
 static void
-gst_text_renderer_bitmap_to_argb (GstTextRender * render, FT_Bitmap * bitmap,
-    guchar * pixbuf, gint x0, gint x1, gint y0, gint y1)
+gst_text_renderer_image_to_argb (GstTextRender * render, guchar * pixbuf,
+    int xpos, int ypos, int stride)
 {
-  int y;                        /* text bitmap coordinates */
-  int rowinc, bit_rowinc;
+  int i, j;
   guchar *p, *bitp;
-  guchar v;
+  int width, height;
 
-  x0 = CLAMP (x0, 0, render->width);
-  x1 = CLAMP (x1, 0, render->width);
+  width = render->image_width;
+  height = render->image_height;
 
-  y0 = CLAMP (y0, 0, render->height);
-  y1 = CLAMP (y1, 0, render->height);
+  for (i = 0; i < height && ypos + i < render->height; i++) {
+    p = pixbuf + (ypos + i) * stride + xpos * 4;
+    bitp = render->text_image + i * width * 4;
+    for (j = 0; j < width && j < render->width; j++) {
+      p[0] = bitp[CAIRO_ARGB_A];
+      p[1] = bitp[CAIRO_ARGB_R];
+      p[2] = bitp[CAIRO_ARGB_G];
+      p[3] = bitp[CAIRO_ARGB_B];
 
+      /* Cairo uses pre-multiplied ARGB, unpremultiply it */
+      CAIRO_UNPREMULTIPLY (p[0], p[1], p[2], p[3]);
 
-  rowinc = render->width - bitmap->width;
-  bit_rowinc = bitmap->pitch - bitmap->width;
-
-  bitp = bitmap->buffer;
-  p = pixbuf + ((x0 + (render->width * y0)) * 4);
-
-  for (y = y0; y < y1; y++) {
-    int n;
-
-    for (n = x0; n < x1; n++) {
-      v = *bitp;
-      if (v) {
-        p[0] = v;
-        p[1] = 255;
-        p[2] = 255;
-        p[3] = 255;
-      }
+      bitp += 4;
       p += 4;
-      bitp++;
     }
-    p += rowinc * 4;
-    bitp += bit_rowinc;
   }
 }
 
@@ -420,14 +470,16 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
   GstTextRender *render;
   GstFlowReturn ret;
   GstBuffer *outbuf;
-  GstCaps *caps = NULL;
-  guint8 *data = GST_BUFFER_DATA (inbuf);
-  guint size = GST_BUFFER_SIZE (inbuf);
+  GstCaps *caps = NULL, *padcaps;
+  guint8 *data;
+  gsize size;
   gint n;
   gint xpos, ypos;
 
   render = GST_TEXT_RENDER (gst_pad_get_parent (pad));
 
+  data = gst_buffer_map (inbuf, &size, NULL, GST_MAP_READ);
+
   /* somehow pango barfs over "\0" buffers... */
   while (size > 0 &&
       (data[size - 1] == '\r' ||
@@ -436,45 +488,40 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
   }
 
   /* render text */
-  GST_DEBUG ("rendering '%*s'", size, data);
+  GST_DEBUG ("rendering '%*s'", (gint) size, data);
   pango_layout_set_markup (render->layout, (gchar *) data, size);
-  gst_text_render_render_text (render);
+  gst_text_render_render_pangocairo (render);
+  gst_buffer_unmap (inbuf, data, size);
 
   gst_text_render_check_argb (render);
 
-  if (!render->use_ARGB) {
-    caps =
-        gst_video_format_new_caps (GST_VIDEO_FORMAT_AYUV, render->width,
-        render->height, 1, 1, 1, 1);
-  } else {
-    caps =
-        gst_video_format_new_caps (GST_VIDEO_FORMAT_ARGB, render->width,
-        render->height, 1, 1, 1, 1);
+  padcaps = gst_pad_get_caps (render->srcpad, NULL);
+  caps = gst_pad_peer_get_caps (render->srcpad, padcaps);
+  gst_caps_unref (padcaps);
+
+  if (!caps || gst_caps_is_empty (caps)) {
+    GST_ELEMENT_ERROR (render, CORE, NEGOTIATION, (NULL), (NULL));
+    ret = GST_FLOW_ERROR;
+    goto done;
   }
 
-  if (!gst_pad_set_caps (render->srcpad, caps)) {
-    gst_caps_unref (caps);
+  gst_caps_truncate (caps);
+  gst_pad_fixate_caps (render->srcpad, caps);
+
+  if (!gst_text_render_src_setcaps (render, caps)) {
     GST_ELEMENT_ERROR (render, CORE, NEGOTIATION, (NULL), (NULL));
     ret = GST_FLOW_ERROR;
     goto done;
   }
 
   GST_DEBUG ("Allocating buffer WxH = %dx%d", render->width, render->height);
-  ret =
-      gst_pad_alloc_buffer_and_set_caps (render->srcpad, GST_BUFFER_OFFSET_NONE,
-      render->width * render->height * 4, caps, &outbuf);
+  outbuf = gst_buffer_new_and_alloc (render->width * render->height * 4);
 
-  if (ret != GST_FLOW_OK)
-    goto done;
-
-  gst_buffer_copy_metadata (outbuf, inbuf, GST_BUFFER_COPY_TIMESTAMPS);
-  data = GST_BUFFER_DATA (outbuf);
+  gst_buffer_copy_into (outbuf, inbuf, GST_BUFFER_COPY_TIMESTAMPS, 0, -1);
+  data = gst_buffer_map (outbuf, &size, NULL, GST_MAP_WRITE);
 
   if (render->use_ARGB) {
-    for (n = 0; n < render->width * render->height; n++) {
-      data[n * 4] = 0;
-      data[n * 4 + 1] = data[n * 4 + 2] = data[n * 4 + 3] = 0;
-    }
+    memset (data, 0, render->width * render->height * 4);
   } else {
     for (n = 0; n < render->width * render->height; n++) {
       data[n * 4] = data[n * 4 + 1] = 0;
@@ -487,10 +534,10 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
       xpos = render->xpad;
       break;
     case GST_TEXT_RENDER_HALIGN_CENTER:
-      xpos = (render->width - render->bitmap.width) / 2;
+      xpos = (render->width - render->image_width) / 2;
       break;
     case GST_TEXT_RENDER_HALIGN_RIGHT:
-      xpos = render->width - render->bitmap.width - render->xpad;
+      xpos = render->width - render->image_width - render->xpad;
       break;
     default:
       xpos = 0;
@@ -498,10 +545,10 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
 
   switch (render->valign) {
     case GST_TEXT_RENDER_VALIGN_BOTTOM:
-      ypos = render->height - render->bitmap.rows - render->ypad;
+      ypos = render->height - render->image_height - render->ypad;
       break;
     case GST_TEXT_RENDER_VALIGN_BASELINE:
-      ypos = render->height - (render->bitmap.rows + render->ypad);
+      ypos = render->height - (render->image_height + render->ypad);
       break;
     case GST_TEXT_RENDER_VALIGN_TOP:
       ypos = render->ypad;
@@ -511,15 +558,16 @@ gst_text_render_chain (GstPad * pad, GstBuffer * inbuf)
       break;
   }
 
-  if (render->bitmap.buffer) {
+  if (render->text_image) {
     if (render->use_ARGB) {
-      gst_text_renderer_bitmap_to_argb (render, &render->bitmap, data, xpos,
-          xpos + render->bitmap.width, ypos, ypos + render->bitmap.rows);
+      gst_text_renderer_image_to_argb (render, data, xpos, ypos,
+          render->width * 4);
     } else {
-      gst_text_renderer_bitmap_to_ayuv (render, &render->bitmap, data, xpos,
-          xpos + render->bitmap.width, ypos, ypos + render->bitmap.rows);
+      gst_text_renderer_image_to_ayuv (render, data, xpos, ypos,
+          render->width * 4);
     }
   }
+  gst_buffer_unmap (outbuf, data, size);
 
   ret = gst_pad_push (render->srcpad, outbuf);
 
@@ -536,7 +584,7 @@ gst_text_render_finalize (GObject * object)
 {
   GstTextRender *render = GST_TEXT_RENDER (object);
 
-  g_free (render->bitmap.buffer);
+  g_free (render->text_image);
 
   if (render->layout)
     g_object_unref (render->layout);
@@ -545,7 +593,7 @@ gst_text_render_finalize (GObject * object)
 }
 
 static void
-gst_text_render_init (GstTextRender * render, GstTextRenderClass * klass)
+gst_text_render_init (GstTextRender * render)
 {
   GstPadTemplate *template;
 
@@ -563,8 +611,6 @@ gst_text_render_init (GstTextRender * render, GstTextRenderClass * klass)
   gst_object_unref (template);
   gst_pad_set_fixatecaps_function (render->srcpad,
       GST_DEBUG_FUNCPTR (gst_text_render_fixate_caps));
-  gst_pad_set_setcaps_function (render->srcpad,
-      GST_DEBUG_FUNCPTR (gst_text_render_setcaps));
 
   gst_element_add_pad (GST_ELEMENT (render), render->srcpad);
 
@@ -573,7 +619,6 @@ gst_text_render_init (GstTextRender * render, GstTextRenderClass * klass)
       pango_layout_new (GST_TEXT_RENDER_GET_CLASS (render)->pango_context);
   pango_layout_set_alignment (render->layout,
       (PangoAlignment) render->line_align);
-  memset (&render->bitmap, 0, sizeof (render->bitmap));
 
   render->halign = DEFAULT_PROP_HALIGNMENT;
   render->valign = DEFAULT_PROP_VALIGNMENT;
@@ -610,7 +655,7 @@ gst_text_render_set_property (GObject * object, guint prop_id,
     case PROP_YPAD:
       render->ypad = g_value_get_int (value);
       break;
-    case ARG_FONT_DESC:
+    case PROP_FONT_DESC:
     {
       PangoFontDescription *desc;
 
@@ -619,8 +664,9 @@ gst_text_render_set_property (GObject * object, guint prop_id,
         GST_LOG ("font description set: %s", g_value_get_string (value));
         GST_OBJECT_LOCK (render);
         pango_layout_set_font_description (render->layout, desc);
+        gst_text_render_adjust_values_with_fontdesc (render, desc);
         pango_font_description_free (desc);
-        gst_text_render_render_text (render);
+        gst_text_render_render_pangocairo (render);
         GST_OBJECT_UNLOCK (render);
       } else {
         GST_WARNING ("font description parse failed: %s",
