@@ -44,23 +44,34 @@
 #include "gsttheoradec.h"
 #include <gst/tag/tag.h>
 #include <gst/video/video.h>
+#include <gst/video/gstmetavideo.h>
+#include <gst/video/gstvideopool.h>
 
 #define GST_CAT_DEFAULT theoradec_debug
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 #define THEORA_DEF_CROP         TRUE
+#define THEORA_DEF_TELEMETRY_MV 0
+#define THEORA_DEF_TELEMETRY_MBMODE 0
+#define THEORA_DEF_TELEMETRY_QI 0
+#define THEORA_DEF_TELEMETRY_BITS 0
+
 enum
 {
-  ARG_0,
-  ARG_CROP
+  PROP_0,
+  PROP_CROP,
+  PROP_TELEMETRY_MV,
+  PROP_TELEMETRY_MBMODE,
+  PROP_TELEMETRY_QI,
+  PROP_TELEMETRY_BITS
 };
 
 static GstStaticPadTemplate theora_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw-yuv, "
-        "format = (fourcc) { I420, Y42B, Y444 }, "
+    GST_STATIC_CAPS ("video/x-raw, "
+        "format = (string) { I420, Y42B, Y444 }, "
         "framerate = (fraction) [0/1, MAX], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
@@ -72,15 +83,16 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("video/x-theora")
     );
 
-GST_BOILERPLATE (GstTheoraDec, gst_theora_dec, GstElement, GST_TYPE_ELEMENT);
+#define gst_theora_dec_parent_class parent_class
+G_DEFINE_TYPE (GstTheoraDec, gst_theora_dec, GST_TYPE_ELEMENT);
 
 static void theora_dec_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void theora_dec_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
+static gboolean theora_dec_setcaps (GstTheoraDec * dec, GstCaps * caps);
 static gboolean theora_dec_sink_event (GstPad * pad, GstEvent * event);
-static gboolean theora_dec_setcaps (GstPad * pad, GstCaps * caps);
 static GstFlowReturn theora_dec_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn theora_dec_change_state (GstElement * element,
     GstStateChange transition);
@@ -98,20 +110,11 @@ static const GstEventMask *theora_get_event_masks (GstPad * pad);
 #endif
 static const GstQueryType *theora_get_query_types (GstPad * pad);
 
-
-static void
-gst_theora_dec_base_init (gpointer g_class)
+static gboolean
+gst_theora_dec_ctl_is_supported (int req)
 {
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&theora_dec_src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&theora_dec_sink_factory));
-  gst_element_class_set_details_simple (element_class,
-      "Theora video decoder", "Codec/Decoder/Video",
-      "decode raw theora streams to raw YUV video",
-      "Benjamin Otte <otte@gnome.org>, Wim Taymans <wim@fluendo.com>");
+  /* should return TH_EFAULT or TH_EINVAL if supported, and TH_EIMPL if not */
+  return (th_decode_ctl (NULL, req, NULL, 0) != TH_EIMPL);
 }
 
 static void
@@ -123,23 +126,75 @@ gst_theora_dec_class_init (GstTheoraDecClass * klass)
   gobject_class->set_property = theora_dec_set_property;
   gobject_class->get_property = theora_dec_get_property;
 
-  g_object_class_install_property (gobject_class, ARG_CROP,
+  g_object_class_install_property (gobject_class, PROP_CROP,
       g_param_spec_boolean ("crop", "Crop",
           "Crop the image to the visible region", THEORA_DEF_CROP,
           (GParamFlags) G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  if (gst_theora_dec_ctl_is_supported (TH_DECCTL_SET_TELEMETRY_MV)) {
+    g_object_class_install_property (gobject_class, PROP_TELEMETRY_MV,
+        g_param_spec_int ("visualize-motion-vectors",
+            "Visualize motion vectors",
+            "Show motion vector selection overlaid on image. "
+            "Value gives a mask for motion vector (MV) modes to show",
+            0, 0xffff, THEORA_DEF_TELEMETRY_MV,
+            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  }
+
+  if (gst_theora_dec_ctl_is_supported (TH_DECCTL_SET_TELEMETRY_MBMODE)) {
+    g_object_class_install_property (gobject_class, PROP_TELEMETRY_MBMODE,
+        g_param_spec_int ("visualize-macroblock-modes",
+            "Visualize macroblock modes",
+            "Show macroblock mode selection overlaid on image. "
+            "Value gives a mask for macroblock (MB) modes to show",
+            0, 0xffff, THEORA_DEF_TELEMETRY_MBMODE,
+            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  }
+
+  if (gst_theora_dec_ctl_is_supported (TH_DECCTL_SET_TELEMETRY_QI)) {
+    g_object_class_install_property (gobject_class, PROP_TELEMETRY_QI,
+        g_param_spec_int ("visualize-quantization-modes",
+            "Visualize adaptive quantization modes",
+            "Show adaptive quantization mode selection overlaid on image. "
+            "Value gives a mask for quantization (QI) modes to show",
+            0, 0xffff, THEORA_DEF_TELEMETRY_QI,
+            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  }
+
+  if (gst_theora_dec_ctl_is_supported (TH_DECCTL_SET_TELEMETRY_BITS)) {
+    /* FIXME: make this a boolean instead? The value scales the bars so
+     * they're less wide. Default is to use full width, and anything else
+     * doesn't seem particularly useful, since the smaller bars just disappear
+     * then (they almost disappear for a value of 2 already). */
+    g_object_class_install_property (gobject_class, PROP_TELEMETRY_BITS,
+        g_param_spec_int ("visualize-bit-usage",
+            "Visualize bitstream usage breakdown",
+            "Sets the bitstream breakdown visualization mode. "
+            "Values influence the width of the bit usage bars to show",
+            0, 0xff, THEORA_DEF_TELEMETRY_BITS,
+            G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  }
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&theora_dec_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&theora_dec_sink_factory));
+  gst_element_class_set_details_simple (gstelement_class,
+      "Theora video decoder", "Codec/Decoder/Video",
+      "decode raw theora streams to raw YUV video",
+      "Benjamin Otte <otte@gnome.org>, Wim Taymans <wim@fluendo.com>");
+
   gstelement_class->change_state = theora_dec_change_state;
 
   GST_DEBUG_CATEGORY_INIT (theoradec_debug, "theoradec", 0, "Theora decoder");
 }
 
 static void
-gst_theora_dec_init (GstTheoraDec * dec, GstTheoraDecClass * g_class)
+gst_theora_dec_init (GstTheoraDec * dec)
 {
   dec->sinkpad =
       gst_pad_new_from_static_template (&theora_dec_sink_factory, "sink");
   gst_pad_set_event_function (dec->sinkpad, theora_dec_sink_event);
-  gst_pad_set_setcaps_function (dec->sinkpad, theora_dec_setcaps);
   gst_pad_set_chain_function (dec->sinkpad, theora_dec_chain);
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
 
@@ -153,6 +208,10 @@ gst_theora_dec_init (GstTheoraDec * dec, GstTheoraDecClass * g_class)
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
   dec->crop = THEORA_DEF_CROP;
+  dec->telemetry_mv = THEORA_DEF_TELEMETRY_MV;
+  dec->telemetry_mbmode = THEORA_DEF_TELEMETRY_MBMODE;
+  dec->telemetry_qi = THEORA_DEF_TELEMETRY_QI;
+  dec->telemetry_bits = THEORA_DEF_TELEMETRY_BITS;
   dec->gather = NULL;
   dec->decode = NULL;
   dec->queued = NULL;
@@ -516,7 +575,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
       GstClockTimeDiff diff;
       GstClockTime timestamp;
 
-      gst_event_parse_qos (event, &proportion, &diff, &timestamp);
+      gst_event_parse_qos (event, NULL, &proportion, &diff, &timestamp);
 
       /* we cannot randomly skip frame decoding since we don't have
        * B frames. we can however use the timestamp and diff to not
@@ -570,29 +629,20 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_EOS:
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_SEGMENT:
     {
-      gboolean update;
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
+      const GstSegment *segment;
 
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
+      gst_event_parse_segment (event, &segment);
 
       /* we need TIME format */
-      if (format != GST_FORMAT_TIME)
+      if (segment->format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
 
-      GST_DEBUG_OBJECT (dec,
-          "newsegment: update %d, rate %g, arate %g, start %" GST_TIME_FORMAT
-          ", stop %" GST_TIME_FORMAT ", time %" GST_TIME_FORMAT,
-          update, rate, arate, GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
-          GST_TIME_ARGS (time));
+      GST_DEBUG_OBJECT (dec, "segment: %" GST_SEGMENT_FORMAT, segment);
 
       /* now configure the values */
-      gst_segment_set_newsegment_full (&dec->segment, update,
-          rate, arate, format, start, stop, time);
+      gst_segment_copy_into (segment, &dec->segment);
       dec->seqnum = gst_event_get_seqnum (event);
 
       /* We don't forward this unless/until the decoder is initialised */
@@ -604,6 +654,15 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       }
       break;
     }
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      ret = theora_dec_setcaps (dec, caps);
+      gst_event_unref (event);
+      break;
+    }
     case GST_EVENT_TAG:
     {
       if (dec->have_header)
@@ -617,7 +676,7 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     default:
-      ret = gst_pad_push_event (dec->srcpad, event);
+      ret = gst_pad_event_default (pad, event);
       break;
   }
 done:
@@ -635,14 +694,11 @@ newseg_wrong_format:
 }
 
 static gboolean
-theora_dec_setcaps (GstPad * pad, GstCaps * caps)
+theora_dec_setcaps (GstTheoraDec * dec, GstCaps * caps)
 {
-  GstTheoraDec *dec;
   GstStructure *s;
   const GValue *codec_data;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
-
   s = gst_caps_get_structure (caps, 0);
 
   /* parse the par, this overrides the encoded par */
@@ -652,48 +708,50 @@ theora_dec_setcaps (GstPad * pad, GstCaps * caps)
   if ((codec_data = gst_structure_get_value (s, "codec_data"))) {
     if (G_VALUE_TYPE (codec_data) == GST_TYPE_BUFFER) {
       GstBuffer *buffer;
-      guint8 *data;
-      guint size;
+      guint8 *data, *ptr;
+      gsize size, left;
       guint offset;
 
       buffer = gst_value_get_buffer (codec_data);
 
       offset = 0;
-      size = GST_BUFFER_SIZE (buffer);
-      data = GST_BUFFER_DATA (buffer);
+      data = gst_buffer_map (buffer, &size, NULL, GST_MAP_READ);
 
-      while (size > 2) {
+      ptr = data;
+      left = size;
+
+      while (left > 2) {
         guint psize;
         GstBuffer *buf;
 
-        psize = (data[0] << 8) | data[1];
+        psize = (ptr[0] << 8) | ptr[1];
         /* skip header */
-        data += 2;
-        size -= 2;
+        ptr += 2;
+        left -= 2;
         offset += 2;
 
         /* make sure we don't read too much */
-        psize = MIN (psize, size);
+        psize = MIN (psize, left);
 
-        buf = gst_buffer_create_sub (buffer, offset, psize);
+        buf =
+            gst_buffer_copy_region (buffer, GST_BUFFER_COPY_ALL, offset, psize);
 
         /* first buffer is a discont buffer */
         if (offset == 2)
           GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
 
         /* now feed it to the decoder we can ignore the error */
-        theora_dec_chain (pad, buf);
+        theora_dec_chain (dec->sinkpad, buf);
 
         /* skip the data */
-        size -= psize;
-        data += psize;
+        left -= psize;
+        ptr += psize;
         offset += psize;
       }
+      gst_buffer_unmap (buffer, data, size);
     }
   }
 
-  gst_object_unref (dec);
-
   return TRUE;
 }
 
@@ -701,20 +759,13 @@ static GstFlowReturn
 theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
   gchar *encoder = NULL;
-  GstBuffer *buf;
   GstTagList *list;
 
   GST_DEBUG_OBJECT (dec, "parsing comment packet");
 
-  buf = gst_buffer_new ();
-  GST_BUFFER_SIZE (buf) = packet->bytes;
-  GST_BUFFER_DATA (buf) = packet->packet;
-
   list =
-      gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "\201theora", 7,
-      &encoder);
-
-  gst_buffer_unref (buf);
+      gst_tag_list_from_vorbiscomment (packet->packet, packet->bytes,
+      (guint8 *) "\201theora", 7, &encoder);
 
   if (!list) {
     GST_ERROR_OBJECT (dec, "couldn't decode comments");
@@ -740,14 +791,69 @@ theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
   return GST_FLOW_OK;
 }
 
+static GstFlowReturn
+theora_negotiate_pool (GstTheoraDec * dec, GstCaps * caps, GstVideoInfo * info)
+{
+  GstQuery *query;
+  GstBufferPool *pool = NULL;
+  guint size, min, max, prefix, alignment;
+  GstStructure *config;
+
+  /* find a pool for the negotiated caps now */
+  query = gst_query_new_allocation (caps, TRUE);
+
+  if (gst_pad_peer_query (dec->srcpad, query)) {
+    GST_DEBUG_OBJECT (dec, "got downstream ALLOCATION hints");
+    /* we got configuration from our peer, parse them */
+    gst_query_parse_allocation_params (query, &size, &min, &max, &prefix,
+        &alignment, &pool);
+    size = MAX (size, info->size);
+  } else {
+    GST_DEBUG_OBJECT (dec, "didn't get downstream ALLOCATION hints");
+    size = info->size;
+    min = max = 0;
+    prefix = 0;
+    alignment = 0;
+  }
+
+  if (pool == NULL) {
+    /* we did not get a pool, make one ourselves then */
+    pool = gst_buffer_pool_new ();
+  }
+
+  if (dec->pool)
+    gst_object_unref (dec->pool);
+  dec->pool = pool;
+
+  config = gst_buffer_pool_get_config (pool);
+  gst_buffer_pool_config_set (config, caps, size, min, max, prefix, alignment);
+  /* just set the option, if the pool can support it we will transparently use
+   * it through the video info API. We could also see if the pool support this
+   * option and only activate it then. */
+  gst_buffer_pool_config_add_option (config, GST_BUFFER_POOL_OPTION_META_VIDEO);
+
+  /* check if downstream supports cropping */
+  dec->use_cropping =
+      gst_query_has_allocation_meta (query, GST_META_API_VIDEO_CROP);
+
+  gst_buffer_pool_set_config (pool, config);
+  /* and activate */
+  gst_buffer_pool_set_active (pool, TRUE);
+
+  gst_query_unref (query);
+
+  return GST_FLOW_OK;
+}
+
 static GstFlowReturn
 theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
   GstCaps *caps;
+  GstVideoFormat format;
+  gint width, height;
   gint par_num, par_den;
   GstFlowReturn ret = GST_FLOW_OK;
   GList *walk;
-  guint32 fourcc;
 
   GST_DEBUG_OBJECT (dec, "fps %d/%d, PAR %d/%d",
       dec->info.fps_numerator, dec->info.fps_denominator,
@@ -782,23 +888,26 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
       dec->info.pic_width, dec->info.pic_height,
       dec->info.pic_x, dec->info.pic_y);
 
-  if (dec->info.pixel_fmt == TH_PF_420) {
-    dec->output_bpp = 12;       /* Average bits per pixel. */
-    fourcc = GST_MAKE_FOURCC ('I', '4', '2', '0');
-  } else if (dec->info.pixel_fmt == TH_PF_422) {
-    dec->output_bpp = 16;
-    fourcc = GST_MAKE_FOURCC ('Y', '4', '2', 'B');
-  } else if (dec->info.pixel_fmt == TH_PF_444) {
-    dec->output_bpp = 24;
-    fourcc = GST_MAKE_FOURCC ('Y', '4', '4', '4');
-  } else {
-    GST_ERROR_OBJECT (dec, "Invalid pixel format %d", dec->info.pixel_fmt);
-    return GST_FLOW_ERROR;
+  switch (dec->info.pixel_fmt) {
+    case TH_PF_444:
+      dec->output_bpp = 24;
+      format = GST_VIDEO_FORMAT_Y444;
+      break;
+    case TH_PF_420:
+      dec->output_bpp = 12;     /* Average bits per pixel. */
+      format = GST_VIDEO_FORMAT_I420;
+      break;
+    case TH_PF_422:
+      dec->output_bpp = 16;
+      format = GST_VIDEO_FORMAT_Y42B;
+      break;
+    default:
+      goto invalid_format;
   }
 
   if (dec->crop) {
-    dec->width = dec->info.pic_width;
-    dec->height = dec->info.pic_height;
+    width = dec->info.pic_width;
+    height = dec->info.pic_height;
     dec->offset_x = dec->info.pic_x;
     dec->offset_y = dec->info.pic_y;
     /* Ensure correct offsets in chroma for formats that need it
@@ -806,37 +915,73 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
      * so no need to handle them ourselves. */
     if (dec->offset_x & 1 && dec->info.pixel_fmt != TH_PF_444) {
       dec->offset_x--;
-      dec->width++;
     }
     if (dec->offset_y & 1 && dec->info.pixel_fmt == TH_PF_420) {
       dec->offset_y--;
-      dec->height++;
     }
   } else {
     /* no cropping, use the encoded dimensions */
-    dec->width = dec->info.frame_width;
-    dec->height = dec->info.frame_height;
+    width = dec->info.frame_width;
+    height = dec->info.frame_height;
     dec->offset_x = 0;
     dec->offset_y = 0;
   }
 
   GST_DEBUG_OBJECT (dec, "after fixup frame dimension %dx%d, offset %d:%d",
-      dec->width, dec->height, dec->offset_x, dec->offset_y);
+      width, height, dec->offset_x, dec->offset_y);
 
   /* done */
   dec->decoder = th_decode_alloc (&dec->info, dec->setup);
 
-  caps = gst_caps_new_simple ("video/x-raw-yuv",
-      "format", GST_TYPE_FOURCC, fourcc,
-      "framerate", GST_TYPE_FRACTION,
-      dec->info.fps_numerator, dec->info.fps_denominator,
-      "pixel-aspect-ratio", GST_TYPE_FRACTION, par_num, par_den,
-      "width", G_TYPE_INT, dec->width, "height", G_TYPE_INT, dec->height,
-      "color-matrix", G_TYPE_STRING, "sdtv",
-      "chroma-site", G_TYPE_STRING, "jpeg", NULL);
+  if (th_decode_ctl (dec->decoder, TH_DECCTL_SET_TELEMETRY_MV,
+          &dec->telemetry_mv, sizeof (dec->telemetry_mv)) != TH_EIMPL) {
+    GST_WARNING_OBJECT (dec, "Could not enable MV visualisation");
+  }
+  if (th_decode_ctl (dec->decoder, TH_DECCTL_SET_TELEMETRY_MBMODE,
+          &dec->telemetry_mbmode, sizeof (dec->telemetry_mbmode)) != TH_EIMPL) {
+    GST_WARNING_OBJECT (dec, "Could not enable MB mode visualisation");
+  }
+  if (th_decode_ctl (dec->decoder, TH_DECCTL_SET_TELEMETRY_QI,
+          &dec->telemetry_qi, sizeof (dec->telemetry_qi)) != TH_EIMPL) {
+    GST_WARNING_OBJECT (dec, "Could not enable QI mode visualisation");
+  }
+  if (th_decode_ctl (dec->decoder, TH_DECCTL_SET_TELEMETRY_BITS,
+          &dec->telemetry_bits, sizeof (dec->telemetry_bits)) != TH_EIMPL) {
+    GST_WARNING_OBJECT (dec, "Could not enable BITS mode visualisation");
+  }
+
+  gst_video_info_set_format (&dec->vinfo, format, width, height);
+  dec->vinfo.fps_n = dec->info.fps_numerator;
+  dec->vinfo.fps_d = dec->info.fps_denominator;
+  dec->vinfo.par_n = par_num;
+  dec->vinfo.par_d = par_den;
+
+  /* these values are for all versions of the colorspace specified in the
+   * theora info */
+  dec->vinfo.chroma_site = GST_VIDEO_CHROMA_SITE_JPEG;
+  dec->vinfo.colorimetry.range = GST_VIDEO_COLOR_RANGE_16_235;
+  dec->vinfo.colorimetry.matrix = GST_VIDEO_COLOR_MATRIX_BT601;
+  dec->vinfo.colorimetry.transfer = GST_VIDEO_TRANSFER_BT709;
+  switch (dec->info.colorspace) {
+    case TH_CS_ITU_REC_470M:
+      dec->vinfo.colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470M;
+      break;
+    case TH_CS_ITU_REC_470BG:
+      dec->vinfo.colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_BT470BG;
+      break;
+    default:
+      dec->vinfo.colorimetry.primaries = GST_VIDEO_COLOR_PRIMARIES_UNKNOWN;
+      break;
+  }
+
+  caps = gst_video_info_to_caps (&dec->vinfo);
   gst_pad_set_caps (dec->srcpad, caps);
   gst_caps_unref (caps);
 
+  /* negotiate a bufferpool */
+  if ((ret = theora_negotiate_pool (dec, caps, &dec->vinfo)) != GST_FLOW_OK)
+    goto no_bufferpool;
+
   dec->have_header = TRUE;
 
   if (dec->pendingevents) {
@@ -853,6 +998,17 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   }
 
   return ret;
+
+  /* ERRORS */
+invalid_format:
+  {
+    GST_ERROR_OBJECT (dec, "Invalid pixel format %d", dec->info.pixel_fmt);
+    return GST_FLOW_ERROR;
+  }
+no_bufferpool:
+  {
+    return ret;
+  }
 }
 
 static GstFlowReturn
@@ -900,7 +1056,7 @@ clip_buffer (GstTheoraDec * dec, GstBuffer * buf)
 {
   gboolean res = TRUE;
   GstClockTime in_ts, in_dur, stop;
-  gint64 cstart, cstop;
+  guint64 cstart, cstop;
 
   in_ts = GST_BUFFER_TIMESTAMP (buf);
   in_dur = GST_BUFFER_DURATION (buf);
@@ -971,57 +1127,96 @@ static GstFlowReturn
 theora_handle_image (GstTheoraDec * dec, th_ycbcr_buffer buf, GstBuffer ** out)
 {
   gint width, height, stride;
+  gint pic_width, pic_height;
   GstFlowReturn result;
-  int i, plane;
-  GstVideoFormat format;
+  int i, comp;
   guint8 *dest, *src;
+  GstVideoFrame frame;
+  GstMetaVideoCrop *crop;
+  gint offset_x, offset_y;
 
-  switch (dec->info.pixel_fmt) {
-    case TH_PF_444:
-      format = GST_VIDEO_FORMAT_Y444;
-      break;
-    case TH_PF_420:
-      format = GST_VIDEO_FORMAT_I420;
-      break;
-    case TH_PF_422:
-      format = GST_VIDEO_FORMAT_Y42B;
-      break;
-    default:
-      g_assert_not_reached ();
+  if (gst_pad_check_reconfigure (dec->srcpad)) {
+    GstCaps *caps;
+
+    caps = gst_pad_get_current_caps (dec->srcpad);
+    theora_negotiate_pool (dec, caps, &dec->vinfo);
+    gst_caps_unref (caps);
   }
 
-  result =
-      gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
-      gst_video_format_get_size (format, dec->width, dec->height),
-      GST_PAD_CAPS (dec->srcpad), out);
-  if (G_UNLIKELY (result != GST_FLOW_OK)) {
-    GST_DEBUG_OBJECT (dec, "could not get buffer, reason: %s",
-        gst_flow_get_name (result));
-    return result;
+  result = gst_buffer_pool_acquire_buffer (dec->pool, out, NULL);
+  if (G_UNLIKELY (result != GST_FLOW_OK))
+    goto no_buffer;
+
+  if (!gst_video_frame_map (&frame, &dec->vinfo, *out, GST_MAP_WRITE))
+    goto invalid_frame;
+
+  if (dec->crop && !dec->use_cropping) {
+    /* we need to crop the hard way */
+    offset_x = dec->info.pic_x;
+    offset_y = dec->info.pic_y;
+    pic_width = dec->info.pic_width;
+    pic_height = dec->info.pic_height;
+    /* Ensure correct offsets in chroma for formats that need it
+     * by rounding the offset. libtheora will add proper pixels,
+     * so no need to handle them ourselves. */
+    if (offset_x & 1 && dec->info.pixel_fmt != TH_PF_444)
+      offset_x--;
+    if (offset_y & 1 && dec->info.pixel_fmt == TH_PF_420)
+      offset_y--;
+  } else {
+    /* copy the whole frame */
+    offset_x = 0;
+    offset_y = 0;
+    pic_width = dec->info.frame_width;
+    pic_height = dec->info.frame_height;
+
+    if (dec->use_cropping) {
+      crop = gst_buffer_add_meta_video_crop (*out);
+      /* we can do things slightly more efficient when we know that
+       * downstream understands clipping */
+      crop->x = dec->info.pic_x;
+      crop->y = dec->info.pic_y;
+      crop->width = dec->info.pic_width;
+      crop->height = dec->info.pic_height;
+    }
   }
 
-  for (plane = 0; plane < 3; plane++) {
-    width = gst_video_format_get_component_width (format, plane, dec->width);
-    height = gst_video_format_get_component_height (format, plane, dec->height);
-    stride = gst_video_format_get_row_stride (format, plane, dec->width);
+  for (comp = 0; comp < 3; comp++) {
+    width =
+        GST_VIDEO_FORMAT_INFO_SCALE_WIDTH (frame.info.finfo, comp, pic_width);
+    height =
+        GST_VIDEO_FORMAT_INFO_SCALE_HEIGHT (frame.info.finfo, comp, pic_height);
+    stride = GST_VIDEO_FRAME_COMP_STRIDE (&frame, comp);
+    dest = GST_VIDEO_FRAME_COMP_DATA (&frame, comp);
 
-    dest =
-        GST_BUFFER_DATA (*out) + gst_video_format_get_component_offset (format,
-        plane, dec->width, dec->height);
-    src = buf[plane].data;
-    src += ((height == dec->height) ? dec->offset_y : dec->offset_y / 2)
-        * buf[plane].stride;
-    src += (width == dec->width) ? dec->offset_x : dec->offset_x / 2;
+    src = buf[comp].data;
+    src += ((height == pic_height) ? offset_y : offset_y / 2)
+        * buf[comp].stride;
+    src += (width == pic_width) ? offset_x : offset_x / 2;
 
     for (i = 0; i < height; i++) {
       memcpy (dest, src, width);
 
       dest += stride;
-      src += buf[plane].stride;
+      src += buf[comp].stride;
     }
   }
+  gst_video_frame_unmap (&frame);
 
   return GST_FLOW_OK;
+
+  /* ERRORS */
+no_buffer:
+  {
+    GST_DEBUG_OBJECT (dec, "could not get buffer, reason: %s",
+        gst_flow_get_name (result));
+    return result;
+  }
+invalid_frame:
+  {
+    GST_DEBUG_OBJECT (dec, "could not map video frame");
+    return GST_FLOW_ERROR;
+  }
 }
 
 static GstFlowReturn
@@ -1187,10 +1382,11 @@ theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf)
   ogg_packet packet;
   GstFlowReturn result = GST_FLOW_OK;
   GstClockTime timestamp, duration;
+  gsize size;
 
   /* make ogg_packet out of the buffer */
-  packet.packet = GST_BUFFER_DATA (buf);
-  packet.bytes = GST_BUFFER_SIZE (buf);
+  packet.packet = gst_buffer_map (buf, &size, NULL, GST_MAP_READ);
+  packet.bytes = size;
   packet.granulepos = -1;
   packet.packetno = 0;          /* we don't really care */
   packet.b_o_s = dec->have_header ? 0 : 1;
@@ -1218,8 +1414,9 @@ theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf)
   } else {
     result = theora_handle_data_packet (dec, &packet, timestamp, duration);
   }
-
 done:
+  gst_buffer_unmap (buf, packet.packet, size);
+
   return result;
 }
 
@@ -1349,7 +1546,7 @@ theora_dec_chain_reverse (GstTheoraDec * dec, gboolean discont, GstBuffer * buf)
     GST_DEBUG_OBJECT (dec, "received discont,gathering buffers");
     while (dec->gather) {
       GstBuffer *gbuf;
-      guint8 *data;
+      guint8 data[1];
 
       gbuf = GST_BUFFER_CAST (dec->gather->data);
       /* remove from the gather list */
@@ -1358,17 +1555,18 @@ theora_dec_chain_reverse (GstTheoraDec * dec, gboolean discont, GstBuffer * buf)
       dec->decode = g_list_prepend (dec->decode, gbuf);
 
       /* if we copied a keyframe, flush and decode the decode queue */
-      data = GST_BUFFER_DATA (gbuf);
-      if ((data[0] & 0x40) == 0) {
-        GST_DEBUG_OBJECT (dec, "copied keyframe");
-        res = theora_dec_flush_decode (dec);
+      if (gst_buffer_extract (gbuf, 0, data, 1) == 1) {
+        if ((data[0] & 0x40) == 0) {
+          GST_DEBUG_OBJECT (dec, "copied keyframe");
+          res = theora_dec_flush_decode (dec);
+        }
       }
     }
   }
 
   /* add buffer to gather queue */
-  GST_DEBUG_OBJECT (dec, "gathering buffer %p, size %u", buf,
-      GST_BUFFER_SIZE (buf));
+  GST_DEBUG_OBJECT (dec, "gathering buffer %p, size %" G_GSIZE_FORMAT, buf,
+      gst_buffer_get_size (buf));
   dec->gather = g_list_prepend (dec->gather, buf);
 
   return res;
@@ -1440,7 +1638,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  ret = parent_class->change_state (element, transition);
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -1453,6 +1651,11 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       th_decode_free (dec->decoder);
       dec->decoder = NULL;
       gst_theora_dec_reset (dec);
+      if (dec->pool) {
+        gst_buffer_pool_set_active (dec->pool, FALSE);
+        gst_object_unref (dec->pool);
+        dec->pool = NULL;
+      }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -1470,9 +1673,21 @@ theora_dec_set_property (GObject * object, guint prop_id,
   GstTheoraDec *dec = GST_THEORA_DEC (object);
 
   switch (prop_id) {
-    case ARG_CROP:
+    case PROP_CROP:
       dec->crop = g_value_get_boolean (value);
       break;
+    case PROP_TELEMETRY_MV:
+      dec->telemetry_mv = g_value_get_int (value);
+      break;
+    case PROP_TELEMETRY_MBMODE:
+      dec->telemetry_mbmode = g_value_get_int (value);
+      break;
+    case PROP_TELEMETRY_QI:
+      dec->telemetry_qi = g_value_get_int (value);
+      break;
+    case PROP_TELEMETRY_BITS:
+      dec->telemetry_bits = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1486,9 +1701,21 @@ theora_dec_get_property (GObject * object, guint prop_id,
   GstTheoraDec *dec = GST_THEORA_DEC (object);
 
   switch (prop_id) {
-    case ARG_CROP:
+    case PROP_CROP:
       g_value_set_boolean (value, dec->crop);
       break;
+    case PROP_TELEMETRY_MV:
+      g_value_set_int (value, dec->telemetry_mv);
+      break;
+    case PROP_TELEMETRY_MBMODE:
+      g_value_set_int (value, dec->telemetry_mbmode);
+      break;
+    case PROP_TELEMETRY_QI:
+      g_value_set_int (value, dec->telemetry_qi);
+      break;
+    case PROP_TELEMETRY_BITS:
+      g_value_set_int (value, dec->telemetry_bits);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
