@@ -64,8 +64,8 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("audio/x-vorbis")
     );
 
-GST_BOILERPLATE (GST_VORBIS_DEC_GLIB_TYPE_NAME, gst_vorbis_dec, GstElement,
-    GST_TYPE_ELEMENT);
+#define gst_vorbis_dec_parent_class parent_class
+G_DEFINE_TYPE (GST_VORBIS_DEC_GLIB_TYPE_NAME, gst_vorbis_dec, GST_TYPE_ELEMENT);
 
 static void vorbis_dec_finalize (GObject * object);
 static gboolean vorbis_dec_sink_event (GstPad * pad, GstEvent * event);
@@ -81,28 +81,10 @@ static gboolean vorbis_dec_src_event (GstPad * pad, GstEvent * event);
 static gboolean vorbis_dec_src_query (GstPad * pad, GstQuery * query);
 static gboolean vorbis_dec_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value);
+    GstFormat dest_format, gint64 * dest_value);
 
 static gboolean vorbis_dec_sink_query (GstPad * pad, GstQuery * query);
 
-static void
-gst_vorbis_dec_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  GstPadTemplate *src_template, *sink_template;
-
-  src_template = gst_static_pad_template_get (&vorbis_dec_src_factory);
-  gst_element_class_add_pad_template (element_class, src_template);
-
-  sink_template = gst_static_pad_template_get (&vorbis_dec_sink_factory);
-  gst_element_class_add_pad_template (element_class, sink_template);
-
-  gst_element_class_set_details_simple (element_class,
-      "Vorbis audio decoder", "Codec/Decoder/Audio",
-      GST_VORBIS_DEC_DESCRIPTION,
-      "Benjamin Otte <otte@gnome.org>, Chris Lord <chris@openedhand.com>");
-}
-
 static void
 gst_vorbis_dec_class_init (GstVorbisDecClass * klass)
 {
@@ -111,6 +93,17 @@ gst_vorbis_dec_class_init (GstVorbisDecClass * klass)
 
   gobject_class->finalize = vorbis_dec_finalize;
 
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&vorbis_dec_src_factory));
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&vorbis_dec_sink_factory));
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "Vorbis audio decoder", "Codec/Decoder/Audio",
+      GST_VORBIS_DEC_DESCRIPTION,
+      "Benjamin Otte <otte@gnome.org>, Chris Lord <chris@openedhand.com>");
+
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (vorbis_dec_change_state);
 }
 
@@ -128,7 +121,7 @@ vorbis_get_query_types (GstPad * pad)
 }
 
 static void
-gst_vorbis_dec_init (GstVorbisDec * dec, GstVorbisDecClass * g_class)
+gst_vorbis_dec_init (GstVorbisDec * dec)
 {
   dec->sinkpad = gst_pad_new_from_static_template (&vorbis_dec_sink_factory,
       "sink");
@@ -207,13 +200,12 @@ gst_vorbis_dec_reset (GstVorbisDec * dec)
 static gboolean
 vorbis_dec_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value)
+    GstFormat dest_format, gint64 * dest_value)
 {
   gboolean res = TRUE;
   GstVorbisDec *dec;
-  guint64 scale = 1;
 
-  if (src_format == *dest_format) {
+  if (src_format == dest_format) {
     *dest_value = src_value;
     return TRUE;
   }
@@ -224,52 +216,12 @@ vorbis_dec_convert (GstPad * pad,
     goto no_header;
 
   if (dec->sinkpad == pad &&
-      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
+      (src_format == GST_FORMAT_BYTES || dest_format == GST_FORMAT_BYTES))
     goto no_format;
 
-  switch (src_format) {
-    case GST_FORMAT_TIME:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          scale = dec->width * dec->vi.channels;
-        case GST_FORMAT_DEFAULT:
-          *dest_value =
-              scale * gst_util_uint64_scale_int (src_value, dec->vi.rate,
-              GST_SECOND);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    case GST_FORMAT_DEFAULT:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          *dest_value = src_value * dec->width * dec->vi.channels;
-          break;
-        case GST_FORMAT_TIME:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, GST_SECOND, dec->vi.rate);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    case GST_FORMAT_BYTES:
-      switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_value = src_value / (dec->width * dec->vi.channels);
-          break;
-        case GST_FORMAT_TIME:
-          *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              dec->vi.rate * dec->width * dec->vi.channels);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    default:
-      res = FALSE;
-  }
+  res = gst_audio_info_convert (&dec->info, src_format, src_value, dest_format,
+      dest_value);
+
 done:
   gst_object_unref (dec);
 
@@ -319,7 +271,7 @@ vorbis_dec_src_query (GstPad * pad, GstQuery * query)
 
       /* and convert to the final format */
       if (!(res =
-              vorbis_dec_convert (pad, GST_FORMAT_TIME, time, &format, &value)))
+              vorbis_dec_convert (pad, GST_FORMAT_TIME, time, format, &value)))
         goto error;
 
       gst_query_set_position (query, format, value);
@@ -345,7 +297,7 @@ vorbis_dec_src_query (GstPad * pad, GstQuery * query)
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
       if (!(res =
-              vorbis_dec_convert (pad, src_fmt, src_val, &dest_fmt, &dest_val)))
+              vorbis_dec_convert (pad, src_fmt, src_val, dest_fmt, &dest_val)))
         goto error;
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
@@ -383,7 +335,7 @@ vorbis_dec_sink_query (GstPad * pad, GstQuery * query)
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
       if (!(res =
-              vorbis_dec_convert (pad, src_fmt, src_val, &dest_fmt, &dest_val)))
+              vorbis_dec_convert (pad, src_fmt, src_val, dest_fmt, &dest_val)))
         goto error;
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
@@ -437,9 +389,9 @@ vorbis_dec_src_event (GstPad * pad, GstEvent * event)
 
       /* First bring the requested format to time */
       tformat = GST_FORMAT_TIME;
-      if (!(res = vorbis_dec_convert (pad, format, cur, &tformat, &tcur)))
+      if (!(res = vorbis_dec_convert (pad, format, cur, tformat, &tcur)))
         goto convert_error;
-      if (!(res = vorbis_dec_convert (pad, format, stop, &tformat, &tstop)))
+      if (!(res = vorbis_dec_convert (pad, format, stop, tformat, &tstop)))
         goto convert_error;
 
       /* then seek with time on the peer */
@@ -493,29 +445,20 @@ vorbis_dec_sink_event (GstPad * pad, GstEvent * event)
       gst_vorbis_dec_reset (dec);
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_SEGMENT:
     {
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
-      gboolean update;
+      const GstSegment *segment;
 
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
+      gst_event_parse_segment (event, &segment);
 
       /* we need time for now */
-      if (format != GST_FORMAT_TIME)
+      if (segment->format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
 
-      GST_DEBUG_OBJECT (dec,
-          "newsegment: update %d, rate %g, arate %g, start %" GST_TIME_FORMAT
-          ", stop %" GST_TIME_FORMAT ", time %" GST_TIME_FORMAT,
-          update, rate, arate, GST_TIME_ARGS (start), GST_TIME_ARGS (stop),
-          GST_TIME_ARGS (time));
+      GST_DEBUG_OBJECT (dec, "segment: %" GST_SEGMENT_FORMAT, segment);
 
       /* now configure the values */
-      gst_segment_set_newsegment_full (&dec->segment, update,
-          rate, arate, format, start, stop, time);
+      gst_segment_copy_into (segment, &dec->segment);
       dec->seqnum = gst_event_get_seqnum (event);
 
       if (dec->initialized)
@@ -541,7 +484,7 @@ vorbis_dec_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     default:
-      ret = gst_pad_push_event (dec->srcpad, event);
+      ret = gst_pad_event_default (pad, event);
       break;
   }
 done:
@@ -561,10 +504,13 @@ static GstFlowReturn
 vorbis_handle_identification_packet (GstVorbisDec * vd)
 {
   GstCaps *caps;
+  GstAudioInfo info;
   const GstAudioChannelPosition *pos = NULL;
-  gint width = GST_VORBIS_DEC_DEFAULT_SAMPLE_WIDTH;
 
-  switch (vd->vi.channels) {
+  gst_audio_info_set_format (&info, GST_VORBIS_AUDIO_FORMAT, vd->vi.rate,
+      vd->vi.channels);
+
+  switch (info.channels) {
     case 1:
     case 2:
       /* nothing */
@@ -575,60 +521,28 @@ vorbis_handle_identification_packet (GstVorbisDec * vd)
     case 6:
     case 7:
     case 8:
-      pos = gst_vorbis_channel_positions[vd->vi.channels - 1];
+      pos = gst_vorbis_channel_positions[info.channels - 1];
       break;
-    default:{
-      gint i;
-      GstAudioChannelPosition *posn =
-          g_new (GstAudioChannelPosition, vd->vi.channels);
+    default:
+    {
+      gint i, max_pos = MAX (info.channels, 64);
 
-      GST_ELEMENT_WARNING (GST_ELEMENT (vd), STREAM, DECODE,
+      GST_ELEMENT_WARNING (vd, STREAM, DECODE,
           (NULL), ("Using NONE channel layout for more than 8 channels"));
-
-      for (i = 0; i < vd->vi.channels; i++)
-        posn[i] = GST_AUDIO_CHANNEL_POSITION_NONE;
-
-      pos = posn;
+      for (i = 0; i < max_pos; i++)
+        info.position[i] = GST_AUDIO_CHANNEL_POSITION_NONE;
     }
   }
 
-  /* negotiate width with downstream */
-  caps = gst_pad_get_allowed_caps (vd->srcpad);
-  if (caps) {
-    if (!gst_caps_is_empty (caps)) {
-      GstStructure *s;
-
-      s = gst_caps_get_structure (caps, 0);
-      /* template ensures 16 or 32 */
-      gst_structure_get_int (s, "width", &width);
-
-      GST_INFO_OBJECT (vd, "using %s with %d channels and %d bit audio depth",
-          gst_structure_get_name (s), vd->vi.channels, width);
-    }
-    gst_caps_unref (caps);
-  }
-  vd->width = width >> 3;
-
-  /* select a copy_samples function, this way we can have specialized versions
-   * for mono/stereo and avoid the depth switch in tremor case */
-  vd->copy_samples = get_copy_sample_func (vd->vi.channels, vd->width);
-
-  caps = gst_caps_copy (gst_pad_get_pad_template_caps (vd->srcpad));
-  gst_caps_set_simple (caps, "rate", G_TYPE_INT, vd->vi.rate,
-      "channels", G_TYPE_INT, vd->vi.channels,
-      "width", G_TYPE_INT, width, NULL);
-
-  if (pos) {
-    gst_audio_set_channel_positions (gst_caps_get_structure (caps, 0), pos);
-  }
-
-  if (vd->vi.channels > 8) {
-    g_free ((GstAudioChannelPosition *) pos);
-  }
-
+  caps = gst_audio_info_to_caps (&info);
   gst_pad_set_caps (vd->srcpad, caps);
   gst_caps_unref (caps);
 
+  vd->info = info;
+  /* select a copy_samples function, this way we can have specialized versions
+   * for mono/stereo and avoid the depth switch in tremor case */
+  vd->copy_samples = get_copy_sample_func (info.channels);
+
   return GST_FLOW_OK;
 }
 
@@ -638,16 +552,16 @@ vorbis_handle_comment_packet (GstVorbisDec * vd, ogg_packet * packet)
   guint bitrate = 0;
   gchar *encoder = NULL;
   GstTagList *list, *old_list;
-  GstBuffer *buf;
+  guint8 *data;
+  gsize size;
 
   GST_DEBUG_OBJECT (vd, "parsing comment packet");
 
-  buf = gst_buffer_new ();
-  GST_BUFFER_DATA (buf) = gst_ogg_packet_data (packet);
-  GST_BUFFER_SIZE (buf) = gst_ogg_packet_size (packet);
+  data = gst_ogg_packet_data (packet);
+  size = gst_ogg_packet_size (packet);
 
   list =
-      gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "\003vorbis", 7,
+      gst_tag_list_from_vorbiscomment (data, size, (guint8 *) "\003vorbis", 7,
       &encoder);
 
   old_list = vd->taglist;
@@ -656,7 +570,6 @@ vorbis_handle_comment_packet (GstVorbisDec * vd, ogg_packet * packet)
   if (old_list)
     gst_tag_list_free (old_list);
   gst_tag_list_free (list);
-  gst_buffer_unref (buf);
 
   if (!vd->taglist) {
     GST_ERROR_OBJECT (vd, "couldn't decode comments");
@@ -809,7 +722,7 @@ vorbis_dec_push_forward (GstVorbisDec * dec, GstBuffer * buf)
 
   /* clip */
   if (!(buf = gst_audio_buffer_clip (buf, &dec->segment, dec->vi.rate,
-              dec->vi.channels * dec->width))) {
+              dec->info.bpf))) {
     GST_LOG_OBJECT (dec, "clipped buffer");
     return GST_FLOW_OK;
   }
@@ -873,15 +786,14 @@ static GstFlowReturn
 vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
     GstClockTime timestamp, GstClockTime duration)
 {
-#ifdef USE_TREMOLO
-  vorbis_sample_t *pcm;
-#else
+#ifndef USE_TREMOLO
   vorbis_sample_t **pcm;
 #endif
   guint sample_count;
   GstBuffer *out = NULL;
   GstFlowReturn result;
-  gint size;
+  guint8 *data;
+  gsize size;
 
   if (G_UNLIKELY (!vd->initialized))
     goto not_initialized;
@@ -915,21 +827,18 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
 #endif
     goto done;
 
-  size = sample_count * vd->vi.channels * vd->width;
-  GST_LOG_OBJECT (vd, "%d samples ready for reading, size %d", sample_count,
-      size);
+  size = sample_count * vd->info.bpf;
+  GST_LOG_OBJECT (vd, "%d samples ready for reading, size %" G_GSIZE_FORMAT,
+      sample_count, size);
 
   /* alloc buffer for it */
-  result =
-      gst_pad_alloc_buffer_and_set_caps (vd->srcpad, GST_BUFFER_OFFSET_NONE,
-      size, GST_PAD_CAPS (vd->srcpad), &out);
-  if (G_UNLIKELY (result != GST_FLOW_OK))
-    goto done;
+  out = gst_buffer_new_and_alloc (size);
 
+  data = gst_buffer_map (out, NULL, NULL, GST_MAP_WRITE);
   /* get samples ready for reading now, should be sample_count */
 #ifdef USE_TREMOLO
-  pcm = GST_BUFFER_DATA (out);
-  if (G_UNLIKELY ((vorbis_dsp_pcmout (&vd->vd, pcm, sample_count)) != sample_count))
+  if (G_UNLIKELY ((vorbis_dsp_pcmout (&vd->vd, data,
+                  sample_count)) != sample_count))
 #else
   if (G_UNLIKELY ((vorbis_synthesis_pcmout (&vd->vd, &pcm)) != sample_count))
 #endif
@@ -937,12 +846,12 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet,
 
 #ifndef USE_TREMOLO
   /* copy samples in buffer */
-  vd->copy_samples ((vorbis_sample_t *) GST_BUFFER_DATA (out), pcm,
-      sample_count, vd->vi.channels, vd->width);
+  vd->copy_samples ((vorbis_sample_t *) data, pcm,
+      sample_count, vd->info.channels);
 #endif
 
-  GST_LOG_OBJECT (vd, "setting output size to %d", size);
-  GST_BUFFER_SIZE (out) = size;
+  GST_LOG_OBJECT (vd, "setting output size to %" G_GSIZE_FORMAT, size);
+  gst_buffer_unmap (out, data, size);
 
   /* this should not overflow */
   if (duration == -1)
@@ -960,7 +869,6 @@ done:
     /* no output, still keep track of timestamps */
     vorbis_do_timestamps (vd, NULL, FALSE, timestamp, duration);
   }
-
 #ifdef USE_TREMOLO
   vorbis_dsp_read (&vd->vd, sample_count);
 #else
@@ -997,6 +905,97 @@ wrong_samples:
   }
 }
 
+static GstFlowReturn
+vorbis_dec_handle_header_buffer (GstVorbisDec * vd, GstBuffer * buffer)
+{
+  ogg_packet *packet;
+  ogg_packet_wrapper packet_wrapper;
+  GstFlowReturn ret;
+
+  gst_ogg_packet_wrapper_map (&packet_wrapper, buffer);
+  packet = gst_ogg_packet_from_wrapper (&packet_wrapper);
+
+  ret = vorbis_handle_header_packet (vd, packet);
+
+  gst_ogg_packet_wrapper_unmap (&packet_wrapper, buffer);
+
+  return ret;
+}
+
+
+#define MIN_NUM_HEADERS 3
+static GstFlowReturn
+vorbis_dec_handle_header_caps (GstVorbisDec * vd, GstBuffer * buffer)
+{
+  GstFlowReturn result = GST_FLOW_OK;
+  GstCaps *caps;
+  GstStructure *s;
+  const GValue *array;
+  const GValue *value = NULL;
+  GstBuffer *buf = NULL;
+
+  if ((caps = gst_pad_get_current_caps (vd->sinkpad)) == NULL)
+    goto no_caps;
+
+  if ((s = gst_caps_get_structure (caps, 0)) == NULL)
+    goto no_caps;
+
+  array = gst_structure_get_value (s, "streamheader");
+
+  if (array == NULL || (gst_value_array_get_size (array) < MIN_NUM_HEADERS))
+    goto array_error;
+
+  /* initial header */
+  value = gst_value_array_get_value (array, 0);
+  buf = gst_value_get_buffer (value);
+  if (!buf)
+    goto null_buffer;
+  result = vorbis_dec_handle_header_buffer (vd, buf);
+  if (result != GST_FLOW_OK)
+    goto buffer_error;
+
+  /* comment header */
+  value = gst_value_array_get_value (array, 1);
+  buf = gst_value_get_buffer (value);
+  if (!buf)
+    goto null_buffer;
+  result = vorbis_dec_handle_header_buffer (vd, buf);
+  if (result != GST_FLOW_OK)
+    goto buffer_error;
+
+  /* bitstream codebook header */
+  value = gst_value_array_get_value (array, 2);
+  buf = gst_value_get_buffer (value);
+  if (!buf)
+    goto null_buffer;
+  result = vorbis_dec_handle_header_buffer (vd, buf);
+  if (result != GST_FLOW_OK)
+    goto buffer_error;
+
+  return result;
+
+no_caps:
+  {
+    GST_WARNING_OBJECT (vd, "no caps negotiated");
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+array_error:
+  {
+    GST_WARNING_OBJECT (vd, "streamheader array not found");
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+null_buffer:
+  {
+    GST_WARNING_OBJECT (vd, "streamheader with null buffer received");
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+buffer_error:
+  {
+    GST_WARNING_OBJECT (vd, "error handling buffer");
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+}
+
 static GstFlowReturn
 vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
 {
@@ -1005,7 +1004,7 @@ vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
   GstFlowReturn result = GST_FLOW_OK;
 
   /* make ogg_packet out of the buffer */
-  gst_ogg_packet_wrapper_from_buffer (&packet_wrapper, buffer);
+  gst_ogg_packet_wrapper_map (&packet_wrapper, buffer);
   packet = gst_ogg_packet_from_wrapper (&packet_wrapper);
   /* set some more stuff */
   packet->granulepos = -1;
@@ -1034,6 +1033,13 @@ vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
   } else {
     GstClockTime timestamp, duration;
 
+    /* try to find header in caps so we can initialize the decoder */
+    if (!vd->initialized) {
+      result = vorbis_dec_handle_header_caps (vd, buffer);
+      if (result != GST_FLOW_OK)
+        goto invalid_caps_header;
+    }
+
     timestamp = GST_BUFFER_TIMESTAMP (buffer);
     duration = GST_BUFFER_DURATION (buffer);
 
@@ -1041,6 +1047,8 @@ vorbis_dec_decode_buffer (GstVorbisDec * vd, GstBuffer * buffer)
   }
 
 done:
+  gst_ogg_packet_wrapper_unmap (&packet_wrapper, buffer);
+
   return result;
 
 empty_buffer:
@@ -1060,6 +1068,13 @@ empty_header:
     vd->discont = TRUE;
     goto done;
   }
+
+invalid_caps_header:
+  {
+    GST_ELEMENT_ERROR (vd, STREAM, DECODE, (NULL),
+        ("invalid streamheader in caps"));
+    goto done;
+  }
 }
 
 /*
@@ -1196,8 +1211,9 @@ vorbis_dec_chain_reverse (GstVorbisDec * vd, gboolean discont, GstBuffer * buf)
 
   if (G_LIKELY (buf)) {
     GST_DEBUG_OBJECT (vd,
-        "gathering buffer %p of size %u, time %" GST_TIME_FORMAT
-        ", dur %" GST_TIME_FORMAT, buf, GST_BUFFER_SIZE (buf),
+        "gathering buffer %p of size %" G_GSIZE_FORMAT
+        ", time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT,
+        buf, gst_buffer_get_size (buf),
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
@@ -1273,7 +1289,7 @@ vorbis_dec_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  res = parent_class->change_state (element, transition);
+  res = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
