@@ -17,22 +17,6 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-
-/**
- * SECTION:element-appsink
- * 
- * Appsink is a sink plugin that supports many different methods for making
- * the application get a handle on the GStreamer data in a pipeline. Unlike
- * most GStreamer elements, Appsink provides external API functions.
- *
- * For the documentation of the API, please see the
- * <link linkend="gst-plugins-base-libs-appsink">libgstapp</link> section in
- * the GStreamer Plugins Base Libraries documentation.
- *
- * Since: 0.10.22
- */
-
-
 /**
  * SECTION:gstappsink
  * @short_description: Easy way for applications to extract buffers from a
@@ -180,25 +164,15 @@ static GstFlowReturn gst_app_sink_render (GstBaseSink * psink,
     GstBuffer * buffer);
 static GstFlowReturn gst_app_sink_render_list (GstBaseSink * psink,
     GstBufferList * list);
-static GstCaps *gst_app_sink_getcaps (GstBaseSink * psink);
+static GstCaps *gst_app_sink_getcaps (GstBaseSink * psink, GstCaps * filter);
 static GstMiniObject *gst_app_sink_pull_object (GstAppSink * appsink);
 
 static guint gst_app_sink_signals[LAST_SIGNAL] = { 0 };
 
-static void
-_do_init (GType filesrc_type)
-{
-  static const GInterfaceInfo urihandler_info = {
-    gst_app_sink_uri_handler_init,
-    NULL,
-    NULL
-  };
-  g_type_add_interface_static (filesrc_type, GST_TYPE_URI_HANDLER,
-      &urihandler_info);
-}
-
-GST_BOILERPLATE_FULL (GstAppSink, gst_app_sink, GstBaseSink, GST_TYPE_BASE_SINK,
-    _do_init);
+#define gst_app_sink_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstAppSink, gst_app_sink, GST_TYPE_BASE_SINK,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
+        gst_app_sink_uri_handler_init));
 
 /* Can't use glib-genmarshal for this, as it doesn't know how to handle
  * GstMiniObject-based types, which are a new fundamental type */
@@ -234,27 +208,15 @@ gst_app_marshal_BUFFER__VOID (GClosure * closure,
   gst_value_take_buffer (return_value, v_return);
 }
 
-static void
-gst_app_sink_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  GST_DEBUG_CATEGORY_INIT (app_sink_debug, "appsink", 0, "appsink element");
-
-  gst_element_class_set_details_simple (element_class, "AppSink",
-      "Generic/Sink", "Allow the application to get access to raw buffer",
-      "David Schleef <ds@schleef.org>, Wim Taymans <wim.taymans@gmail.com>");
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_app_sink_template));
-}
-
 static void
 gst_app_sink_class_init (GstAppSinkClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
+  GstElementClass *element_class = (GstElementClass *) klass;
   GstBaseSinkClass *basesink_class = (GstBaseSinkClass *) klass;
 
+  GST_DEBUG_CATEGORY_INIT (app_sink_debug, "appsink", 0, "appsink element");
+
   gobject_class->dispose = gst_app_sink_dispose;
   gobject_class->finalize = gst_app_sink_finalize;
 
@@ -303,10 +265,10 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * GstAppSink::new-preroll:
    * @appsink: the appsink element that emited the signal
    *
-   * Signal that a new preroll buffer is available. 
+   * Signal that a new preroll buffer is available.
    *
    * This signal is emited from the steaming thread and only when the
-   * "emit-signals" property is %TRUE. 
+   * "emit-signals" property is %TRUE.
    *
    * The new preroll buffer can be retrieved with the "pull-preroll" action
    * signal or gst_app_sink_pull_preroll() either from this signal callback
@@ -326,7 +288,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * Signal that a new buffer is available.
    *
    * This signal is emited from the steaming thread and only when the
-   * "emit-signals" property is %TRUE. 
+   * "emit-signals" property is %TRUE.
    *
    * The new buffer can be retrieved with the "pull-buffer" action
    * signal or gst_app_sink_pull_buffer() either from this signal callback
@@ -346,7 +308,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * Signal that a new bufferlist is available.
    *
    * This signal is emited from the steaming thread and only when the
-   * "emit-signals" property is %TRUE. 
+   * "emit-signals" property is %TRUE.
    *
    * The new buffer can be retrieved with the "pull-buffer-list" action
    * signal or gst_app_sink_pull_buffer_list() either from this signal callback
@@ -376,10 +338,10 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * when calling gst_app_sink_pull_buffer() or the "pull-buffer" action signal.
    *
    * If an EOS event was received before any buffers, this function returns
-   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
    *
    * This function blocks until a preroll buffer or EOS is received or the appsink
-   * element is set to the READY/NULL state. 
+   * element is set to the READY/NULL state.
    *
    * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
    */
@@ -393,11 +355,11 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * @appsink: the appsink element to emit this signal on
    *
    * This function blocks until a buffer or EOS becomes available or the appsink
-   * element is set to the READY/NULL state. 
+   * element is set to the READY/NULL state.
    *
    * This function will only return buffers when the appsink is in the PLAYING
    * state. All rendered buffers will be put in a queue so that the application
-   * can pull buffers at its own rate. 
+   * can pull buffers at its own rate.
    *
    * Note that when the application does not pull buffers fast enough, the
    * queued buffers could consume a lot of memory, especially when dealing with
@@ -405,7 +367,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * the "drop" and "max-buffers" properties.
    *
    * If an EOS event was received before any buffers, this function returns
-   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
    *
    * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
    */
@@ -419,11 +381,11 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * @appsink: the appsink element to emit this signal on
    *
    * This function blocks until a buffer list or EOS becomes available or the appsink
-   * element is set to the READY/NULL state. 
+   * element is set to the READY/NULL state.
    *
    * This function will only return bufferlists when the appsink is in the PLAYING
    * state. All rendered bufferlists will be put in a queue so that the application
-   * can pull bufferlists at its own rate. 
+   * can pull bufferlists at its own rate.
    *
    * Note that when the application does not pull bufferlists fast enough, the
    * queued bufferlists could consume a lot of memory, especially when dealing with
@@ -431,7 +393,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
    * the "drop" and "max-buffers" properties.
    *
    * If an EOS event was received before any buffers, this function returns
-   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+   * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
    *
    * Returns: a #GstBufferList or NULL when the appsink is stopped or EOS.
    */
@@ -441,6 +403,13 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
           pull_buffer_list), NULL, NULL, gst_app_marshal_BUFFER__VOID,
       GST_TYPE_BUFFER_LIST, 0, G_TYPE_NONE);
 
+  gst_element_class_set_details_simple (element_class, "AppSink",
+      "Generic/Sink", "Allow the application to get access to raw buffer",
+      "David Schleef <ds@schleef.org>, Wim Taymans <wim.taymans@gmail.com>");
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_app_sink_template));
+
   basesink_class->unlock = gst_app_sink_unlock_start;
   basesink_class->unlock_stop = gst_app_sink_unlock_stop;
   basesink_class->start = gst_app_sink_start;
@@ -459,7 +428,7 @@ gst_app_sink_class_init (GstAppSinkClass * klass)
 }
 
 static void
-gst_app_sink_init (GstAppSink * appsink, GstAppSinkClass * klass)
+gst_app_sink_init (GstAppSink * appsink)
 {
   GstAppSinkPrivate *priv;
 
@@ -833,10 +802,10 @@ gst_app_sink_render (GstBaseSink * psink, GstBuffer * buffer)
 static GstFlowReturn
 gst_app_sink_render_list (GstBaseSink * sink, GstBufferList * list)
 {
-  GstBufferListIterator *it;
   GstFlowReturn flow;
   GstAppSink *appsink;
-  GstBuffer *group;
+  GstBuffer *buffer;
+  guint i, len;
 
   appsink = GST_APP_SINK_CAST (sink);
 
@@ -847,42 +816,33 @@ gst_app_sink_render_list (GstBaseSink * sink, GstBufferList * list)
    * then and push them one-by-one */
   GST_INFO_OBJECT (sink, "chaining each group in list as a merged buffer");
 
-  it = gst_buffer_list_iterate (list);
+  len = gst_buffer_list_len (list);
 
-  if (gst_buffer_list_iterator_next_group (it)) {
-    do {
-      group = gst_buffer_list_iterator_merge_group (it);
-      if (group == NULL) {
-        group = gst_buffer_new ();
-        GST_DEBUG_OBJECT (sink, "chaining empty group");
-      } else {
-        GST_DEBUG_OBJECT (sink, "chaining group");
-      }
-      flow = gst_app_sink_render (sink, group);
-      gst_buffer_unref (group);
-    } while (flow == GST_FLOW_OK && gst_buffer_list_iterator_next_group (it));
-  } else {
-    GST_DEBUG_OBJECT (sink, "chaining empty group");
-    group = gst_buffer_new ();
-    flow = gst_app_sink_render (sink, group);
-    gst_buffer_unref (group);
+  flow = GST_FLOW_OK;
+  for (i = 0; i < len; i++) {
+    buffer = gst_buffer_list_get (list, i);
+    flow = gst_app_sink_render (sink, buffer);
+    if (flow != GST_FLOW_OK)
+      break;
   }
 
-  gst_buffer_list_iterator_free (it);
-
   return flow;
 }
 
 static GstCaps *
-gst_app_sink_getcaps (GstBaseSink * psink)
+gst_app_sink_getcaps (GstBaseSink * psink, GstCaps * filter)
 {
   GstCaps *caps;
   GstAppSink *appsink = GST_APP_SINK_CAST (psink);
   GstAppSinkPrivate *priv = appsink->priv;
 
   GST_OBJECT_LOCK (appsink);
-  if ((caps = priv->caps))
-    gst_caps_ref (caps);
+  if ((caps = priv->caps)) {
+    if (filter)
+      caps = gst_caps_intersect_full (filter, caps, GST_CAPS_INTERSECT_FIRST);
+    else
+      gst_caps_ref (caps);
+  }
   GST_DEBUG_OBJECT (appsink, "got caps %" GST_PTR_FORMAT, caps);
   GST_OBJECT_UNLOCK (appsink);
 
@@ -948,7 +908,7 @@ not_started:
  * Set the capabilities on the appsink element.  This function takes
  * a copy of the caps structure. After calling this method, the sink will only
  * accept caps that match @caps. If @caps is non-fixed, you must check the caps
- * on the buffers to get the actual used caps. 
+ * on the buffers to get the actual used caps.
  *
  * Since: 0.10.22
  */
@@ -1233,10 +1193,10 @@ gst_app_sink_get_drop (GstAppSink * appsink)
  * when calling gst_app_sink_pull_buffer().
  *
  * If an EOS event was received before any buffers, this function returns
- * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+ * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
  *
  * This function blocks until a preroll buffer or EOS is received or the appsink
- * element is set to the READY/NULL state. 
+ * element is set to the READY/NULL state.
  *
  * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
  *
@@ -1295,7 +1255,7 @@ not_started:
  * @appsink: a #GstAppSink
  *
  * This function blocks until a buffer or EOS becomes available or the appsink
- * element is set to the READY/NULL state. 
+ * element is set to the READY/NULL state.
  *
  * This function will only return buffers when the appsink is in the PLAYING
  * state. All rendered buffers will be put in a queue so that the application
@@ -1304,7 +1264,7 @@ not_started:
  * especially when dealing with raw video frames.
  *
  * If an EOS event was received before any buffers, this function returns
- * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+ * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
  *
  * Returns: a #GstBuffer or NULL when the appsink is stopped or EOS.
  *
@@ -1323,7 +1283,7 @@ gst_app_sink_pull_buffer (GstAppSink * appsink)
  * @appsink: a #GstAppSink
  *
  * This function blocks until a buffer list or EOS becomes available or the
- * appsink element is set to the READY/NULL state. 
+ * appsink element is set to the READY/NULL state.
  *
  * This function will only return buffer lists when the appsink is in the
  * PLAYING state. All rendered buffer lists will be put in a queue so that
@@ -1333,7 +1293,7 @@ gst_app_sink_pull_buffer (GstAppSink * appsink)
  * video frames.
  *
  * If an EOS event was received before any buffer lists, this function returns
- * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition. 
+ * %NULL. Use gst_app_sink_is_eos () to check for the EOS condition.
  *
  * Returns: a #GstBufferList or NULL when the appsink is stopped or EOS.
  */
@@ -1399,13 +1359,13 @@ gst_app_sink_set_callbacks (GstAppSink * appsink,
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
 static GstURIType
-gst_app_sink_uri_get_type (void)
+gst_app_sink_uri_get_type (GType type)
 {
   return GST_URI_SINK;
 }
 
 static gchar **
-gst_app_sink_uri_get_protocols (void)
+gst_app_sink_uri_get_protocols (GType type)
 {
   static gchar *protocols[] = { (char *) "appsink", NULL };
 
