@@ -149,10 +149,6 @@
 #include "config.h"
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include "gstaudiodecoder.h"
 #include <gst/pbutils/descriptions.h>
 
@@ -181,8 +177,6 @@ enum
 #define DEFAULT_LATENCY    0
 #define DEFAULT_TOLERANCE  0
 #define DEFAULT_PLC        FALSE
-#define DEFAULT_DRAINABLE  TRUE
-#define DEFAULT_NEEDS_FORMAT  FALSE
 
 typedef struct _GstAudioDecoderContext
 {
@@ -264,8 +258,6 @@ struct _GstAudioDecoderPrivate
   GstClockTime latency;
   GstClockTime tolerance;
   gboolean plc;
-  gboolean drainable;
-  gboolean needs_format;
 
   /* pending serialized sink events, will be sent from finish_frame() */
   GList *pending_events;
@@ -286,23 +278,48 @@ static GstStateChangeReturn gst_audio_decoder_change_state (GstElement *
     element, GstStateChange transition);
 static gboolean gst_audio_decoder_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_audio_decoder_src_event (GstPad * pad, GstEvent * event);
-static gboolean gst_audio_decoder_sink_setcaps (GstPad * pad, GstCaps * caps);
-static gboolean gst_audio_decoder_src_setcaps (GstPad * pad, GstCaps * caps);
+static gboolean gst_audio_decoder_sink_setcaps (GstAudioDecoder * dec,
+    GstCaps * caps);
+gboolean gst_audio_decoder_src_setcaps (GstAudioDecoder * dec, GstCaps * caps);
 static GstFlowReturn gst_audio_decoder_chain (GstPad * pad, GstBuffer * buf);
 static gboolean gst_audio_decoder_src_query (GstPad * pad, GstQuery * query);
 static gboolean gst_audio_decoder_sink_query (GstPad * pad, GstQuery * query);
 static const GstQueryType *gst_audio_decoder_get_query_types (GstPad * pad);
 static void gst_audio_decoder_reset (GstAudioDecoder * dec, gboolean full);
 
+static GstElementClass *parent_class = NULL;
 
-GST_BOILERPLATE (GstAudioDecoder, gst_audio_decoder, GstElement,
-    GST_TYPE_ELEMENT);
+static void gst_audio_decoder_class_init (GstAudioDecoderClass * klass);
+static void gst_audio_decoder_init (GstAudioDecoder * dec,
+    GstAudioDecoderClass * klass);
 
-static void
-gst_audio_decoder_base_init (gpointer g_class)
+GType
+gst_audio_decoder_get_type (void)
 {
+  static volatile gsize audio_decoder_type = 0;
+
+  if (g_once_init_enter (&audio_decoder_type)) {
+    GType _type;
+    static const GTypeInfo audio_decoder_info = {
+      sizeof (GstAudioDecoderClass),
+      NULL,
+      NULL,
+      (GClassInitFunc) gst_audio_decoder_class_init,
+      NULL,
+      NULL,
+      sizeof (GstAudioDecoder),
+      0,
+      (GInstanceInitFunc) gst_audio_decoder_init,
+    };
+
+    _type = g_type_register_static (GST_TYPE_ELEMENT,
+        "GstAudioDecoder", &audio_decoder_info, G_TYPE_FLAG_ABSTRACT);
+    g_once_init_leave (&audio_decoder_type, _type);
+  }
+  return audio_decoder_type;
 }
 
+
 static void
 gst_audio_decoder_class_init (GstAudioDecoderClass * klass)
 {
@@ -361,8 +378,6 @@ gst_audio_decoder_init (GstAudioDecoder * dec, GstAudioDecoderClass * klass)
   dec->sinkpad = gst_pad_new_from_template (pad_template, "sink");
   gst_pad_set_event_function (dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_audio_decoder_sink_event));
-  gst_pad_set_setcaps_function (dec->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_audio_decoder_sink_setcaps));
   gst_pad_set_chain_function (dec->sinkpad,
       GST_DEBUG_FUNCPTR (gst_audio_decoder_chain));
   gst_pad_set_query_function (dec->sinkpad,
@@ -376,8 +391,6 @@ gst_audio_decoder_init (GstAudioDecoder * dec, GstAudioDecoderClass * klass)
   g_return_if_fail (pad_template != NULL);
 
   dec->srcpad = gst_pad_new_from_template (pad_template, "src");
-  gst_pad_set_setcaps_function (dec->srcpad,
-      GST_DEBUG_FUNCPTR (gst_audio_decoder_src_setcaps));
   gst_pad_set_event_function (dec->srcpad,
       GST_DEBUG_FUNCPTR (gst_audio_decoder_src_event));
   gst_pad_set_query_function (dec->srcpad,
@@ -398,8 +411,6 @@ gst_audio_decoder_init (GstAudioDecoder * dec, GstAudioDecoderClass * klass)
   dec->priv->latency = DEFAULT_LATENCY;
   dec->priv->tolerance = DEFAULT_TOLERANCE;
   dec->priv->plc = DEFAULT_PLC;
-  dec->priv->drainable = DEFAULT_DRAINABLE;
-  dec->priv->needs_format = DEFAULT_NEEDS_FORMAT;
 
   /* init state */
   gst_audio_decoder_reset (dec, TRUE);
@@ -421,9 +432,8 @@ gst_audio_decoder_reset (GstAudioDecoder * dec, gboolean full)
     dec->priv->error_count = 0;
     gst_audio_decoder_clear_queues (dec);
 
-    gst_audio_info_clear (&dec->priv->ctx.info);
+    gst_audio_info_init (&dec->priv->ctx.info);
     memset (&dec->priv->ctx, 0, sizeof (dec->priv->ctx));
-    dec->priv->ctx.max_errors = GST_AUDIO_DECODER_MAX_ERRORS;
 
     if (dec->priv->taglist) {
       gst_tag_list_free (dec->priv->taglist);
@@ -473,17 +483,22 @@ gst_audio_decoder_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-/* automagically perform sanity checking of src caps;
- * also extracts output data format */
-static gboolean
-gst_audio_decoder_src_setcaps (GstPad * pad, GstCaps * caps)
+/**
+ * gst_audio_decoder_set_outcaps:
+ * @dec: a #GstAudioDecoder
+ * @caps: #GstCaps
+ *
+ * Configure output @caps on the srcpad of @dec. Also perform
+ * sanity checking of @caps and extracts output data format
+ *
+ * Returns: %TRUE on success.
+ * */
+gboolean
+gst_audio_decoder_set_outcaps (GstAudioDecoder * dec, GstCaps * caps)
 {
-  GstAudioDecoder *dec;
   gboolean res = TRUE;
   guint old_rate;
 
-  dec = GST_AUDIO_DECODER (gst_pad_get_parent (pad));
-
   GST_DEBUG_OBJECT (dec, "setting src caps %" GST_PTR_FORMAT, caps);
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
@@ -507,7 +522,8 @@ gst_audio_decoder_src_setcaps (GstPad * pad, GstCaps * caps)
 done:
   GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
-  gst_object_unref (dec);
+  res = gst_pad_set_caps (dec->srcpad, caps);
+
   return res;
 
   /* ERRORS */
@@ -520,13 +536,11 @@ refuse_caps:
 }
 
 static gboolean
-gst_audio_decoder_sink_setcaps (GstPad * pad, GstCaps * caps)
+gst_audio_decoder_sink_setcaps (GstAudioDecoder * dec, GstCaps * caps)
 {
-  GstAudioDecoder *dec;
   GstAudioDecoderClass *klass;
   gboolean res = TRUE;
 
-  dec = GST_AUDIO_DECODER (gst_pad_get_parent (pad));
   klass = GST_AUDIO_DECODER_GET_CLASS (dec);
 
   GST_DEBUG_OBJECT (dec, "caps: %" GST_PTR_FORMAT, caps);
@@ -534,18 +548,19 @@ gst_audio_decoder_sink_setcaps (GstPad * pad, GstCaps * caps)
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
   /* NOTE pbutils only needed here */
   /* TODO maybe (only) upstream demuxer/parser etc should handle this ? */
+#if 0
   if (dec->priv->taglist)
     gst_tag_list_free (dec->priv->taglist);
   dec->priv->taglist = gst_tag_list_new ();
   gst_pb_utils_add_codec_description_to_tag_list (dec->priv->taglist,
       GST_TAG_AUDIO_CODEC, caps);
+#endif
 
   if (klass->set_format)
     res = klass->set_format (dec, caps);
 
   GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
-  g_object_unref (dec);
   return res;
 }
 
@@ -565,79 +580,7 @@ gst_audio_decoder_setup (GstAudioDecoder * dec)
   gst_query_unref (query);
 
   /* normalize to bool */
-  dec->priv->agg = ! !res;
-}
-
-static GstFlowReturn
-gst_audio_decoder_push_forward (GstAudioDecoder * dec, GstBuffer * buf)
-{
-  GstAudioDecoderClass *klass;
-  GstAudioDecoderPrivate *priv;
-  GstAudioDecoderContext *ctx;
-  GstFlowReturn ret = GST_FLOW_OK;
-
-  klass = GST_AUDIO_DECODER_GET_CLASS (dec);
-  priv = dec->priv;
-  ctx = &dec->priv->ctx;
-
-  g_return_val_if_fail (ctx->info.bpf != 0, GST_FLOW_ERROR);
-
-  if (G_UNLIKELY (!buf)) {
-    g_assert_not_reached ();
-    return GST_FLOW_OK;
-  }
-
-  GST_LOG_OBJECT (dec, "clipping buffer of size %d with ts %" GST_TIME_FORMAT
-      ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buf),
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-      GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
-
-  /* clip buffer */
-  buf = gst_audio_buffer_clip (buf, &dec->segment, ctx->info.rate,
-      ctx->info.bpf);
-  if (G_UNLIKELY (!buf)) {
-    GST_DEBUG_OBJECT (dec, "no data after clipping to segment");
-    goto exit;
-  }
-
-  /* decorate */
-  gst_buffer_set_caps (buf, GST_PAD_CAPS (dec->srcpad));
-
-  if (G_UNLIKELY (priv->discont)) {
-    GST_LOG_OBJECT (dec, "marking discont");
-    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
-    priv->discont = FALSE;
-  }
-
-  /* track where we are */
-  if (G_LIKELY (GST_BUFFER_TIMESTAMP_IS_VALID (buf))) {
-    /* duration should always be valid for raw audio */
-    g_assert (GST_BUFFER_DURATION_IS_VALID (buf));
-    dec->segment.last_stop =
-        GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf);
-  }
-
-  if (klass->pre_push) {
-    /* last chance for subclass to do some dirty stuff */
-    ret = klass->pre_push (dec, &buf);
-    if (ret != GST_FLOW_OK || !buf) {
-      GST_DEBUG_OBJECT (dec, "subclass returned %s, buf %p",
-          gst_flow_get_name (ret), buf);
-      if (buf)
-        gst_buffer_unref (buf);
-      goto exit;
-    }
-  }
-
-  GST_LOG_OBJECT (dec, "pushing buffer of size %d with ts %" GST_TIME_FORMAT
-      ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buf),
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-      GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
-
-  ret = gst_pad_push (dec->srcpad, buf);
-
-exit:
-  return ret;
+  dec->priv->agg = !!res;
 }
 
 /* mini aggregator combining output buffers into fewer larger ones,
@@ -645,20 +588,42 @@ exit:
 static GstFlowReturn
 gst_audio_decoder_output (GstAudioDecoder * dec, GstBuffer * buf)
 {
+  GstAudioDecoderClass *klass;
   GstAudioDecoderPrivate *priv;
+  GstAudioDecoderContext *ctx;
   GstFlowReturn ret = GST_FLOW_OK;
   GstBuffer *inbuf = NULL;
 
+  klass = GST_AUDIO_DECODER_GET_CLASS (dec);
   priv = dec->priv;
+  ctx = &dec->priv->ctx;
 
   if (G_UNLIKELY (priv->agg < 0))
     gst_audio_decoder_setup (dec);
 
   if (G_LIKELY (buf)) {
-    GST_LOG_OBJECT (dec, "output buffer of size %d with ts %" GST_TIME_FORMAT
-        ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buf),
+    g_return_val_if_fail (ctx->info.bpf != 0, GST_FLOW_ERROR);
+
+    GST_LOG_OBJECT (dec,
+        "output buffer of size %" G_GSIZE_FORMAT " with ts %" GST_TIME_FORMAT
+        ", duration %" GST_TIME_FORMAT, gst_buffer_get_size (buf),
         GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+
+    /* clip buffer */
+    buf = gst_audio_buffer_clip (buf, &dec->segment, ctx->info.rate,
+        ctx->info.bpf);
+    if (G_UNLIKELY (!buf)) {
+      GST_DEBUG_OBJECT (dec, "no data after clipping to segment");
+    } else {
+      GST_LOG_OBJECT (dec,
+          "buffer after segment clipping has size %" G_GSIZE_FORMAT " with ts %"
+          GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT,
+          gst_buffer_get_size (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+    }
+  } else {
+    GST_DEBUG_OBJECT (dec, "no output buffer");
   }
 
 again:
@@ -691,7 +656,7 @@ again:
       }
       gst_adapter_push (priv->adapter_out, buf);
       priv->out_dur += GST_BUFFER_DURATION (buf);
-      av += GST_BUFFER_SIZE (buf);
+      av += gst_buffer_get_size (buf);
       buf = NULL;
     }
     if (priv->out_dur > dec->priv->latency)
@@ -708,8 +673,39 @@ again:
   }
 
   if (G_LIKELY (buf)) {
+
+    if (G_UNLIKELY (priv->discont)) {
+      GST_LOG_OBJECT (dec, "marking discont");
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+      priv->discont = FALSE;
+    }
+
+    if (G_LIKELY (GST_BUFFER_TIMESTAMP_IS_VALID (buf))) {
+      /* duration should always be valid for raw audio */
+      g_assert (GST_BUFFER_DURATION_IS_VALID (buf));
+      dec->segment.position =
+          GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf);
+    }
+
+    if (klass->pre_push) {
+      /* last chance for subclass to do some dirty stuff */
+      ret = klass->pre_push (dec, &buf);
+      if (ret != GST_FLOW_OK || !buf) {
+        GST_DEBUG_OBJECT (dec, "subclass returned %s, buf %p",
+            gst_flow_get_name (ret), buf);
+        if (buf)
+          gst_buffer_unref (buf);
+        goto exit;
+      }
+    }
+
+    GST_LOG_OBJECT (dec, "pushing buffer of size %d with ts %" GST_TIME_FORMAT
+        ", duration %" GST_TIME_FORMAT, gst_buffer_get_size (buf),
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
+
     if (dec->segment.rate > 0.0) {
-      ret = gst_audio_decoder_push_forward (dec, buf);
+      ret = gst_pad_push (dec->srcpad, buf);
       GST_LOG_OBJECT (dec, "buffer pushed: %s", gst_flow_get_name (ret));
     } else {
       ret = GST_FLOW_OK;
@@ -717,6 +713,7 @@ again:
       GST_LOG_OBJECT (dec, "buffer queued");
     }
 
+  exit:
     if (inbuf) {
       buf = inbuf;
       goto again;
@@ -726,27 +723,6 @@ again:
   return ret;
 }
 
-/**
- * gst_audio_decoder_finish_frame:
- * @dec: a #GstAudioDecoder
- * @buf: decoded data
- * @frames: number of decoded frames represented by decoded data
- *
- * Collects decoded data and pushes it downstream.
- *
- * @buf may be NULL in which case the indicated number of frames
- * are discarded and considered to have produced no output
- * (e.g. lead-in or setup frames).
- * Otherwise, source pad caps must be set when it is called with valid
- * data in @buf.
- *
- * Note that a frame received in gst_audio_decoder_handle_frame() may be
- * invalidated by a call to this function.
- *
- * Returns: a #GstFlowReturn that should be escalated to caller (of caller)
- *
- * Since: 0.10.36
- */
 GstFlowReturn
 gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
     gint frames)
@@ -755,27 +731,24 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
   GstAudioDecoderContext *ctx;
   gint samples = 0;
   GstClockTime ts, next_ts;
+  gsize size;
   GstFlowReturn ret = GST_FLOW_OK;
 
   /* subclass should know what it is producing by now */
-  g_return_val_if_fail (buf == NULL || GST_PAD_CAPS (dec->srcpad) != NULL,
+  g_return_val_if_fail (buf == NULL || gst_pad_has_current_caps (dec->srcpad),
       GST_FLOW_ERROR);
   /* subclass should not hand us no data */
-  g_return_val_if_fail (buf == NULL || GST_BUFFER_SIZE (buf) > 0,
+  g_return_val_if_fail (buf == NULL || gst_buffer_get_size (buf) > 0,
       GST_FLOW_ERROR);
   /* no dummy calls please */
   g_return_val_if_fail (frames != 0, GST_FLOW_ERROR);
 
   priv = dec->priv;
   ctx = &dec->priv->ctx;
-
-  /* must know the output format by now */
-  g_return_val_if_fail (buf == NULL || GST_AUDIO_INFO_IS_VALID (&ctx->info),
-      GST_FLOW_ERROR);
+  size = buf ? gst_buffer_get_size (buf) : 0;
 
   GST_LOG_OBJECT (dec, "accepting %d bytes == %d samples for %d frames",
-      buf ? GST_BUFFER_SIZE (buf) : -1,
-      buf ? GST_BUFFER_SIZE (buf) / ctx->info.bpf : -1, frames);
+      buf ? size : -1, buf ? size / ctx->info.bpf : -1, frames);
 
   GST_AUDIO_DECODER_STREAM_LOCK (dec);
 
@@ -793,10 +766,10 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
 
   /* output shoud be whole number of sample frames */
   if (G_LIKELY (buf && ctx->info.bpf)) {
-    if (GST_BUFFER_SIZE (buf) % ctx->info.bpf)
+    if (size % ctx->info.bpf)
       goto wrong_buffer;
     /* per channel least */
-    samples = GST_BUFFER_SIZE (buf) / ctx->info.bpf;
+    samples = size / ctx->info.bpf;
   }
 
   /* frame and ts book-keeping */
@@ -827,15 +800,15 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
     frames--;
   }
 
-  if (G_UNLIKELY (!buf))
-    goto exit;
-
   /* lock on */
   if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (priv->base_ts))) {
     priv->base_ts = ts;
     GST_DEBUG_OBJECT (dec, "base_ts now %" GST_TIME_FORMAT, GST_TIME_ARGS (ts));
   }
 
+  if (G_UNLIKELY (!buf))
+    goto exit;
+
   /* slightly convoluted approach caters for perfect ts if subclass desires */
   if (GST_CLOCK_TIME_IS_VALID (ts)) {
     if (dec->priv->tolerance > 0) {
@@ -843,18 +816,17 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
 
       g_assert (GST_CLOCK_TIME_IS_VALID (priv->base_ts));
       next_ts = priv->base_ts +
-          gst_util_uint64_scale (priv->samples, GST_SECOND, ctx->info.rate);
-      GST_LOG_OBJECT (dec,
-          "buffer is %" G_GUINT64_FORMAT " samples past base_ts %"
-          GST_TIME_FORMAT ", expected ts %" GST_TIME_FORMAT, priv->samples,
+          gst_util_uint64_scale (samples, GST_SECOND, ctx->info.rate);
+      GST_LOG_OBJECT (dec, "buffer is %d samples past base_ts %" GST_TIME_FORMAT
+          ", expected ts %" GST_TIME_FORMAT, samples,
           GST_TIME_ARGS (priv->base_ts), GST_TIME_ARGS (next_ts));
       diff = GST_CLOCK_DIFF (next_ts, ts);
       GST_LOG_OBJECT (dec, "ts diff %d ms", (gint) (diff / GST_MSECOND));
       /* if within tolerance,
        * discard buffer ts and carry on producing perfect stream,
        * otherwise resync to ts */
-      if (G_UNLIKELY (diff < (gint64) - dec->priv->tolerance ||
-              diff > (gint64) dec->priv->tolerance)) {
+      if (G_UNLIKELY (diff < -dec->priv->tolerance ||
+              diff > dec->priv->tolerance)) {
         GST_DEBUG_OBJECT (dec, "base_ts resync");
         priv->base_ts = ts;
         priv->samples = 0;
@@ -877,7 +849,7 @@ gst_audio_decoder_finish_frame (GstAudioDecoder * dec, GstBuffer * buf,
     priv->taglist = NULL;
   }
 
-  buf = gst_buffer_make_metadata_writable (buf);
+  buf = gst_buffer_make_writable (buf);
   if (G_LIKELY (GST_CLOCK_TIME_IS_VALID (priv->base_ts))) {
     GST_BUFFER_TIMESTAMP (buf) =
         priv->base_ts +
@@ -908,8 +880,7 @@ exit:
 wrong_buffer:
   {
     GST_ELEMENT_ERROR (dec, STREAM, ENCODE, (NULL),
-        ("buffer size %d not a multiple of %d", GST_BUFFER_SIZE (buf),
-            ctx->info.bpf));
+        ("buffer size %d not a multiple of %d", size, ctx->info.bpf));
     gst_buffer_unref (buf);
     ret = GST_FLOW_ERROR;
     goto exit;
@@ -931,13 +902,13 @@ gst_audio_decoder_handle_frame (GstAudioDecoder * dec,
     GstAudioDecoderClass * klass, GstBuffer * buffer)
 {
   if (G_LIKELY (buffer)) {
+    gsize size = gst_buffer_get_size (buffer);
     /* keep around for admin */
     GST_LOG_OBJECT (dec, "tracking frame size %d, ts %" GST_TIME_FORMAT,
-        GST_BUFFER_SIZE (buffer),
-        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
+        size, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)));
     g_queue_push_tail (&dec->priv->frames, buffer);
     dec->priv->ctx.delay = dec->priv->frames.length;
-    dec->priv->bytes_in += GST_BUFFER_SIZE (buffer);
+    dec->priv->bytes_in += size;
   } else {
     GST_LOG_OBJECT (dec, "providing subclass with NULL frame");
   }
@@ -1005,7 +976,6 @@ gst_audio_decoder_push_buffers (GstAudioDecoder * dec, gboolean force)
           break;
         } else if (ret == GST_FLOW_OK) {
           GST_LOG_OBJECT (dec, "frame at offset %d of length %d", offset, len);
-          g_assert (len);
           g_assert (offset + len <= av);
           priv->sync_flush = 0;
         } else {
@@ -1023,16 +993,12 @@ gst_audio_decoder_push_buffers (GstAudioDecoder * dec, gboolean force)
         priv->prev_ts = ts;
       }
       buffer = gst_adapter_take_buffer (priv->adapter, len);
-      buffer = gst_buffer_make_metadata_writable (buffer);
+      buffer = gst_buffer_make_writable (buffer);
       GST_BUFFER_TIMESTAMP (buffer) = ts;
       flush += len;
     } else {
       if (!force)
         break;
-      if (!priv->drainable) {
-        priv->drained = TRUE;
-        break;
-      }
       buffer = NULL;
     }
 
@@ -1064,7 +1030,7 @@ gst_audio_decoder_drain (GstAudioDecoder * dec)
 {
   GstFlowReturn ret;
 
-  if (dec->priv->drained && !dec->priv->gather)
+  if (dec->priv->drained)
     return GST_FLOW_OK;
   else {
     /* dispatch reverse pending buffers */
@@ -1122,14 +1088,7 @@ gst_audio_decoder_flush (GstAudioDecoder * dec, gboolean hard)
 static GstFlowReturn
 gst_audio_decoder_chain_forward (GstAudioDecoder * dec, GstBuffer * buffer)
 {
-  GstFlowReturn ret = GST_FLOW_OK;
-
-  /* discard silly case, though maybe ts may be of value ?? */
-  if (G_UNLIKELY (GST_BUFFER_SIZE (buffer) == 0)) {
-    GST_DEBUG_OBJECT (dec, "discarding empty buffer");
-    gst_buffer_unref (buffer);
-    goto exit;
-  }
+  GstFlowReturn ret;
 
   /* grab buffer */
   gst_adapter_push (dec->priv->adapter, buffer);
@@ -1140,7 +1099,6 @@ gst_audio_decoder_chain_forward (GstAudioDecoder * dec, GstBuffer * buffer)
   /* hand to subclass */
   ret = gst_audio_decoder_push_buffers (dec, FALSE);
 
-exit:
   GST_LOG_OBJECT (dec, "chain-done");
   return ret;
 }
@@ -1229,7 +1187,6 @@ gst_audio_decoder_flush_decode (GstAudioDecoder * dec)
 {
   GstAudioDecoderPrivate *priv = dec->priv;
   GstFlowReturn res = GST_FLOW_OK;
-  GstClockTime timestamp;
   GList *walk;
 
   walk = priv->decode;
@@ -1267,39 +1224,20 @@ gst_audio_decoder_flush_decode (GstAudioDecoder * dec)
   gst_audio_decoder_drain (dec);
 
   /* now send queued data downstream */
-  timestamp = GST_CLOCK_TIME_NONE;
   while (priv->queued) {
     GstBuffer *buf = GST_BUFFER_CAST (priv->queued->data);
 
-    /* duration should always be valid for raw audio */
-    g_assert (GST_BUFFER_DURATION_IS_VALID (buf));
-
-    /* interpolate (backward) if needed */
-    if (G_LIKELY (timestamp != -1))
-      timestamp -= GST_BUFFER_DURATION (buf);
-
-    if (!GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
-      GST_LOG_OBJECT (dec, "applying reverse interpolated ts %"
-          GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
-      GST_BUFFER_TIMESTAMP (buf) = timestamp;
-    } else {
-      /* track otherwise */
-      timestamp = GST_BUFFER_TIMESTAMP (buf);
-      GST_LOG_OBJECT (dec, "tracking ts %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (timestamp));
-    }
-
     if (G_LIKELY (res == GST_FLOW_OK)) {
       GST_DEBUG_OBJECT (dec, "pushing buffer %p of size %u, "
           "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
-          GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+          gst_buffer_get_size (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
           GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
       /* should be already, but let's be sure */
-      buf = gst_buffer_make_metadata_writable (buf);
+      buf = gst_buffer_make_writable (buf);
       /* avoid stray DISCONT from forward processing,
        * which have no meaning in reverse pushing */
       GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DISCONT);
-      res = gst_audio_decoder_push_forward (dec, buf);
+      res = gst_pad_push (dec->srcpad, buf);
     } else {
       gst_buffer_unref (buf);
     }
@@ -1335,7 +1273,7 @@ gst_audio_decoder_chain_reverse (GstAudioDecoder * dec, GstBuffer * buf)
   if (G_LIKELY (buf)) {
     GST_DEBUG_OBJECT (dec, "gathering buffer %p of size %u, "
         "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
-        GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+        gst_buffer_get_size (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
         GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
 
     /* add buffer to gather queue */
@@ -1353,12 +1291,9 @@ gst_audio_decoder_chain (GstPad * pad, GstBuffer * buffer)
 
   dec = GST_AUDIO_DECODER (GST_PAD_PARENT (pad));
 
-  if (G_UNLIKELY (!GST_PAD_CAPS (pad) && dec->priv->needs_format))
-    goto not_negotiated;
-
   GST_LOG_OBJECT (dec,
       "received buffer of size %d with ts %" GST_TIME_FORMAT
-      ", duration %" GST_TIME_FORMAT, GST_BUFFER_SIZE (buffer),
+      ", duration %" GST_TIME_FORMAT, gst_buffer_get_size (buffer),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
 
@@ -1393,15 +1328,6 @@ gst_audio_decoder_chain (GstPad * pad, GstBuffer * buffer)
   GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
 
   return ret;
-
-  /* ERRORS */
-not_negotiated:
-  {
-    GST_ELEMENT_ERROR (dec, CORE, NEGOTIATION, (NULL),
-        ("decoder not initialized"));
-    gst_buffer_unref (buffer);
-    return GST_FLOW_NOT_NEGOTIATED;
-  }
 }
 
 /* perform upstream byte <-> time conversion (duration, seeking)
@@ -1419,47 +1345,36 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
   gboolean handled = FALSE;
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_SEGMENT:
     {
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
-      gboolean update;
+      GstSegment seg;
 
       GST_AUDIO_DECODER_STREAM_LOCK (dec);
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
+      gst_event_copy_segment (event, &seg);
 
-      if (format == GST_FORMAT_TIME) {
-        GST_DEBUG_OBJECT (dec, "received TIME NEW_SEGMENT %" GST_TIME_FORMAT
-            " -- %" GST_TIME_FORMAT ", time %" GST_TIME_FORMAT
-            ", rate %g, applied_rate %g",
-            GST_TIME_ARGS (start), GST_TIME_ARGS (stop), GST_TIME_ARGS (time),
-            rate, arate);
+      if (seg.format == GST_FORMAT_TIME) {
+        GST_DEBUG_OBJECT (dec, "received TIME SEGMENT %" GST_PTR_FORMAT, &seg);
       } else {
-        GstFormat dformat = GST_FORMAT_TIME;
-
-        GST_DEBUG_OBJECT (dec, "received NEW_SEGMENT %" G_GINT64_FORMAT
-            " -- %" G_GINT64_FORMAT ", time %" G_GINT64_FORMAT
-            ", rate %g, applied_rate %g", start, stop, time, rate, arate);
+        gint64 nstart;
+        GST_DEBUG_OBJECT (dec, "received SEGMENT %" GST_PTR_FORMAT, &seg);
         /* handle newsegment resulting from legacy simple seeking */
         /* note that we need to convert this whether or not enough data
          * to handle initial newsegment */
         if (dec->priv->ctx.do_byte_time &&
-            gst_pad_query_convert (dec->sinkpad, GST_FORMAT_BYTES, start,
-                &dformat, &start)) {
+            gst_pad_query_convert (dec->sinkpad, GST_FORMAT_BYTES, seg.start,
+                GST_FORMAT_TIME, &nstart)) {
           /* best attempt convert */
           /* as these are only estimates, stop is kept open-ended to avoid
            * premature cutting */
           GST_DEBUG_OBJECT (dec, "converted to TIME start %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (start));
-          format = GST_FORMAT_TIME;
-          time = start;
-          stop = GST_CLOCK_TIME_NONE;
+              GST_TIME_ARGS (nstart));
+          seg.format = GST_FORMAT_TIME;
+          seg.start = nstart;
+          seg.time = nstart;
+          seg.stop = GST_CLOCK_TIME_NONE;
           /* replace event */
           gst_event_unref (event);
-          event = gst_event_new_new_segment_full (update, rate, arate,
-              GST_FORMAT_TIME, start, stop, time);
+          event = gst_event_new_segment (&seg);
         } else {
           GST_DEBUG_OBJECT (dec, "unsupported format; ignoring");
           GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
@@ -1470,41 +1385,42 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
       /* finish current segment */
       gst_audio_decoder_drain (dec);
 
+#if 0
       if (update) {
         /* time progressed without data, see if we can fill the gap with
          * some concealment data */
         GST_DEBUG_OBJECT (dec,
-            "segment update: plc %d, do_plc %d, last_stop %" GST_TIME_FORMAT,
+            "segment update: plc %d, do_plc %d, position %" GST_TIME_FORMAT,
             dec->priv->plc, dec->priv->ctx.do_plc,
-            GST_TIME_ARGS (dec->segment.last_stop));
+            GST_TIME_ARGS (dec->segment.position));
         if (dec->priv->plc && dec->priv->ctx.do_plc &&
-            dec->segment.rate > 0.0 && dec->segment.last_stop < start) {
+            dec->segment.rate > 0.0 && dec->segment.position < start) {
           GstAudioDecoderClass *klass;
           GstBuffer *buf;
 
           klass = GST_AUDIO_DECODER_GET_CLASS (dec);
           /* hand subclass empty frame with duration that needs covering */
           buf = gst_buffer_new ();
-          GST_BUFFER_DURATION (buf) = start - dec->segment.last_stop;
+          GST_BUFFER_DURATION (buf) = start - dec->segment.position;
           /* best effort, not much error handling */
           gst_audio_decoder_handle_frame (dec, klass, buf);
         }
-      } else {
+      } else
+#endif
+      {
         /* prepare for next one */
         gst_audio_decoder_flush (dec, FALSE);
         /* and that's where we time from,
          * in case upstream does not come up with anything better
          * (e.g. upstream BYTE) */
-        if (format != GST_FORMAT_TIME) {
-          dec->priv->base_ts = start;
+        if (seg.format != GST_FORMAT_TIME) {
+          dec->priv->base_ts = seg.start;
           dec->priv->samples = 0;
         }
       }
 
       /* and follow along with segment */
-      gst_segment_set_newsegment_full (&dec->segment, update, rate, arate,
-          format, start, stop, time);
-
+      dec->segment = seg;
       dec->priv->pending_events =
           g_list_append (dec->priv->pending_events, event);
       handled = TRUE;
@@ -1532,6 +1448,16 @@ gst_audio_decoder_sink_eventfunc (GstAudioDecoder * dec, GstEvent * event)
       GST_AUDIO_DECODER_STREAM_UNLOCK (dec);
       break;
 
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      gst_audio_decoder_sink_setcaps (dec, caps);
+      gst_event_unref (event);
+      handled = TRUE;
+      break;
+    }
     default:
       break;
   }
@@ -1624,13 +1550,12 @@ gst_audio_decoder_do_seek (GstAudioDecoder * dec, GstEvent * event)
   }
 
   memcpy (&seek_segment, &dec->segment, sizeof (seek_segment));
-  gst_segment_set_seek (&seek_segment, rate, format, flags, start_type,
+  gst_segment_do_seek (&seek_segment, rate, format, flags, start_type,
       start_time, end_type, end_time, NULL);
-  start_time = seek_segment.last_stop;
+  start_time = seek_segment.position;
 
-  format = GST_FORMAT_BYTES;
   if (!gst_pad_query_convert (dec->sinkpad, GST_FORMAT_TIME, start_time,
-          &format, &start)) {
+          GST_FORMAT_BYTES, &start)) {
     GST_DEBUG_OBJECT (dec, "conversion failed");
     return FALSE;
   }
@@ -1653,10 +1578,6 @@ gst_audio_decoder_src_event (GstPad * pad, GstEvent * event)
   gboolean res = FALSE;
 
   dec = GST_AUDIO_DECODER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (dec == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
 
   GST_DEBUG_OBJECT (dec, "received event %d, %s", GST_EVENT_TYPE (event),
       GST_EVENT_TYPE_NAME (event));
@@ -1664,7 +1585,7 @@ gst_audio_decoder_src_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      GstFormat format, tformat;
+      GstFormat format;
       gdouble rate;
       GstSeekFlags flags;
       GstSeekType cur_type, stop_type;
@@ -1689,10 +1610,12 @@ gst_audio_decoder_src_event (GstPad * pad, GstEvent * event)
 
       /* ... though a non-time seek can be aided as well */
       /* First bring the requested format to time */
-      tformat = GST_FORMAT_TIME;
-      if (!(res = gst_pad_query_convert (pad, format, cur, &tformat, &tcur)))
+      if (!(res =
+              gst_pad_query_convert (pad, format, cur, GST_FORMAT_TIME, &tcur)))
         goto convert_error;
-      if (!(res = gst_pad_query_convert (pad, format, stop, &tformat, &tstop)))
+      if (!(res =
+              gst_pad_query_convert (pad, format, stop, GST_FORMAT_TIME,
+                  &tstop)))
         goto convert_error;
 
       /* then seek with time on the peer */
@@ -1855,9 +1778,6 @@ gst_audio_decoder_src_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
 
   dec = GST_AUDIO_DECODER (GST_PAD_PARENT (pad));
-  if (G_UNLIKELY (dec == NULL))
-    return FALSE;
-
   peerpad = gst_pad_get_peer (GST_PAD (dec->sinkpad));
 
   GST_LOG_OBJECT (dec, "handling query: %" GST_PTR_FORMAT, query);
@@ -1876,12 +1796,11 @@ gst_audio_decoder_src_query (GstPad * pad, GstQuery * query)
       if (format == GST_FORMAT_TIME && gst_audio_decoder_do_byte (dec)) {
         gint64 value;
 
-        format = GST_FORMAT_BYTES;
-        if (gst_pad_query_peer_duration (dec->sinkpad, &format, &value)) {
+        if (gst_pad_query_peer_duration (dec->sinkpad, GST_FORMAT_BYTES,
+                &value)) {
           GST_LOG_OBJECT (dec, "upstream size %" G_GINT64_FORMAT, value);
-          format = GST_FORMAT_TIME;
           if (gst_pad_query_convert (dec->sinkpad, GST_FORMAT_BYTES, value,
-                  &format, &value)) {
+                  GST_FORMAT_TIME, &value)) {
             gst_query_set_duration (query, GST_FORMAT_TIME, value);
             res = TRUE;
           }
@@ -1900,7 +1819,7 @@ gst_audio_decoder_src_query (GstPad * pad, GstQuery * query)
       }
 
       /* we start from the last seen time */
-      time = dec->segment.last_stop;
+      time = dec->segment.position;
       /* correct for the segment values */
       time = gst_segment_to_stream_time (&dec->segment, GST_FORMAT_TIME, time);
 
@@ -1910,7 +1829,7 @@ gst_audio_decoder_src_query (GstPad * pad, GstQuery * query)
       /* and convert to the final format */
       gst_query_parse_position (query, &format, NULL);
       if (!(res = gst_pad_query_convert (pad, GST_FORMAT_TIME, time,
-                  &format, &value)))
+                  format, &value)))
         break;
 
       gst_query_set_position (query, format, value);
@@ -2087,7 +2006,7 @@ gst_audio_decoder_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  ret = parent_class->change_state (element, transition);
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
@@ -2241,8 +2160,7 @@ gst_audio_decoder_get_delay (GstAudioDecoder * dec)
  * @num: max tolerated errors
  *
  * Sets numbers of tolerated decoder errors, where a tolerated one is then only
- * warned about, but more than tolerated will lead to fatal error.  Default
- * is set to GST_AUDIO_DECODER_MAX_ERRORS.
+ * warned about, but more than tolerated will lead to fatal error.
  *
  * Since: 0.10.36
  */
@@ -2482,105 +2400,3 @@ gst_audio_decoder_get_tolerance (GstAudioDecoder * dec)
 
   return result;
 }
-
-/**
- * gst_audio_decoder_set_drainable:
- * @dec: a #GstAudioDecoder
- * @enabled: new state
- *
- * Configures decoder drain handling.  If drainable, subclass might
- * be handed a NULL buffer to have it return any leftover decoded data.
- * Otherwise, it is not considered so capable and will only ever be passed
- * real data.
- *
- * MT safe.
- *
- * Since: 0.10.36
- */
-void
-gst_audio_decoder_set_drainable (GstAudioDecoder * dec, gboolean enabled)
-{
-  g_return_if_fail (GST_IS_AUDIO_DECODER (dec));
-
-  GST_OBJECT_LOCK (dec);
-  dec->priv->drainable = enabled;
-  GST_OBJECT_UNLOCK (dec);
-}
-
-/**
- * gst_audio_decoder_get_drainable:
- * @dec: a #GstAudioDecoder
- *
- * Queries decoder drain handling.
- *
- * Returns: TRUE if drainable handling is enabled.
- *
- * MT safe.
- *
- * Since: 0.10.36
- */
-gboolean
-gst_audio_decoder_get_drainable (GstAudioDecoder * dec)
-{
-  gboolean result;
-
-  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), 0);
-
-  GST_OBJECT_LOCK (dec);
-  result = dec->priv->drainable;
-  GST_OBJECT_UNLOCK (dec);
-
-  return result;
-}
-
-/**
- * gst_audio_decoder_set_needs_format:
- * @dec: a #GstAudioDecoder
- * @enabled: new state
- *
- * Configures decoder format needs.  If enabled, subclass needs to be
- * negotiated with format caps before it can process any data.  It will then
- * never be handed any data before it has been configured.
- * Otherwise, it might be handed data without having been configured and
- * is then expected being able to do so either by default
- * or based on the input data.
- *
- * MT safe.
- *
- * Since: 0.10.36
- */
-void
-gst_audio_decoder_set_needs_format (GstAudioDecoder * dec, gboolean enabled)
-{
-  g_return_if_fail (GST_IS_AUDIO_DECODER (dec));
-
-  GST_OBJECT_LOCK (dec);
-  dec->priv->needs_format = enabled;
-  GST_OBJECT_UNLOCK (dec);
-}
-
-/**
- * gst_audio_decoder_get_needs_format:
- * @dec: a #GstAudioDecoder
- *
- * Queries decoder required format handling.
- *
- * Returns: TRUE if required format handling is enabled.
- *
- * MT safe.
- *
- * Since: 0.10.36
- */
-gboolean
-gst_audio_decoder_get_needs_format (GstAudioDecoder * dec)
-{
-  gboolean result;
-
-  g_return_val_if_fail (GST_IS_AUDIO_DECODER (dec), 0);
-
-  GST_OBJECT_LOCK (dec);
-  result = dec->priv->needs_format;
-  GST_OBJECT_UNLOCK (dec);
-
-  return result;
-}
