@@ -170,8 +170,8 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
 
   ring_parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_audioringbuffer_dispose);
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_audioringbuffer_finalize);
+  gobject_class->dispose = gst_audioringbuffer_dispose;
+  gobject_class->finalize = gst_audioringbuffer_finalize;
 
   gstringbuffer_class->open_device =
       GST_DEBUG_FUNCPTR (gst_audioringbuffer_open_device);
@@ -191,7 +191,7 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
       GST_DEBUG_FUNCPTR (gst_audioringbuffer_activate);
 }
 
-typedef guint (*WriteFunc) (GstAudioSink * sink, gpointer data, guint length);
+typedef gint (*WriteFunc) (GstAudioSink * sink, gpointer data, guint length);
 
 /* this internal thread does nothing else but write samples to the audio device.
  * It will write each segment in the ringbuffer and will update the play
@@ -394,8 +394,8 @@ gst_audioringbuffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   /* set latency to one more segment as we need some headroom */
   spec->seglatency = spec->segtotal + 1;
 
-  buf->data = gst_buffer_new_and_alloc (spec->segtotal * spec->segsize);
-  memset (GST_BUFFER_DATA (buf->data), 0, GST_BUFFER_SIZE (buf->data));
+  buf->size = spec->segtotal * spec->segsize;
+  buf->memory = g_malloc0 (buf->size);
 
   return TRUE;
 
@@ -468,8 +468,8 @@ gst_audioringbuffer_release (GstRingBuffer * buf)
   csink = GST_AUDIO_SINK_GET_CLASS (sink);
 
   /* free the buffer */
-  gst_buffer_unref (buf->data);
-  buf->data = NULL;
+  g_free (buf->memory);
+  buf->memory = NULL;
 
   if (csink->unprepare)
     result = csink->unprepare (sink);
@@ -574,20 +574,15 @@ enum
   ARG_0,
 };
 
-#define _do_init(bla) \
+#define _do_init \
     GST_DEBUG_CATEGORY_INIT (gst_audio_sink_debug, "audiosink", 0, "audiosink element");
-
-GST_BOILERPLATE_FULL (GstAudioSink, gst_audio_sink, GstBaseAudioSink,
+#define gst_audio_sink_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstAudioSink, gst_audio_sink,
     GST_TYPE_BASE_AUDIO_SINK, _do_init);
 
 static GstRingBuffer *gst_audio_sink_create_ringbuffer (GstBaseAudioSink *
     sink);
 
-static void
-gst_audio_sink_base_init (gpointer g_class)
-{
-}
-
 static void
 gst_audio_sink_class_init (GstAudioSinkClass * klass)
 {
@@ -602,7 +597,7 @@ gst_audio_sink_class_init (GstAudioSinkClass * klass)
 }
 
 static void
-gst_audio_sink_init (GstAudioSink * audiosink, GstAudioSinkClass * g_class)
+gst_audio_sink_init (GstAudioSink * audiosink)
 {
 }
 
