@@ -48,20 +48,24 @@ GST_DEBUG_CATEGORY_STATIC (gst_base_audio_src_debug);
 GType
 gst_base_audio_src_slave_method_get_type (void)
 {
-  static GType slave_method_type = 0;
+  static volatile gsize slave_method_type = 0;
+  /* FIXME 0.11: nick should be "retimestamp" not "re-timestamp" */
   static const GEnumValue slave_method[] = {
-    {GST_BASE_AUDIO_SRC_SLAVE_RESAMPLE, "Resampling slaving", "resample"},
-    {GST_BASE_AUDIO_SRC_SLAVE_RETIMESTAMP, "Re-timestamp", "re-timestamp"},
-    {GST_BASE_AUDIO_SRC_SLAVE_SKEW, "Skew", "skew"},
-    {GST_BASE_AUDIO_SRC_SLAVE_NONE, "No slaving", "none"},
+    {GST_BASE_AUDIO_SRC_SLAVE_RESAMPLE,
+        "GST_BASE_AUDIO_SRC_SLAVE_RESAMPLE", "resample"},
+    {GST_BASE_AUDIO_SRC_SLAVE_RETIMESTAMP,
+        "GST_BASE_AUDIO_SRC_SLAVE_RETIMESTAMP", "re-timestamp"},
+    {GST_BASE_AUDIO_SRC_SLAVE_SKEW, "GST_BASE_AUDIO_SRC_SLAVE_SKEW", "skew"},
+    {GST_BASE_AUDIO_SRC_SLAVE_NONE, "GST_BASE_AUDIO_SRC_SLAVE_NONE", "none"},
     {0, NULL, NULL},
   };
 
-  if (!slave_method_type) {
-    slave_method_type =
+  if (g_once_init_enter (&slave_method_type)) {
+    GType tmp =
         g_enum_register_static ("GstBaseAudioSrcSlaveMethod", slave_method);
+    g_once_init_leave (&slave_method_type, tmp);
   }
-  return slave_method_type;
+  return (GType) slave_method_type;
 }
 
 #define GST_BASE_AUDIO_SRC_GET_PRIVATE(obj)  \
@@ -115,8 +119,9 @@ _do_init (GType type)
 #endif /* ENABLE_NLS */
 }
 
-GST_BOILERPLATE_FULL (GstBaseAudioSrc, gst_base_audio_src, GstPushSrc,
-    GST_TYPE_PUSH_SRC, _do_init);
+#define gst_base_audio_src_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstBaseAudioSrc, gst_base_audio_src, GST_TYPE_PUSH_SRC,
+    _do_init (g_define_type_id));
 
 static void gst_base_audio_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -133,7 +138,6 @@ static GstClockTime gst_base_audio_src_get_time (GstClock * clock,
 
 static GstFlowReturn gst_base_audio_src_create (GstBaseSrc * bsrc,
     guint64 offset, guint length, GstBuffer ** buf);
-static gboolean gst_base_audio_src_check_get_range (GstBaseSrc * bsrc);
 
 static gboolean gst_base_audio_src_event (GstBaseSrc * bsrc, GstEvent * event);
 static void gst_base_audio_src_get_times (GstBaseSrc * bsrc,
@@ -144,11 +148,6 @@ static void gst_base_audio_src_fixate (GstBaseSrc * bsrc, GstCaps * caps);
 
 /* static guint gst_base_audio_src_signals[LAST_SIGNAL] = { 0 }; */
 
-static void
-gst_base_audio_src_base_init (gpointer g_class)
-{
-}
-
 static void
 gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
 {
@@ -162,11 +161,9 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
 
   g_type_class_add_private (klass, sizeof (GstBaseAudioSrcPrivate));
 
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_base_audio_src_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_base_audio_src_get_property);
-  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_base_audio_src_dispose);
+  gobject_class->set_property = gst_base_audio_src_set_property;
+  gobject_class->get_property = gst_base_audio_src_get_property;
+  gobject_class->dispose = gst_base_audio_src_dispose;
 
   g_object_class_install_property (gobject_class, PROP_BUFFER_TIME,
       g_param_spec_int64 ("buffer-time", "Buffer Time",
@@ -191,7 +188,7 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
       g_param_spec_int64 ("actual-buffer-time", "Actual Buffer Time",
           "Actual configured size of audio buffer in microseconds",
           DEFAULT_ACTUAL_BUFFER_TIME, G_MAXINT64, DEFAULT_ACTUAL_BUFFER_TIME,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   /**
    * GstBaseAudioSrc:actual-latency-time:
@@ -204,7 +201,7 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
       g_param_spec_int64 ("actual-latency-time", "Actual Latency Time",
           "Actual configured audio latency in microseconds",
           DEFAULT_ACTUAL_LATENCY_TIME, G_MAXINT64, DEFAULT_ACTUAL_LATENCY_TIME,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_PROVIDE_CLOCK,
       g_param_spec_boolean ("provide-clock", "Provide Clock",
@@ -228,8 +225,6 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
   gstbasesrc_class->get_times =
       GST_DEBUG_FUNCPTR (gst_base_audio_src_get_times);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_base_audio_src_create);
-  gstbasesrc_class->check_get_range =
-      GST_DEBUG_FUNCPTR (gst_base_audio_src_check_get_range);
   gstbasesrc_class->fixate = GST_DEBUG_FUNCPTR (gst_base_audio_src_fixate);
 
   /* ref class from a thread-safe context to work around missing bit of
@@ -239,8 +234,7 @@ gst_base_audio_src_class_init (GstBaseAudioSrcClass * klass)
 }
 
 static void
-gst_base_audio_src_init (GstBaseAudioSrc * baseaudiosrc,
-    GstBaseAudioSrcClass * g_class)
+gst_base_audio_src_init (GstBaseAudioSrc * baseaudiosrc)
 {
   baseaudiosrc->priv = GST_BASE_AUDIO_SRC_GET_PRIVATE (baseaudiosrc);
 
@@ -269,9 +263,11 @@ gst_base_audio_src_dispose (GObject * object)
   src = GST_BASE_AUDIO_SRC (object);
 
   GST_OBJECT_LOCK (src);
-  if (src->clock)
+  if (src->clock) {
+    gst_audio_clock_invalidate (src->clock);
     gst_object_unref (src->clock);
-  src->clock = NULL;
+    src->clock = NULL;
+  }
 
   if (src->ringbuffer) {
     gst_object_unparent (GST_OBJECT_CAST (src->ringbuffer));
@@ -327,7 +323,8 @@ gst_base_audio_src_get_time (GstClock * clock, GstBaseAudioSrc * src)
   guint delay;
   GstClockTime result;
 
-  if (G_UNLIKELY (src->ringbuffer == NULL || src->ringbuffer->spec.rate == 0))
+  if (G_UNLIKELY (src->ringbuffer == NULL
+          || src->ringbuffer->spec.info.rate == 0))
     return GST_CLOCK_TIME_NONE;
 
   raw = samples = gst_ring_buffer_samples_done (src->ringbuffer);
@@ -339,25 +336,16 @@ gst_base_audio_src_get_time (GstClock * clock, GstBaseAudioSrc * src)
   samples += delay;
 
   result = gst_util_uint64_scale_int (samples, GST_SECOND,
-      src->ringbuffer->spec.rate);
+      src->ringbuffer->spec.info.rate);
 
   GST_DEBUG_OBJECT (src,
-      "processed samples: raw %llu, delay %u, real %llu, time %"
-      GST_TIME_FORMAT, raw, delay, samples, GST_TIME_ARGS (result));
+      "processed samples: raw %" G_GUINT64_FORMAT ", delay %u, real %"
+      G_GUINT64_FORMAT ", time %" GST_TIME_FORMAT, raw, delay, samples,
+      GST_TIME_ARGS (result));
 
   return result;
 }
 
-static gboolean
-gst_base_audio_src_check_get_range (GstBaseSrc * bsrc)
-{
-  /* we allow limited pull base operation of which the details
-   * will eventually exposed in an as of yet non-existing query.
-   * Basically pulling can be done on any number of bytes as long
-   * as the offset is -1 or sequentially increasing. */
-  return TRUE;
-}
-
 /**
  * gst_base_audio_src_set_provide_clock:
  * @src: a #GstBaseAudioSrc
@@ -522,26 +510,16 @@ static void
 gst_base_audio_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
 {
   GstStructure *s;
-  gint width, depth;
 
   s = gst_caps_get_structure (caps, 0);
 
   /* fields for all formats */
-  gst_structure_fixate_field_nearest_int (s, "rate", 44100);
-  gst_structure_fixate_field_nearest_int (s, "channels", 2);
-  gst_structure_fixate_field_nearest_int (s, "width", 16);
+  gst_structure_fixate_field_nearest_int (s, "rate", GST_AUDIO_DEF_RATE);
+  gst_structure_fixate_field_nearest_int (s, "channels",
+      GST_AUDIO_DEF_CHANNELS);
+  gst_structure_fixate_field_string (s, "format", GST_AUDIO_DEF_FORMAT);
 
-  /* fields for int */
-  if (gst_structure_has_field (s, "depth")) {
-    gst_structure_get_int (s, "width", &width);
-    /* round width to nearest multiple of 8 for the depth */
-    depth = GST_ROUND_UP_8 (width);
-    gst_structure_fixate_field_nearest_int (s, "depth", depth);
-  }
-  if (gst_structure_has_field (s, "signed"))
-    gst_structure_fixate_field_boolean (s, "signed", TRUE);
-  if (gst_structure_has_field (s, "endianness"))
-    gst_structure_fixate_field_nearest_int (s, "endianness", G_BYTE_ORDER);
+  GST_BASE_SRC_CLASS (parent_class)->fixate (bsrc, caps);
 }
 
 static gboolean
@@ -549,20 +527,28 @@ gst_base_audio_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
 {
   GstBaseAudioSrc *src = GST_BASE_AUDIO_SRC (bsrc);
   GstRingBufferSpec *spec;
+  gint bpf, rate;
 
   spec = &src->ringbuffer->spec;
 
   spec->buffer_time = src->buffer_time;
   spec->latency_time = src->latency_time;
 
-  if (!gst_ring_buffer_parse_caps (spec, caps))
+  GST_OBJECT_LOCK (src);
+  if (!gst_ring_buffer_parse_caps (spec, caps)) {
+    GST_OBJECT_UNLOCK (src);
     goto parse_error;
+  }
+
+  bpf = GST_AUDIO_INFO_BPF (&spec->info);
+  rate = GST_AUDIO_INFO_RATE (&spec->info);
 
   /* calculate suggested segsize and segtotal */
-  spec->segsize =
-      spec->rate * spec->bytes_per_sample * spec->latency_time / GST_MSECOND;
+  spec->segsize = rate * bpf * spec->latency_time / GST_MSECOND;
   spec->segtotal = spec->buffer_time / spec->latency_time;
 
+  GST_OBJECT_UNLOCK (src);
+
   GST_DEBUG ("release old ringbuffer");
 
   gst_ring_buffer_release (src->ringbuffer);
@@ -575,11 +561,9 @@ gst_base_audio_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
     goto acquire_error;
 
   /* calculate actual latency and buffer times */
-  spec->latency_time =
-      spec->segsize * GST_MSECOND / (spec->rate * spec->bytes_per_sample);
+  spec->latency_time = spec->segsize * GST_MSECOND / (rate * bpf);
   spec->buffer_time =
-      spec->segtotal * spec->segsize * GST_MSECOND / (spec->rate *
-      spec->bytes_per_sample);
+      spec->segtotal * spec->segsize * GST_MSECOND / (rate * bpf);
 
   gst_ring_buffer_debug_spec_buff (spec);
 
@@ -622,21 +606,27 @@ gst_base_audio_src_query (GstBaseSrc * bsrc, GstQuery * query)
     {
       GstClockTime min_latency, max_latency;
       GstRingBufferSpec *spec;
+      gint bpf, rate;
 
+      GST_OBJECT_LOCK (src);
       if (G_UNLIKELY (src->ringbuffer == NULL
-              || src->ringbuffer->spec.rate == 0))
+              || src->ringbuffer->spec.info.rate == 0)) {
+        GST_OBJECT_UNLOCK (src);
         goto done;
+      }
 
       spec = &src->ringbuffer->spec;
+      rate = GST_AUDIO_INFO_RATE (&spec->info);
+      bpf = GST_AUDIO_INFO_BPF (&spec->info);
 
       /* we have at least 1 segment of latency */
       min_latency =
-          gst_util_uint64_scale_int (spec->segsize, GST_SECOND,
-          spec->rate * spec->bytes_per_sample);
+          gst_util_uint64_scale_int (spec->segsize, GST_SECOND, rate * bpf);
       /* we cannot delay more than the buffersize else we lose data */
       max_latency =
           gst_util_uint64_scale_int (spec->segtotal * spec->segsize, GST_SECOND,
-          spec->rate * spec->bytes_per_sample);
+          rate * bpf);
+      GST_OBJECT_UNLOCK (src);
 
       GST_DEBUG_OBJECT (src,
           "report latency min %" GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
@@ -649,6 +639,17 @@ gst_base_audio_src_query (GstBaseSrc * bsrc, GstQuery * query)
       res = TRUE;
       break;
     }
+    case GST_QUERY_SCHEDULING:
+    {
+      /* we allow limited pull base operation of which the details
+       * will eventually exposed in an as of yet non-existing query.
+       * Basically pulling can be done on any number of bytes as long
+       * as the offset is -1 or sequentially increasing. */
+      gst_query_set_scheduling (query, TRUE, FALSE, TRUE, 1, -1, 1);
+
+      res = TRUE;
+      break;
+    }
     default:
       res = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
       break;
@@ -700,33 +701,41 @@ gst_base_audio_src_get_offset (GstBaseAudioSrc * src)
 
   /* assume we can append to the previous sample */
   sample = src->next_sample;
-  /* no previous sample, try to read from position 0 */
-  if (sample == -1)
-    sample = 0;
 
   sps = src->ringbuffer->samples_per_seg;
   segtotal = src->ringbuffer->spec.segtotal;
 
-  /* figure out the segment and the offset inside the segment where
-   * the sample should be read from. */
-  readseg = sample / sps;
-
   /* get the currently processed segment */
   segdone = g_atomic_int_get (&src->ringbuffer->segdone)
       - src->ringbuffer->segbase;
 
-  GST_DEBUG_OBJECT (src, "reading from %d, we are at %d", readseg, segdone);
+  if (sample != -1) {
+    GST_DEBUG_OBJECT (src, "at segment %d and sample %" G_GUINT64_FORMAT,
+        segdone, sample);
+    /* figure out the segment and the offset inside the segment where
+     * the sample should be read from. */
+    readseg = sample / sps;
 
-  /* see how far away it is from the read segment, normally segdone (where new
-   * data is written in the ringbuffer) is bigger than readseg (where we are
-   * reading). */
-  diff = segdone - readseg;
-  if (diff >= segtotal) {
-    GST_DEBUG_OBJECT (src, "dropped, align to segment %d", segdone);
-    /* sample would be dropped, position to next playable position */
+    /* see how far away it is from the read segment, normally segdone (where new
+     * data is written in the ringbuffer) is bigger than readseg (where we are
+     * reading). */
+    diff = segdone - readseg;
+    if (diff >= segtotal) {
+      GST_DEBUG_OBJECT (src, "dropped, align to segment %d", segdone);
+      /* sample would be dropped, position to next playable position */
+      sample = ((guint64) (segdone)) * sps;
+    }
+  } else {
+    /* no previous sample, go to the current position */
+    GST_DEBUG_OBJECT (src, "first sample, align to current %d", segdone);
     sample = ((guint64) (segdone)) * sps;
+    readseg = segdone;
   }
 
+  GST_DEBUG_OBJECT (src,
+      "reading from %d, we are at %d, sample %" G_GUINT64_FORMAT, readseg,
+      segdone, sample);
+
   return sample;
 }
 
@@ -734,12 +743,13 @@ static GstFlowReturn
 gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     GstBuffer ** outbuf)
 {
+  GstFlowReturn ret;
   GstBaseAudioSrc *src = GST_BASE_AUDIO_SRC (bsrc);
   GstBuffer *buf;
-  guchar *data;
+  guchar *data, *ptr;
   guint samples, total_samples;
   guint64 sample;
-  gint bps;
+  gint bpf, rate;
   GstRingBuffer *ringbuffer;
   GstRingBufferSpec *spec;
   guint read;
@@ -752,18 +762,19 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
   if (G_UNLIKELY (!gst_ring_buffer_is_acquired (ringbuffer)))
     goto wrong_state;
 
-  bps = spec->bytes_per_sample;
+  bpf = GST_AUDIO_INFO_BPF (&spec->info);
+  rate = GST_AUDIO_INFO_RATE (&spec->info);
 
   if ((length == 0 && bsrc->blocksize == 0) || length == -1)
     /* no length given, use the default segment size */
     length = spec->segsize;
   else
     /* make sure we round down to an integral number of samples */
-    length -= length % bps;
+    length -= length % bpf;
 
   /* figure out the offset in the ringbuffer */
   if (G_UNLIKELY (offset != -1)) {
-    sample = offset / bps;
+    sample = offset / bpf;
     /* if a specific offset was given it must be the next sequential
      * offset we expect or we fail for now. */
     if (src->next_sample != -1 && sample != src->next_sample)
@@ -774,17 +785,20 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     sample = gst_base_audio_src_get_offset (src);
   }
 
-  GST_DEBUG_OBJECT (src, "reading from sample %" G_GUINT64_FORMAT, sample);
+  GST_DEBUG_OBJECT (src, "reading from sample %" G_GUINT64_FORMAT " length %u",
+      sample, length);
 
   /* get the number of samples to read */
-  total_samples = samples = length / bps;
+  total_samples = samples = length / bpf;
 
-  /* FIXME, using a bufferpool would be nice here */
-  buf = gst_buffer_new_and_alloc (length);
-  data = GST_BUFFER_DATA (buf);
+  /* use the basesrc allocation code to use bufferpools or custom allocators */
+  ret = GST_BASE_SRC_CLASS (parent_class)->alloc (bsrc, offset, length, &buf);
+  if (G_UNLIKELY (ret != GST_FLOW_OK))
+    goto alloc_failed;
 
+  data = ptr = gst_buffer_map (buf, NULL, NULL, GST_MAP_WRITE);
   do {
-    read = gst_ring_buffer_read (ringbuffer, sample, data, samples);
+    read = gst_ring_buffer_read (ringbuffer, sample, ptr, samples);
     GST_DEBUG_OBJECT (src, "read %u of %u", read, samples);
     /* if we read all, we're done */
     if (read == samples)
@@ -800,8 +814,9 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     /* read next samples */
     sample += read;
     samples -= read;
-    data += read * bps;
+    ptr += read * bpf;
   } while (TRUE);
+  gst_buffer_unmap (buf, data, length);
 
   /* mark discontinuity if needed */
   if (G_UNLIKELY (sample != src->next_sample) && src->next_sample != -1) {
@@ -819,9 +834,9 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
   src->next_sample = sample + samples;
 
   /* get the normal timestamp to get the duration. */
-  timestamp = gst_util_uint64_scale_int (sample, GST_SECOND, spec->rate);
+  timestamp = gst_util_uint64_scale_int (sample, GST_SECOND, rate);
   duration = gst_util_uint64_scale_int (src->next_sample, GST_SECOND,
-      spec->rate) - timestamp;
+      rate) - timestamp;
 
   GST_OBJECT_LOCK (src);
   if (!(clock = GST_ELEMENT_CLOCK (src)))
@@ -868,7 +883,7 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
 
         /* the running_time converted to a sample (relative to the ringbuffer) */
         running_time_sample =
-            gst_util_uint64_scale_int (running_time, spec->rate, GST_SECOND);
+            gst_util_uint64_scale_int (running_time, rate, GST_SECOND);
 
         /* the segmentnr corrensponding to running_time, round down */
         running_time_segment = running_time_sample / sps;
@@ -922,8 +937,7 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
           new_sample = ((guint64) new_read_segment) * sps;
 
           /* and get the relative time to this -> our new timestamp */
-          timestamp =
-              gst_util_uint64_scale_int (new_sample, GST_SECOND, spec->rate);
+          timestamp = gst_util_uint64_scale_int (new_sample, GST_SECOND, rate);
 
           /* we update the next sample accordingly */
           src->next_sample = new_sample + samples;
@@ -946,14 +960,13 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
         timestamp = gst_clock_get_time (clock);
         base_time = GST_ELEMENT_CAST (src)->base_time;
 
-        if (timestamp > base_time)
+        if (GST_CLOCK_DIFF (timestamp, base_time) < 0)
           timestamp -= base_time;
         else
           timestamp = 0;
 
         /* subtract latency */
-        latency =
-            gst_util_uint64_scale_int (total_samples, GST_SECOND, spec->rate);
+        latency = gst_util_uint64_scale_int (total_samples, GST_SECOND, rate);
         if (timestamp > latency)
           timestamp -= latency;
         else
@@ -971,7 +984,7 @@ gst_base_audio_src_create (GstBaseSrc * bsrc, guint64 offset, guint length,
     /* we are not slaved, subtract base_time */
     base_time = GST_ELEMENT_CAST (src)->base_time;
 
-    if (timestamp > base_time) {
+    if (GST_CLOCK_DIFF (timestamp, base_time) < 0) {
       timestamp -= base_time;
       GST_LOG_OBJECT (src,
           "buffer timestamp %" GST_TIME_FORMAT " (base_time %" GST_TIME_FORMAT
@@ -1010,6 +1023,11 @@ wrong_offset:
             G_GUINT64_FORMAT " was given", offset));
     return GST_FLOW_ERROR;
   }
+alloc_failed:
+  {
+    GST_DEBUG_OBJECT (src, "alloc failed: %s", gst_flow_get_name (ret));
+    return ret;
+  }
 stopped:
   {
     gst_buffer_unref (buf);
@@ -1068,6 +1086,15 @@ gst_base_audio_src_change_state (GstElement * element,
       src->next_sample = -1;
       gst_ring_buffer_set_flushing (src->ringbuffer, FALSE);
       gst_ring_buffer_may_start (src->ringbuffer, FALSE);
+      /* Only post clock-provide messages if this is the clock that
+       * we've created. If the subclass has overriden it the subclass
+       * should post this messages whenever necessary */
+      if (src->clock && GST_IS_AUDIO_CLOCK (src->clock) &&
+          GST_AUDIO_CLOCK_CAST (src->clock)->func ==
+          (GstAudioClockGetTimeFunc) gst_base_audio_src_get_time)
+        gst_element_post_message (element,
+            gst_message_new_clock_provide (GST_OBJECT_CAST (element),
+                src->clock, TRUE));
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_DEBUG_OBJECT (src, "PAUSED->PLAYING");
@@ -1080,6 +1107,14 @@ gst_base_audio_src_change_state (GstElement * element,
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       GST_DEBUG_OBJECT (src, "PAUSED->READY");
+      /* Only post clock-lost messages if this is the clock that
+       * we've created. If the subclass has overriden it the subclass
+       * should post this messages whenever necessary */
+      if (src->clock && GST_IS_AUDIO_CLOCK (src->clock) &&
+          GST_AUDIO_CLOCK_CAST (src->clock)->func ==
+          (GstAudioClockGetTimeFunc) gst_base_audio_src_get_time)
+        gst_element_post_message (element,
+            gst_message_new_clock_lost (GST_OBJECT_CAST (element), src->clock));
       gst_ring_buffer_set_flushing (src->ringbuffer, TRUE);
       break;
     default:
