@@ -24,6 +24,7 @@
 #define __GST_RING_BUFFER_H__
 
 #include <gst/gst.h>
+#include <gst/audio/audio.h>
 
 G_BEGIN_DECLS
 
@@ -47,7 +48,7 @@ typedef struct _GstRingBufferSpec GstRingBufferSpec;
  * @user_data: user data
  *
  * This function is set with gst_ring_buffer_set_callback() and is
- * called to fill the memory at @data with @len bytes of samples. 
+ * called to fill the memory at @data with @len bytes of samples.
  */
 typedef void (*GstRingBufferCallback) (GstRingBuffer *rbuf, guint8* data, guint len, gpointer user_data);
 
@@ -83,24 +84,24 @@ typedef enum {
 
 /**
  * GstBufferFormatType:
- * @GST_BUFTYPE_LINEAR: samples in linear PCM
- * @GST_BUFTYPE_FLOAT: samples in float 
+ * @GST_BUFTYPE_RAW: samples in linear or float
  * @GST_BUFTYPE_MU_LAW: samples in mulaw
  * @GST_BUFTYPE_A_LAW: samples in alaw
  * @GST_BUFTYPE_IMA_ADPCM: samples in ima adpcm
- * @GST_BUFTYPE_MPEG: samples in mpeg audio format
+ * @GST_BUFTYPE_MPEG: samples in mpeg audio (but not AAC) format
  * @GST_BUFTYPE_GSM: samples in gsm format
  * @GST_BUFTYPE_IEC958: samples in IEC958 frames (e.g. AC3)
  * @GST_BUFTYPE_AC3: samples in AC3 format
  * @GST_BUFTYPE_EAC3: samples in EAC3 format
  * @GST_BUFTYPE_DTS: samples in DTS format
+ * @GST_BUFTYPE_MPEG2_AAC: samples in MPEG-2 AAC format
+ * @GST_BUFTYPE_MPEG4_AAC: samples in MPEG-4 AAC format
  *
  * The format of the samples in the ringbuffer.
  */
 typedef enum
 {
-  GST_BUFTYPE_LINEAR,
-  GST_BUFTYPE_FLOAT,
+  GST_BUFTYPE_RAW,
   GST_BUFTYPE_MU_LAW,
   GST_BUFTYPE_A_LAW,
   GST_BUFTYPE_IMA_ADPCM,
@@ -109,78 +110,22 @@ typedef enum
   GST_BUFTYPE_IEC958,
   GST_BUFTYPE_AC3,
   GST_BUFTYPE_EAC3,
-  GST_BUFTYPE_DTS
+  GST_BUFTYPE_DTS,
+  GST_BUFTYPE_MPEG2_AAC,
+  GST_BUFTYPE_MPEG4_AAC,
 } GstBufferFormatType;
 
-typedef enum
-{
-  GST_UNKNOWN,
-
-  GST_S8,
-  GST_U8,
-
-  GST_S16_LE,
-  GST_S16_BE,
-  GST_U16_LE,
-  GST_U16_BE,
-
-  GST_S24_LE,
-  GST_S24_BE,
-  GST_U24_LE,
-  GST_U24_BE,
-
-  GST_S32_LE,
-  GST_S32_BE,
-  GST_U32_LE,
-  GST_U32_BE,
-
-  GST_S24_3LE,
-  GST_S24_3BE,
-  GST_U24_3LE,
-  GST_U24_3BE,
-  GST_S20_3LE,
-  GST_S20_3BE,
-  GST_U20_3LE,
-  GST_U20_3BE,
-  GST_S18_3LE,
-  GST_S18_3BE,
-  GST_U18_3LE,
-  GST_U18_3BE,
-
-  GST_FLOAT32_LE,
-  GST_FLOAT32_BE,
-
-  GST_FLOAT64_LE,
-  GST_FLOAT64_BE,
-
-  GST_MU_LAW,
-  GST_A_LAW,
-  GST_IMA_ADPCM,
-  GST_MPEG,
-  GST_GSM,
-  GST_IEC958,
-  GST_AC3,
-  GST_EAC3,
-  GST_DTS
-} GstBufferFormat;
-
 /**
  * GstRingBufferSpec:
  * @caps: The caps that generated the Spec.
  * @type: the sample type
- * @format: the sample format
- * @sign: the sample sign
- * @bigend: the endianness of the samples
- * @width: the width of the samples
- * @depth: th depth of the samples
- * @rate: the samplerate
- * @channels: the number of channels
+ * @info: the #GstAudioInfo
  * @latency_time: the latency in microseconds
  * @buffer_time: the total buffer size in microseconds
  * @segsize: the size of one segment in bytes
  * @segtotal: the total number of segments
- * @bytes_per_sample: number of bytes in one sample
- * @silence_sample: bytes representing one sample of silence
+ * @seglatency: number of segments queued in the lower level device,
+ *  defaults to segtotal
  *
  * The structure containing the format specification of the ringbuffer.
  */
@@ -191,15 +136,9 @@ struct _GstRingBufferSpec
   GstCaps  *caps;               /* the caps of the buffer */
 
   /* in/out */
-  GstBufferFormatType   type;
-  GstBufferFormat format;
-  gboolean  sign;
-  gboolean  bigend;
-  gint      width;
-  gint      depth;
-  gint      rate;
-  gint      channels;
-  
+  GstBufferFormatType type;
+  GstAudioInfo        info;
+
   guint64  latency_time;        /* the required/actual latency time, this is the
 				 * actual the size of one segment and the
 				 * minimum possible latency we can achieve. */
@@ -213,17 +152,12 @@ struct _GstRingBufferSpec
 				 * number of segments of @segsize and should be
 				 * chosen so that it matches buffer_time as
 				 * close as possible. */
-  /* out */
-  gint     bytes_per_sample;    /* number of bytes of one sample */
-  guint8   silence_sample[32];  /* bytes representing silence */
-
   /* ABI added 0.10.20 */
   gint     seglatency;          /* number of segments queued in the lower
 				 * level device, defaults to segtotal. */
 
   /*< private >*/
-  /* gpointer _gst_reserved[GST_PADDING]; */
-  guint8 _gst_reserved[(sizeof (gpointer) * GST_PADDING) - sizeof (gint)];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 #define GST_RING_BUFFER_GET_COND(buf) (((GstRingBuffer *)buf)->cond)
@@ -236,7 +170,8 @@ struct _GstRingBufferSpec
  * @cond: used to signal start/stop/pause/resume actions
  * @open: boolean indicating that the ringbuffer is open
  * @acquired: boolean indicating that the ringbuffer is acquired
- * @data: data in the ringbuffer
+ * @memory: data in the ringbuffer
+ * @size: size of data in the ringbuffer
  * @spec: format and layout of the ringbuffer data
  * @segstate: status of each segment in the ringbuffer (unused)
  * @samples_per_seg: number of samples in one segment
@@ -255,7 +190,8 @@ struct _GstRingBuffer {
   GCond                 *cond;
   gboolean               open;
   gboolean               acquired;
-  GstBuffer             *data;
+  guint8                *memory;
+  gsize                  size;
   GstRingBufferSpec      spec;
   GstRingBufferSegState *segstate;
   gint                   samples_per_seg;
@@ -271,20 +207,18 @@ struct _GstRingBuffer {
   GstRingBufferCallback  callback;
   gpointer               cb_data;
 
+  gboolean               flushing;
+  /* ATOMIC */
+  gint                   may_start;
+  gboolean               active;
+
   /*< private >*/
-  union {
-    struct {
-      gboolean           flushing;
-      /* ATOMIC */
-      gint               may_start;
-    } ABI;
-    /* adding + 0 to mark ABI change to be undone later */
-    gpointer _gst_reserved[GST_PADDING + 0];
-  } abidata;
+  gpointer _gst_reserved[GST_PADDING + 0];
 };
 
 /**
  * GstRingBufferClass:
+ * @parent_class: parent class
  * @open_device:  open the device, don't set any params or allocate anything
  * @acquire: allocate the resources for the ringbuffer using the given spec
  * @release: free resources of the ringbuffer
@@ -294,6 +228,10 @@ struct _GstRingBuffer {
  * @resume: resume processing of samples after pause
  * @stop: stop processing of samples
  * @delay: get number of samples queued in device
+ * @activate: activate the thread that starts pulling and monitoring the
+ * consumed segments in the device. Since 0.10.22
+ * @commit: write samples into the ringbuffer
+ * @clear_all: clear the entire ringbuffer Since 0.10.24
  *
  * The vmethods that subclasses can override to implement the ringbuffer.
  */
@@ -313,20 +251,33 @@ struct _GstRingBufferClass {
 
   guint        (*delay)        (GstRingBuffer *buf);
 
+  /* ABI added */
+  gboolean     (*activate)     (GstRingBuffer *buf, gboolean active);
+
+  guint        (*commit)       (GstRingBuffer * buf, guint64 *sample,
+                                guchar * data, gint in_samples,
+                                gint out_samples, gint * accum);
+
+  void         (*clear_all)    (GstRingBuffer * buf);
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING];
+  gpointer _gst_reserved[GST_PADDING - 3];
 };
 
 GType gst_ring_buffer_get_type(void);
 
 /* callback stuff */
-void            gst_ring_buffer_set_callback    (GstRingBuffer *buf, GstRingBufferCallback cb, 
+void            gst_ring_buffer_set_callback    (GstRingBuffer *buf, GstRingBufferCallback cb,
                                                  gpointer user_data);
 
 gboolean        gst_ring_buffer_parse_caps      (GstRingBufferSpec *spec, GstCaps *caps);
 void            gst_ring_buffer_debug_spec_caps (GstRingBufferSpec *spec);
 void            gst_ring_buffer_debug_spec_buff (GstRingBufferSpec *spec);
 
+gboolean        gst_ring_buffer_convert         (GstRingBuffer * buf, GstFormat src_fmt,
+                                                 gint64 src_val, GstFormat dest_fmt,
+						 gint64 * dest_val);
+
 /* device state */
 gboolean        gst_ring_buffer_open_device     (GstRingBuffer *buf);
 gboolean        gst_ring_buffer_close_device    (GstRingBuffer *buf);
@@ -339,6 +290,10 @@ gboolean        gst_ring_buffer_release         (GstRingBuffer *buf);
 
 gboolean        gst_ring_buffer_is_acquired     (GstRingBuffer *buf);
 
+/* activating */
+gboolean        gst_ring_buffer_activate        (GstRingBuffer *buf, gboolean active);
+gboolean        gst_ring_buffer_is_active       (GstRingBuffer *buf);
+
 /* flushing */
 void            gst_ring_buffer_set_flushing    (GstRingBuffer *buf, gboolean flushing);
 
@@ -357,14 +312,14 @@ void            gst_ring_buffer_set_sample      (GstRingBuffer *buf, guint64 sam
 void            gst_ring_buffer_clear_all       (GstRingBuffer *buf);
 
 /* commit samples */
-guint           gst_ring_buffer_commit          (GstRingBuffer *buf, guint64 sample, 
+guint           gst_ring_buffer_commit          (GstRingBuffer *buf, guint64 sample,
                                                  guchar *data, guint len);
 guint           gst_ring_buffer_commit_full     (GstRingBuffer * buf, guint64 *sample,
-		                                 guchar * data, gint in_samples, 
+		                                 guchar * data, gint in_samples,
 						 gint out_samples, gint * accum);
 
 /* read samples */
-guint           gst_ring_buffer_read            (GstRingBuffer *buf, guint64 sample, 
+guint           gst_ring_buffer_read            (GstRingBuffer *buf, guint64 sample,
                                                  guchar *data, guint len);
 
 /* mostly protected */
