@@ -88,6 +88,7 @@
 #include <string.h>
 #include <stdlib.h>             /* for strtol */
 
+#include <gst/audio/audio.h>
 #include "gstcddabasesrc.h"
 #include "gst/gst-i18n-plugin.h"
 
@@ -114,6 +115,8 @@ enum
   ARG_TOC_BIAS
 };
 
+static void gst_cdda_base_src_uri_handler_init (gpointer g_iface,
+    gpointer iface_data);
 static void gst_cdda_base_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_cdda_base_src_set_property (GObject * object, guint prop_id,
@@ -125,7 +128,6 @@ static gboolean gst_cdda_base_src_handle_event (GstBaseSrc * basesrc,
     GstEvent * event);
 static gboolean gst_cdda_base_src_do_seek (GstBaseSrc * basesrc,
     GstSegment * segment);
-static void gst_cdda_base_src_setup_interfaces (GType type);
 static gboolean gst_cdda_base_src_start (GstBaseSrc * basesrc);
 static gboolean gst_cdda_base_src_stop (GstBaseSrc * basesrc);
 static GstFlowReturn gst_cdda_base_src_create (GstPushSrc * pushsrc,
@@ -135,15 +137,14 @@ static void gst_cdda_base_src_update_duration (GstCddaBaseSrc * src);
 static void gst_cdda_base_src_set_index (GstElement * src, GstIndex * index);
 static GstIndex *gst_cdda_base_src_get_index (GstElement * src);
 
-GST_BOILERPLATE_FULL (GstCddaBaseSrc, gst_cdda_base_src, GstPushSrc,
-    GST_TYPE_PUSH_SRC, gst_cdda_base_src_setup_interfaces);
+#define gst_cdda_base_src_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (GstCddaBaseSrc, gst_cdda_base_src, GST_TYPE_PUSH_SRC,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
+        gst_cdda_base_src_uri_handler_init));
 
 #define SRC_CAPS \
-  "audio/x-raw-int, "               \
-  "endianness = (int) BYTE_ORDER, " \
-  "signed = (boolean) true, "       \
-  "width = (int) 16, "              \
-  "depth = (int) 16, "              \
+  "audio/x-raw, "               \
+  "format = (string) " GST_AUDIO_NE(S16) ", " \
   "rate = (int) 44100, "            \
   "channels = (int) 2"              \
 
@@ -177,12 +178,20 @@ gst_cdda_base_src_mode_get_type (void)
 }
 
 static void
-gst_cdda_base_src_base_init (gpointer g_class)
+gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
 {
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstElementClass *element_class;
+  GstPushSrcClass *pushsrc_class;
+  GstBaseSrcClass *basesrc_class;
+  GObjectClass *gobject_class;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_cdda_base_src_src_template));
+  gobject_class = (GObjectClass *) klass;
+  element_class = (GstElementClass *) klass;
+  basesrc_class = (GstBaseSrcClass *) klass;
+  pushsrc_class = (GstPushSrcClass *) klass;
+
+  GST_DEBUG_CATEGORY_INIT (gst_cdda_base_src_debug, "cddabasesrc", 0,
+      "CDDA Base Source");
 
   /* our very own formats */
   track_format = gst_format_register ("track", "CD track");
@@ -196,23 +205,6 @@ gst_cdda_base_src_base_init (gpointer g_class)
   gst_tag_register (GST_TAG_CDDA_TRACK_TAGS, GST_TAG_FLAG_META, GST_TYPE_TAG_LIST, "track-tags", "CDDA taglist for one track", gst_tag_merge_use_first);        ///////////// FIXME: right function??? ///////
 #endif
 
-  GST_DEBUG_CATEGORY_INIT (gst_cdda_base_src_debug, "cddabasesrc", 0,
-      "CDDA Base Source");
-}
-
-static void
-gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
-{
-  GstElementClass *element_class;
-  GstPushSrcClass *pushsrc_class;
-  GstBaseSrcClass *basesrc_class;
-  GObjectClass *gobject_class;
-
-  gobject_class = (GObjectClass *) klass;
-  element_class = (GstElementClass *) klass;
-  basesrc_class = (GstBaseSrcClass *) klass;
-  pushsrc_class = (GstPushSrcClass *) klass;
-
   gobject_class->set_property = gst_cdda_base_src_set_property;
   gobject_class->get_property = gst_cdda_base_src_get_property;
   gobject_class->finalize = gst_cdda_base_src_finalize;
@@ -247,6 +239,9 @@ gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 #endif
 
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cdda_base_src_src_template));
+
   element_class->set_index = GST_DEBUG_FUNCPTR (gst_cdda_base_src_set_index);
   element_class->get_index = GST_DEBUG_FUNCPTR (gst_cdda_base_src_get_index);
 
@@ -262,7 +257,7 @@ gst_cdda_base_src_class_init (GstCddaBaseSrcClass * klass)
 }
 
 static void
-gst_cdda_base_src_init (GstCddaBaseSrc * src, GstCddaBaseSrcClass * klass)
+gst_cdda_base_src_init (GstCddaBaseSrc * src)
 {
   gst_pad_set_query_type_function (GST_BASE_SRC_PAD (src),
       GST_DEBUG_FUNCPTR (gst_cdda_base_src_get_query_types));
@@ -914,13 +909,13 @@ gst_cdda_base_src_handle_event (GstBaseSrc * basesrc, GstEvent * event)
 }
 
 static GstURIType
-gst_cdda_base_src_uri_get_type (void)
+gst_cdda_base_src_uri_get_type (GType type)
 {
   return GST_URI_SRC;
 }
 
 static gchar **
-gst_cdda_base_src_uri_get_protocols (void)
+gst_cdda_base_src_uri_get_protocols (GType type)
 {
   static gchar *protocols[] = { (char *) "cdda", NULL };
 
@@ -1035,18 +1030,6 @@ gst_cdda_base_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
   iface->get_protocols = gst_cdda_base_src_uri_get_protocols;
 }
 
-static void
-gst_cdda_base_src_setup_interfaces (GType type)
-{
-  static const GInterfaceInfo urihandler_info = {
-    gst_cdda_base_src_uri_handler_init,
-    NULL,
-    NULL,
-  };
-
-  g_type_add_interface_static (type, GST_TYPE_URI_HANDLER, &urihandler_info);
-}
-
 /**
  * gst_cdda_base_src_add_track:
  * @src: a #GstCddaBaseSrc
@@ -1095,24 +1078,20 @@ static void
 gst_cdda_base_src_update_duration (GstCddaBaseSrc * src)
 {
   GstBaseSrc *basesrc;
-  GstFormat format;
-  gint64 duration;
+  gint64 dur;
 
   basesrc = GST_BASE_SRC (src);
 
-  format = GST_FORMAT_TIME;
-  if (gst_pad_query_duration (GST_BASE_SRC_PAD (src), &format, &duration)) {
-    gst_segment_set_duration (&basesrc->segment, GST_FORMAT_TIME, duration);
-  } else {
-    gst_segment_set_duration (&basesrc->segment, GST_FORMAT_TIME, -1);
-    duration = GST_CLOCK_TIME_NONE;
+  if (!gst_pad_query_duration (GST_BASE_SRC_PAD (src), GST_FORMAT_TIME, &dur)) {
+    dur = GST_CLOCK_TIME_NONE;
   }
+  basesrc->segment.duration = dur;
 
   gst_element_post_message (GST_ELEMENT (src),
       gst_message_new_duration (GST_OBJECT (src), GST_FORMAT_TIME, -1));
 
   GST_LOG_OBJECT (src, "duration updated to %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (duration));
+      GST_TIME_ARGS (dur));
 }
 
 #define CD_MSF_OFFSET 150
@@ -1574,7 +1553,6 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
   GstCddaBaseSrcClass *klass = GST_CDDA_BASE_SRC_GET_CLASS (pushsrc);
   GstCddaBaseSrc *src = GST_CDDA_BASE_SRC (pushsrc);
   GstBuffer *buf;
-  GstFormat format;
   gboolean eos;
 
   GstClockTime position = GST_CLOCK_TIME_NONE;
@@ -1628,18 +1606,15 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
     return GST_FLOW_ERROR;
   }
 
-  if (GST_BUFFER_CAPS (buf) == NULL) {
-    gst_buffer_set_caps (buf, GST_PAD_CAPS (GST_BASE_SRC_PAD (src)));
-  }
-
-  format = GST_FORMAT_TIME;
-  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &qry_position)) {
+  if (gst_pad_query_position (GST_BASE_SRC_PAD (src), GST_FORMAT_TIME,
+          &qry_position)) {
     gint64 next_ts = 0;
 
     position = (GstClockTime) qry_position;
 
     ++src->cur_sector;
-    if (gst_pad_query_position (GST_BASE_SRC_PAD (src), &format, &next_ts)) {
+    if (gst_pad_query_position (GST_BASE_SRC_PAD (src), GST_FORMAT_TIME,
+            &next_ts)) {
       duration = (GstClockTime) (next_ts - qry_position);
     }
     --src->cur_sector;
@@ -1647,7 +1622,7 @@ gst_cdda_base_src_create (GstPushSrc * pushsrc, GstBuffer ** buffer)
 
   /* fallback duration: 4 bytes per sample, 44100 samples per second */
   if (duration == GST_CLOCK_TIME_NONE) {
-    duration = gst_util_uint64_scale_int (GST_BUFFER_SIZE (buf) >> 2,
+    duration = gst_util_uint64_scale_int (gst_buffer_get_size (buf) >> 2,
         GST_SECOND, 44100);
   }
 
