@@ -20,6 +20,7 @@
 #include <glib.h>
 #include <math.h>
 
+#include "_kiss_fft_guts_s16.h"
 #include "kiss_fftr_s16.h"
 #include "gstfft.h"
 #include "gstffts16.h"
@@ -74,13 +75,18 @@ GstFFTS16 *
 gst_fft_s16_new (gint len, gboolean inverse)
 {
   GstFFTS16 *self;
+  gsize subsize = 0, memneeded;
 
   g_return_val_if_fail (len > 0, NULL);
   g_return_val_if_fail (len % 2 == 0, NULL);
 
-  self = g_new (GstFFTS16, 1);
+  kiss_fftr_s16_alloc (len, (inverse) ? 1 : 0, NULL, &subsize);
+  memneeded = ALIGN_STRUCT (sizeof (GstFFTS16)) + subsize;
 
-  self->cfg = kiss_fftr_s16_alloc (len, (inverse) ? 1 : 0, NULL, NULL);
+  self = (GstFFTS16 *) g_malloc0 (memneeded);
+
+  self->cfg = (((guint8 *) self) + ALIGN_STRUCT (sizeof (GstFFTS16)));
+  self->cfg = kiss_fftr_s16_alloc (len, (inverse) ? 1 : 0, self->cfg, &subsize);
   g_assert (self->cfg);
 
   self->inverse = inverse;
@@ -152,7 +158,6 @@ gst_fft_s16_inverse_fft (GstFFTS16 * self, const GstFFTS16Complex * freqdata,
 void
 gst_fft_s16_free (GstFFTS16 * self)
 {
-  kiss_fftr_s16_free (self->cfg);
   g_free (self);
 }
 
@@ -181,11 +186,11 @@ gst_fft_s16_window (GstFFTS16 * self, gint16 * timedata, GstFFTWindow window)
       break;
     case GST_FFT_WINDOW_HAMMING:
       for (i = 0; i < len; i++)
-        timedata[i] *= (0.53836 - 0.46164 * cos (2.0 * M_PI * i / len));
+        timedata[i] *= (0.53836 - 0.46164 * cos (2.0 * G_PI * i / len));
       break;
     case GST_FFT_WINDOW_HANN:
       for (i = 0; i < len; i++)
-        timedata[i] *= (0.5 - 0.5 * cos (2.0 * M_PI * i / len));
+        timedata[i] *= (0.5 - 0.5 * cos (2.0 * G_PI * i / len));
       break;
     case GST_FFT_WINDOW_BARTLETT:
       for (i = 0; i < len; i++)
