@@ -41,6 +41,11 @@
  * </refsect2>
  */
 
+/* FIXME 0.11: check if we need to add API for sometimes-supportedness
+ * (aka making up for GstImplementsInterface removal) */
+
+/* FIXME 0.11: replace signals with messages (+ make API thread-safe) */
+
 enum
 {
   VALUE_CHANGED,
@@ -71,8 +76,6 @@ gst_color_balance_get_type (void)
 
     gst_color_balance_type = g_type_register_static (G_TYPE_INTERFACE,
         "GstColorBalance", &gst_color_balance_info, 0);
-    g_type_interface_add_prerequisite (gst_color_balance_type,
-        GST_TYPE_IMPLEMENTS_INTERFACE);
   }
 
   return gst_color_balance_type;
@@ -124,7 +127,11 @@ gst_color_balance_class_init (GstColorBalanceClass * klass)
 const GList *
 gst_color_balance_list_channels (GstColorBalance * balance)
 {
-  GstColorBalanceClass *klass = GST_COLOR_BALANCE_GET_CLASS (balance);
+  GstColorBalanceClass *klass;
+
+  g_return_val_if_fail (GST_IS_COLOR_BALANCE (balance), NULL);
+
+  klass = GST_COLOR_BALANCE_GET_CLASS (balance);
 
   if (klass->list_channels) {
     return klass->list_channels (balance);
@@ -142,8 +149,8 @@ gst_color_balance_list_channels (GstColorBalance * balance)
  * Sets the current value of the channel to the passed value, which must
  * be between min_value and max_value.
  * 
- * See Also: The #GstColorBalanceChannel::min_value and
- *         #GstColorBalanceChannel::max_value members of the
+ * See Also: The #GstColorBalanceChannel.min_value and
+ *         #GstColorBalanceChannel.max_value members of the
  *         #GstColorBalanceChannel object.
  */
 void
@@ -165,8 +172,8 @@ gst_color_balance_set_value (GstColorBalance * balance,
  * Retrieve the current value of the indicated channel, between min_value
  * and max_value.
  * 
- * See Also: The #GstColorBalanceChannel::min_value and
- *         #GstColorBalanceChannel::max_value members of the
+ * See Also: The #GstColorBalanceChannel.min_value and
+ *         #GstColorBalanceChannel.max_value members of the
  *         #GstColorBalanceChannel object.
  * 
  * Returns: The current value of the channel.
@@ -175,7 +182,11 @@ gint
 gst_color_balance_get_value (GstColorBalance * balance,
     GstColorBalanceChannel * channel)
 {
-  GstColorBalanceClass *klass = GST_COLOR_BALANCE_GET_CLASS (balance);
+  GstColorBalanceClass *klass;
+
+  g_return_val_if_fail (GST_IS_COLOR_BALANCE (balance), 0);
+
+  klass = GST_COLOR_BALANCE_GET_CLASS (balance);
 
   if (klass->get_value) {
     return klass->get_value (balance, channel);
@@ -184,6 +195,29 @@ gst_color_balance_get_value (GstColorBalance * balance,
   return channel->min_value;
 }
 
+/**
+ * gst_color_balance_get_balance_type:
+ * @balance: The #GstColorBalance implementation
+ *
+ * Get the #GstColorBalanceType of this implementation.
+ *
+ * Returns: A the #GstColorBalanceType.
+ *
+ * Since: 0.10.24
+ */
+GstColorBalanceType
+gst_color_balance_get_balance_type (GstColorBalance * balance)
+{
+  GstColorBalanceClass *klass;
+
+  g_return_val_if_fail (GST_IS_COLOR_BALANCE (balance),
+      GST_COLOR_BALANCE_SOFTWARE);
+
+  klass = GST_COLOR_BALANCE_GET_CLASS (balance);
+
+  return klass->balance_type;
+}
+
 /**
  * gst_color_balance_value_changed:
  * @balance: A #GstColorBalance instance
@@ -199,6 +233,9 @@ void
 gst_color_balance_value_changed (GstColorBalance * balance,
     GstColorBalanceChannel * channel, gint value)
 {
+
+  g_return_if_fail (GST_IS_COLOR_BALANCE (balance));
+
   g_signal_emit (G_OBJECT (balance),
       gst_color_balance_signals[VALUE_CHANGED], 0, channel, value);
 
