@@ -50,6 +50,9 @@
  * macro (in C) and the mixer API can be used.
  */
 
+/* FIXME 0.11: check if we need to add API for sometimes-supportedness
+ * (aka making up for GstImplementsInterface removal) */
+
 #ifndef GST_DISABLE_DEPRECATED
 enum
 {
@@ -86,8 +89,6 @@ gst_mixer_get_type (void)
 
     gst_mixer_type = g_type_register_static (G_TYPE_INTERFACE,
         "GstMixer", &gst_mixer_info, 0);
-    g_type_interface_add_prerequisite (gst_mixer_type,
-        GST_TYPE_IMPLEMENTS_INTERFACE);
   }
 
   return gst_mixer_type;
@@ -544,7 +545,7 @@ gst_mixer_option_changed (GstMixer * mixer,
  * options of a given options object has changed.
  *
  * The new options are not contained in the message on purpose. Applications
- * should call gst_mixer_option_get_values() on @opts to make @opts update
+ * should call gst_mixer_options_get_values() on @opts to make @opts update
  * its internal state and obtain the new list of values.
  *
  * This function only works for GstElements that are implementing the
