@@ -69,6 +69,11 @@
  * </refsect2>
  */
 
+/* FIXME 0.11: check if we need to add API for sometimes-supportedness
+ * (aka making up for GstImplementsInterface removal) */
+
+/* FIXME 0.11: replace signals with messages (+ make API thread-safe) */
+
 enum
 {
   NORM_CHANGED,
@@ -102,8 +107,6 @@ gst_tuner_get_type (void)
 
     gst_tuner_type = g_type_register_static (G_TYPE_INTERFACE,
         "GstTuner", &gst_tuner_info, 0);
-    g_type_interface_add_prerequisite (gst_tuner_type,
-        GST_TYPE_IMPLEMENTS_INTERFACE);
   }
 
   return gst_tuner_type;
@@ -221,7 +224,8 @@ gst_tuner_list_channels (GstTuner * tuner)
  * @tuner: the #GstTuner (a #GstElement) that owns the channel.
  * @channel: the channel to tune to.
  *
- * Tunes the object to the given channel.
+ * Tunes the object to the given channel, which should be one of the
+ * channels returned by gst_tuner_list_channels().
  */
 
 void
@@ -293,7 +297,8 @@ gst_tuner_list_norms (GstTuner * tuner)
  * @tuner: the #GstTuner (a #GstElement) to set the norm on.
  * @norm: the norm to use for the current channel.
  *
- * Changes the video norm on this tuner to the given norm.
+ * Changes the video norm on this tuner to the given norm, which should be
+ * one of the norms returned by gst_tuner_list_norms().
  */
 
 void
@@ -336,7 +341,7 @@ gst_tuner_get_norm (GstTuner * tuner)
 
 /**
  * gst_tuner_set_frequency:
- * @tuner: The #Gsttuner (a #GstElement) that owns the given channel.
+ * @tuner: The #GstTuner (a #GstElement) that owns the given channel.
  * @channel: The #GstTunerChannel to set the frequency on.
  * @frequency: The frequency to tune in to.
  *
