@@ -39,10 +39,13 @@ static GstDiscovererAudioInfo
 static GstDiscovererVideoInfo
     * gst_discoverer_video_info_copy_int (GstDiscovererVideoInfo * ptr);
 
+static GstDiscovererSubtitleInfo
+    * gst_discoverer_subtitle_info_copy_int (GstDiscovererSubtitleInfo * ptr);
+
 /* Per-stream information */
 
 G_DEFINE_TYPE (GstDiscovererStreamInfo, gst_discoverer_stream_info,
-    GST_TYPE_MINI_OBJECT);
+    G_TYPE_OBJECT);
 
 static void
 gst_discoverer_stream_info_init (GstDiscovererStreamInfo * info)
@@ -51,10 +54,12 @@ gst_discoverer_stream_info_init (GstDiscovererStreamInfo * info)
 }
 
 static void
-gst_discoverer_stream_info_finalize (GstDiscovererStreamInfo * info)
+gst_discoverer_stream_info_finalize (GObject * object)
 {
+  GstDiscovererStreamInfo *info = (GstDiscovererStreamInfo *) object;
+
   if (info->next)
-    gst_mini_object_unref ((GstMiniObject *) info->next);
+    g_object_unref ((GObject *) info->next);
 
   if (info->caps)
     gst_caps_unref (info->caps);
@@ -66,25 +71,17 @@ gst_discoverer_stream_info_finalize (GstDiscovererStreamInfo * info)
     gst_structure_free (info->misc);
 }
 
-static GstDiscovererStreamInfo *
-gst_discoverer_stream_info_copy (GstDiscovererStreamInfo * info)
-{
-  return gst_discoverer_info_copy_int (info, NULL);
-}
-
 static void
-gst_discoverer_stream_info_class_init (GstMiniObjectClass * klass)
+gst_discoverer_stream_info_class_init (GObjectClass * klass)
 {
-  klass->finalize =
-      (GstMiniObjectFinalizeFunction) gst_discoverer_stream_info_finalize;
-  klass->copy = (GstMiniObjectCopyFunction) gst_discoverer_stream_info_copy;
+  klass->finalize = gst_discoverer_stream_info_finalize;
 }
 
 static GstDiscovererStreamInfo *
 gst_discoverer_stream_info_new (void)
 {
   return (GstDiscovererStreamInfo *)
-      gst_mini_object_new (GST_TYPE_DISCOVERER_STREAM_INFO);
+      g_object_new (GST_TYPE_DISCOVERER_STREAM_INFO, NULL);
 }
 
 static GstDiscovererStreamInfo *
@@ -110,6 +107,11 @@ gst_discoverer_info_copy_int (GstDiscovererStreamInfo * info,
     ret = (GstDiscovererStreamInfo *)
         gst_discoverer_video_info_copy_int ((GstDiscovererVideoInfo *) info);
 
+  } else if (ltyp == GST_TYPE_DISCOVERER_SUBTITLE_INFO) {
+    ret = (GstDiscovererStreamInfo *)
+        gst_discoverer_subtitle_info_copy_int ((GstDiscovererSubtitleInfo *)
+        info);
+
   } else
     ret = gst_discoverer_stream_info_new ();
 
@@ -147,28 +149,28 @@ static GstDiscovererContainerInfo *
 gst_discoverer_container_info_new (void)
 {
   return (GstDiscovererContainerInfo *)
-      gst_mini_object_new (GST_TYPE_DISCOVERER_CONTAINER_INFO);
+      g_object_new (GST_TYPE_DISCOVERER_CONTAINER_INFO, NULL);
 }
 
 static void
-gst_discoverer_container_info_finalize (GstDiscovererContainerInfo * info)
+gst_discoverer_container_info_finalize (GObject * object)
 {
+  GstDiscovererContainerInfo *info = (GstDiscovererContainerInfo *) object;
   GList *tmp;
 
   for (tmp = ((GstDiscovererContainerInfo *) info)->streams; tmp;
       tmp = tmp->next)
-    gst_mini_object_unref ((GstMiniObject *) tmp->data);
+    g_object_unref ((GObject *) tmp->data);
 
   gst_discoverer_stream_info_list_free (info->streams);
 
-  gst_discoverer_stream_info_finalize ((GstDiscovererStreamInfo *) info);
+  gst_discoverer_stream_info_finalize ((GObject *) info);
 }
 
 static void
-gst_discoverer_container_info_class_init (GstMiniObjectClass * klass)
+gst_discoverer_container_info_class_init (GObjectClass * klass)
 {
-  klass->finalize =
-      (GstMiniObjectFinalizeFunction) gst_discoverer_container_info_finalize;
+  klass->finalize = gst_discoverer_container_info_finalize;
 }
 
 static GstDiscovererContainerInfo *
@@ -199,22 +201,32 @@ G_DEFINE_TYPE (GstDiscovererAudioInfo, gst_discoverer_audio_info,
     GST_TYPE_DISCOVERER_STREAM_INFO);
 
 static void
-gst_discoverer_audio_info_class_init (GstDiscovererAudioInfoClass * klass)
+gst_discoverer_audio_info_finalize (GObject * object)
 {
-  /* Nothing to initialize */
+  GstDiscovererAudioInfo *info = (GstDiscovererAudioInfo *) object;
+
+  g_free (info->language);
+
+  G_OBJECT_CLASS (gst_discoverer_audio_info_parent_class)->finalize (object);
+}
+
+static void
+gst_discoverer_audio_info_class_init (GObjectClass * klass)
+{
+  klass->finalize = gst_discoverer_audio_info_finalize;
 }
 
 static void
 gst_discoverer_audio_info_init (GstDiscovererAudioInfo * info)
 {
-  /* Nothing to initialize */
+  info->language = NULL;
 }
 
 static GstDiscovererAudioInfo *
 gst_discoverer_audio_info_new (void)
 {
   return (GstDiscovererAudioInfo *)
-      gst_mini_object_new (GST_TYPE_DISCOVERER_AUDIO_INFO);
+      g_object_new (GST_TYPE_DISCOVERER_AUDIO_INFO, NULL);
 }
 
 static GstDiscovererAudioInfo *
@@ -229,6 +241,52 @@ gst_discoverer_audio_info_copy_int (GstDiscovererAudioInfo * ptr)
   ret->depth = ptr->depth;
   ret->bitrate = ptr->bitrate;
   ret->max_bitrate = ptr->max_bitrate;
+  ret->language = g_strdup (ptr->language);
+
+  return ret;
+}
+
+/* Subtitle information */
+G_DEFINE_TYPE (GstDiscovererSubtitleInfo, gst_discoverer_subtitle_info,
+    GST_TYPE_DISCOVERER_STREAM_INFO);
+
+static void
+gst_discoverer_subtitle_info_init (GstDiscovererSubtitleInfo * info)
+{
+  info->language = NULL;
+}
+
+static void
+gst_discoverer_subtitle_info_finalize (GObject * object)
+{
+  GstDiscovererSubtitleInfo *info = (GstDiscovererSubtitleInfo *) object;
+
+  g_free (info->language);
+
+  G_OBJECT_CLASS (gst_discoverer_subtitle_info_parent_class)->finalize (object);
+}
+
+static void
+gst_discoverer_subtitle_info_class_init (GObjectClass * klass)
+{
+  klass->finalize = gst_discoverer_subtitle_info_finalize;
+}
+
+static GstDiscovererSubtitleInfo *
+gst_discoverer_subtitle_info_new (void)
+{
+  return (GstDiscovererSubtitleInfo *)
+      g_object_new (GST_TYPE_DISCOVERER_SUBTITLE_INFO, NULL);
+}
+
+static GstDiscovererSubtitleInfo *
+gst_discoverer_subtitle_info_copy_int (GstDiscovererSubtitleInfo * ptr)
+{
+  GstDiscovererSubtitleInfo *ret;
+
+  ret = gst_discoverer_subtitle_info_new ();
+
+  ret->language = g_strdup (ptr->language);
 
   return ret;
 }
@@ -238,7 +296,7 @@ G_DEFINE_TYPE (GstDiscovererVideoInfo, gst_discoverer_video_info,
     GST_TYPE_DISCOVERER_STREAM_INFO);
 
 static void
-gst_discoverer_video_info_class_init (GstMiniObjectClass * klass)
+gst_discoverer_video_info_class_init (GObjectClass * klass)
 {
   /* Nothing to initialize */
 }
@@ -253,7 +311,7 @@ static GstDiscovererVideoInfo *
 gst_discoverer_video_info_new (void)
 {
   return (GstDiscovererVideoInfo *)
-      gst_mini_object_new (GST_TYPE_DISCOVERER_VIDEO_INFO);
+      g_object_new (GST_TYPE_DISCOVERER_VIDEO_INFO, NULL);
 }
 
 static GstDiscovererVideoInfo *
@@ -279,7 +337,7 @@ gst_discoverer_video_info_copy_int (GstDiscovererVideoInfo * ptr)
 }
 
 /* Global stream information */
-G_DEFINE_TYPE (GstDiscovererInfo, gst_discoverer_info, GST_TYPE_MINI_OBJECT);
+G_DEFINE_TYPE (GstDiscovererInfo, gst_discoverer_info, G_TYPE_OBJECT);
 
 static void
 gst_discoverer_info_init (GstDiscovererInfo * info)
@@ -288,12 +346,13 @@ gst_discoverer_info_init (GstDiscovererInfo * info)
 }
 
 static void
-gst_discoverer_info_finalize (GstDiscovererInfo * info)
+gst_discoverer_info_finalize (GObject * object)
 {
+  GstDiscovererInfo *info = (GstDiscovererInfo *) object;
   g_free (info->uri);
 
   if (info->stream_info)
-    gst_mini_object_unref ((GstMiniObject *) info->stream_info);
+    g_object_unref ((GObject *) info->stream_info);
 
   if (info->misc)
     gst_structure_free (info->misc);
@@ -307,7 +366,7 @@ gst_discoverer_info_finalize (GstDiscovererInfo * info)
 static GstDiscovererInfo *
 gst_discoverer_info_new (void)
 {
-  return (GstDiscovererInfo *) gst_mini_object_new (GST_TYPE_DISCOVERER_INFO);
+  return (GstDiscovererInfo *) g_object_new (GST_TYPE_DISCOVERER_INFO, NULL);
 }
 
 GstDiscovererInfo *
@@ -350,11 +409,9 @@ gst_discoverer_info_copy (GstDiscovererInfo * ptr)
 }
 
 static void
-gst_discoverer_info_class_init (GstMiniObjectClass * klass)
+gst_discoverer_info_class_init (GObjectClass * klass)
 {
-  klass->finalize =
-      (GstMiniObjectFinalizeFunction) gst_discoverer_info_finalize;
-  klass->copy = (GstMiniObjectCopyFunction) gst_discoverer_info_copy;
+  klass->finalize = gst_discoverer_info_finalize;
 }
 
 /**
@@ -377,15 +434,16 @@ gst_discoverer_stream_info_list_free (GList * infos)
 /**
  * gst_discoverer_info_get_streams:
  * @info: a #GstDiscovererInfo
- * @streamtype: a #GType of #GstDiscovererStreamInfo
+ * @streamtype: a #GType derived from #GstDiscovererStreamInfo
  *
  * Finds the #GstDiscovererStreamInfo contained in @info that match the
  * given @streamtype.
  *
- * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
- * free it with #gst_discoverer_stream_info_list_free.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
+ * matching #GstDiscovererStreamInfo. The caller should free it with
+ * gst_discoverer_stream_info_list_free().
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GList *
 gst_discoverer_info_get_streams (GstDiscovererInfo * info, GType streamtype)
@@ -408,10 +466,11 @@ gst_discoverer_info_get_streams (GstDiscovererInfo * info, GType streamtype)
  *
  * Finds all the #GstDiscovererAudioInfo contained in @info
  *
- * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
- * free it with #gst_discoverer_stream_info_list_free.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
+ * matching #GstDiscovererStreamInfo. The caller should free it with
+ * gst_discoverer_stream_info_list_free().
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GList *
 gst_discoverer_info_get_audio_streams (GstDiscovererInfo * info)
@@ -425,10 +484,11 @@ gst_discoverer_info_get_audio_streams (GstDiscovererInfo * info)
  *
  * Finds all the #GstDiscovererVideoInfo contained in @info
  *
- * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
- * free it with #gst_discoverer_stream_info_list_free.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
+ * matching #GstDiscovererStreamInfo. The caller should free it with
+ * gst_discoverer_stream_info_list_free().
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GList *
 gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
@@ -436,16 +496,36 @@ gst_discoverer_info_get_video_streams (GstDiscovererInfo * info)
   return gst_discoverer_info_get_streams (info, GST_TYPE_DISCOVERER_VIDEO_INFO);
 }
 
+/**
+ * gst_discoverer_info_get_subtitle_streams:
+ * @info: a #GstDiscovererInfo
+ *
+ * Finds all the #GstDiscovererSubtitleInfo contained in @info
+ *
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
+ * matching #GstDiscovererStreamInfo. The caller should free it with
+ * gst_discoverer_stream_info_list_free().
+ *
+ * Since: 0.10.36
+ */
+GList *
+gst_discoverer_info_get_subtitle_streams (GstDiscovererInfo * info)
+{
+  return gst_discoverer_info_get_streams (info,
+      GST_TYPE_DISCOVERER_SUBTITLE_INFO);
+}
+
 /**
  * gst_discoverer_info_get_container_streams:
  * @info: a #GstDiscovererInfo
  *
  * Finds all the #GstDiscovererContainerInfo contained in @info
  *
- * Returns: A #GList of matching #GstDiscovererStreamInfo. The caller should
- * free it with #gst_discoverer_stream_info_list_free.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): A #GList of
+ * matching #GstDiscovererStreamInfo. The caller should free it with
+ * gst_discoverer_stream_info_list_free().
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GList *
 gst_discoverer_info_get_container_streams (GstDiscovererInfo * info)
@@ -461,7 +541,7 @@ gst_discoverer_info_get_container_streams (GstDiscovererInfo * info)
  * Returns: a human readable name for the stream type of the given @info (ex : "audio",
  * "container",...).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 const gchar *
 gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
@@ -477,6 +557,8 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
     else
       return "video";
   }
+  if (GST_IS_DISCOVERER_SUBTITLE_INFO (info))
+    return "subtitles";
   return "unknown";
 }
 
@@ -493,10 +575,11 @@ gst_discoverer_stream_info_get_stream_type_nick (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_previous:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the previous #GstDiscovererStreamInfo in a chain. %NULL for starting
- * points. Unref with #gst_discoverer_stream_info_unref after usage.
+ * Returns: (transfer full): the previous #GstDiscovererStreamInfo in a chain.
+ * %NULL for starting points. Unref with #gst_discoverer_stream_info_unref
+ * after usage.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GstDiscovererStreamInfo *
 gst_discoverer_stream_info_get_previous (GstDiscovererStreamInfo * info)
@@ -512,10 +595,11 @@ gst_discoverer_stream_info_get_previous (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_next:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the next #GstDiscovererStreamInfo in a chain. %NULL for final streams.
+ * Returns: (transfer full): the next #GstDiscovererStreamInfo in a chain. %NULL
+ * for final streams.
  * Unref with #gst_discoverer_stream_info_unref after usage.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GstDiscovererStreamInfo *
 gst_discoverer_stream_info_get_next (GstDiscovererStreamInfo * info)
@@ -532,9 +616,10 @@ gst_discoverer_stream_info_get_next (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_caps:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the #GstCaps of the stream. Unref with #gst_caps_unref after usage.
+ * Returns: (transfer full): the #GstCaps of the stream. Unref with
+ * #gst_caps_unref after usage.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GstCaps *
 gst_discoverer_stream_info_get_caps (GstDiscovererStreamInfo * info)
@@ -550,10 +635,10 @@ gst_discoverer_stream_info_get_caps (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_tags:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the tags contained in this stream. If you wish to use the tags after
- * the life-time of @info you will need to copy them.
+ * Returns: (transfer none): the tags contained in this stream. If you wish to
+ * use the tags after the life-time of @info you will need to copy them.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 const GstTagList *
@@ -568,11 +653,11 @@ gst_discoverer_stream_info_get_tags (GstDiscovererStreamInfo * info)
  * gst_discoverer_stream_info_get_misc:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: additional information regarding the stream (for example codec version,
- * profile, etc..). If you wish to use the #GstStructure after the life-time of
- * @info you will need to copy it.
+ * Returns: (transfer none): additional information regarding the stream (for
+ * example codec version, profile, etc..). If you wish to use the #GstStructure
+ * after the life-time of @info you will need to copy it.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 const GstStructure *
 gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
@@ -588,10 +673,11 @@ gst_discoverer_stream_info_get_misc (GstDiscovererStreamInfo * info)
  * gst_discoverer_container_info_get_streams:
  * @info: a #GstDiscovererStreamInfo
  *
- * Returns: the list of #GstDiscovererStreamInfo this container stream offers.
- * Free with #gst_discoverer_stream_info_list_free after usage.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): the list of
+ * #GstDiscovererStreamInfo this container stream offers.
+ * Free with gst_discoverer_stream_info_list_free() after usage.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 GList *
@@ -622,7 +708,7 @@ gst_discoverer_container_info_get_streams (GstDiscovererContainerInfo * info)
  *
  * Returns: the number of channels in the stream.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (channels, guint, 0);
@@ -633,7 +719,7 @@ AUDIO_INFO_ACCESSOR_CODE (channels, guint, 0);
  *
  * Returns: the sample rate of the stream in Hertz.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (sample_rate, guint, 0);
@@ -644,7 +730,7 @@ AUDIO_INFO_ACCESSOR_CODE (sample_rate, guint, 0);
  *
  * Returns: the number of bits used per sample in each channel.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (depth, guint, 0);
@@ -655,7 +741,7 @@ AUDIO_INFO_ACCESSOR_CODE (depth, guint, 0);
  *
  * Returns: the average or nominal bitrate of the stream in bits/second.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
@@ -666,11 +752,22 @@ AUDIO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
  *
  * Returns: the maximum bitrate of the stream in bits/second.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 AUDIO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
 
+/**
+ * gst_discoverer_audio_info_get_language:
+ * @info: a #GstDiscovererAudioInfo
+ *
+ * Returns: the language of the stream, or NULL if unknown.
+ *
+ * Since: 0.10.36
+ */
+
+AUDIO_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
+
 /* GstDiscovererVideoInfo */
 
 #define VIDEO_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
@@ -684,7 +781,7 @@ AUDIO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
  *
  * Returns: the width of the video stream in pixels.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (width, guint, 0);
@@ -695,7 +792,7 @@ VIDEO_INFO_ACCESSOR_CODE (width, guint, 0);
  *
  * Returns: the height of the video stream in pixels.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (height, guint, 0);
@@ -704,10 +801,9 @@ VIDEO_INFO_ACCESSOR_CODE (height, guint, 0);
  * gst_discoverer_video_info_get_depth:
  * @info: a #GstDiscovererVideoInfo
  *
- * Returns: the depth in bits of the video stream (only relevant for
- * video streams).
+ * Returns: the depth in bits of the video stream.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (depth, guint, 0);
@@ -718,7 +814,7 @@ VIDEO_INFO_ACCESSOR_CODE (depth, guint, 0);
  *
  * Returns: the framerate of the video stream (numerator).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (framerate_num, guint, 0);
@@ -729,7 +825,7 @@ VIDEO_INFO_ACCESSOR_CODE (framerate_num, guint, 0);
  *
  * Returns: the framerate of the video stream (denominator).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (framerate_denom, guint, 0);
@@ -740,7 +836,7 @@ VIDEO_INFO_ACCESSOR_CODE (framerate_denom, guint, 0);
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (numerator).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (par_num, guint, 0);
@@ -751,7 +847,7 @@ VIDEO_INFO_ACCESSOR_CODE (par_num, guint, 0);
  *
  * Returns: the Pixel Aspect Ratio (PAR) of the video stream (denominator).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (par_denom, guint, 0);
@@ -762,7 +858,7 @@ VIDEO_INFO_ACCESSOR_CODE (par_denom, guint, 0);
  *
  * Returns: %TRUE if the stream is interlaced, else %FALSE.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 gboolean
 gst_discoverer_video_info_is_interlaced (const GstDiscovererVideoInfo * info)
@@ -778,7 +874,7 @@ gst_discoverer_video_info_is_interlaced (const GstDiscovererVideoInfo * info)
  *
  * Returns: the average or nominal bitrate of the video stream in bits/second.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
@@ -789,7 +885,7 @@ VIDEO_INFO_ACCESSOR_CODE (bitrate, guint, 0);
  *
  * Returns: the maximum bitrate of the video stream in bits/second.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 VIDEO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
@@ -801,7 +897,7 @@ VIDEO_INFO_ACCESSOR_CODE (max_bitrate, guint, 0);
  * Returns: #TRUE if the video stream corresponds to an image (i.e. only contains
  * one frame).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 gboolean
 gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
@@ -811,6 +907,24 @@ gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
   return info->is_image;
 }
 
+/* GstDiscovererSubtitleInfo */
+
+#define SUBTITLE_INFO_ACCESSOR_CODE(fieldname, type, failval)                     \
+  GENERIC_ACCESSOR_CODE(gst_discoverer_subtitle_info, GstDiscovererSubtitleInfo*, \
+			GST_TYPE_DISCOVERER_SUBTITLE_INFO,                        \
+			fieldname, type, failval)
+
+/**
+ * gst_discoverer_subtitle_info_get_language:
+ * @info: a #GstDiscovererSubtitleInfo
+ *
+ * Returns: the language of the stream, or NULL if unknown.
+ *
+ * Since: 0.10.36
+ */
+
+SUBTITLE_INFO_ACCESSOR_CODE (language, const gchar *, NULL);
+
 /* GstDiscovererInfo */
 
 #define DISCOVERER_INFO_ACCESSOR_CODE(fieldname, type, failval)		\
@@ -822,10 +936,10 @@ gst_discoverer_video_info_is_image (const GstDiscovererVideoInfo * info)
  * gst_discoverer_info_get_uri:
  * @info: a #GstDiscovererInfo
  *
- * Returns: the URI to which this information corresponds to. Copy it if you
- * wish to use it after the life-time of @info.
+ * Returns: (transfer none): the URI to which this information corresponds to.
+ * Copy it if you wish to use it after the life-time of @info.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (uri, const gchar *, NULL);
@@ -836,7 +950,7 @@ DISCOVERER_INFO_ACCESSOR_CODE (uri, const gchar *, NULL);
  *
  * Returns: the result of the discovery as a #GstDiscovererResult.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (result, GstDiscovererResult, GST_DISCOVERER_OK);
@@ -845,11 +959,12 @@ DISCOVERER_INFO_ACCESSOR_CODE (result, GstDiscovererResult, GST_DISCOVERER_OK);
  * gst_discoverer_info_get_stream_info:
  * @info: a #GstDiscovererInfo
  *
- * Returns: the structure (or topology) of the URI as a #GstDiscovererStreamInfo.
+ * Returns: (transfer full): the structure (or topology) of the URI as a
+ * #GstDiscovererStreamInfo.
  * This structure can be traversed to see the original hierarchy. Unref with
- * #gst_discoverer_stream_info_unref after usage.
+ * gst_discoverer_stream_info_unref() after usage.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 GstDiscovererStreamInfo *
@@ -866,10 +981,11 @@ gst_discoverer_info_get_stream_info (GstDiscovererInfo * info)
  * gst_discoverer_info_get_stream_list:
  * @info: a #GstDiscovererInfo
  *
- * Returns: the list of all streams contained in the #info. Free after usage
- * with #gst_discoverer_stream_info_list_free.
+ * Returns: (transfer full) (element-type Gst.DiscovererStreamInfo): the list of
+ * all streams contained in the #info. Free after usage
+ * with gst_discoverer_stream_info_list_free().
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 GList *
 gst_discoverer_info_get_stream_list (GstDiscovererInfo * info)
@@ -892,20 +1008,31 @@ gst_discoverer_info_get_stream_list (GstDiscovererInfo * info)
  *
  * Returns: the duration of the URI in #GstClockTime (nanoseconds).
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (duration, GstClockTime, GST_CLOCK_TIME_NONE);
 
+/**
+ * gst_discoverer_info_get_seekable:
+ * @info: a #GstDiscovererInfo
+ *
+ * Returns: the wheter the URI is seekable.
+ *
+ * Since: 0.10.32
+ */
+
+DISCOVERER_INFO_ACCESSOR_CODE (seekable, gboolean, FALSE);
+
 /**
  * gst_discoverer_info_get_misc:
  * @info: a #GstDiscovererInfo
  *
- * Returns: Miscellaneous information stored as a #GstStructure (for example:
- * information about missing plugins). If you wish to use the #GstStructure
- * after the life-time of @info, you will need to copy it.
+ * Returns: (transfer none): Miscellaneous information stored as a #GstStructure
+ * (for example: information about missing plugins). If you wish to use the
+ * #GstStructure after the life-time of @info, you will need to copy it.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
@@ -914,10 +1041,50 @@ DISCOVERER_INFO_ACCESSOR_CODE (misc, const GstStructure *, NULL);
  * gst_discoverer_info_get_tags:
  * @info: a #GstDiscovererInfo
  *
- * Returns: all tags contained in the %URI. If you wish to use the tags after
- * the life-time of @info, you will need to copy them.
+ * Returns: (transfer none): all tags contained in the URI. If you wish to use
+ * the tags after the life-time of @info, you will need to copy them.
  *
- * Since 0.10.31
+ * Since: 0.10.31
  */
 
 DISCOVERER_INFO_ACCESSOR_CODE (tags, const GstTagList *, NULL);
+
+/**
+ * gst_discoverer_info_ref:
+ * @info: a #GstDiscovererInfo
+ *
+ * Increments the reference count of @info.
+ *
+ * Returns: the same #GstDiscovererInfo object
+ *
+ * Since: 0.10.31
+ */
+
+/**
+ * gst_discoverer_info_unref:
+ * @info: a #GstDiscovererInfo
+ *
+ * Decrements the reference count of @info.
+ *
+ * Since: 0.10.31
+ */
+
+/**
+ * gst_discoverer_stream_info_ref:
+ * @info: a #GstDiscovererStreamInfo
+ *
+ * Increments the reference count of @info.
+ *
+ * Returns: the same #GstDiscovererStreamInfo object
+ *
+ * Since: 0.10.31
+ */
+
+/**
+ * gst_discoverer_stream_info_unref:
+ * @info: a #GstDiscovererStreamInfo
+ *
+ * Decrements the reference count of @info.
+ *
+ * Since: 0.10.31
+ */
