@@ -26,7 +26,7 @@
 #include "riff-ids.h"
 #include "riff-media.h"
 
-#include <gst/audio/multichannel.h>
+#include <gst/audio/audio.h>
 
 #include <string.h>
 #include <math.h>
@@ -68,21 +68,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
       if (strf) {
         if (bpp == 8) {
-          caps = gst_caps_new_simple ("video/x-raw-rgb",
-              "bpp", G_TYPE_INT, 8, "depth", G_TYPE_INT, 8,
-              "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+          caps = gst_caps_new_simple ("video/x-raw",
+              "format", G_TYPE_STRING, "RGB8_PALETTED", NULL);
         } else if (bpp == 24) {
-          caps = gst_caps_new_simple ("video/x-raw-rgb",
-              "bpp", G_TYPE_INT, 24, "depth", G_TYPE_INT, 24,
-              "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-              "red_mask", G_TYPE_INT, 0xff, "green_mask", G_TYPE_INT, 0xff00,
-              "blue_mask", G_TYPE_INT, 0xff0000, NULL);
+          caps = gst_caps_new_simple ("video/x-raw",
+              "format", G_TYPE_STRING, "BGR", NULL);
         } else if (bpp == 32) {
-          caps = gst_caps_new_simple ("video/x-raw-rgb",
-              "bpp", G_TYPE_INT, 32, "depth", G_TYPE_INT, 24,
-              "endianness", G_TYPE_INT, G_BIG_ENDIAN,
-              "red_mask", G_TYPE_INT, 0xff00, "green_mask", G_TYPE_INT,
-              0xff0000, "blue_mask", G_TYPE_INT, 0xff000000, NULL);
+          caps = gst_caps_new_simple ("video/x-raw",
+              "format", G_TYPE_STRING, "BGRx", NULL);
         } else {
           GST_WARNING ("Unhandled DIB RGB depth: %d", bpp);
           return NULL;
@@ -90,8 +83,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       } else {
         /* for template */
         caps =
-            gst_caps_from_string ("video/x-raw-rgb, bpp = (int) { 8, 24, 32 }, "
-            "depth = (int) { 8, 24}");
+            gst_caps_from_string ("video/x-raw, format = (string) "
+            "{ RGB8_PALETTED, BGR, BGRx }");
       }
 
       palette = strf_data;
@@ -105,38 +98,38 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
     }
     case GST_MAKE_FOURCC ('I', '4', '2', '0'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
+      caps = gst_caps_new_simple ("video/x-raw",
+          "format", G_TYPE_STRING, "I420", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed planar YUV 4:2:0");
       break;
 
     case GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'):
     case GST_MAKE_FOURCC ('Y', 'U', 'N', 'V'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('Y', 'U', 'Y', '2'),
-          NULL);
+      caps = gst_caps_new_simple ("video/x-raw",
+          "format", G_TYPE_STRING, "YUY2", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YUV 4:2:2");
       break;
 
     case GST_MAKE_FOURCC ('Y', 'V', 'U', '9'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
+      caps = gst_caps_new_simple ("video/x-raw",
+          "format", G_TYPE_STRING, "YVU9", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YVU 4:1:0");
       break;
 
     case GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
+    case GST_MAKE_FOURCC ('2', 'v', 'u', 'y'):
+      caps = gst_caps_new_simple ("video/x-raw",
+          "format", G_TYPE_STRING, "UYVY", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YUV 4:2:2");
       break;
 
     case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
-      caps = gst_caps_new_simple ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, codec_fcc, NULL);
+      caps = gst_caps_new_simple ("video/x-raw",
+          "format", G_TYPE_STRING, "YV12", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Uncompressed packed YVU 4:2:2");
       break;
@@ -470,6 +463,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
     case GST_MAKE_FOURCC ('V', 'I', 'D', 'M'):
     case GST_MAKE_FOURCC ('M', '4', 'T', '3'):
     case GST_MAKE_FOURCC ('G', 'E', 'O', 'X'):
+    case GST_MAKE_FOURCC ('M', 'P', '4', 'V'):
+    case GST_MAKE_FOURCC ('m', 'p', '4', 'v'):
       caps = gst_caps_new_simple ("video/mpeg",
           "mpegversion", G_TYPE_INT, 4, NULL);
       if (codec_name)
@@ -543,16 +538,14 @@ gst_riff_create_video_caps (guint32 codec_fcc,
 
     case GST_MAKE_FOURCC ('W', 'M', 'V', 'A'):
       caps = gst_caps_new_simple ("video/x-wmv",
-          "wmvversion", G_TYPE_INT, 3, "format", GST_TYPE_FOURCC,
-          codec_fcc, NULL);
+          "wmvversion", G_TYPE_INT, 3, "format", G_TYPE_STRING, "WMVA", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft Windows Media Advanced Profile");
       break;
 
     case GST_MAKE_FOURCC ('W', 'V', 'C', '1'):
       caps = gst_caps_new_simple ("video/x-wmv",
-          "wmvversion", G_TYPE_INT, 3, "format", GST_TYPE_FOURCC,
-          codec_fcc, NULL);
+          "wmvversion", G_TYPE_INT, 3, "format", G_TYPE_STRING, "WVC1", NULL);
       if (codec_name)
         *codec_name = g_strdup ("Microsoft Windows Media VC-1");
       break;
@@ -713,6 +706,22 @@ gst_riff_create_video_caps (guint32 codec_fcc,
       break;
     }
 
+    case GST_MAKE_FOURCC ('C', 'S', 'C', 'D'):
+    {
+      if (strf) {
+        gint depth = (strf->bit_cnt != 0) ? (gint) strf->bit_cnt : 24;
+
+        caps = gst_caps_new_simple ("video/x-camstudio", "depth", G_TYPE_INT,
+            depth, NULL);
+      } else {
+        /* template caps */
+        caps = gst_caps_new_simple ("video/x-camstudio", NULL);
+      }
+      if (codec_name)
+        *codec_name = g_strdup ("Camstudio");
+      break;
+    }
+
     case GST_MAKE_FOURCC ('V', 'C', 'R', '1'):
       caps = gst_caps_new_simple ("video/x-ati-vcr",
           "vcrversion", G_TYPE_INT, 1, NULL);
@@ -802,11 +811,34 @@ gst_riff_create_video_caps (guint32 codec_fcc,
     case GST_MAKE_FOURCC ('V', 'P', '6', '0'):
     case GST_MAKE_FOURCC ('v', 'p', '6', '1'):
     case GST_MAKE_FOURCC ('V', 'P', '6', '1'):
+    case GST_MAKE_FOURCC ('V', 'p', '6', '2'):
+    case GST_MAKE_FOURCC ('V', 'P', '6', '2'):
       caps = gst_caps_new_simple ("video/x-vp6", NULL);
       if (codec_name)
         *codec_name = g_strdup ("On2 VP6");
       break;
 
+    case GST_MAKE_FOURCC ('V', 'P', '6', 'F'):
+    case GST_MAKE_FOURCC ('v', 'p', '6', 'f'):
+    case GST_MAKE_FOURCC ('F', 'L', 'V', '4'):
+      caps = gst_caps_new_simple ("video/x-vp6-flash", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("On2 VP6");
+      break;
+
+    case GST_MAKE_FOURCC ('v', 'p', '7', '0'):
+    case GST_MAKE_FOURCC ('V', 'P', '7', '0'):
+      caps = gst_caps_new_simple ("video/x-vp7", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("On2 VP7");
+      break;
+
+    case GST_MAKE_FOURCC ('V', 'P', '8', '0'):
+      caps = gst_caps_new_simple ("video/x-vp8", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("On2 VP8");
+      break;
+
     case GST_MAKE_FOURCC ('L', 'M', '2', '0'):
       caps = gst_caps_new_simple ("video/x-mimic", NULL);
       if (codec_name)
@@ -848,8 +880,8 @@ gst_riff_create_video_caps (guint32 codec_fcc,
         "height", G_TYPE_INT, strf->height, NULL);
   } else {
     gst_caps_set_simple (caps,
-        "width", GST_TYPE_INT_RANGE, 16, 4096,
-        "height", GST_TYPE_INT_RANGE, 16, 4096, NULL);
+        "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
+        "height", GST_TYPE_INT_RANGE, 1, G_MAXINT, NULL);
   }
 
   /* extradata */
@@ -865,18 +897,19 @@ gst_riff_create_video_caps (guint32 codec_fcc,
   if (palette) {
     GstBuffer *copy;
     guint num_colors;
+    gsize size;
 
     if (strf != NULL)
       num_colors = strf->num_colors;
     else
       num_colors = 256;
 
-    if (GST_BUFFER_SIZE (palette) >= (num_colors * 4)) {
+    size = gst_buffer_get_size (palette);
+
+    if (size >= (num_colors * 4)) {
       /* palette is always at least 256*4 bytes */
-      copy =
-          gst_buffer_new_and_alloc (MAX (GST_BUFFER_SIZE (palette), 256 * 4));
-      memcpy (GST_BUFFER_DATA (copy), GST_BUFFER_DATA (palette),
-          GST_BUFFER_SIZE (palette));
+      copy = gst_buffer_new_and_alloc (MAX (size, 256 * 4));
+      gst_buffer_copy_into (copy, palette, GST_BUFFER_COPY_MEMORY, 0, size);
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
       {
@@ -917,19 +950,13 @@ static const struct
   0x00100, GST_AUDIO_CHANNEL_POSITION_REAR_CENTER}, {
   0x00200, GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT}, {
   0x00400, GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT}, {
-  0x00800, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_CENTER       */
-  {
-  0x01000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_LEFT   */
-  {
-  0x02000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_CENTER */
-  {
-  0x04000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_FRONT_RIGHT  */
-  {
-  0x08000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_BACK_LEFT    */
-  {
-  0x10000, GST_AUDIO_CHANNEL_POSITION_INVALID}, /* TOP_BACK_CENTER  */
-  {
-  0x20000, GST_AUDIO_CHANNEL_POSITION_INVALID}  /* TOP_BACK_RIGHT   */
+  0x00800, GST_AUDIO_CHANNEL_POSITION_TOP_CENTER}, {
+  0x01000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_LEFT}, {
+  0x02000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_CENTER}, {
+  0x04000, GST_AUDIO_CHANNEL_POSITION_TOP_FRONT_RIGHT}, {
+  0x08000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_LEFT}, {
+  0x10000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_CENTER}, {
+  0x20000, GST_AUDIO_CHANNEL_POSITION_TOP_REAR_RIGHT}
 };
 
 #define MAX_CHANNEL_POSITIONS G_N_ELEMENTS (layout_mapping)
@@ -1088,6 +1115,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
         gint ba = strf->blockalign;
         gint ch = strf->channels;
         gint wd, ws;
+        GstAudioFormat format;
 
         /* If we have an empty blockalign, we take the width contained in 
          * strf->size */
@@ -1108,11 +1136,12 @@ gst_riff_create_audio_caps (guint16 codec_id,
         /* For reference, the actual depth is in strf->size */
         ws = wd;
 
-        caps = gst_caps_new_simple ("audio/x-raw-int",
-            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-            "channels", G_TYPE_INT, ch,
-            "width", G_TYPE_INT, wd,
-            "depth", G_TYPE_INT, ws, "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
+        format =
+            gst_audio_format_build_integer (wd != 8, G_LITTLE_ENDIAN, wd, ws);
+
+        caps = gst_caps_new_simple ("audio/x-raw",
+            "format", G_TYPE_STRING, gst_audio_format_to_string (format),
+            "channels", G_TYPE_INT, ch, NULL);
 
         /* Add default channel layout. In theory this should be done
          * for 1 and 2 channels too but apparently breaks too many
@@ -1128,10 +1157,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
         }
       } else {
         /* FIXME: this is pretty useless - we need fixed caps */
-        caps = gst_caps_from_string ("audio/x-raw-int, "
-            "endianness = (int) LITTLE_ENDIAN, "
-            "signed = (boolean) { true, false }, "
-            "width = (int) { 8, 16, 24, 32 }, " "depth = (int) [ 1, 32 ]");
+        caps = gst_caps_from_string ("audio/x-raw, "
+            "format = (string) { S8, U8, S16LE, U16LE, S24LE, "
+            "U24LE, S32LE, U32LE }");
       }
       if (codec_name && strf)
         *codec_name = g_strdup_printf ("Uncompressed %d-bit PCM audio",
@@ -1155,9 +1183,9 @@ gst_riff_create_audio_caps (guint16 codec_id,
         gint ch = strf->channels;
         gint wd = ba * 8 / ch;
 
-        caps = gst_caps_new_simple ("audio/x-raw-float",
-            "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
-            "channels", G_TYPE_INT, ch, "width", G_TYPE_INT, wd, NULL);
+        caps = gst_caps_new_simple ("audio/x-raw",
+            "format", G_TYPE_STRING, wd == 64 ? "F64LE" : "F32LE",
+            "channels", G_TYPE_INT, ch, NULL);
 
         /* Add default channel layout. In theory this should be done
          * for 1 and 2 channels too but apparently breaks too many
@@ -1173,8 +1201,8 @@ gst_riff_create_audio_caps (guint16 codec_id,
         }
       } else {
         /* FIXME: this is pretty useless - we need fixed caps */
-        caps = gst_caps_from_string ("audio/x-raw-float, "
-            "endianness = (int) LITTLE_ENDIAN, " "width = (int) { 32, 64 }");
+        caps = gst_caps_from_string ("audio/x-raw, "
+            "format = (string) { F32LE, F64LE }");
       }
       if (codec_name && strf)
         *codec_name = g_strdup_printf ("Uncompressed %d-bit IEEE float audio",
@@ -1423,25 +1451,28 @@ gst_riff_create_audio_caps (guint16 codec_id,
       guint16 valid_bits_per_sample;
       guint32 channel_mask;
       guint32 subformat_guid[4];
-      const guint8 *data;
+      guint8 *data;
+      gsize size;
 
       channels_max = 8;
 
       /* should be at least 22 bytes */
-      if (strf_data == NULL || GST_BUFFER_SIZE (strf_data) < 22) {
-        GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %d (expected: 22)",
-            (strf_data) ? GST_BUFFER_SIZE (strf_data) : -1);
+      size = gst_buffer_get_size (strf_data);
+
+      if (strf_data == NULL || size < 22) {
+        GST_WARNING ("WAVE_FORMAT_EXTENSIBLE data size is %" G_GSIZE_FORMAT
+            " (expected: 22)", (strf_data) ? size : -1);
         return NULL;
       }
 
-      data = GST_BUFFER_DATA (strf_data);
-
+      data = gst_buffer_map (strf_data, &size, NULL, GST_MAP_READ);
       valid_bits_per_sample = GST_READ_UINT16_LE (data);
       channel_mask = GST_READ_UINT32_LE (data + 2);
       subformat_guid[0] = GST_READ_UINT32_LE (data + 6);
       subformat_guid[1] = GST_READ_UINT32_LE (data + 10);
       subformat_guid[2] = GST_READ_UINT32_LE (data + 14);
       subformat_guid[3] = GST_READ_UINT32_LE (data + 18);
+      gst_buffer_unmap (strf_data, data, size);
 
       GST_DEBUG ("valid bps    = %u", valid_bits_per_sample);
       GST_DEBUG ("channel mask = 0x%08x", channel_mask);
@@ -1456,6 +1487,7 @@ gst_riff_create_audio_caps (guint16 codec_id,
             gint ba = strf->blockalign;
             gint wd = ba * 8 / strf->channels;
             gint ws;
+            GstAudioFormat format;
 
             /* in riff, the depth is stored in the size field but it just
              * means that the _least_ significant bits are cleared. We can
@@ -1467,13 +1499,14 @@ gst_riff_create_audio_caps (guint16 codec_id,
              * if (valid_bits_per_sample != 0)
              *   ws = valid_bits_per_sample; */
 
-            caps = gst_caps_new_simple ("audio/x-raw-int",
-                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            format =
+                gst_audio_format_build_integer (wd != 8, G_LITTLE_ENDIAN, wd,
+                ws);
+
+            caps = gst_caps_new_simple ("audio/x-raw",
+                "format", G_TYPE_STRING, gst_audio_format_to_string (format),
                 "channels", G_TYPE_INT, strf->channels,
-                "width", G_TYPE_INT, wd,
-                "depth", G_TYPE_INT, ws,
-                "rate", G_TYPE_INT, strf->rate,
-                "signed", G_TYPE_BOOLEAN, wd != 8, NULL);
+                "rate", G_TYPE_INT, strf->rate, NULL);
 
             /* If channel_mask == 0 and channels > 2 let's
              * assume default layout as some wav files don't have the
@@ -1502,10 +1535,10 @@ gst_riff_create_audio_caps (guint16 codec_id,
             gint ba = strf->blockalign;
             gint wd = ba * 8 / strf->channels;
 
-            caps = gst_caps_new_simple ("audio/x-raw-float",
-                "endianness", G_TYPE_INT, G_LITTLE_ENDIAN,
+            caps = gst_caps_new_simple ("audio/x-raw",
+                "format", G_TYPE_STRING, wd == 32 ? "F32LE" : "F64LE",
                 "channels", G_TYPE_INT, strf->channels,
-                "width", G_TYPE_INT, wd, "rate", G_TYPE_INT, strf->rate, NULL);
+                "rate", G_TYPE_INT, strf->rate, NULL);
 
             /* If channel_mask == 0 and channels > 2 let's
              * assume default layout as some wav files don't have the
@@ -1710,6 +1743,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('A', 'S', 'V', '1'),
     GST_MAKE_FOURCC ('A', 'S', 'V', '2'),
     GST_MAKE_FOURCC ('C', 'L', 'J', 'R'),
+    GST_MAKE_FOURCC ('C', 'S', 'C', 'D'),
     GST_MAKE_FOURCC ('C', 'Y', 'U', 'V'),
     GST_MAKE_FOURCC ('D', 'I', 'B', ' '),
     GST_MAKE_FOURCC ('D', 'I', 'V', '3'),
@@ -1720,6 +1754,7 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('D', 'X', '5', '0'),
     GST_MAKE_FOURCC ('M', '4', 'C', 'C'),
     GST_MAKE_FOURCC ('F', 'L', 'V', '1'),
+    GST_MAKE_FOURCC ('F', 'L', 'V', '4'),
     GST_MAKE_FOURCC ('H', '2', '6', '3'),
     GST_MAKE_FOURCC ('V', 'X', '1', 'K'),
     GST_MAKE_FOURCC ('H', '2', '6', '4'),
@@ -1772,6 +1807,9 @@ gst_riff_create_video_template_caps (void)
     GST_MAKE_FOURCC ('K', 'M', 'V', 'C'),
     GST_MAKE_FOURCC ('V', 'P', '5', '0'),
     GST_MAKE_FOURCC ('V', 'P', '6', '0'),
+    GST_MAKE_FOURCC ('V', 'P', '6', 'F'),
+    GST_MAKE_FOURCC ('V', 'P', '7', '0'),
+    GST_MAKE_FOURCC ('V', 'P', '8', '0'),
     GST_MAKE_FOURCC ('L', 'M', '2', '0'),
     GST_MAKE_FOURCC ('R', 'P', 'Z', 'A'),
     GST_MAKE_FOURCC ('T', 'H', 'E', 'O'),
