@@ -21,12 +21,9 @@
  * SECTION:gstbasertpaudiopayload
  * @short_description: Base class for audio RTP payloader
  *
- * <refsect2>
- * <para>
  * Provides a base class for audio RTP payloaders for frame or sample based
  * audio codecs (constant bitrate)
- * </para>
- * <para>
+ *
  * This class derives from GstBaseRTPPayload. It can be used for payloading
  * audio codecs. It will only work with constant bitrate codecs. It supports
  * both frame based and sample based codecs. It takes care of packing up the
@@ -38,7 +35,8 @@
  * equal to min-ptime (if set). If min-ptime is not set, any residual data is
  * sent in a last RTP packet. In the case of frame based codecs, the resulting
  * RTP packets always contain full frames.
- * </para>
+ *
+ * <refsect2>
  * <title>Usage</title>
  * <para>
  * To use this base class, your child element needs to call either
@@ -158,16 +156,12 @@ static GstFlowReturn gst_base_rtp_audio_payload_handle_buffer (GstBaseRTPPayload
 static GstStateChangeReturn gst_base_rtp_payload_audio_change_state (GstElement
     * element, GstStateChange transition);
 
-static gboolean gst_base_rtp_payload_audio_handle_event (GstPad * pad,
-    GstEvent * event);
+static gboolean gst_base_rtp_payload_audio_handle_event (GstBaseRTPPayload
+    * payload, GstEvent * event);
 
-GST_BOILERPLATE (GstBaseRTPAudioPayload, gst_base_rtp_audio_payload,
-    GstBaseRTPPayload, GST_TYPE_BASE_RTP_PAYLOAD);
-
-static void
-gst_base_rtp_audio_payload_base_init (gpointer klass)
-{
-}
+#define gst_base_rtp_audio_payload_parent_class parent_class
+G_DEFINE_TYPE (GstBaseRTPAudioPayload, gst_base_rtp_audio_payload,
+    GST_TYPE_BASE_RTP_PAYLOAD);
 
 static void
 gst_base_rtp_audio_payload_class_init (GstBaseRTPAudioPayloadClass * klass)
@@ -204,8 +198,7 @@ gst_base_rtp_audio_payload_class_init (GstBaseRTPAudioPayloadClass * klass)
 }
 
 static void
-gst_base_rtp_audio_payload_init (GstBaseRTPAudioPayload * payload,
-    GstBaseRTPAudioPayloadClass * klass)
+gst_base_rtp_audio_payload_init (GstBaseRTPAudioPayload * payload)
 {
   payload->priv = GST_BASE_RTP_AUDIO_PAYLOAD_GET_PRIVATE (payload);
 
@@ -406,19 +399,23 @@ gst_base_rtp_audio_payload_set_meta (GstBaseRTPAudioPayload * payload,
 {
   GstBaseRTPPayload *basepayload;
   GstBaseRTPAudioPayloadPrivate *priv;
+  GstRTPBuffer rtp;
 
   basepayload = GST_BASE_RTP_PAYLOAD_CAST (payload);
   priv = payload->priv;
 
   /* set payload type */
-  gst_rtp_buffer_set_payload_type (buffer, basepayload->pt);
+  gst_rtp_buffer_map (buffer, GST_MAP_WRITE, &rtp);
+  gst_rtp_buffer_set_payload_type (&rtp, basepayload->pt);
   /* set marker bit for disconts */
   if (priv->discont) {
     GST_DEBUG_OBJECT (payload, "Setting marker and DISCONT");
-    gst_rtp_buffer_set_marker (buffer, TRUE);
+    gst_rtp_buffer_set_marker (&rtp, TRUE);
     GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
     priv->discont = FALSE;
   }
+  gst_rtp_buffer_unmap (&rtp);
+
   GST_BUFFER_TIMESTAMP (buffer) = timestamp;
 
   /* get the offset in RTP time */
@@ -458,6 +455,7 @@ gst_base_rtp_audio_payload_push (GstBaseRTPAudioPayload * baseaudiopayload,
   GstBuffer *outbuf;
   guint8 *payload;
   GstFlowReturn ret;
+  GstRTPBuffer rtp;
 
   basepayload = GST_BASE_RTP_PAYLOAD (baseaudiopayload);
 
@@ -468,8 +466,10 @@ gst_base_rtp_audio_payload_push (GstBaseRTPAudioPayload * baseaudiopayload,
   outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
 
   /* copy payload */
-  payload = gst_rtp_buffer_get_payload (outbuf);
+  gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
+  payload = gst_rtp_buffer_get_payload (&rtp);
   memcpy (payload, data, payload_len);
+  gst_rtp_buffer_unmap (&rtp);
 
   /* set metadata */
   gst_base_rtp_audio_payload_set_meta (baseaudiopayload, outbuf, payload_len,
@@ -494,7 +494,7 @@ gst_base_rtp_audio_payload_push_buffer (GstBaseRTPAudioPayload *
   priv = baseaudiopayload->priv;
   basepayload = GST_BASE_RTP_PAYLOAD (baseaudiopayload);
 
-  payload_len = GST_BUFFER_SIZE (buffer);
+  payload_len = gst_buffer_get_size (buffer);
 
   GST_DEBUG_OBJECT (baseaudiopayload, "Pushing %d bytes ts %" GST_TIME_FORMAT,
       payload_len, GST_TIME_ARGS (timestamp));
@@ -513,24 +513,29 @@ gst_base_rtp_audio_payload_push_buffer (GstBaseRTPAudioPayload *
 
   if (priv->buffer_list) {
     GstBufferList *list;
-    GstBufferListIterator *it;
+    guint i, len;
 
     list = gst_buffer_list_new ();
-    it = gst_buffer_list_iterate (list);
+    len = gst_buffer_list_len (list);
 
-    /* add both buffers to the buffer list */
-    gst_buffer_list_iterator_add_group (it);
-    gst_buffer_list_iterator_add (it, outbuf);
-    gst_buffer_list_iterator_add (it, buffer);
-
-    gst_buffer_list_iterator_free (it);
+    for (i = 0; i < len; i++) {
+      /* FIXME */
+      g_warning ("bufferlist not implemented");
+      gst_buffer_list_add (list, outbuf);
+      gst_buffer_list_add (list, buffer);
+    }
 
     GST_DEBUG_OBJECT (baseaudiopayload, "Pushing list %p", list);
     ret = gst_basertppayload_push_list (basepayload, list);
   } else {
+    GstRTPBuffer rtp;
+
     /* copy payload */
-    payload = gst_rtp_buffer_get_payload (outbuf);
-    memcpy (payload, GST_BUFFER_DATA (buffer), payload_len);
+    gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
+    payload = gst_rtp_buffer_get_payload (&rtp);
+    gst_buffer_extract (buffer, 0, payload, payload_len);
+    gst_rtp_buffer_unmap (&rtp);
+
     gst_buffer_unref (buffer);
 
     GST_DEBUG_OBJECT (baseaudiopayload, "Pushing buffer %p", outbuf);
@@ -609,13 +614,17 @@ gst_base_rtp_audio_payload_flush (GstBaseRTPAudioPayload * baseaudiopayload,
         gst_base_rtp_audio_payload_push_buffer (baseaudiopayload, buffer,
         timestamp);
   } else {
+    GstRTPBuffer rtp;
+
     /* create buffer to hold the payload */
     outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
 
     /* copy payload */
-    payload = gst_rtp_buffer_get_payload (outbuf);
+    gst_rtp_buffer_map (outbuf, GST_MAP_WRITE, &rtp);
+    payload = gst_rtp_buffer_get_payload (&rtp);
     gst_adapter_copy (adapter, payload, 0, payload_len);
     gst_adapter_flush (adapter, payload_len);
+    gst_rtp_buffer_unmap (&rtp);
 
     /* set metadata */
     gst_base_rtp_audio_payload_set_meta (baseaudiopayload, outbuf, payload_len,
@@ -863,7 +872,7 @@ gst_base_rtp_audio_payload_handle_buffer (GstBaseRTPPayload *
       "Calculated min_payload_len %u and max_payload_len %u",
       min_payload_len, max_payload_len);
 
-  size = GST_BUFFER_SIZE (buffer);
+  size = gst_buffer_get_size (buffer);
 
   /* shortcut, we don't need to use the adapter when the packet can be pushed
    * through directly. */
@@ -944,12 +953,13 @@ gst_base_rtp_payload_audio_change_state (GstElement * element,
 }
 
 static gboolean
-gst_base_rtp_payload_audio_handle_event (GstPad * pad, GstEvent * event)
+gst_base_rtp_payload_audio_handle_event (GstBaseRTPPayload * basep,
+    GstEvent * event)
 {
   GstBaseRTPAudioPayload *payload;
   gboolean res = FALSE;
 
-  payload = GST_BASE_RTP_AUDIO_PAYLOAD (gst_pad_get_parent (pad));
+  payload = GST_BASE_RTP_AUDIO_PAYLOAD (basep);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -963,9 +973,9 @@ gst_base_rtp_payload_audio_handle_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-  gst_object_unref (payload);
+  /* let parent handle the remainder of the event */
+  res = GST_BASE_RTP_PAYLOAD_CLASS (parent_class)->handle_event (basep, event);
 
-  /* return FALSE to let parent handle the remainder of the event */
   return res;
 }
 
