@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Philippe Khalaf <burger@speedy.org> 
+ * Copyright (C) <2005> Philippe Khalaf <burger@speedy.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -40,15 +40,6 @@ G_BEGIN_DECLS
 #define GST_BASE_RTP_DEPAYLOAD_SINKPAD(depayload) (GST_BASE_RTP_DEPAYLOAD (depayload)->sinkpad)
 #define GST_BASE_RTP_DEPAYLOAD_SRCPAD(depayload)  (GST_BASE_RTP_DEPAYLOAD (depayload)->srcpad)
 
-#ifndef GST_DISABLE_DEPRECATED
-/* this was presumably never meant to be public API, or should at least
- * have been prefixed if it was. Don't use. (FIXME: remove in 0.11) */
-#define QUEUE_LOCK_INIT(base)   (g_static_rec_mutex_init(&base->queuelock))
-#define QUEUE_LOCK_FREE(base)   (g_static_rec_mutex_free(&base->queuelock))
-#define QUEUE_LOCK(base)        (g_static_rec_mutex_lock(&base->queuelock))
-#define QUEUE_UNLOCK(base)      (g_static_rec_mutex_unlock(&base->queuelock))
-#endif
-
 typedef struct _GstBaseRTPDepayload      GstBaseRTPDepayload;
 typedef struct _GstBaseRTPDepayloadClass GstBaseRTPDepayloadClass;
 typedef struct _GstBaseRTPDepayloadPrivate GstBaseRTPDepayloadPrivate;
@@ -59,30 +50,9 @@ struct _GstBaseRTPDepayload
 
   GstPad *sinkpad, *srcpad;
 
-#ifndef GST_REMOVE_DEPRECATED
-  /* lock to protect the queue, deprecated */
-  GStaticRecMutex queuelock;
-
-  /* deprecated */
-  gboolean thread_running;
-  /* the releaser thread, deprecated */
-  GThread *thread;
-#endif
-
   /* this attribute must be set by the child */
   guint clock_rate;
 
-#ifndef GST_REMOVE_DEPRECATED
-  /* this value can be modified by the child if needed, deprecated */
-  guint queue_delay;
-#endif
-
-  /* we will queue up to RTP_QUEUEDELAY ms of packets,
-   * reordering them if necessary
-   * dropping any packets that are more than
-   * RTP_QUEUEDELAY ms late, deprecated */
-  GQueue *queue;
-
   GstSegment segment;
   gboolean need_newsegment;
 
@@ -92,21 +62,30 @@ struct _GstBaseRTPDepayload
   gpointer _gst_reserved[GST_PADDING-1];
 };
 
+/**
+ * GstBaseRTPDepayloadClass:
+ * @parent_class: the parent class
+ * @set_caps: configure the depayloader
+ * @add_to_queue: (deprecated)
+ * @process: process incoming rtp packets
+ * @set_gst_timestamp: convert from RTP timestamp to GST timestamp
+ * @packet_lost: signal the depayloader about packet loss
+ * @handle_event: custom event handling
+ *
+ * Base class for audio RTP payloader.
+ */
 struct _GstBaseRTPDepayloadClass
 {
   GstElementClass parent_class;
 
-  /* virtuals */
+  /* virtuals, inform the subclass of the caps. */
   gboolean (*set_caps) (GstBaseRTPDepayload *filter, GstCaps *caps);
 
-  /* non-pure function, default implementation in base class
-   * this does buffering, reordering and dropping, deprecated */
-  GstFlowReturn (*add_to_queue) (GstBaseRTPDepayload *filter, GstBuffer *in);
-
   /* pure virtual function, child must use this to process incoming
-   * rtp packets. If the child returns a buffer, the timestamp of @in will be
-   * applied to the result buffer and the buffer will be pushed. If this
-   * function returns %NULL, nothing is pushed.  */
+   * rtp packets. If the child returns a buffer without a valid timestamp,
+   * the timestamp of @in will be applied to the result buffer and the
+   * buffer will be pushed. If this function returns %NULL, nothing is
+   * pushed.  */
   GstBuffer * (*process) (GstBaseRTPDepayload *base, GstBuffer *in);
 
   /* non-pure function used to convert from RTP timestamp to GST timestamp
@@ -118,8 +97,13 @@ struct _GstBaseRTPDepayloadClass
    * The default implementation of this message pushes a segment update. */
   gboolean (*packet_lost) (GstBaseRTPDepayload *filter, GstEvent *event);
 
+  /* the default implementation does the default actions for events but
+   * implementation can override.
+   * Since: 0.10.32   */
+  gboolean (*handle_event) (GstBaseRTPDepayload * filter, GstEvent * event);
+
   /*< private >*/
-  gpointer _gst_reserved[GST_PADDING-1];
+  gpointer _gst_reserved[GST_PADDING-2];
 };
 
 GType gst_base_rtp_depayload_get_type (void);
@@ -127,6 +111,8 @@ GType gst_base_rtp_depayload_get_type (void);
 GstFlowReturn   gst_base_rtp_depayload_push              (GstBaseRTPDepayload *filter, GstBuffer *out_buf);
 GstFlowReturn   gst_base_rtp_depayload_push_ts           (GstBaseRTPDepayload *filter,
 		                                          guint32 timestamp, GstBuffer *out_buf);
+GstFlowReturn   gst_base_rtp_depayload_push_list         (GstBaseRTPDepayload *filter, GstBufferList *out_list);
+
 
 G_END_DECLS
 
