@@ -29,7 +29,6 @@
 #include "gstchannelmix.h"
 #include "gstaudioquantize.h"
 #include "audioconvert.h"
-#include "gst/floatcast/floatcast.h"
 #include "gstaudioconvertorc.h"
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
@@ -41,6 +40,25 @@
 #define orc_audio_convert_unpack_u32_be orc_audio_convert_unpack_u32_swap
 #define orc_audio_convert_unpack_s32_le orc_audio_convert_unpack_s32
 #define orc_audio_convert_unpack_s32_be orc_audio_convert_unpack_s32_swap
+#define orc_audio_convert_unpack_float_le orc_audio_convert_unpack_float_s32
+#define orc_audio_convert_unpack_float_be orc_audio_convert_unpack_float_s32_swap
+#define orc_audio_convert_unpack_double_le orc_audio_convert_unpack_double_s32
+#define orc_audio_convert_unpack_double_be orc_audio_convert_unpack_double_s32_swap
+#define orc_audio_convert_unpack_float_hq_le orc_audio_convert_unpack_float_double
+#define orc_audio_convert_unpack_float_hq_be orc_audio_convert_unpack_float_double_swap
+#define orc_audio_convert_unpack_double_hq_le orc_audio_convert_unpack_double_double
+#define orc_audio_convert_unpack_double_hq_be orc_audio_convert_unpack_double_double_swap
+#define orc_audio_convert_unpack_u8_float orc_audio_convert_unpack_u8_double
+#define orc_audio_convert_unpack_u16_le_float orc_audio_convert_unpack_u16_double
+#define orc_audio_convert_unpack_u16_be_float orc_audio_convert_unpack_u16_double_swap
+#define orc_audio_convert_unpack_u32_le_float orc_audio_convert_unpack_u32_double
+#define orc_audio_convert_unpack_u32_be_float orc_audio_convert_unpack_u32_double_swap
+#define orc_audio_convert_unpack_s8_float orc_audio_convert_unpack_s8_double
+#define orc_audio_convert_unpack_s16_le_float orc_audio_convert_unpack_s16_double
+#define orc_audio_convert_unpack_s16_be_float orc_audio_convert_unpack_s16_double_swap
+#define orc_audio_convert_unpack_s32_le_float orc_audio_convert_unpack_s32_double
+#define orc_audio_convert_unpack_s32_be_float orc_audio_convert_unpack_s32_double_swap
+#define orc_audio_convert_pack_u8 orc_audio_convert_pack_u8
 #define orc_audio_convert_pack_u16_le orc_audio_convert_pack_u16
 #define orc_audio_convert_pack_u16_be orc_audio_convert_pack_u16_swap
 #define orc_audio_convert_pack_s16_le orc_audio_convert_pack_s16
@@ -49,6 +67,22 @@
 #define orc_audio_convert_pack_u32_be orc_audio_convert_pack_u32_swap
 #define orc_audio_convert_pack_s32_le orc_audio_convert_pack_s32
 #define orc_audio_convert_pack_s32_be orc_audio_convert_pack_s32_swap
+#define orc_audio_convert_pack_float_le orc_audio_convert_pack_s32_float
+#define orc_audio_convert_pack_float_be orc_audio_convert_pack_s32_float_swap
+#define orc_audio_convert_pack_double_le orc_audio_convert_pack_s32_double
+#define orc_audio_convert_pack_double_be orc_audio_convert_pack_s32_double_swap
+#define orc_audio_convert_pack_float_hq_le orc_audio_convert_pack_double_float
+#define orc_audio_convert_pack_float_hq_be orc_audio_convert_pack_double_float_swap
+#define orc_audio_convert_pack_s8_float orc_audio_convert_pack_double_s8
+#define orc_audio_convert_pack_s16_le_float orc_audio_convert_pack_double_s16
+#define orc_audio_convert_pack_s16_be_float orc_audio_convert_pack_double_s16_swap
+#define orc_audio_convert_pack_s32_le_float orc_audio_convert_pack_double_s32
+#define orc_audio_convert_pack_s32_be_float orc_audio_convert_pack_double_s32_swap
+#define orc_audio_convert_pack_u8_float orc_audio_convert_pack_double_u8
+#define orc_audio_convert_pack_u16_le_float orc_audio_convert_pack_double_u16
+#define orc_audio_convert_pack_u16_be_float orc_audio_convert_pack_double_u16_swap
+#define orc_audio_convert_pack_u32_le_float orc_audio_convert_pack_double_u32
+#define orc_audio_convert_pack_u32_be_float orc_audio_convert_pack_double_u32_swap
 #else
 #define orc_audio_convert_unpack_u16_be orc_audio_convert_unpack_u16
 #define orc_audio_convert_unpack_u16_le orc_audio_convert_unpack_u16_swap
@@ -58,6 +92,25 @@
 #define orc_audio_convert_unpack_u32_le orc_audio_convert_unpack_u32_swap
 #define orc_audio_convert_unpack_s32_be orc_audio_convert_unpack_s32
 #define orc_audio_convert_unpack_s32_le orc_audio_convert_unpack_s32_swap
+#define orc_audio_convert_unpack_float_be orc_audio_convert_unpack_float_s32
+#define orc_audio_convert_unpack_float_le orc_audio_convert_unpack_float_s32_swap
+#define orc_audio_convert_unpack_double_be orc_audio_convert_unpack_double_s32
+#define orc_audio_convert_unpack_double_le orc_audio_convert_unpack_double_s32_swap
+#define orc_audio_convert_unpack_float_hq_be orc_audio_convert_unpack_float_double
+#define orc_audio_convert_unpack_float_hq_le orc_audio_convert_unpack_float_double_swap
+#define orc_audio_convert_unpack_double_hq_be orc_audio_convert_unpack_double_double
+#define orc_audio_convert_unpack_double_hq_le orc_audio_convert_unpack_double_double_swap
+#define orc_audio_convert_unpack_u8_float orc_audio_convert_unpack_u8_double
+#define orc_audio_convert_unpack_u16_be_float orc_audio_convert_unpack_u16_double
+#define orc_audio_convert_unpack_u16_le_float orc_audio_convert_unpack_u16_double_swap
+#define orc_audio_convert_unpack_u32_be_float orc_audio_convert_unpack_u32_double
+#define orc_audio_convert_unpack_u32_le_float orc_audio_convert_unpack_u32_double_swap
+#define orc_audio_convert_unpack_s8_float orc_audio_convert_unpack_s8_double
+#define orc_audio_convert_unpack_s16_be_float orc_audio_convert_unpack_s16_double
+#define orc_audio_convert_unpack_s16_le_float orc_audio_convert_unpack_s16_double_swap
+#define orc_audio_convert_unpack_s32_be_float orc_audio_convert_unpack_s32_double
+#define orc_audio_convert_unpack_s32_le_float orc_audio_convert_unpack_s32_double_swap
+#define orc_audio_convert_pack_u8 orc_audio_convert_pack_u8
 #define orc_audio_convert_pack_u16_be orc_audio_convert_pack_u16
 #define orc_audio_convert_pack_u16_le orc_audio_convert_pack_u16_swap
 #define orc_audio_convert_pack_s16_be orc_audio_convert_pack_s16
@@ -66,6 +119,22 @@
 #define orc_audio_convert_pack_u32_le orc_audio_convert_pack_u32_swap
 #define orc_audio_convert_pack_s32_be orc_audio_convert_pack_s32
 #define orc_audio_convert_pack_s32_le orc_audio_convert_pack_s32_swap
+#define orc_audio_convert_pack_float_be orc_audio_convert_pack_s32_float
+#define orc_audio_convert_pack_float_le orc_audio_convert_pack_s32_float_swap
+#define orc_audio_convert_pack_double_be orc_audio_convert_pack_s32_double
+#define orc_audio_convert_pack_double_le orc_audio_convert_pack_s32_double_swap
+#define orc_audio_convert_pack_float_hq_be orc_audio_convert_pack_double_float
+#define orc_audio_convert_pack_float_hq_le orc_audio_convert_pack_double_float_swap
+#define orc_audio_convert_pack_s8_float orc_audio_convert_pack_double_s8
+#define orc_audio_convert_pack_s16_be_float orc_audio_convert_pack_double_s16
+#define orc_audio_convert_pack_s16_le_float orc_audio_convert_pack_double_s16_swap
+#define orc_audio_convert_pack_s32_be_float orc_audio_convert_pack_double_s32
+#define orc_audio_convert_pack_s32_le_float orc_audio_convert_pack_double_s32_swap
+#define orc_audio_convert_pack_u8_float orc_audio_convert_pack_double_u8
+#define orc_audio_convert_pack_u16_be_float orc_audio_convert_pack_double_u16
+#define orc_audio_convert_pack_u16_le_float orc_audio_convert_pack_double_u16_swap
+#define orc_audio_convert_pack_u32_be_float orc_audio_convert_pack_double_u32
+#define orc_audio_convert_pack_u32_le_float orc_audio_convert_pack_double_u32_swap
 #endif
 
 /* sign bit in the intermediate format */
@@ -114,6 +183,14 @@ MAKE_UNPACK_FUNC_NAME (name) (type * src, gint32 * dst, gint s, gint count)   \
   }                                                                           \
 }
 
+/* unpack from float to signed integer 32 with orc */
+#define MAKE_UNPACK_FUNC_ORC_FI(name, type, READ_FUNC)                        \
+static void                                                                   \
+MAKE_UNPACK_FUNC_NAME (name) (type * src, gint32 * dst, gint s, gint count)   \
+{                                                                             \
+  MAKE_ORC_UNPACK_FUNC_NAME (name) ((guint32 *) dst, src, count);                         \
+}
+
 /* unpack from float to float 64 (double) */
 #define MAKE_UNPACK_FUNC_FF(name, type, FUNC)                                 \
 static void                                                                   \
@@ -124,6 +201,15 @@ MAKE_UNPACK_FUNC_NAME (name) (type * src, gdouble * dst, gint s,              \
     *dst++ = (gdouble) FUNC (*src++);                                         \
 }
 
+/* unpack from float to float 64 (double) with orc */
+#define MAKE_UNPACK_FUNC_ORC_FF(name, type, FUNC)                                 \
+static void                                                                   \
+MAKE_UNPACK_FUNC_NAME (name) (type * src, gdouble * dst, gint s,              \
+    gint count)                                                               \
+{                                                                             \
+  MAKE_ORC_UNPACK_FUNC_NAME (name) ((gdouble *) dst, src, count);             \
+}
+
 /* unpack from int to float 64 (double) */
 #define MAKE_UNPACK_FUNC_IF(name, stride, sign, READ_FUNC)                    \
 static void                                                                   \
@@ -138,6 +224,14 @@ MAKE_UNPACK_FUNC_NAME (name) (guint8 * src, gdouble * dst, gint scale,        \
   }                                                                           \
 }
 
+#define MAKE_UNPACK_FUNC_ORC_IF(name, stride, sign, READ_FUNC)          \
+static void                                                             \
+MAKE_UNPACK_FUNC_NAME (name) (guint8 *src, gdouble *dst,                \
+        gint scale, gint count)                                         \
+{                                                                       \
+  MAKE_ORC_UNPACK_FUNC_NAME (name) (dst, src, scale, count);            \
+}
+
 #define READ8(p)          GST_READ_UINT8(p)
 #define READ16_FROM_LE(p) GST_READ_UINT16_LE (p)
 #define READ16_FROM_BE(p) GST_READ_UINT16_BE (p)
@@ -162,28 +256,28 @@ MAKE_UNPACK_FUNC_ORC_II (u32_le, 4, SIGNED, READ32_FROM_LE);
 MAKE_UNPACK_FUNC_ORC_II (s32_le, 4, 0, READ32_FROM_LE);
 MAKE_UNPACK_FUNC_ORC_II (u32_be, 4, SIGNED, READ32_FROM_BE);
 MAKE_UNPACK_FUNC_ORC_II (s32_be, 4, 0, READ32_FROM_BE);
-MAKE_UNPACK_FUNC_FI (float_le, gfloat, GFLOAT_FROM_LE);
-MAKE_UNPACK_FUNC_FI (float_be, gfloat, GFLOAT_FROM_BE);
-MAKE_UNPACK_FUNC_FI (double_le, gdouble, GDOUBLE_FROM_LE);
-MAKE_UNPACK_FUNC_FI (double_be, gdouble, GDOUBLE_FROM_BE);
-MAKE_UNPACK_FUNC_FF (float_hq_le, gfloat, GFLOAT_FROM_LE);
-MAKE_UNPACK_FUNC_FF (float_hq_be, gfloat, GFLOAT_FROM_BE);
-MAKE_UNPACK_FUNC_FF (double_hq_le, gdouble, GDOUBLE_FROM_LE);
-MAKE_UNPACK_FUNC_FF (double_hq_be, gdouble, GDOUBLE_FROM_BE);
-MAKE_UNPACK_FUNC_IF (u8_float, 1, SIGNED, READ8);
-MAKE_UNPACK_FUNC_IF (s8_float, 1, 0, READ8);
-MAKE_UNPACK_FUNC_IF (u16_le_float, 2, SIGNED, READ16_FROM_LE);
-MAKE_UNPACK_FUNC_IF (s16_le_float, 2, 0, READ16_FROM_LE);
-MAKE_UNPACK_FUNC_IF (u16_be_float, 2, SIGNED, READ16_FROM_BE);
-MAKE_UNPACK_FUNC_IF (s16_be_float, 2, 0, READ16_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_FI (float_le, gfloat, GFLOAT_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_FI (float_be, gfloat, GFLOAT_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_FI (double_le, gdouble, GDOUBLE_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_FI (double_be, gdouble, GDOUBLE_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_FF (float_hq_le, gfloat, GFLOAT_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_FF (float_hq_be, gfloat, GFLOAT_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_FF (double_hq_le, gdouble, GDOUBLE_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_FF (double_hq_be, gdouble, GDOUBLE_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_IF (u8_float, 1, SIGNED, READ8);
+MAKE_UNPACK_FUNC_ORC_IF (s8_float, 1, 0, READ8);
+MAKE_UNPACK_FUNC_ORC_IF (u16_le_float, 2, SIGNED, READ16_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_IF (s16_le_float, 2, 0, READ16_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_IF (u16_be_float, 2, SIGNED, READ16_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_IF (s16_be_float, 2, 0, READ16_FROM_BE);
 MAKE_UNPACK_FUNC_IF (u24_le_float, 3, SIGNED, READ24_FROM_LE);
 MAKE_UNPACK_FUNC_IF (s24_le_float, 3, 0, READ24_FROM_LE);
 MAKE_UNPACK_FUNC_IF (u24_be_float, 3, SIGNED, READ24_FROM_BE);
 MAKE_UNPACK_FUNC_IF (s24_be_float, 3, 0, READ24_FROM_BE);
-MAKE_UNPACK_FUNC_IF (u32_le_float, 4, SIGNED, READ32_FROM_LE);
-MAKE_UNPACK_FUNC_IF (s32_le_float, 4, 0, READ32_FROM_LE);
-MAKE_UNPACK_FUNC_IF (u32_be_float, 4, SIGNED, READ32_FROM_BE);
-MAKE_UNPACK_FUNC_IF (s32_be_float, 4, 0, READ32_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_IF (u32_le_float, 4, SIGNED, READ32_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_IF (s32_le_float, 4, 0, READ32_FROM_LE);
+MAKE_UNPACK_FUNC_ORC_IF (u32_be_float, 4, SIGNED, READ32_FROM_BE);
+MAKE_UNPACK_FUNC_ORC_IF (s32_be_float, 4, 0, READ32_FROM_BE);
 
 /* One of the double_hq_* functions generated above is ineffecient, but it's
  * never used anyway.  The same is true for one of the s32_* functions. */
@@ -231,6 +325,15 @@ MAKE_PACK_FUNC_NAME (name) (gint32 *src, guint8 * dst,                  \
   }                                                                     \
 }
 
+/* pack from signed integer 32 to float using orc */
+#define MAKE_PACK_FUNC_ORC_IF(name, type, FUNC)                         \
+static void                                                             \
+MAKE_PACK_FUNC_NAME (name) (gint32 * src, type * dst, gint scale,       \
+    gint count)                                                         \
+{                                                                       \
+  MAKE_PACK_FUNC_NAME_ORC (name) (dst, src, count);                     \
+}
+
 /* pack from signed integer 32 to float */
 #define MAKE_PACK_FUNC_IF(name, type, FUNC)                             \
 static void                                                             \
@@ -251,6 +354,15 @@ MAKE_PACK_FUNC_NAME (name) (gdouble * src, type * dst, gint s,          \
     *dst++ = FUNC ((type) (*src++));                                    \
 }
 
+/* pack from float 64 (double) to float with orc */
+#define MAKE_PACK_FUNC_ORC_FF(name, type, FUNC)                             \
+static void                                                             \
+MAKE_PACK_FUNC_NAME (name) (gdouble * src, type * dst, gint s,          \
+    gint count)                                                         \
+{                                                                       \
+  MAKE_PACK_FUNC_NAME_ORC (name) (dst, src, count);                     \
+}
+
 /* pack from float 64 (double) to signed int.
  * the floats are already in the correct range. Only a cast is needed.
  */
@@ -287,6 +399,28 @@ MAKE_PACK_FUNC_NAME (name) (gdouble * src, guint8 * dst, gint scale,    \
   }                                                                     \
 }
 
+/* pack from float 64 (double) to signed int with orc.
+ * the floats are already in the correct range. Only a cast is needed.
+ */
+#define MAKE_PACK_FUNC_ORC_FI_S(name, stride, WRITE_FUNC)               \
+static void                                                             \
+MAKE_PACK_FUNC_NAME (name) (gdouble * src, guint8 * dst, gint scale,    \
+    gint count)                                                         \
+{                                                                       \
+  MAKE_PACK_FUNC_NAME_ORC (name) (dst, src, scale, count);              \
+}
+
+/* pack from float 64 (double) to unsigned int with orc.
+ * the floats are already in the correct range. Only a cast is needed.
+ */
+#define MAKE_PACK_FUNC_ORC_FI_U(name, stride, WRITE_FUNC)               \
+static void                                                             \
+MAKE_PACK_FUNC_NAME (name) (gdouble * src, guint8 * dst, gint scale,    \
+    gint count)                                                         \
+{                                                                       \
+  MAKE_PACK_FUNC_NAME_ORC (name) (dst, src, scale, count);              \
+}
+
 #define WRITE8(p, v)       GST_WRITE_UINT8 (p, v)
 #define WRITE16_TO_LE(p,v) GST_WRITE_UINT16_LE (p, (guint16)(v))
 #define WRITE16_TO_BE(p,v) GST_WRITE_UINT16_BE (p, (guint16)(v))
@@ -309,26 +443,26 @@ MAKE_PACK_FUNC_ORC_II (u32_le, 4, SIGNED, WRITE32_TO_LE);
 MAKE_PACK_FUNC_ORC_II (s32_le, 4, 0, WRITE32_TO_LE);
 MAKE_PACK_FUNC_ORC_II (u32_be, 4, SIGNED, WRITE32_TO_BE);
 MAKE_PACK_FUNC_ORC_II (s32_be, 4, 0, WRITE32_TO_BE);
-MAKE_PACK_FUNC_IF (float_le, gfloat, GFLOAT_TO_LE);
-MAKE_PACK_FUNC_IF (float_be, gfloat, GFLOAT_TO_BE);
-MAKE_PACK_FUNC_IF (double_le, gdouble, GDOUBLE_TO_LE);
-MAKE_PACK_FUNC_IF (double_be, gdouble, GDOUBLE_TO_BE);
-MAKE_PACK_FUNC_FF (float_hq_le, gfloat, GFLOAT_TO_LE);
-MAKE_PACK_FUNC_FF (float_hq_be, gfloat, GFLOAT_TO_BE);
-MAKE_PACK_FUNC_FI_U (u8_float, 1, WRITE8);
-MAKE_PACK_FUNC_FI_S (s8_float, 1, WRITE8);
-MAKE_PACK_FUNC_FI_U (u16_le_float, 2, WRITE16_TO_LE);
-MAKE_PACK_FUNC_FI_S (s16_le_float, 2, WRITE16_TO_LE);
-MAKE_PACK_FUNC_FI_U (u16_be_float, 2, WRITE16_TO_BE);
-MAKE_PACK_FUNC_FI_S (s16_be_float, 2, WRITE16_TO_BE);
+MAKE_PACK_FUNC_ORC_IF (float_le, gfloat, GFLOAT_TO_LE);
+MAKE_PACK_FUNC_ORC_IF (float_be, gfloat, GFLOAT_TO_BE);
+MAKE_PACK_FUNC_ORC_IF (double_le, gdouble, GDOUBLE_TO_LE);
+MAKE_PACK_FUNC_ORC_IF (double_be, gdouble, GDOUBLE_TO_BE);
+MAKE_PACK_FUNC_ORC_FF (float_hq_le, gfloat, GFLOAT_TO_LE);
+MAKE_PACK_FUNC_ORC_FF (float_hq_be, gfloat, GFLOAT_TO_BE);
+MAKE_PACK_FUNC_ORC_FI_U (u8_float, 1, WRITE8);
+MAKE_PACK_FUNC_ORC_FI_S (s8_float, 1, WRITE8);
+MAKE_PACK_FUNC_ORC_FI_U (u16_le_float, 2, WRITE16_TO_LE);
+MAKE_PACK_FUNC_ORC_FI_S (s16_le_float, 2, WRITE16_TO_LE);
+MAKE_PACK_FUNC_ORC_FI_U (u16_be_float, 2, WRITE16_TO_BE);
+MAKE_PACK_FUNC_ORC_FI_S (s16_be_float, 2, WRITE16_TO_BE);
 MAKE_PACK_FUNC_FI_U (u24_le_float, 3, WRITE24_TO_LE);
 MAKE_PACK_FUNC_FI_S (s24_le_float, 3, WRITE24_TO_LE);
 MAKE_PACK_FUNC_FI_U (u24_be_float, 3, WRITE24_TO_BE);
 MAKE_PACK_FUNC_FI_S (s24_be_float, 3, WRITE24_TO_BE);
-MAKE_PACK_FUNC_FI_U (u32_le_float, 4, WRITE32_TO_LE);
-MAKE_PACK_FUNC_FI_S (s32_le_float, 4, WRITE32_TO_LE);
-MAKE_PACK_FUNC_FI_U (u32_be_float, 4, WRITE32_TO_BE);
-MAKE_PACK_FUNC_FI_S (s32_be_float, 4, WRITE32_TO_BE);
+MAKE_PACK_FUNC_ORC_FI_U (u32_le_float, 4, WRITE32_TO_LE);
+MAKE_PACK_FUNC_ORC_FI_S (s32_le_float, 4, WRITE32_TO_LE);
+MAKE_PACK_FUNC_ORC_FI_U (u32_be_float, 4, WRITE32_TO_BE);
+MAKE_PACK_FUNC_ORC_FI_S (s32_be_float, 4, WRITE32_TO_BE);
 
 /* For double_hq, packing and unpacking is the same, so we reuse the unpacking
  * functions here. */
@@ -421,24 +555,27 @@ static AudioConvertPack pack_funcs[] = {
   (AudioConvertPack) MAKE_PACK_FUNC_NAME (s32_be_float),
 };
 
-#define DOUBLE_INTERMEDIATE_FORMAT(ctx)                                 \
-    ((!ctx->in.is_int && !ctx->out.is_int) || (ctx->ns != NOISE_SHAPING_NONE))
+#define DOUBLE_INTERMEDIATE_FORMAT(ctx)                   \
+    ((!GST_AUDIO_FORMAT_INFO_IS_INTEGER (ctx->in.finfo) &&    \
+      !GST_AUDIO_FORMAT_INFO_IS_INTEGER (ctx->out.finfo)) ||  \
+     (ctx->ns != NOISE_SHAPING_NONE))
 
 static gint
-audio_convert_get_func_index (AudioConvertCtx * ctx, AudioConvertFmt * fmt)
+audio_convert_get_func_index (AudioConvertCtx * ctx,
+    const GstAudioFormatInfo * fmt)
 {
   gint index = 0;
 
-  if (fmt->is_int) {
-    index += (fmt->width / 8 - 1) * 4;
-    index += fmt->endianness == G_LITTLE_ENDIAN ? 0 : 2;
-    index += fmt->sign ? 1 : 0;
+  if (GST_AUDIO_FORMAT_INFO_IS_INTEGER (fmt)) {
+    index += (GST_AUDIO_FORMAT_INFO_WIDTH (fmt) / 8 - 1) * 4;
+    index += GST_AUDIO_FORMAT_INFO_IS_LITTLE_ENDIAN (fmt) ? 0 : 2;
+    index += GST_AUDIO_FORMAT_INFO_IS_SIGNED (fmt) ? 1 : 0;
     index += (ctx->ns == NOISE_SHAPING_NONE) ? 0 : 24;
   } else {
     /* this is float/double */
     index = 16;
-    index += (fmt->width == 32) ? 0 : 2;
-    index += (fmt->endianness == G_LITTLE_ENDIAN) ? 0 : 1;
+    index += (GST_AUDIO_FORMAT_INFO_WIDTH (fmt) == 32) ? 0 : 2;
+    index += GST_AUDIO_FORMAT_INFO_IS_LITTLE_ENDIAN (fmt) ? 0 : 1;
     index += (DOUBLE_INTERMEDIATE_FORMAT (ctx)) ? 4 : 0;
   }
 
@@ -446,34 +583,22 @@ audio_convert_get_func_index (AudioConvertCtx * ctx, AudioConvertFmt * fmt)
 }
 
 static inline gboolean
-check_default (AudioConvertCtx * ctx, AudioConvertFmt * fmt)
+check_default (AudioConvertCtx * ctx, const GstAudioFormatInfo * fmt)
 {
   if (!DOUBLE_INTERMEDIATE_FORMAT (ctx)) {
-    return (fmt->width == 32 && fmt->depth == 32 &&
-        fmt->endianness == G_BYTE_ORDER && fmt->sign == TRUE);
+    return GST_AUDIO_FORMAT_INFO_FORMAT (fmt) == GST_AUDIO_FORMAT_S32;
   } else {
-    return (fmt->width == 64 && fmt->endianness == G_BYTE_ORDER);
+    return GST_AUDIO_FORMAT_INFO_FORMAT (fmt) == GST_AUDIO_FORMAT_F64;
   }
 }
 
 gboolean
-audio_convert_clean_fmt (AudioConvertFmt * fmt)
-{
-  g_return_val_if_fail (fmt != NULL, FALSE);
-
-  g_free (fmt->pos);
-  fmt->pos = NULL;
-
-  return TRUE;
-}
-
-
-gboolean
-audio_convert_prepare_context (AudioConvertCtx * ctx, AudioConvertFmt * in,
-    AudioConvertFmt * out, GstAudioConvertDithering dither,
+audio_convert_prepare_context (AudioConvertCtx * ctx, GstAudioInfo * in,
+    GstAudioInfo * out, GstAudioConvertDithering dither,
     GstAudioConvertNoiseShaping ns)
 {
   gint idx_in, idx_out;
+  gint in_depth, out_depth;
 
   g_return_val_if_fail (ctx != NULL, FALSE);
   g_return_val_if_fail (in != NULL, FALSE);
@@ -482,18 +607,18 @@ audio_convert_prepare_context (AudioConvertCtx * ctx, AudioConvertFmt * in,
   /* first clean the existing context */
   audio_convert_clean_context (ctx);
 
-  g_return_val_if_fail (in->unpositioned_layout == out->unpositioned_layout,
-      FALSE);
-
   ctx->in = *in;
   ctx->out = *out;
 
+  in_depth = GST_AUDIO_FORMAT_INFO_DEPTH (in->finfo);
+  out_depth = GST_AUDIO_FORMAT_INFO_DEPTH (out->finfo);
+
   /* Don't dither or apply noise shaping if target depth is bigger than 20 bits
    * as DA converters only can do a SNR up to 20 bits in reality.
    * Also don't dither or apply noise shaping if target depth is larger than
    * source depth. */
-  if (ctx->out.depth <= 20 && (!ctx->in.is_int
-          || ctx->in.depth >= ctx->out.depth)) {
+  if (out_depth <= 20 && (!GST_AUDIO_FORMAT_INFO_IS_INTEGER (in->finfo)
+          || in_depth >= out_depth)) {
     ctx->dither = dither;
     ctx->ns = ns;
   } else {
@@ -503,15 +628,15 @@ audio_convert_prepare_context (AudioConvertCtx * ctx, AudioConvertFmt * in,
 
   /* Use simple error feedback when output sample rate is smaller than
    * 32000 as the other methods might move the noise to audible ranges */
-  if (ctx->ns > NOISE_SHAPING_ERROR_FEEDBACK && ctx->out.rate < 32000)
+  if (ctx->ns > NOISE_SHAPING_ERROR_FEEDBACK && out->rate < 32000)
     ctx->ns = NOISE_SHAPING_ERROR_FEEDBACK;
 
   gst_channel_mix_setup_matrix (ctx);
 
-  idx_in = audio_convert_get_func_index (ctx, in);
+  idx_in = audio_convert_get_func_index (ctx, in->finfo);
   ctx->unpack = unpack_funcs[idx_in];
 
-  idx_out = audio_convert_get_func_index (ctx, out);
+  idx_out = audio_convert_get_func_index (ctx, out->finfo);
   ctx->pack = pack_funcs[idx_out];
 
   /* if both formats are float/double or we use noise shaping use double as
@@ -523,20 +648,22 @@ audio_convert_prepare_context (AudioConvertCtx * ctx, AudioConvertFmt * in,
     GST_INFO ("use float mixing");
     ctx->channel_mix = (AudioConvertMix) gst_channel_mix_mix_float;
   }
-  GST_INFO ("unitsizes: %d -> %d", in->unit_size, out->unit_size);
+  GST_INFO ("unitsizes: %d -> %d", in->bpf, out->bpf);
 
   /* check if input is in default format */
-  ctx->in_default = check_default (ctx, in);
+  ctx->in_default = check_default (ctx, in->finfo);
   /* check if channel mixer is passthrough */
   ctx->mix_passthrough = gst_channel_mix_passthrough (ctx);
   /* check if output is in default format */
-  ctx->out_default = check_default (ctx, out);
+  ctx->out_default = check_default (ctx, out->finfo);
 
   GST_INFO ("in default %d, mix passthrough %d, out default %d",
       ctx->in_default, ctx->mix_passthrough, ctx->out_default);
 
-  ctx->in_scale = (in->is_int) ? (32 - in->depth) : 0;
-  ctx->out_scale = (out->is_int) ? (32 - out->depth) : 0;
+  ctx->in_scale =
+      GST_AUDIO_FORMAT_INFO_IS_INTEGER (in->finfo) ? (32 - in_depth) : 0;
+  ctx->out_scale =
+      GST_AUDIO_FORMAT_INFO_IS_INTEGER (out->finfo) ? (32 - out_depth) : 0;
 
   gst_audio_quantize_setup (ctx);
 
@@ -549,8 +676,8 @@ audio_convert_clean_context (AudioConvertCtx * ctx)
   g_return_val_if_fail (ctx != NULL, FALSE);
 
   gst_audio_quantize_free (ctx);
-  audio_convert_clean_fmt (&ctx->in);
-  audio_convert_clean_fmt (&ctx->out);
+  gst_audio_info_init (&ctx->in);
+  gst_audio_info_init (&ctx->out);
   gst_channel_mix_unset_matrix (ctx);
 
   g_free (ctx->tmpbuf);
@@ -567,9 +694,9 @@ audio_convert_get_sizes (AudioConvertCtx * ctx, gint samples, gint * srcsize,
   g_return_val_if_fail (ctx != NULL, FALSE);
 
   if (srcsize)
-    *srcsize = samples * ctx->in.unit_size;
+    *srcsize = samples * ctx->in.bpf;
   if (dstsize)
-    *dstsize = samples * ctx->out.unit_size;
+    *dstsize = samples * ctx->out.bpf;
 
   return TRUE;
 }
@@ -581,6 +708,7 @@ audio_convert_convert (AudioConvertCtx * ctx, gpointer src,
   guint insize, outsize, size;
   gpointer outbuf, tmpbuf;
   guint intemp = 0, outtemp = 0, biggest;
+  gint in_width, out_width;
 
   g_return_val_if_fail (ctx != NULL, FALSE);
   g_return_val_if_fail (src != NULL, FALSE);
@@ -590,23 +718,26 @@ audio_convert_convert (AudioConvertCtx * ctx, gpointer src,
   if (samples == 0)
     return TRUE;
 
-  insize = ctx->in.unit_size * samples;
-  outsize = ctx->out.unit_size * samples;
+  insize = ctx->in.bpf * samples;
+  outsize = ctx->out.bpf * samples;
+
+  in_width = GST_AUDIO_FORMAT_INFO_WIDTH (ctx->in.finfo);
+  out_width = GST_AUDIO_FORMAT_INFO_WIDTH (ctx->out.finfo);
 
   /* find biggest temp buffer size */
   size = (DOUBLE_INTERMEDIATE_FORMAT (ctx)) ? sizeof (gdouble)
       : sizeof (gint32);
 
   if (!ctx->in_default)
-    intemp = gst_util_uint64_scale (insize, size * 8, ctx->in.width);
+    intemp = gst_util_uint64_scale (insize, size * 8, in_width);
   if (!ctx->mix_passthrough || !ctx->out_default)
-    outtemp = gst_util_uint64_scale (outsize, size * 8, ctx->out.width);
+    outtemp = gst_util_uint64_scale (outsize, size * 8, out_width);
   biggest = MAX (intemp, outtemp);
 
   /* see if one of the buffers can be used as temp */
-  if ((outsize >= biggest) && (ctx->out.unit_size <= size))
+  if ((outsize >= biggest) && (ctx->out.bpf <= size))
     tmpbuf = dst;
-  else if ((insize >= biggest) && src_writable && (ctx->in.unit_size >= size))
+  else if ((insize >= biggest) && src_writable && (ctx->in.bpf >= size))
     tmpbuf = src;
   else {
     if (biggest > ctx->tmpbufsize) {
@@ -644,7 +775,7 @@ audio_convert_convert (AudioConvertCtx * ctx, gpointer src,
   }
 
   /* we only need to quantize if output format is int */
-  if (ctx->out.is_int) {
+  if (GST_AUDIO_FORMAT_INFO_IS_INTEGER (ctx->out.finfo)) {
     if (ctx->out_default)
       outbuf = dst;
     else
