@@ -78,6 +78,7 @@ enum
 
 #define DEFAULT_SILENT     TRUE
 #define DEFAULT_TOLERANCE  0
+#define DEFAULT_SKIP_TO_FIRST FALSE
 
 enum
 {
@@ -88,28 +89,23 @@ enum
   ARG_DROP,
   ARG_SILENT,
   ARG_TOLERANCE,
-  /* FILL ME */
+  ARG_SKIP_TO_FIRST
 };
 
 static GstStaticPadTemplate gst_audio_rate_src_template =
-    GST_STATIC_PAD_TEMPLATE ("src",
+GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS ";"
-        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS (GST_AUDIO_CAPS_MAKE (GST_AUDIO_FORMATS_ALL))
     );
 
 static GstStaticPadTemplate gst_audio_rate_sink_template =
-    GST_STATIC_PAD_TEMPLATE ("sink",
+GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS ";"
-        GST_AUDIO_FLOAT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS (GST_AUDIO_CAPS_MAKE (GST_AUDIO_FORMATS_ALL))
     );
 
-static void gst_audio_rate_base_init (gpointer g_class);
-static void gst_audio_rate_class_init (GstAudioRateClass * klass);
-static void gst_audio_rate_init (GstAudioRate * audiorate);
 static gboolean gst_audio_rate_sink_event (GstPad * pad, GstEvent * event);
 static gboolean gst_audio_rate_src_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_audio_rate_chain (GstPad * pad, GstBuffer * buf);
@@ -122,50 +118,13 @@ static void gst_audio_rate_get_property (GObject * object,
 static GstStateChangeReturn gst_audio_rate_change_state (GstElement * element,
     GstStateChange transition);
 
-static GstElementClass *parent_class = NULL;
-
 /*static guint gst_audio_rate_signals[LAST_SIGNAL] = { 0 }; */
 
-static GType
-gst_audio_rate_get_type (void)
-{
-  static GType audio_rate_type = 0;
+static GParamSpec *pspec_drop = NULL;
+static GParamSpec *pspec_add = NULL;
 
-  if (!audio_rate_type) {
-    static const GTypeInfo audio_rate_info = {
-      sizeof (GstAudioRateClass),
-      gst_audio_rate_base_init,
-      NULL,
-      (GClassInitFunc) gst_audio_rate_class_init,
-      NULL,
-      NULL,
-      sizeof (GstAudioRate),
-      0,
-      (GInstanceInitFunc) gst_audio_rate_init,
-    };
-
-    audio_rate_type = g_type_register_static (GST_TYPE_ELEMENT,
-        "GstAudioRate", &audio_rate_info, 0);
-  }
-
-  return audio_rate_type;
-}
-
-static void
-gst_audio_rate_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_set_details_simple (element_class,
-      "Audio rate adjuster", "Filter/Effect/Audio",
-      "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
-      "Wim Taymans <wim@fluendo.com>");
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_rate_sink_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_audio_rate_src_template));
-}
+#define gst_audio_rate_parent_class parent_class
+G_DEFINE_TYPE (GstAudioRate, gst_audio_rate, GST_TYPE_ELEMENT);
 
 static void
 gst_audio_rate_class_init (GstAudioRateClass * klass)
@@ -173,8 +132,6 @@ gst_audio_rate_class_init (GstAudioRateClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
-
   object_class->set_property = gst_audio_rate_set_property;
   object_class->get_property = gst_audio_rate_get_property;
 
@@ -185,12 +142,12 @@ gst_audio_rate_class_init (GstAudioRateClass * klass)
   g_object_class_install_property (object_class, ARG_OUT,
       g_param_spec_uint64 ("out", "Out", "Number of output samples", 0,
           G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_ADD,
-      g_param_spec_uint64 ("add", "Add", "Number of added samples", 0,
-          G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (object_class, ARG_DROP,
-      g_param_spec_uint64 ("drop", "Drop", "Number of dropped samples", 0,
-          G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+  pspec_add = g_param_spec_uint64 ("add", "Add", "Number of added samples",
+      0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_ADD, pspec_add);
+  pspec_drop = g_param_spec_uint64 ("drop", "Drop", "Number of dropped samples",
+      0, G_MAXUINT64, 0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, ARG_DROP, pspec_drop);
   g_object_class_install_property (object_class, ARG_SILENT,
       g_param_spec_boolean ("silent", "silent",
           "Don't emit notify for dropped and duplicated frames", DEFAULT_SILENT,
@@ -209,6 +166,28 @@ gst_audio_rate_class_init (GstAudioRateClass * klass)
           0, G_MAXUINT64, DEFAULT_TOLERANCE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstAudioRate:skip-to-first:
+   *
+   * Don't produce buffers before the first one we receive.
+   *
+   * Since: 0.10.33
+   **/
+  g_object_class_install_property (object_class, ARG_SKIP_TO_FIRST,
+      g_param_spec_boolean ("skip-to-first", "Skip to first buffer",
+          "Don't produce buffers before the first one we receive",
+          DEFAULT_SKIP_TO_FIRST, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_element_class_set_details_simple (element_class,
+      "Audio rate adjuster", "Filter/Effect/Audio",
+      "Drops/duplicates/adjusts timestamps on audio samples to make a perfect stream",
+      "Wim Taymans <wim@fluendo.com>");
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_audio_rate_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_audio_rate_src_template));
+
   element_class->change_state = gst_audio_rate_change_state;
 }
 
@@ -225,51 +204,22 @@ gst_audio_rate_reset (GstAudioRate * audiorate)
 }
 
 static gboolean
-gst_audio_rate_setcaps (GstPad * pad, GstCaps * caps)
+gst_audio_rate_setcaps (GstAudioRate * audiorate, GstCaps * caps)
 {
-  GstAudioRate *audiorate;
-  GstStructure *structure;
-  GstPad *otherpad;
-  gboolean ret = FALSE;
-  gint channels, width, rate;
+  GstAudioInfo info;
 
-  audiorate = GST_AUDIO_RATE (gst_pad_get_parent (pad));
-
-  structure = gst_caps_get_structure (caps, 0);
-
-  if (!gst_structure_get_int (structure, "channels", &channels))
-    goto wrong_caps;
-  if (!gst_structure_get_int (structure, "width", &width))
-    goto wrong_caps;
-  if (!gst_structure_get_int (structure, "rate", &rate))
+  if (!gst_audio_info_from_caps (&info, caps))
     goto wrong_caps;
 
-  audiorate->bytes_per_sample = channels * (width / 8);
-  if (audiorate->bytes_per_sample == 0)
-    goto wrong_format;
+  audiorate->info = info;
 
-  audiorate->rate = rate;
-
-  /* the format is correct, configure caps on other pad */
-  otherpad = (pad == audiorate->srcpad) ? audiorate->sinkpad :
-      audiorate->srcpad;
-
-  ret = gst_pad_set_caps (otherpad, caps);
-
-done:
-  gst_object_unref (audiorate);
-  return ret;
+  return TRUE;
 
   /* ERRORS */
 wrong_caps:
   {
-    GST_DEBUG_OBJECT (audiorate, "could not get channels/width from caps");
-    goto done;
-  }
-wrong_format:
-  {
-    GST_DEBUG_OBJECT (audiorate, "bytes_per_samples gave 0");
-    goto done;
+    GST_DEBUG_OBJECT (audiorate, "could not parse caps");
+    return FALSE;
   }
 }
 
@@ -280,14 +230,12 @@ gst_audio_rate_init (GstAudioRate * audiorate)
       gst_pad_new_from_static_template (&gst_audio_rate_sink_template, "sink");
   gst_pad_set_event_function (audiorate->sinkpad, gst_audio_rate_sink_event);
   gst_pad_set_chain_function (audiorate->sinkpad, gst_audio_rate_chain);
-  gst_pad_set_setcaps_function (audiorate->sinkpad, gst_audio_rate_setcaps);
   gst_pad_set_getcaps_function (audiorate->sinkpad, gst_pad_proxy_getcaps);
   gst_element_add_pad (GST_ELEMENT (audiorate), audiorate->sinkpad);
 
   audiorate->srcpad =
       gst_pad_new_from_static_template (&gst_audio_rate_src_template, "src");
   gst_pad_set_event_function (audiorate->srcpad, gst_audio_rate_src_event);
-  gst_pad_set_setcaps_function (audiorate->srcpad, gst_audio_rate_setcaps);
   gst_pad_set_getcaps_function (audiorate->srcpad, gst_pad_proxy_getcaps);
   gst_element_add_pad (GST_ELEMENT (audiorate), audiorate->srcpad);
 
@@ -328,46 +276,52 @@ gst_audio_rate_sink_event (GstPad * pad, GstEvent * event)
   audiorate = GST_AUDIO_RATE (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      if ((res = gst_audio_rate_setcaps (audiorate, caps))) {
+        res = gst_pad_push_event (audiorate->srcpad, event);
+      } else {
+        gst_event_unref (event);
+      }
+      break;
+    }
     case GST_EVENT_FLUSH_STOP:
       GST_DEBUG_OBJECT (audiorate, "handling FLUSH_STOP");
       gst_audio_rate_reset (audiorate);
       res = gst_pad_push_event (audiorate->srcpad, event);
       break;
-    case GST_EVENT_NEWSEGMENT:
+    case GST_EVENT_SEGMENT:
     {
-      GstFormat format;
-      gdouble rate, arate;
-      gint64 start, stop, time;
-      gboolean update;
-
-      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
-          &start, &stop, &time);
+      gst_event_copy_segment (event, &audiorate->sink_segment);
 
       GST_DEBUG_OBJECT (audiorate, "handle NEWSEGMENT");
+#if 0
       /* FIXME: bad things will likely happen if rate < 0 ... */
       if (!update) {
         /* a new segment starts. We need to figure out what will be the next
          * sample offset. We mark the offsets as invalid so that the _chain
          * function will perform this calculation. */
         gst_audio_rate_fill_to_time (audiorate, audiorate->src_segment.stop);
+#endif
         audiorate->next_offset = -1;
         audiorate->next_ts = -1;
+#if 0
       } else {
         gst_audio_rate_fill_to_time (audiorate, audiorate->src_segment.start);
       }
-
-      /* we accept all formats */
-      gst_segment_set_newsegment_full (&audiorate->sink_segment, update, rate,
-          arate, format, start, stop, time);
+#endif
 
       GST_DEBUG_OBJECT (audiorate, "updated segment: %" GST_SEGMENT_FORMAT,
           &audiorate->sink_segment);
 
-      if (format == GST_FORMAT_TIME) {
+      if (audiorate->sink_segment.format == GST_FORMAT_TIME) {
         /* TIME formats can be copied to src and forwarded */
         res = gst_pad_push_event (audiorate->srcpad, event);
-        memcpy (&audiorate->src_segment, &audiorate->sink_segment,
-            sizeof (GstSegment));
+        gst_segment_copy_into (&audiorate->sink_segment,
+            &audiorate->src_segment);
       } else {
         /* other formats will be handled in the _chain function */
         gst_event_unref (event);
@@ -376,7 +330,9 @@ gst_audio_rate_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
-      /* FIXME, fill last segment */
+      /* Fill segment until the end */
+      if (GST_CLOCK_TIME_IS_VALID (audiorate->src_segment.stop))
+        gst_audio_rate_fill_to_time (audiorate, audiorate->src_segment.stop);
       res = gst_pad_push_event (audiorate->srcpad, event);
       break;
     default:
@@ -410,58 +366,10 @@ gst_audio_rate_src_event (GstPad * pad, GstEvent * event)
 
 static gboolean
 gst_audio_rate_convert (GstAudioRate * audiorate,
-    GstFormat src_fmt, gint64 src_val, GstFormat dest_fmt, gint64 * dest_val)
+    GstFormat src_fmt, guint64 src_val, GstFormat dest_fmt, guint64 * dest_val)
 {
-  if (src_fmt == dest_fmt) {
-    *dest_val = src_val;
-    return TRUE;
-  }
-
-  switch (src_fmt) {
-    case GST_FORMAT_DEFAULT:
-      switch (dest_fmt) {
-        case GST_FORMAT_BYTES:
-          *dest_val = src_val * audiorate->bytes_per_sample;
-          break;
-        case GST_FORMAT_TIME:
-          *dest_val =
-              gst_util_uint64_scale_int (src_val, GST_SECOND, audiorate->rate);
-          break;
-        default:
-          return FALSE;;
-      }
-      break;
-    case GST_FORMAT_BYTES:
-      switch (dest_fmt) {
-        case GST_FORMAT_DEFAULT:
-          *dest_val = src_val / audiorate->bytes_per_sample;
-          break;
-        case GST_FORMAT_TIME:
-          *dest_val = gst_util_uint64_scale_int (src_val, GST_SECOND,
-              audiorate->rate * audiorate->bytes_per_sample);
-          break;
-        default:
-          return FALSE;;
-      }
-      break;
-    case GST_FORMAT_TIME:
-      switch (dest_fmt) {
-        case GST_FORMAT_BYTES:
-          *dest_val = gst_util_uint64_scale_int (src_val,
-              audiorate->rate * audiorate->bytes_per_sample, GST_SECOND);
-          break;
-        case GST_FORMAT_DEFAULT:
-          *dest_val =
-              gst_util_uint64_scale_int (src_val, audiorate->rate, GST_SECOND);
-          break;
-        default:
-          return FALSE;;
-      }
-      break;
-    default:
-      return FALSE;
-  }
-  return TRUE;
+  return gst_audio_info_convert (&audiorate->info, src_fmt, src_val, dest_fmt,
+      (gint64 *) dest_val);
 }
 
 
@@ -478,19 +386,38 @@ gst_audio_rate_convert_segments (GstAudioRate * audiorate)
 		dst_fmt, &audiorate->src_segment.field);
 
   audiorate->sink_segment.rate = audiorate->src_segment.rate;
-  audiorate->sink_segment.abs_rate = audiorate->src_segment.abs_rate;
   audiorate->sink_segment.flags = audiorate->src_segment.flags;
   audiorate->sink_segment.applied_rate = audiorate->src_segment.applied_rate;
   CONVERT_VAL (start);
   CONVERT_VAL (stop);
   CONVERT_VAL (time);
-  CONVERT_VAL (accum);
-  CONVERT_VAL (last_stop);
+  CONVERT_VAL (base);
+  CONVERT_VAL (position);
 #undef CONVERT_VAL
 
   return TRUE;
 }
 
+static void
+gst_audio_rate_notify_drop (GstAudioRate * audiorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) audiorate, "drop");
+#else
+  g_object_notify_by_pspec ((GObject *) audiorate, pspec_drop);
+#endif
+}
+
+static void
+gst_audio_rate_notify_add (GstAudioRate * audiorate)
+{
+#if !GLIB_CHECK_VERSION(2,26,0)
+  g_object_notify ((GObject *) audiorate, "add");
+#else
+  g_object_notify_by_pspec ((GObject *) audiorate, pspec_add);
+#endif
+}
+
 static GstFlowReturn
 gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
 {
@@ -500,11 +427,15 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
   guint in_size;
   GstFlowReturn ret = GST_FLOW_OK;
   GstClockTimeDiff diff;
+  gint rate, bpf;
 
   audiorate = GST_AUDIO_RATE (gst_pad_get_parent (pad));
 
+  rate = GST_AUDIO_INFO_RATE (&audiorate->info);
+  bpf = GST_AUDIO_INFO_BPF (&audiorate->info);
+
   /* need to be negotiated now */
-  if (audiorate->bytes_per_sample == 0)
+  if (bpf == 0)
     goto not_negotiated;
 
   /* we have a new pending segment */
@@ -521,7 +452,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
      */
     /* convert first timestamp of segment to sample position */
     pos = gst_util_uint64_scale_int (audiorate->src_segment.start,
-        audiorate->rate, GST_SECOND);
+        GST_AUDIO_INFO_RATE (&audiorate->info), GST_SECOND);
 
     GST_DEBUG_OBJECT (audiorate, "resync to offset %" G_GINT64_FORMAT, pos);
 
@@ -530,7 +461,17 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
 
     audiorate->next_offset = pos;
     audiorate->next_ts = gst_util_uint64_scale_int (audiorate->next_offset,
-        GST_SECOND, audiorate->rate);
+        GST_SECOND, GST_AUDIO_INFO_RATE (&audiorate->info));
+
+    if (audiorate->skip_to_first && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
+      GST_DEBUG_OBJECT (audiorate, "but skipping to first buffer instead");
+      pos = gst_util_uint64_scale_int (GST_BUFFER_TIMESTAMP (buf),
+          GST_AUDIO_INFO_RATE (&audiorate->info), GST_SECOND);
+      GST_DEBUG_OBJECT (audiorate, "so resync to offset %" G_GINT64_FORMAT,
+          pos);
+      audiorate->next_offset = pos;
+      audiorate->next_ts = GST_BUFFER_TIMESTAMP (buf);
+    }
   }
 
   audiorate->in++;
@@ -541,12 +482,11 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
     in_time = audiorate->next_ts;
   }
 
-  in_size = GST_BUFFER_SIZE (buf);
-  in_samples = in_size / audiorate->bytes_per_sample;
+  in_size = gst_buffer_get_size (buf);
+  in_samples = in_size / bpf;
 
   /* calculate the buffer offset */
-  in_offset = gst_util_uint64_scale_int_round (in_time, audiorate->rate,
-      GST_SECOND);
+  in_offset = gst_util_uint64_scale_int_round (in_time, rate, GST_SECOND);
   in_offset_end = in_offset + in_samples;
 
   GST_LOG_OBJECT (audiorate,
@@ -554,7 +494,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
       ", in_size:%u, in_offset:%" G_GUINT64_FORMAT ", in_offset_end:%"
       G_GUINT64_FORMAT ", ->next_offset:%" G_GUINT64_FORMAT ", ->next_ts:%"
       GST_TIME_FORMAT, GST_TIME_ARGS (in_time),
-      GST_TIME_ARGS (GST_FRAMES_TO_CLOCK_TIME (in_samples, audiorate->rate)),
+      GST_TIME_ARGS (GST_FRAMES_TO_CLOCK_TIME (in_samples, rate)),
       in_size, in_offset, in_offset_end, audiorate->next_offset,
       GST_TIME_ARGS (audiorate->next_ts));
 
@@ -584,14 +524,18 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
     fillsamples = in_offset - audiorate->next_offset;
 
     while (fillsamples > 0) {
-      guint64 cursamples = MIN (fillsamples, audiorate->rate);
+      guint64 cursamples = MIN (fillsamples, rate);
+      guint8 *data;
 
       fillsamples -= cursamples;
-      fillsize = cursamples * audiorate->bytes_per_sample;
+      fillsize = cursamples * bpf;
 
       fill = gst_buffer_new_and_alloc (fillsize);
+
+      data = gst_buffer_map (fill, NULL, NULL, GST_MAP_WRITE);
       /* FIXME, 0 might not be the silence byte for the negotiated format. */
-      memset (GST_BUFFER_DATA (fill), 0, fillsize);
+      memset (data, 0, fillsize);
+      gst_buffer_unmap (fill, data, fillsize);
 
       GST_DEBUG_OBJECT (audiorate, "inserting %" G_GUINT64_FORMAT " samples",
           cursamples);
@@ -605,7 +549,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
        * streams */
       GST_BUFFER_TIMESTAMP (fill) = audiorate->next_ts;
       audiorate->next_ts = gst_util_uint64_scale_int (audiorate->next_offset,
-          GST_SECOND, audiorate->rate);
+          GST_SECOND, rate);
       GST_BUFFER_DURATION (fill) = audiorate->next_ts -
           GST_BUFFER_TIMESTAMP (fill);
 
@@ -617,7 +561,6 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
         GST_BUFFER_FLAG_SET (fill, GST_BUFFER_FLAG_DISCONT);
         audiorate->discont = FALSE;
       }
-      gst_buffer_set_caps (fill, GST_PAD_CAPS (audiorate->srcpad));
 
       ret = gst_pad_push (audiorate->srcpad, fill);
       if (ret != GST_FLOW_OK)
@@ -626,13 +569,13 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
       audiorate->add += cursamples;
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "add");
+        gst_audio_rate_notify_add (audiorate);
     }
 
   } else if (in_offset < audiorate->next_offset) {
     /* need to remove samples */
     if (in_offset_end <= audiorate->next_offset) {
-      guint64 drop = in_size / audiorate->bytes_per_sample;
+      guint64 drop = in_size / bpf;
 
       audiorate->drop += drop;
 
@@ -644,7 +587,7 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
       buf = NULL;
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "drop");
+        gst_audio_rate_notify_drop (audiorate);
 
       goto beach;
     } else {
@@ -654,27 +597,27 @@ gst_audio_rate_chain (GstPad * pad, GstBuffer * buf)
 
       /* truncate buffer */
       truncsamples = audiorate->next_offset - in_offset;
-      truncsize = truncsamples * audiorate->bytes_per_sample;
+      truncsize = truncsamples * bpf;
       leftsize = in_size - truncsize;
 
-      trunc = gst_buffer_create_sub (buf, truncsize, leftsize);
+      trunc =
+          gst_buffer_copy_region (buf, GST_BUFFER_COPY_ALL, truncsize,
+          leftsize);
 
       gst_buffer_unref (buf);
       buf = trunc;
 
-      gst_buffer_set_caps (buf, GST_PAD_CAPS (audiorate->srcpad));
-
       audiorate->drop += truncsamples;
       GST_DEBUG_OBJECT (audiorate, "truncating %" G_GUINT64_FORMAT " samples",
           truncsamples);
 
       if (!audiorate->silent)
-        g_object_notify (G_OBJECT (audiorate), "drop");
+        gst_audio_rate_notify_drop (audiorate);
     }
   }
 
 send:
-  if (GST_BUFFER_SIZE (buf) == 0)
+  if (gst_buffer_get_size (buf) == 0)
     goto beach;
 
   /* Now calculate parameters for whichever buffer (either the original
@@ -684,26 +627,26 @@ send:
 
   GST_BUFFER_TIMESTAMP (buf) = audiorate->next_ts;
   audiorate->next_ts = gst_util_uint64_scale_int (in_offset_end,
-      GST_SECOND, audiorate->rate);
+      GST_SECOND, rate);
   GST_BUFFER_DURATION (buf) = audiorate->next_ts - GST_BUFFER_TIMESTAMP (buf);
 
   if (audiorate->discont) {
     /* we need to output a discont buffer, do so now */
     GST_DEBUG_OBJECT (audiorate, "marking DISCONT on output buffer");
-    buf = gst_buffer_make_metadata_writable (buf);
+    buf = gst_buffer_make_writable (buf);
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
     audiorate->discont = FALSE;
   } else if (GST_BUFFER_IS_DISCONT (buf)) {
     /* else we make everything continuous so we can safely remove the DISCONT
      * flag from the buffer if there was one */
     GST_DEBUG_OBJECT (audiorate, "removing DISCONT from buffer");
-    buf = gst_buffer_make_metadata_writable (buf);
+    buf = gst_buffer_make_writable (buf);
     GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DISCONT);
   }
 
   /* set last_stop on segment */
-  gst_segment_set_last_stop (&audiorate->src_segment, GST_FORMAT_TIME,
-      GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf));
+  audiorate->src_segment.position =
+      GST_BUFFER_TIMESTAMP (buf) + GST_BUFFER_DURATION (buf);
 
   ret = gst_pad_push (audiorate->srcpad, buf);
   buf = NULL;
@@ -743,6 +686,9 @@ gst_audio_rate_set_property (GObject * object,
     case ARG_TOLERANCE:
       audiorate->tolerance = g_value_get_uint64 (value);
       break;
+    case ARG_SKIP_TO_FIRST:
+      audiorate->skip_to_first = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -774,6 +720,9 @@ gst_audio_rate_get_property (GObject * object,
     case ARG_TOLERANCE:
       g_value_set_uint64 (value, audiorate->tolerance);
       break;
+    case ARG_SKIP_TO_FIRST:
+      g_value_set_boolean (value, audiorate->skip_to_first);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -792,18 +741,15 @@ gst_audio_rate_change_state (GstElement * element, GstStateChange transition)
       audiorate->in = 0;
       audiorate->out = 0;
       audiorate->drop = 0;
-      audiorate->bytes_per_sample = 0;
       audiorate->add = 0;
+      gst_audio_info_init (&audiorate->info);
       gst_audio_rate_reset (audiorate);
       break;
     default:
       break;
   }
 
-  if (parent_class->change_state)
-    return parent_class->change_state (element, transition);
-
-  return GST_STATE_CHANGE_SUCCESS;
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 }
 
 static gboolean
