@@ -43,8 +43,10 @@ G_DEFINE_TYPE (GstStreamSplitter, gst_stream_splitter, GST_TYPE_ELEMENT);
 
 static void gst_stream_splitter_dispose (GObject * object);
 
+static gboolean gst_stream_splitter_sink_setcaps (GstPad * pad, GstCaps * caps);
+
 static GstPad *gst_stream_splitter_request_new_pad (GstElement * element,
-    GstPadTemplate * templ, const gchar * name);
+    GstPadTemplate * templ, const gchar * name, const GstCaps * caps);
 static void gst_stream_splitter_release_pad (GstElement * element,
     GstPad * pad);
 
@@ -88,6 +90,11 @@ gst_stream_splitter_dispose (GObject * object)
     stream_splitter->lock = NULL;
   }
 
+  g_list_foreach (stream_splitter->pending_events, (GFunc) gst_event_unref,
+      NULL);
+  g_list_free (stream_splitter->pending_events);
+  stream_splitter->pending_events = NULL;
+
   G_OBJECT_CLASS (gst_stream_splitter_parent_class)->dispose (object);
 }
 
@@ -151,6 +158,16 @@ gst_stream_splitter_sink_event (GstPad * pad, GstEvent * event)
       GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      res = gst_stream_splitter_sink_setcaps (pad, caps);
+
+      store = TRUE;
+      break;
+    }
     case GST_EVENT_FLUSH_STOP:
       flushpending = TRUE;
       toall = TRUE;
@@ -173,9 +190,8 @@ gst_stream_splitter_sink_event (GstPad * pad, GstEvent * event)
   }
 
   if (flushpending) {
-    GList *tmp;
-    for (tmp = stream_splitter->pending_events; tmp; tmp = tmp->next)
-      gst_event_unref ((GstEvent *) tmp->data);
+    g_list_foreach (stream_splitter->pending_events, (GFunc) gst_event_unref,
+        NULL);
     g_list_free (stream_splitter->pending_events);
     stream_splitter->pending_events = NULL;
   }
@@ -238,7 +254,7 @@ beach:
 }
 
 static GstCaps *
-gst_stream_splitter_sink_getcaps (GstPad * pad)
+gst_stream_splitter_sink_getcaps (GstPad * pad, GstCaps * filter)
 {
   GstStreamSplitter *stream_splitter =
       (GstStreamSplitter *) GST_PAD_PARENT (pad);
@@ -252,7 +268,7 @@ gst_stream_splitter_sink_getcaps (GstPad * pad)
 
 resync:
   if (G_UNLIKELY (stream_splitter->srcpads == NULL)) {
-    res = gst_caps_new_any ();
+    res = (filter ? gst_caps_ref (filter) : gst_caps_new_any ());
     goto beach;
   }
 
@@ -264,10 +280,13 @@ resync:
     GstPad *srcpad = (GstPad *) tmp->data;
 
     STREAMS_UNLOCK (stream_splitter);
-    if (res)
-      gst_caps_merge (res, gst_pad_peer_get_caps_reffed (srcpad));
-    else
-      res = gst_pad_peer_get_caps (srcpad);
+    if (res) {
+      GstCaps *peercaps = gst_pad_peer_get_caps (srcpad, filter);
+      if (peercaps)
+        gst_caps_merge (res, gst_caps_make_writable (peercaps));
+    } else {
+      res = gst_pad_peer_get_caps (srcpad, filter);
+    }
     STREAMS_LOCK (stream_splitter);
 
     if (G_UNLIKELY (cookie != stream_splitter->cookie)) {
@@ -312,7 +331,7 @@ resync:
     GstCaps *peercaps;
 
     STREAMS_UNLOCK (stream_splitter);
-    peercaps = gst_pad_peer_get_caps_reffed (srcpad);
+    peercaps = gst_pad_peer_get_caps (srcpad, NULL);
     if (peercaps) {
       res = gst_caps_can_intersect (caps, peercaps);
       gst_caps_unref (peercaps);
@@ -374,8 +393,6 @@ gst_stream_splitter_init (GstStreamSplitter * stream_splitter)
       gst_stream_splitter_sink_event);
   gst_pad_set_getcaps_function (stream_splitter->sinkpad,
       gst_stream_splitter_sink_getcaps);
-  gst_pad_set_setcaps_function (stream_splitter->sinkpad,
-      gst_stream_splitter_sink_setcaps);
   gst_element_add_pad (GST_ELEMENT (stream_splitter), stream_splitter->sinkpad);
 
   stream_splitter->lock = g_mutex_new ();
@@ -383,7 +400,7 @@ gst_stream_splitter_init (GstStreamSplitter * stream_splitter)
 
 static GstPad *
 gst_stream_splitter_request_new_pad (GstElement * element,
-    GstPadTemplate * templ, const gchar * name)
+    GstPadTemplate * templ, const gchar * name, const GstCaps * caps)
 {
   GstStreamSplitter *stream_splitter = (GstStreamSplitter *) element;
   GstPad *srcpad;
