@@ -27,14 +27,11 @@
 #include <gst/gst-i18n-plugin.h>
 #include <gst/pbutils/pbutils.h>
 
-#include "gststreamselector.h"
-#include "gststreaminfo.h"
+#include "gstplayback.h"
 #include "gstplaysink.h"
+#include "gststreamselector.h"
 #include "gstsubtitleoverlay.h"
 
-gboolean gst_play_bin_plugin_init (GstPlugin * plugin);
-gboolean gst_play_bin2_plugin_init (GstPlugin * plugin);
-
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
@@ -51,14 +48,15 @@ plugin_init (GstPlugin * plugin)
 
   /* ref class from a thread-safe context to work around missing bit of
    * thread-safety in GObject */
-  g_type_class_ref (GST_TYPE_STREAM_INFO);
   g_type_class_ref (GST_TYPE_STREAM_SELECTOR);
 
-  res = gst_play_bin_plugin_init (plugin);
-  res &= gst_play_bin2_plugin_init (plugin);
+  res = gst_play_bin2_plugin_init (plugin);
   res &= gst_play_sink_plugin_init (plugin);
   res &= gst_subtitle_overlay_plugin_init (plugin);
 
+  res &= gst_decode_bin_plugin_init (plugin);
+  res &= gst_uri_decode_bin_plugin_init (plugin);
+
   return res;
 }
 
