@@ -54,8 +54,9 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink_%d",
 
 static const gboolean passthrough = TRUE;
 
-GST_BOILERPLATE (GstStreamSynchronizer, gst_stream_synchronizer,
-    GstElement, GST_TYPE_ELEMENT);
+#define gst_stream_synchronizer_parent_class parent_class
+G_DEFINE_TYPE (GstStreamSynchronizer, gst_stream_synchronizer,
+    GST_TYPE_ELEMENT);
 
 typedef struct
 {
@@ -120,8 +121,12 @@ gst_stream_synchronizer_iterate_internal_links (GstPad * pad)
 
   opad = gst_stream_get_other_pad_from_pad (pad);
   if (opad) {
-    it = gst_iterator_new_single (GST_TYPE_PAD, opad,
-        (GstCopyFunction) gst_object_ref, (GFreeFunc) gst_object_unref);
+    GValue value = { 0, };
+
+    g_value_init (&value, GST_TYPE_PAD);
+    g_value_set_object (&value, opad);
+    it = gst_iterator_new_single (GST_TYPE_PAD, &value);
+    g_value_unset (&value);
     gst_object_unref (opad);
   }
 
@@ -146,19 +151,19 @@ gst_stream_synchronizer_query (GstPad * pad, GstQuery * query)
 }
 
 static GstCaps *
-gst_stream_synchronizer_getcaps (GstPad * pad)
+gst_stream_synchronizer_getcaps (GstPad * pad, GstCaps * filter)
 {
   GstPad *opad;
   GstCaps *ret = NULL;
 
   opad = gst_stream_get_other_pad_from_pad (pad);
   if (opad) {
-    ret = gst_pad_peer_get_caps (opad);
+    ret = gst_pad_peer_get_caps (opad, filter);
     gst_object_unref (opad);
   }
 
   if (ret == NULL)
-    ret = gst_caps_new_any ();
+    ret = (filter ? gst_caps_ref (filter) : gst_caps_new_any ());
 
   GST_LOG_OBJECT (pad, "Returning caps: %" GST_PTR_FORMAT, ret);
 
@@ -196,7 +201,7 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstEvent * event)
     goto skip_adjustments;
 
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
-      GST_EVENT_TYPE_NAME (event), event->structure);
+      GST_EVENT_TYPE_NAME (event), event);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_QOS:{
@@ -206,7 +211,7 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstEvent * event)
       gint64 running_time_diff;
       GstStream *stream;
 
-      gst_event_parse_qos (event, &proportion, &diff, &timestamp);
+      gst_event_parse_qos (event, NULL, &proportion, &diff, &timestamp);
       gst_event_unref (event);
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
@@ -240,7 +245,9 @@ gst_stream_synchronizer_src_event (GstPad * pad, GstEvent * event)
         goto out;
       }
 
-      event = gst_event_new_qos (proportion, diff, timestamp);
+      event =
+          gst_event_new_qos (GST_QOS_TYPE_UNDERFLOW, proportion, diff,
+          timestamp);
       break;
     }
     default:
@@ -274,16 +281,14 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
     goto skip_adjustments;
 
   GST_LOG_OBJECT (pad, "Handling event %s: %" GST_PTR_FORMAT,
-      GST_EVENT_TYPE_NAME (event), event->structure);
+      GST_EVENT_TYPE_NAME (event), event);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SINK_MESSAGE:{
       GstMessage *message;
 
       gst_event_parse_sink_message (event, &message);
-      if (message->structure
-          && gst_structure_has_name (message->structure,
-              "playbin2-stream-changed")) {
+      if (gst_message_has_name (message, "playbin-stream-changed")) {
         GstStream *stream;
 
         GST_STREAM_SYNCHRONIZER_LOCK (self);
@@ -306,29 +311,29 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
               break;
           }
           if (all_wait) {
-            gint64 last_stop = 0;
+            gint64 position = 0;
 
             GST_DEBUG_OBJECT (self, "All streams have changed -- unblocking");
 
             for (l = self->streams; l; l = l->next) {
               GstStream *ostream = l->data;
               gint64 stop_running_time;
-              gint64 last_stop_running_time;
+              gint64 position_running_time;
 
               ostream->wait = FALSE;
 
               stop_running_time =
                   gst_segment_to_running_time (&ostream->segment,
                   GST_FORMAT_TIME, ostream->segment.stop);
-              last_stop_running_time =
+              position_running_time =
                   gst_segment_to_running_time (&ostream->segment,
-                  GST_FORMAT_TIME, ostream->segment.last_stop);
-              last_stop =
-                  MAX (last_stop, MAX (stop_running_time,
-                      last_stop_running_time));
+                  GST_FORMAT_TIME, ostream->segment.position);
+              position =
+                  MAX (position, MAX (stop_running_time,
+                      position_running_time));
             }
-            last_stop = MAX (0, last_stop);
-            self->group_start_time = MAX (self->group_start_time, last_stop);
+            position = MAX (0, position);
+            self->group_start_time = MAX (self->group_start_time, position);
 
             GST_DEBUG_OBJECT (self, "New group start time: %" GST_TIME_FORMAT,
                 GST_TIME_ARGS (self->group_start_time));
@@ -341,15 +346,11 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
       gst_message_unref (message);
       break;
     }
-    case GST_EVENT_NEWSEGMENT:{
+    case GST_EVENT_SEGMENT:{
       GstStream *stream;
-      gboolean update;
-      gdouble rate, applied_rate;
-      GstFormat format;
-      gint64 start, stop, position;
+      GstSegment segment;
 
-      gst_event_parse_new_segment_full (event,
-          &update, &rate, &applied_rate, &format, &start, &stop, &position);
+      gst_event_copy_segment (event, &segment);
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
@@ -369,41 +370,35 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
         goto done;
       }
 
-      if (stream && format == GST_FORMAT_TIME) {
+      if (stream && segment.format == GST_FORMAT_TIME) {
         if (stream->new_stream) {
-          gint64 last_stop_running_time = 0;
+          gint64 position_running_time = 0;
           gint64 stop_running_time = 0;
 
           if (stream->segment.format == GST_FORMAT_TIME) {
-            last_stop_running_time =
+            position_running_time =
                 gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-                stream->segment.last_stop);
-            last_stop_running_time = MAX (last_stop_running_time, 0);
+                stream->segment.position);
+            position_running_time = MAX (position_running_time, 0);
             stop_running_time =
                 gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
                 stream->segment.stop);
-            stop_running_time = MAX (last_stop_running_time, 0);
+            stop_running_time = MAX (position_running_time, 0);
 
-            if (stop_running_time != last_stop_running_time) {
+            if (stop_running_time != position_running_time) {
               GST_WARNING_OBJECT (pad,
-                  "Gap between last_stop and segment stop: %" GST_TIME_FORMAT
+                  "Gap between position and segment stop: %" GST_TIME_FORMAT
                   " != %" GST_TIME_FORMAT, GST_TIME_ARGS (stop_running_time),
-                  GST_TIME_ARGS (last_stop_running_time));
+                  GST_TIME_ARGS (position_running_time));
             }
 
-            if (stop_running_time < last_stop_running_time) {
+            if (stop_running_time < position_running_time) {
               GST_DEBUG_OBJECT (pad, "Updating stop position");
+              stream->segment.stop = stream->segment.position;
               gst_pad_push_event (stream->srcpad,
-                  gst_event_new_new_segment_full (TRUE, stream->segment.rate,
-                      stream->segment.applied_rate, GST_FORMAT_TIME,
-                      stream->segment.start, stream->segment.last_stop,
-                      stream->segment.time));
-              gst_segment_set_newsegment_full (&stream->segment, TRUE,
-                  stream->segment.rate, stream->segment.applied_rate,
-                  GST_FORMAT_TIME, stream->segment.start,
-                  stream->segment.last_stop, stream->segment.time);
+                  gst_event_new_segment (&stream->segment));
             }
-            stop_running_time = MAX (stop_running_time, last_stop_running_time);
+            stop_running_time = MAX (stop_running_time, position_running_time);
             GST_DEBUG_OBJECT (pad,
                 "Stop running time of last group: %" GST_TIME_FORMAT,
                 GST_TIME_ARGS (stop_running_time));
@@ -417,27 +412,23 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
             GST_DEBUG_OBJECT (pad,
                 "Advancing running time for other streams by: %"
                 GST_TIME_FORMAT, GST_TIME_ARGS (diff));
-            gst_pad_push_event (stream->srcpad,
-                gst_event_new_new_segment_full (FALSE, 1.0, 1.0,
-                    GST_FORMAT_TIME, 0, diff, 0));
-            gst_segment_set_newsegment_full (&stream->segment, FALSE, 1.0, 1.0,
-                GST_FORMAT_TIME, 0, diff, 0);
+
+            segment.base += diff;
           }
         }
 
         GST_DEBUG_OBJECT (pad, "Segment was: %" GST_SEGMENT_FORMAT,
             &stream->segment);
-        gst_segment_set_newsegment_full (&stream->segment, update, rate,
-            applied_rate, format, start, stop, position);
+        gst_segment_copy_into (&segment, &stream->segment);
         GST_DEBUG_OBJECT (pad, "Segment now is: %" GST_SEGMENT_FORMAT,
             &stream->segment);
 
         GST_DEBUG_OBJECT (pad, "Stream start running time: %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (stream->segment.accum));
-        stream->running_time_diff = stream->segment.accum;
+            GST_TIME_ARGS (stream->segment.base));
+        stream->running_time_diff = stream->segment.base;
       } else if (stream) {
         GST_WARNING_OBJECT (pad, "Non-TIME segment: %s",
-            gst_format_get_name (format));
+            gst_format_get_name (segment.format));
         gst_segment_init (&stream->segment, GST_FORMAT_UNDEFINED);
       }
       GST_STREAM_SYNCHRONIZER_UNLOCK (self);
@@ -472,11 +463,15 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
 
       GST_STREAM_SYNCHRONIZER_LOCK (self);
       stream = gst_pad_get_element_private (pad);
-      if (stream) {
-        GST_DEBUG_OBJECT (pad, "Have EOS for stream %d", stream->stream_number);
-        stream->is_eos = TRUE;
+      if (!stream) {
+        GST_STREAM_SYNCHRONIZER_UNLOCK (self);
+        GST_WARNING_OBJECT (pad, "EOS for unknown stream");
+        break;
       }
 
+      GST_DEBUG_OBJECT (pad, "Have EOS for stream %d", stream->stream_number);
+      stream->is_eos = TRUE;
+
       seen_data = stream->seen_data;
       srcpad = gst_object_ref (stream->srcpad);
 
@@ -519,7 +514,6 @@ gst_stream_synchronizer_sink_event (GstPad * pad, GstEvent * event)
         if (!seen_data) {
           GstBuffer *buf = gst_buffer_new ();
 
-          GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_PREROLL);
           gst_pad_push (srcpad, buf);
         }
       }
@@ -545,26 +539,6 @@ done:
   return ret;
 }
 
-static GstFlowReturn
-gst_stream_synchronizer_sink_bufferalloc (GstPad * pad, guint64 offset,
-    guint size, GstCaps * caps, GstBuffer ** buf)
-{
-  GstPad *opad;
-  GstFlowReturn ret = GST_FLOW_ERROR;
-
-  GST_LOG_OBJECT (pad, "Allocating buffer: size=%u", size);
-
-  opad = gst_stream_get_other_pad_from_pad (pad);
-  if (opad) {
-    ret = gst_pad_alloc_buffer (opad, offset, size, caps, buf);
-    gst_object_unref (opad);
-  }
-
-  GST_LOG_OBJECT (pad, "Allocation: %s", gst_flow_get_name (ret));
-
-  return ret;
-}
-
 static GstFlowReturn
 gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -585,10 +559,10 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
     goto done;
   }
 
-  GST_LOG_OBJECT (pad, "Handling buffer %p: size=%u, timestamp=%"
-      GST_TIME_FORMAT " duration=%" GST_TIME_FORMAT
+  GST_LOG_OBJECT (pad, "Handling buffer %p: size=%" G_GSIZE_FORMAT
+      ", timestamp=%" GST_TIME_FORMAT " duration=%" GST_TIME_FORMAT
       " offset=%" G_GUINT64_FORMAT " offset_end=%" G_GUINT64_FORMAT,
-      buffer, GST_BUFFER_SIZE (buffer),
+      buffer, gst_buffer_get_size (buffer),
       GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buffer)),
       GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)),
       GST_BUFFER_OFFSET (buffer), GST_BUFFER_OFFSET_END (buffer));
@@ -603,7 +577,7 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
 
   stream->seen_data = TRUE;
   if (stream && stream->drop_discont) {
-    buffer = gst_buffer_make_metadata_writable (buffer);
+    buffer = gst_buffer_make_writable (buffer);
     GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
     stream->drop_discont = FALSE;
   }
@@ -612,8 +586,8 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
       && GST_CLOCK_TIME_IS_VALID (timestamp)) {
     GST_LOG_OBJECT (pad,
         "Updating last-stop from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (stream->segment.last_stop), GST_TIME_ARGS (timestamp));
-    gst_segment_set_last_stop (&stream->segment, GST_FORMAT_TIME, timestamp);
+        GST_TIME_ARGS (stream->segment.position), GST_TIME_ARGS (timestamp));
+    stream->segment.position = timestamp;
   }
   GST_STREAM_SYNCHRONIZER_UNLOCK (self);
 
@@ -633,28 +607,27 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
         && GST_CLOCK_TIME_IS_VALID (timestamp_end)) {
       GST_LOG_OBJECT (pad,
           "Updating last-stop from %" GST_TIME_FORMAT " to %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (stream->segment.last_stop),
+          GST_TIME_ARGS (stream->segment.position),
           GST_TIME_ARGS (timestamp_end));
-      gst_segment_set_last_stop (&stream->segment, GST_FORMAT_TIME,
-          timestamp_end);
+      stream->segment.position = timestamp_end;
     }
 
     /* Advance EOS streams if necessary. For non-EOS
      * streams the demuxers should already do this! */
     for (l = self->streams; l; l = l->next) {
       GstStream *ostream = l->data;
-      gint64 last_stop;
+      gint64 position;
 
       if (!ostream->is_eos || ostream->segment.format != GST_FORMAT_TIME)
         continue;
 
-      if (ostream->segment.last_stop != -1)
-        last_stop = ostream->segment.last_stop;
+      if (ostream->segment.position != -1)
+        position = ostream->segment.position;
       else
-        last_stop = ostream->segment.start;
+        position = ostream->segment.start;
 
       /* Is there a 1 second lag? */
-      if (last_stop != -1 && last_stop + GST_SECOND < timestamp_end) {
+      if (position != -1 && position + GST_SECOND < timestamp_end) {
         gint64 new_start, new_stop;
 
         new_start = timestamp_end - GST_SECOND;
@@ -665,18 +638,16 @@ gst_stream_synchronizer_sink_chain (GstPad * pad, GstBuffer * buffer)
 
         GST_DEBUG_OBJECT (ostream->sinkpad,
             "Advancing stream %u from %" GST_TIME_FORMAT " to %"
-            GST_TIME_FORMAT, ostream->stream_number, GST_TIME_ARGS (last_stop),
+            GST_TIME_FORMAT, ostream->stream_number, GST_TIME_ARGS (position),
             GST_TIME_ARGS (new_start));
 
+        ostream->segment.start = new_start;
+        ostream->segment.stop = new_stop;
+        ostream->segment.time = new_start;
+        ostream->segment.position = new_start;
+
         gst_pad_push_event (ostream->srcpad,
-            gst_event_new_new_segment_full (TRUE, ostream->segment.rate,
-                ostream->segment.applied_rate, ostream->segment.format,
-                new_start, new_stop, new_start));
-        gst_segment_set_newsegment_full (&ostream->segment, TRUE,
-            ostream->segment.rate, ostream->segment.applied_rate,
-            ostream->segment.format, new_start, new_stop, new_start);
-        gst_segment_set_last_stop (&ostream->segment, GST_FORMAT_TIME,
-            new_start);
+            gst_event_new_segment (&ostream->segment));
       }
     }
     GST_STREAM_SYNCHRONIZER_UNLOCK (self);
@@ -692,7 +663,7 @@ done:
 /* GstElement vfuncs */
 static GstPad *
 gst_stream_synchronizer_request_new_pad (GstElement * element,
-    GstPadTemplate * temp, const gchar * name)
+    GstPadTemplate * temp, const gchar * name, const GstCaps * caps)
 {
   GstStreamSynchronizer *self = GST_STREAM_SYNCHRONIZER (element);
   GstStream *stream;
@@ -722,8 +693,6 @@ gst_stream_synchronizer_request_new_pad (GstElement * element,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_sink_event));
   gst_pad_set_chain_function (stream->sinkpad,
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_sink_chain));
-  gst_pad_set_bufferalloc_function (stream->sinkpad,
-      GST_DEBUG_FUNCPTR (gst_stream_synchronizer_sink_bufferalloc));
 
   tmp = g_strdup_printf ("src_%d", self->current_stream_number);
   stream->srcpad = gst_pad_new_from_static_template (&srctemplate, tmp);
@@ -790,15 +759,15 @@ gst_stream_synchronizer_release_stream (GstStreamSynchronizer * self,
 
   if (stream->segment.format == GST_FORMAT_TIME) {
     gint64 stop_running_time;
-    gint64 last_stop_running_time;
+    gint64 position_running_time;
 
     stop_running_time =
         gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
         stream->segment.stop);
-    last_stop_running_time =
+    position_running_time =
         gst_segment_to_running_time (&stream->segment, GST_FORMAT_TIME,
-        stream->segment.last_stop);
-    stop_running_time = MAX (stop_running_time, last_stop_running_time);
+        stream->segment.position);
+    stop_running_time = MAX (stop_running_time, position_running_time);
 
     GST_DEBUG_OBJECT (stream->sinkpad,
         "Stop running time was: %" GST_TIME_FORMAT,
@@ -939,29 +908,12 @@ gst_stream_synchronizer_finalize (GObject * object)
 
 /* GObject type initialization */
 static void
-gst_stream_synchronizer_init (GstStreamSynchronizer * self,
-    GstStreamSynchronizerClass * klass)
+gst_stream_synchronizer_init (GstStreamSynchronizer * self)
 {
   self->lock = g_mutex_new ();
   self->stream_finish_cond = g_cond_new ();
 }
 
-static void
-gst_stream_synchronizer_base_init (gpointer g_class)
-{
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&srctemplate));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sinktemplate));
-
-  gst_element_class_set_details_simple (gstelement_class,
-      "Stream Synchronizer", "Generic",
-      "Synchronizes a group of streams to have equal durations and starting points",
-      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
-}
-
 static void
 gst_stream_synchronizer_class_init (GstStreamSynchronizerClass * klass)
 {
@@ -973,6 +925,16 @@ gst_stream_synchronizer_class_init (GstStreamSynchronizerClass * klass)
 
   gobject_class->finalize = gst_stream_synchronizer_finalize;
 
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sinktemplate));
+
+  gst_element_class_set_details_simple (element_class,
+      "Stream Synchronizer", "Generic",
+      "Synchronizes a group of streams to have equal durations and starting points",
+      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
+
   element_class->change_state =
       GST_DEBUG_FUNCPTR (gst_stream_synchronizer_change_state);
   element_class->request_new_pad =
