@@ -153,8 +153,6 @@ typedef struct {
 
   gboolean discont;
 
-  GstTCPProtocol protocol;
-
   gboolean caps_sent;
   gboolean new_connection;
 
@@ -176,6 +174,8 @@ typedef struct {
   guint64 last_activity_time;
   guint64 dropped_buffers;
   guint64 avg_queue_size;
+  guint64 first_buffer_ts;
+  guint64 last_buffer_ts;
 } GstTCPClient;
 
 #define CLIENTS_LOCK_INIT(fdsink)       (g_static_rec_mutex_init(&fdsink->clientslock))
@@ -206,9 +206,9 @@ struct _GstMultiFdSink {
   GSList *streamheader; /* GSList of GstBuffers to use as streamheader */
   gboolean previous_buffer_in_caps;
 
-  GstTCPProtocol protocol;
   guint mtu;
   gint qos_dscp;
+  gboolean handle_read;
 
   GArray *bufqueue;     /* global queue of buffers */
 
@@ -234,6 +234,8 @@ struct _GstMultiFdSink {
   gint64 time_min;	/* min time to queue */
   gint   buffers_min;   /* min number of buffers to queue */
 
+  gboolean resend_streamheader; /* resend streamheader if it changes */
+
   /* stats */
   gint buffers_queued;  /* number of queued buffers */
   gint bytes_queued;    /* number of queued bytes */
