@@ -19,9 +19,18 @@
  */
 
 /**
- * SECTION:tcpclientsink
+ * SECTION:element-tcpclientsink
  * @see_also: #tcpclientsrc
  *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * # server:
+ * nc -l -p 3000
+ * # client:
+ * gst-launch fdsrc fd=1 ! tcpclientsink protocol=none port=3000
+ * ]| everything you type in the client is shown on the server
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,13 +42,6 @@
 #include "gsttcpclientsink.h"
 #include <string.h>             /* memset */
 
-/* elementfactory information */
-static const GstElementDetails gst_tcp_client_sink_details =
-GST_ELEMENT_DETAILS ("TCP client sink",
-    "Sink/Network",
-    "Send data as a client over the network via TCP",
-    "Thomas Vander Stichele <thomas at apestaart dot org>");
-
 /* TCPClientSink signals and args */
 enum
 {
@@ -55,9 +57,7 @@ enum
 {
   ARG_0,
   ARG_HOST,
-  ARG_PORT,
-  ARG_PROTOCOL
-      /* FILL ME */
+  ARG_PORT
 };
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -65,9 +65,6 @@ static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-static void gst_tcp_client_sink_base_init (gpointer g_class);
-static void gst_tcp_client_sink_class_init (GstTCPClientSink * klass);
-static void gst_tcp_client_sink_init (GstTCPClientSink * tcpclientsink);
 static void gst_tcp_client_sink_finalize (GObject * gobject);
 
 static gboolean gst_tcp_client_sink_setcaps (GstBaseSink * bsink,
@@ -83,50 +80,13 @@ static void gst_tcp_client_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 
-static GstElementClass *parent_class = NULL;
-
 /*static guint gst_tcp_client_sink_signals[LAST_SIGNAL] = { 0 }; */
 
-GType
-gst_tcp_client_sink_get_type (void)
-{
-  static GType tcpclientsink_type = 0;
-
-
-  if (!tcpclientsink_type) {
-    static const GTypeInfo tcpclientsink_info = {
-      sizeof (GstTCPClientSinkClass),
-      gst_tcp_client_sink_base_init,
-      NULL,
-      (GClassInitFunc) gst_tcp_client_sink_class_init,
-      NULL,
-      NULL,
-      sizeof (GstTCPClientSink),
-      0,
-      (GInstanceInitFunc) gst_tcp_client_sink_init,
-      NULL
-    };
-
-    tcpclientsink_type =
-        g_type_register_static (GST_TYPE_BASE_SINK, "GstTCPClientSink",
-        &tcpclientsink_info, 0);
-  }
-  return tcpclientsink_type;
-}
+#define gst_tcp_client_sink_parent_class parent_class
+G_DEFINE_TYPE (GstTCPClientSink, gst_tcp_client_sink, GST_TYPE_BASE_SINK);
 
 static void
-gst_tcp_client_sink_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sinktemplate));
-
-  gst_element_class_set_details (element_class, &gst_tcp_client_sink_details);
-}
-
-static void
-gst_tcp_client_sink_class_init (GstTCPClientSink * klass)
+gst_tcp_client_sink_class_init (GstTCPClientSinkClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
@@ -149,10 +109,14 @@ gst_tcp_client_sink_class_init (GstTCPClientSink * klass)
       g_param_spec_int ("port", "Port", "The port to send the packets to",
           0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, ARG_PROTOCOL,
-      g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
-          GST_TYPE_TCP_PROTOCOL, GST_TCP_PROTOCOL_NONE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sinktemplate));
+
+  gst_element_class_set_details_simple (gstelement_class,
+      "TCP client sink", "Sink/Network",
+      "Send data as a client over the network via TCP",
+      "Thomas Vander Stichele <thomas at apestaart dot org>");
 
   gstelement_class->change_state = gst_tcp_client_sink_change_state;
 
@@ -169,7 +133,6 @@ gst_tcp_client_sink_init (GstTCPClientSink * this)
   this->port = TCP_DEFAULT_PORT;
 
   this->sock_fd.fd = -1;
-  this->protocol = GST_TCP_PROTOCOL_NONE;
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SINK_OPEN);
 }
 
@@ -186,45 +149,7 @@ gst_tcp_client_sink_finalize (GObject * gobject)
 static gboolean
 gst_tcp_client_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
 {
-  GstTCPClientSink *sink;
-
-  sink = GST_TCP_CLIENT_SINK (bsink);
-
-  /* write the buffer header if we have one */
-  switch (sink->protocol) {
-    case GST_TCP_PROTOCOL_NONE:
-      break;
-
-    case GST_TCP_PROTOCOL_GDP:
-      /* if we haven't send caps yet, send them first */
-      if (!sink->caps_sent) {
-        const GstCaps *caps;
-        gchar *string;
-
-        caps = GST_PAD_CAPS (GST_PAD_PEER (GST_BASE_SINK_PAD (bsink)));
-        string = gst_caps_to_string (caps);
-        GST_DEBUG_OBJECT (sink, "Sending caps %s through GDP", string);
-        g_free (string);
-
-        if (!gst_tcp_gdp_write_caps (GST_ELEMENT (sink), sink->sock_fd.fd,
-                caps, TRUE, sink->host, sink->port))
-          goto gdp_write_error;
-
-        sink->caps_sent = TRUE;
-      }
-      break;
-    default:
-      g_warning ("Unhandled protocol type");
-      break;
-  }
-
   return TRUE;
-
-  /* ERRORS */
-gdp_write_error:
-  {
-    return FALSE;
-  }
 }
 
 static GstFlowReturn
@@ -232,33 +157,21 @@ gst_tcp_client_sink_render (GstBaseSink * bsink, GstBuffer * buf)
 {
   size_t wrote = 0;
   GstTCPClientSink *sink;
-  gint size;
+  guint8 *data;
+  gsize size;
 
   sink = GST_TCP_CLIENT_SINK (bsink);
 
   g_return_val_if_fail (GST_OBJECT_FLAG_IS_SET (sink, GST_TCP_CLIENT_SINK_OPEN),
       GST_FLOW_WRONG_STATE);
 
-  size = GST_BUFFER_SIZE (buf);
-
-  GST_LOG_OBJECT (sink, "writing %d bytes for buffer data", size);
-
-  /* write the buffer header if we have one */
-  switch (sink->protocol) {
-    case GST_TCP_PROTOCOL_NONE:
-      break;
-    case GST_TCP_PROTOCOL_GDP:
-      GST_LOG_OBJECT (sink, "Sending buffer header through GDP");
-      if (!gst_tcp_gdp_write_buffer (GST_ELEMENT (sink), sink->sock_fd.fd, buf,
-              TRUE, sink->host, sink->port))
-        goto gdp_write_error;
-      break;
-    default:
-      break;
-  }
+  data = gst_buffer_map (buf, &size, NULL, GST_MAP_READ);
+  GST_LOG_OBJECT (sink, "writing %" G_GSIZE_FORMAT " bytes for buffer data",
+      size);
 
   /* write buffer data */
-  wrote = gst_tcp_socket_write (sink->sock_fd.fd, GST_BUFFER_DATA (buf), size);
+  wrote = gst_tcp_socket_write (sink->sock_fd.fd, data, size);
+  gst_buffer_unmap (buf, data, size);
 
   if (wrote < size)
     goto write_error;
@@ -268,16 +181,12 @@ gst_tcp_client_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   return GST_FLOW_OK;
 
   /* ERRORS */
-gdp_write_error:
-  {
-    return FALSE;
-  }
 write_error:
   {
     GST_ELEMENT_ERROR (sink, RESOURCE, WRITE,
         (_("Error while sending data to \"%s:%d\"."), sink->host, sink->port),
-        ("Only %" G_GSIZE_FORMAT " of %u bytes written: %s",
-            wrote, GST_BUFFER_SIZE (buf), g_strerror (errno)));
+        ("Only %" G_GSIZE_FORMAT " of %" G_GSIZE_FORMAT " bytes written: %s",
+            wrote, size, g_strerror (errno)));
     return GST_FLOW_ERROR;
   }
 }
@@ -303,9 +212,6 @@ gst_tcp_client_sink_set_property (GObject * object, guint prop_id,
     case ARG_PORT:
       tcpclientsink->port = g_value_get_int (value);
       break;
-    case ARG_PROTOCOL:
-      tcpclientsink->protocol = g_value_get_enum (value);
-      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -329,9 +235,6 @@ gst_tcp_client_sink_get_property (GObject * object, guint prop_id,
     case ARG_PORT:
       g_value_set_int (value, tcpclientsink->port);
       break;
-    case ARG_PROTOCOL:
-      g_value_set_enum (value, tcpclientsink->protocol);
-      break;
 
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -432,7 +335,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      if (!gst_tcp_client_sink_start (GST_TCP_CLIENT_SINK (element)))
+      if (!gst_tcp_client_sink_start (sink))
         goto start_failure;
       break;
     default:
@@ -442,7 +345,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_tcp_client_sink_stop (GST_TCP_CLIENT_SINK (element));
+      gst_tcp_client_sink_stop (sink);
     default:
       break;
   }
