@@ -49,7 +49,6 @@ GST_DEBUG_CATEGORY_STATIC (video_test_src_debug);
 #define DEFAULT_PATTERN            GST_VIDEO_TEST_SRC_SMPTE
 #define DEFAULT_TIMESTAMP_OFFSET   0
 #define DEFAULT_IS_LIVE            FALSE
-#define DEFAULT_PEER_ALLOC         TRUE
 #define DEFAULT_COLOR_SPEC         GST_VIDEO_TEST_SRC_BT601
 #define DEFAULT_FOREGROUND_COLOR   0xffffffff
 #define DEFAULT_BACKGROUND_COLOR   0xff000000
@@ -61,8 +60,6 @@ enum
   PROP_PATTERN,
   PROP_TIMESTAMP_OFFSET,
   PROP_IS_LIVE,
-  PROP_PEER_ALLOC,
-  PROP_COLOR_SPEC,
   PROP_K0,
   PROP_KX,
   PROP_KY,
@@ -82,9 +79,8 @@ enum
 };
 
 
-GST_BOILERPLATE (GstVideoTestSrc, gst_video_test_src, GstPushSrc,
-    GST_TYPE_PUSH_SRC);
-
+#define gst_video_test_src_parent_class parent_class
+G_DEFINE_TYPE (GstVideoTestSrc, gst_video_test_src, GST_TYPE_PUSH_SRC);
 
 static void gst_video_test_src_set_pattern (GstVideoTestSrc * videotestsrc,
     int pattern_type);
@@ -93,9 +89,10 @@ static void gst_video_test_src_set_property (GObject * object, guint prop_id,
 static void gst_video_test_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
-static GstCaps *gst_video_test_src_getcaps (GstBaseSrc * bsrc);
+static GstCaps *gst_video_test_src_getcaps (GstBaseSrc * bsrc,
+    GstCaps * filter);
 static gboolean gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps);
-static void gst_video_test_src_src_fixate (GstPad * pad, GstCaps * caps);
+static void gst_video_test_src_src_fixate (GstBaseSrc * bsrc, GstCaps * caps);
 
 static gboolean gst_video_test_src_is_seekable (GstBaseSrc * psrc);
 static gboolean gst_video_test_src_do_seek (GstBaseSrc * bsrc,
@@ -104,9 +101,12 @@ static gboolean gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query);
 
 static void gst_video_test_src_get_times (GstBaseSrc * basesrc,
     GstBuffer * buffer, GstClockTime * start, GstClockTime * end);
-static GstFlowReturn gst_video_test_src_create (GstPushSrc * psrc,
-    GstBuffer ** buffer);
+static gboolean gst_video_test_src_decide_allocation (GstBaseSrc * bsrc,
+    GstQuery * query);
+static GstFlowReturn gst_video_test_src_fill (GstPushSrc * psrc,
+    GstBuffer * buffer);
 static gboolean gst_video_test_src_start (GstBaseSrc * basesrc);
+static gboolean gst_video_test_src_stop (GstBaseSrc * basesrc);
 
 #define GST_TYPE_VIDEO_TEST_SRC_PATTERN (gst_video_test_src_pattern_get_type ())
 static GType
@@ -146,46 +146,17 @@ gst_video_test_src_pattern_get_type (void)
   return video_test_src_pattern_type;
 }
 
-#define GST_TYPE_VIDEO_TEST_SRC_COLOR_SPEC (gst_video_test_src_color_spec_get_type ())
-static GType
-gst_video_test_src_color_spec_get_type (void)
-{
-  static GType video_test_src_color_spec_type = 0;
-  static const GEnumValue color_spec_types[] = {
-    {GST_VIDEO_TEST_SRC_BT601, "ITU-R Rec. BT.601", "bt601"},
-    {GST_VIDEO_TEST_SRC_BT709, "ITU-R Rec. BT.709", "bt709"},
-    {0, NULL, NULL}
-  };
-
-  if (!video_test_src_color_spec_type) {
-    video_test_src_color_spec_type =
-        g_enum_register_static ("GstVideoTestSrcColorSpec", color_spec_types);
-  }
-  return video_test_src_color_spec_type;
-}
-
-static void
-gst_video_test_src_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_set_details_simple (element_class,
-      "Video test source", "Source/Video",
-      "Creates a test video stream", "David A. Schleef <ds@schleef.org>");
-
-  gst_element_class_add_pad_template (element_class,
-      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-          gst_video_test_src_getcaps (NULL)));
-}
-
 static void
 gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
   GstPushSrcClass *gstpushsrc_class;
+  GstCaps *templ_caps;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
   gstpushsrc_class = (GstPushSrcClass *) klass;
 
@@ -204,17 +175,6 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
       g_param_spec_boolean ("is-live", "Is Live",
           "Whether to act as a live source", DEFAULT_IS_LIVE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_PEER_ALLOC,
-      g_param_spec_boolean ("peer-alloc", "Peer Alloc",
-          "Ask the peer to allocate an output buffer", DEFAULT_PEER_ALLOC,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_COLOR_SPEC,
-      g_param_spec_enum ("colorspec", "Color Specification",
-          "Generate video in the given color specification (Deprecated: "
-          "use a caps filter with video/x-raw-yuv,color-matrix=\"sdtv\" or "
-          "\"hdtv\" instead)",
-          GST_TYPE_VIDEO_TEST_SRC_COLOR_SPEC,
-          DEFAULT_COLOR_SPEC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_K0,
       g_param_spec_int ("k0", "Zoneplate zero order phase",
           "Zoneplate zero order phase, for generating plain fields or phase offsets",
@@ -303,24 +263,32 @@ gst_video_test_src_class_init (GstVideoTestSrcClass * klass)
           G_MININT32, G_MAXINT32, DEFAULT_HORIZONTAL_SPEED,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  gst_element_class_set_details_simple (gstelement_class,
+      "Video test source", "Source/Video",
+      "Creates a test video stream", "David A. Schleef <ds@schleef.org>");
+
+  templ_caps = gst_video_test_src_getcaps (NULL, NULL);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS, templ_caps));
+  gst_caps_unref (templ_caps);
+
   gstbasesrc_class->get_caps = gst_video_test_src_getcaps;
   gstbasesrc_class->set_caps = gst_video_test_src_setcaps;
+  gstbasesrc_class->fixate = gst_video_test_src_src_fixate;
   gstbasesrc_class->is_seekable = gst_video_test_src_is_seekable;
   gstbasesrc_class->do_seek = gst_video_test_src_do_seek;
   gstbasesrc_class->query = gst_video_test_src_query;
   gstbasesrc_class->get_times = gst_video_test_src_get_times;
   gstbasesrc_class->start = gst_video_test_src_start;
+  gstbasesrc_class->stop = gst_video_test_src_stop;
+  gstbasesrc_class->decide_allocation = gst_video_test_src_decide_allocation;
 
-  gstpushsrc_class->create = gst_video_test_src_create;
+  gstpushsrc_class->fill = gst_video_test_src_fill;
 }
 
 static void
-gst_video_test_src_init (GstVideoTestSrc * src, GstVideoTestSrcClass * g_class)
+gst_video_test_src_init (GstVideoTestSrc * src)
 {
-  GstPad *pad = GST_BASE_SRC_PAD (src);
-
-  gst_pad_set_fixatecaps_function (pad, gst_video_test_src_src_fixate);
-
   gst_video_test_src_set_pattern (src, DEFAULT_PATTERN);
 
   src->timestamp_offset = DEFAULT_TIMESTAMP_OFFSET;
@@ -331,11 +299,10 @@ gst_video_test_src_init (GstVideoTestSrc * src, GstVideoTestSrcClass * g_class)
   /* we operate in time */
   gst_base_src_set_format (GST_BASE_SRC (src), GST_FORMAT_TIME);
   gst_base_src_set_live (GST_BASE_SRC (src), DEFAULT_IS_LIVE);
-  src->peer_alloc = DEFAULT_PEER_ALLOC;
 }
 
 static void
-gst_video_test_src_src_fixate (GstPad * pad, GstCaps * caps)
+gst_video_test_src_src_fixate (GstBaseSrc * bsrc, GstCaps * caps)
 {
   GstStructure *structure;
 
@@ -347,13 +314,15 @@ gst_video_test_src_src_fixate (GstPad * pad, GstCaps * caps)
   if (gst_structure_has_field (structure, "pixel-aspect-ratio"))
     gst_structure_fixate_field_nearest_fraction (structure,
         "pixel-aspect-ratio", 1, 1);
-  if (gst_structure_has_field (structure, "color-matrix"))
-    gst_structure_fixate_field_string (structure, "color-matrix", "sdtv");
+  if (gst_structure_has_field (structure, "colorimetry"))
+    gst_structure_fixate_field_string (structure, "colorimetry", "bt601");
   if (gst_structure_has_field (structure, "chroma-site"))
     gst_structure_fixate_field_string (structure, "chroma-site", "mpeg2");
 
   if (gst_structure_has_field (structure, "interlaced"))
     gst_structure_fixate_field_boolean (structure, "interlaced", FALSE);
+
+  GST_BASE_SRC_CLASS (parent_class)->fixate (bsrc, caps);
 }
 
 static void
@@ -449,11 +418,6 @@ gst_video_test_src_set_property (GObject * object, guint prop_id,
     case PROP_IS_LIVE:
       gst_base_src_set_live (GST_BASE_SRC (src), g_value_get_boolean (value));
       break;
-    case PROP_PEER_ALLOC:
-      src->peer_alloc = g_value_get_boolean (value);
-      break;
-    case PROP_COLOR_SPEC:
-      break;
     case PROP_K0:
       src->k0 = g_value_get_int (value);
       break;
@@ -519,11 +483,6 @@ gst_video_test_src_get_property (GObject * object, guint prop_id,
     case PROP_IS_LIVE:
       g_value_set_boolean (value, gst_base_src_is_live (GST_BASE_SRC (src)));
       break;
-    case PROP_PEER_ALLOC:
-      g_value_set_boolean (value, src->peer_alloc);
-      break;
-    case PROP_COLOR_SPEC:
-      break;
     case PROP_K0:
       g_value_set_int (value, src->k0);
       break;
@@ -577,7 +536,7 @@ gst_video_test_src_get_property (GObject * object, guint prop_id,
 
 /* threadsafe because this gets called as the plugin is loaded */
 static GstCaps *
-gst_video_test_src_getcaps (GstBaseSrc * bsrc)
+gst_video_test_src_getcaps (GstBaseSrc * bsrc, GstCaps * filter)
 {
   static GstCaps *capslist = NULL;
 
@@ -587,8 +546,8 @@ gst_video_test_src_getcaps (GstBaseSrc * bsrc)
     int i;
 
     caps = gst_caps_new_empty ();
-    for (i = 0; i < n_fourccs; i++) {
-      structure = paint_get_structure (fourcc_list + i);
+    for (i = 0; i < n_formats; i++) {
+      structure = paint_get_structure (format_list + i);
       gst_structure_set (structure,
           "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
           "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
@@ -599,61 +558,42 @@ gst_video_test_src_getcaps (GstBaseSrc * bsrc)
     capslist = caps;
   }
 
-  return gst_caps_copy (capslist);
+  if (filter)
+    return gst_caps_intersect_full (filter, capslist, GST_CAPS_INTERSECT_FIRST);
+  else
+    return gst_caps_ref (capslist);
 }
 
 static gboolean
 gst_video_test_src_parse_caps (const GstCaps * caps,
-    gint * width, gint * height, gint * rate_numerator, gint * rate_denominator,
-    struct fourcc_list_struct **fourcc, GstVideoTestSrcColorSpec * color_spec)
+    gint * width, gint * height, gint * fps_n, gint * fps_d,
+    GstVideoColorimetry * colorimetry)
 {
   const GstStructure *structure;
   GstPadLinkReturn ret;
   const GValue *framerate;
-  const char *csp;
+  const gchar *csp;
 
   GST_DEBUG ("parsing caps");
 
-  if (gst_caps_get_size (caps) < 1)
-    return FALSE;
-
   structure = gst_caps_get_structure (caps, 0);
 
-  if (!(*fourcc = paintinfo_find_by_structure (structure)))
-    goto unknown_format;
-
   ret = gst_structure_get_int (structure, "width", width);
   ret &= gst_structure_get_int (structure, "height", height);
   framerate = gst_structure_get_value (structure, "framerate");
 
   if (framerate) {
-    *rate_numerator = gst_value_get_fraction_numerator (framerate);
-    *rate_denominator = gst_value_get_fraction_denominator (framerate);
+    *fps_n = gst_value_get_fraction_numerator (framerate);
+    *fps_d = gst_value_get_fraction_denominator (framerate);
   } else
     goto no_framerate;
 
-  csp = gst_structure_get_string (structure, "color-matrix");
-  if (csp) {
-    if (strcmp (csp, "sdtv") == 0) {
-      *color_spec = GST_VIDEO_TEST_SRC_BT601;
-    } else if (strcmp (csp, "hdtv") == 0) {
-      *color_spec = GST_VIDEO_TEST_SRC_BT709;
-    } else {
-      GST_DEBUG ("unknown color-matrix");
-      return FALSE;
-    }
-  } else {
-    *color_spec = GST_VIDEO_TEST_SRC_BT601;
-  }
+  if ((csp = gst_structure_get_string (structure, "colorimetry")))
+    gst_video_colorimetry_from_string (colorimetry, csp);
 
   return ret;
 
   /* ERRORS */
-unknown_format:
-  {
-    GST_DEBUG ("videotestsrc format not found");
-    return FALSE;
-  }
 no_framerate:
   {
     GST_DEBUG ("videotestsrc no framerate given");
@@ -662,33 +602,89 @@ no_framerate:
 }
 
 static gboolean
-gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
+gst_video_test_src_decide_allocation (GstBaseSrc * bsrc, GstQuery * query)
 {
-  gboolean res;
-  gint width, height, rate_denominator, rate_numerator;
-  struct fourcc_list_struct *fourcc;
   GstVideoTestSrc *videotestsrc;
-  GstVideoTestSrcColorSpec color_spec;
+  GstBufferPool *pool;
+  guint size, min, max, prefix, alignment;
 
   videotestsrc = GST_VIDEO_TEST_SRC (bsrc);
 
-  res = gst_video_test_src_parse_caps (caps, &width, &height,
-      &rate_numerator, &rate_denominator, &fourcc, &color_spec);
-  if (res) {
-    /* looks ok here */
-    videotestsrc->fourcc = fourcc;
-    videotestsrc->width = width;
-    videotestsrc->height = height;
-    videotestsrc->rate_numerator = rate_numerator;
-    videotestsrc->rate_denominator = rate_denominator;
-    videotestsrc->bpp = videotestsrc->fourcc->bitspp;
-    videotestsrc->color_spec = color_spec;
+  gst_query_parse_allocation_params (query, &size, &min, &max, &prefix,
+      &alignment, &pool);
+  /* adjust size */
+  size = MAX (size, videotestsrc->info.size);
 
-    GST_DEBUG_OBJECT (videotestsrc, "size %dx%d, %d/%d fps",
-        videotestsrc->width, videotestsrc->height,
-        videotestsrc->rate_numerator, videotestsrc->rate_denominator);
+  if (pool) {
+    GstStructure *config;
+
+    config = gst_buffer_pool_get_config (pool);
+    gst_buffer_pool_config_add_option (config,
+        GST_BUFFER_POOL_OPTION_META_VIDEO);
+    gst_buffer_pool_set_config (pool, config);
+  }
+  gst_query_set_allocation_params (query, size, min, max, prefix,
+      alignment, pool);
+
+  return TRUE;
+}
+
+static gboolean
+gst_video_test_src_setcaps (GstBaseSrc * bsrc, GstCaps * caps)
+{
+  struct format_list_struct *format;
+  const GstStructure *structure;
+  GstVideoTestSrc *videotestsrc;
+  GstVideoInfo info;
+
+  videotestsrc = GST_VIDEO_TEST_SRC (bsrc);
+
+  structure = gst_caps_get_structure (caps, 0);
+
+  if (gst_structure_has_name (structure, "video/x-raw")) {
+    /* we can use the parsing code */
+    if (!gst_video_info_from_caps (&info, caps))
+      goto parse_failed;
+
+  } else if (gst_structure_has_name (structure, "video/x-raw-bayer")) {
+    if (!gst_video_test_src_parse_caps (caps, &info.width, &info.height,
+            &info.fps_n, &info.fps_d, &info.colorimetry))
+      goto parse_failed;
+
+    info.size =
+        gst_video_test_src_get_size (videotestsrc, info.width, info.height);
+  }
+
+  if (!(format = paintinfo_find_by_structure (structure)))
+    goto unknown_format;
+
+  /* looks ok here */
+  videotestsrc->format = format;
+  videotestsrc->info = info;
+
+  GST_DEBUG_OBJECT (videotestsrc, "size %dx%d, %d/%d fps",
+      info.width, info.height, info.fps_n, info.fps_d);
+
+  g_free (videotestsrc->tmpline);
+  g_free (videotestsrc->tmpline2);
+  g_free (videotestsrc->tmpline_u8);
+  videotestsrc->tmpline_u8 = g_malloc (info.width + 8);
+  videotestsrc->tmpline = g_malloc ((info.width + 8) * 4);
+  videotestsrc->tmpline2 = g_malloc ((info.width + 8) * 4);
+
+  return TRUE;
+
+  /* ERRORS */
+parse_failed:
+  {
+    GST_DEBUG_OBJECT (bsrc, "failed to parse caps");
+    return FALSE;
+  }
+unknown_format:
+  {
+    GST_DEBUG ("videotestsrc format not found");
+    return FALSE;
   }
-  return res;
 }
 
 static gboolean
@@ -706,61 +702,17 @@ gst_video_test_src_query (GstBaseSrc * bsrc, GstQuery * query)
       gint64 src_val, dest_val;
 
       gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
-      if (src_fmt == dest_fmt) {
-        dest_val = src_val;
-        goto done;
-      }
-
-      switch (src_fmt) {
-        case GST_FORMAT_DEFAULT:
-          switch (dest_fmt) {
-            case GST_FORMAT_TIME:
-              /* frames to time */
-              if (src->rate_numerator) {
-                dest_val = gst_util_uint64_scale (src_val,
-                    src->rate_denominator * GST_SECOND, src->rate_numerator);
-              } else {
-                dest_val = 0;
-              }
-              break;
-            default:
-              goto error;
-          }
-          break;
-        case GST_FORMAT_TIME:
-          switch (dest_fmt) {
-            case GST_FORMAT_DEFAULT:
-              /* time to frames */
-              if (src->rate_numerator) {
-                dest_val = gst_util_uint64_scale (src_val,
-                    src->rate_numerator, src->rate_denominator * GST_SECOND);
-              } else {
-                dest_val = 0;
-              }
-              break;
-            default:
-              goto error;
-          }
-          break;
-        default:
-          goto error;
-      }
-    done:
+      res =
+          gst_video_info_convert (&src->info, src_fmt, src_val, dest_fmt,
+          &dest_val);
       gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
-      res = TRUE;
       break;
     }
     default:
       res = GST_BASE_SRC_CLASS (parent_class)->query (bsrc, query);
+      break;
   }
   return res;
-
-  /* ERROR */
-error:
-  {
-    GST_DEBUG_OBJECT (src, "query failed");
-    return FALSE;
-  }
 }
 
 static void
@@ -789,30 +741,30 @@ gst_video_test_src_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
 static gboolean
 gst_video_test_src_do_seek (GstBaseSrc * bsrc, GstSegment * segment)
 {
-  GstClockTime time;
+  GstClockTime position;
   GstVideoTestSrc *src;
 
   src = GST_VIDEO_TEST_SRC (bsrc);
 
   segment->time = segment->start;
-  time = segment->last_stop;
+  position = segment->position;
 
-  /* now move to the time indicated */
-  if (src->rate_numerator) {
-    src->n_frames = gst_util_uint64_scale (time,
-        src->rate_numerator, src->rate_denominator * GST_SECOND);
+  /* now move to the position indicated */
+  if (src->info.fps_n) {
+    src->n_frames = gst_util_uint64_scale (position,
+        src->info.fps_n, src->info.fps_d * GST_SECOND);
   } else {
     src->n_frames = 0;
   }
-  if (src->rate_numerator) {
+  if (src->info.fps_n) {
     src->running_time = gst_util_uint64_scale (src->n_frames,
-        src->rate_denominator * GST_SECOND, src->rate_numerator);
+        src->info.fps_d * GST_SECOND, src->info.fps_n);
   } else {
     /* FIXME : Not sure what to set here */
     src->running_time = 0;
   }
 
-  g_assert (src->running_time <= time);
+  g_assert (src->running_time <= position);
 
   return TRUE;
 }
@@ -825,85 +777,47 @@ gst_video_test_src_is_seekable (GstBaseSrc * psrc)
 }
 
 static GstFlowReturn
-gst_video_test_src_create (GstPushSrc * psrc, GstBuffer ** buffer)
+gst_video_test_src_fill (GstPushSrc * psrc, GstBuffer * buffer)
 {
   GstVideoTestSrc *src;
-  gulong newsize, size;
-  GstBuffer *outbuf = NULL;
-  GstFlowReturn res;
   GstClockTime next_time;
+  GstVideoFrame frame;
 
   src = GST_VIDEO_TEST_SRC (psrc);
 
-  if (G_UNLIKELY (src->fourcc == NULL))
+  if (G_UNLIKELY (src->format == NULL))
     goto not_negotiated;
 
   /* 0 framerate and we are at the second frame, eos */
-  if (G_UNLIKELY (src->rate_numerator == 0 && src->n_frames == 1))
+  if (G_UNLIKELY (src->info.fps_n == 0 && src->n_frames == 1))
     goto eos;
 
-  newsize = gst_video_test_src_get_size (src, src->width, src->height);
-
-  g_return_val_if_fail (newsize > 0, GST_FLOW_ERROR);
-
   GST_LOG_OBJECT (src,
-      "creating buffer of %lu bytes with %dx%d image for frame %d", newsize,
-      src->width, src->height, (gint) src->n_frames);
+      "creating buffer from pool for frame %d", (gint) src->n_frames);
 
-  if (src->peer_alloc) {
-    res =
-        gst_pad_alloc_buffer_and_set_caps (GST_BASE_SRC_PAD (psrc),
-        GST_BUFFER_OFFSET_NONE, newsize, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)),
-        &outbuf);
-    if (res != GST_FLOW_OK)
-      goto no_buffer;
+  if (!gst_video_frame_map (&frame, &src->info, buffer, GST_MAP_WRITE))
+    goto invalid_frame;
 
-    /* the buffer could have renegotiated, we need to discard any buffers of the
-     * wrong size. */
-    size = GST_BUFFER_SIZE (outbuf);
-    newsize = gst_video_test_src_get_size (src, src->width, src->height);
+  src->make_image (src, &frame);
 
-    if (size != newsize) {
-      gst_buffer_unref (outbuf);
-      outbuf = NULL;
-    }
-  }
+  gst_video_frame_unmap (&frame);
 
-  if (outbuf == NULL) {
-    outbuf = gst_buffer_new_and_alloc (newsize);
-    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (GST_BASE_SRC_PAD (psrc)));
-  }
-
-  memset (GST_BUFFER_DATA (outbuf), 0, GST_BUFFER_SIZE (outbuf));
-  src->tmpline_u8 = g_malloc (src->width + 8);
-  src->tmpline = g_malloc ((src->width + 8) * 4);
-  src->tmpline2 = g_malloc ((src->width + 8) * 4);
-
-  src->make_image (src, (void *) GST_BUFFER_DATA (outbuf),
-      src->width, src->height);
-
-  g_free (src->tmpline);
-  g_free (src->tmpline2);
-  g_free (src->tmpline_u8);
-
-  GST_BUFFER_TIMESTAMP (outbuf) = src->timestamp_offset + src->running_time;
-  GST_BUFFER_OFFSET (outbuf) = src->n_frames;
+  GST_BUFFER_TIMESTAMP (buffer) = src->timestamp_offset + src->running_time;
+  GST_BUFFER_OFFSET (buffer) = src->n_frames;
   src->n_frames++;
-  GST_BUFFER_OFFSET_END (outbuf) = src->n_frames;
-  if (src->rate_numerator) {
+  GST_BUFFER_OFFSET_END (buffer) = src->n_frames;
+  if (src->info.fps_n) {
     next_time = gst_util_uint64_scale_int (src->n_frames * GST_SECOND,
-        src->rate_denominator, src->rate_numerator);
-    GST_BUFFER_DURATION (outbuf) = next_time - src->running_time;
+        src->info.fps_d, src->info.fps_n);
+    GST_BUFFER_DURATION (buffer) = next_time - src->running_time;
   } else {
     next_time = src->timestamp_offset;
     /* NONE means forever */
-    GST_BUFFER_DURATION (outbuf) = GST_CLOCK_TIME_NONE;
+    GST_BUFFER_DURATION (buffer) = GST_CLOCK_TIME_NONE;
   }
 
   src->running_time = next_time;
 
-  *buffer = outbuf;
-
   return GST_FLOW_OK;
 
 not_negotiated:
@@ -917,11 +831,10 @@ eos:
     GST_DEBUG_OBJECT (src, "eos: 0 framerate, frame %d", (gint) src->n_frames);
     return GST_FLOW_UNEXPECTED;
   }
-no_buffer:
+invalid_frame:
   {
-    GST_DEBUG_OBJECT (src, "could not allocate buffer, reason %s",
-        gst_flow_get_name (res));
-    return res;
+    GST_DEBUG_OBJECT (src, "invalid frame");
+    return GST_FLOW_OK;
   }
 }
 
@@ -936,6 +849,21 @@ gst_video_test_src_start (GstBaseSrc * basesrc)
   return TRUE;
 }
 
+static gboolean
+gst_video_test_src_stop (GstBaseSrc * basesrc)
+{
+  GstVideoTestSrc *src = GST_VIDEO_TEST_SRC (basesrc);
+
+  g_free (src->tmpline);
+  src->tmpline = NULL;
+  g_free (src->tmpline2);
+  src->tmpline2 = NULL;
+  g_free (src->tmpline_u8);
+  src->tmpline_u8 = NULL;
+
+  return TRUE;
+}
+
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
