@@ -573,7 +573,6 @@ GST_START_TEST (test_add_pad)
   GstElement *bin, *src1, *src2, *adder, *sink;
   GstBus *bus;
   GstPad *srcpad;
-  GstStreamConsistency *consist;
   gboolean res;
 
   GST_INFO ("preparing test");
@@ -600,7 +599,6 @@ GST_START_TEST (test_add_pad)
   fail_unless (res == TRUE, NULL);
 
   srcpad = gst_element_get_static_pad (adder, "src");
-  consist = gst_consistency_checker_new (srcpad);
   gst_object_unref (srcpad);
 
   main_loop = g_main_loop_new (NULL, FALSE);
@@ -656,7 +654,6 @@ GST_START_TEST (test_remove_pad)
   GstBus *bus;
   GstPad *pad, *srcpad;
   gboolean res;
-  GstStreamConsistency *consist;
 
   GST_INFO ("preparing test");
 
@@ -682,7 +679,6 @@ GST_START_TEST (test_remove_pad)
   fail_if (pad == NULL, NULL);
 
   srcpad = gst_element_get_static_pad (adder, "src");
-  consist = gst_consistency_checker_new (srcpad);
   gst_object_unref (srcpad);
 
   main_loop = g_main_loop_new (NULL, FALSE);
@@ -744,6 +740,7 @@ handoff_buffer_cb (GstElement * fakesink, GstBuffer * buffer, GstPad * pad,
 /* check if clipping works as expected */
 GST_START_TEST (test_clip)
 {
+  GstSegment segment;
   GstElement *bin, *adder, *sink;
   GstBus *bus;
   GstPad *sinkpad;
@@ -751,7 +748,7 @@ GST_START_TEST (test_clip)
   GstFlowReturn ret;
   GstEvent *event;
   GstBuffer *buffer;
-  GstCaps *caps;
+//FIXME:  GstCaps *caps;
 
   GST_INFO ("preparing test");
 
@@ -784,22 +781,24 @@ GST_START_TEST (test_clip)
   fail_if (sinkpad == NULL, NULL);
 
   /* send segment to adder */
-  event = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME,
-      GST_SECOND, 2 * GST_SECOND, 0);
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  segment.start = GST_SECOND;
+  segment.stop = 2 * GST_SECOND;
+  segment.time = 0;
+  event = gst_event_new_segment (&segment);
   gst_pad_send_event (sinkpad, event);
 
-  caps = gst_caps_new_simple ("audio/x-raw-int",
+/*FIXME:  caps = gst_caps_new_simple ("audio/x-raw",
+      "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
       "rate", G_TYPE_INT, 44100,
-      "channels", G_TYPE_INT, 2,
-      "endianness", G_TYPE_INT, G_BYTE_ORDER,
-      "width", G_TYPE_INT, 16,
-      "depth", G_TYPE_INT, 16, "signed", G_TYPE_BOOLEAN, TRUE, NULL);
+      "channels", G_TYPE_INT, 2, NULL);
+*/
 
   /* should be clipped and ok */
   buffer = gst_buffer_new_and_alloc (44100);
   GST_BUFFER_TIMESTAMP (buffer) = 0;
   GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  gst_buffer_set_caps (buffer, caps);
+//FIXME:  gst_buffer_set_caps (buffer, caps);
   GST_DEBUG ("pushing buffer %p", buffer);
   ret = gst_pad_chain (sinkpad, buffer);
   fail_unless (ret == GST_FLOW_OK);
@@ -809,7 +808,7 @@ GST_START_TEST (test_clip)
   buffer = gst_buffer_new_and_alloc (44100);
   GST_BUFFER_TIMESTAMP (buffer) = 900 * GST_MSECOND;
   GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  gst_buffer_set_caps (buffer, caps);
+//FIXME:  gst_buffer_set_caps (buffer, caps);
   GST_DEBUG ("pushing buffer %p", buffer);
   ret = gst_pad_chain (sinkpad, buffer);
   fail_unless (ret == GST_FLOW_OK);
@@ -820,7 +819,7 @@ GST_START_TEST (test_clip)
   buffer = gst_buffer_new_and_alloc (44100);
   GST_BUFFER_TIMESTAMP (buffer) = 1 * GST_SECOND;
   GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  gst_buffer_set_caps (buffer, caps);
+//FIXME:  gst_buffer_set_caps (buffer, caps);
   GST_DEBUG ("pushing buffer %p", buffer);
   ret = gst_pad_chain (sinkpad, buffer);
   fail_unless (ret == GST_FLOW_OK);
@@ -831,7 +830,7 @@ GST_START_TEST (test_clip)
   buffer = gst_buffer_new_and_alloc (44100);
   GST_BUFFER_TIMESTAMP (buffer) = 2 * GST_SECOND;
   GST_BUFFER_DURATION (buffer) = 250 * GST_MSECOND;
-  gst_buffer_set_caps (buffer, caps);
+//FIXME:  gst_buffer_set_caps (buffer, caps);
   GST_DEBUG ("pushing buffer %p", buffer);
   ret = gst_pad_chain (sinkpad, buffer);
   fail_unless (ret == GST_FLOW_OK);
