@@ -157,21 +157,20 @@ check_chain_final_state (gpointer key, ChainState * state, gpointer data)
   return TRUE;
 }
 
-static gboolean
-eos_buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
+static GstProbeReturn
+eos_buffer_probe (GstPad * pad, GstProbeType type, GstBuffer * buffer,
+    gpointer unused)
 {
   gint ret;
   gint size;
-  guint8 *data;
   gchar *oggbuffer;
   ChainState *state = NULL;
   gboolean has_video = FALSE;
 
-  size = GST_BUFFER_SIZE (buffer);
-  data = GST_BUFFER_DATA (buffer);
+  size = gst_buffer_get_size (buffer);
 
   oggbuffer = ogg_sync_buffer (&oggsync, size);
-  memcpy (oggbuffer, data, size);
+  gst_buffer_extract (buffer, 0, oggbuffer, size);
   ogg_sync_wrote (&oggsync, size);
 
   do {
@@ -197,7 +196,7 @@ eos_buffer_probe (GstPad * pad, GstBuffer * buffer, gpointer unused)
     }
   }
 
-  return TRUE;
+  return GST_PROBE_OK;
 }
 
 static void
@@ -210,7 +209,8 @@ start_pipeline (GstElement * bin, GstPad * pad)
   eos_chain_states =
       g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
   probe_id =
-      gst_pad_add_buffer_probe (pad, G_CALLBACK (eos_buffer_probe), NULL);
+      gst_pad_add_probe (pad, GST_PROBE_TYPE_BUFFER,
+      (GstPadProbeCallback) eos_buffer_probe, NULL, NULL);
 
   ret = gst_element_set_state (bin, GST_STATE_PLAYING);
   fail_if (ret == GST_STATE_CHANGE_FAILURE, "Could not start test pipeline");
@@ -232,7 +232,7 @@ stop_pipeline (GstElement * bin, GstPad * pad)
     fail_if (ret != GST_STATE_CHANGE_SUCCESS, "Could not stop test pipeline");
   }
 
-  gst_pad_remove_buffer_probe (pad, (guint) probe_id);
+  gst_pad_remove_probe (pad, probe_id);
   ogg_sync_clear (&oggsync);
 
   /* check end conditions, such as EOS flags */
@@ -258,6 +258,7 @@ test_pipeline (const char *pipeline)
   GError *error = NULL;
   GMainLoop *loop;
   GstPadLinkReturn linkret;
+  guint bus_watch = 0;
 
   bin = gst_parse_launch (pipeline, &error);
   fail_unless (bin != NULL, "Error parsing pipeline: %s",
@@ -280,7 +281,7 @@ test_pipeline (const char *pipeline)
   /* run until we receive EOS */
   loop = g_main_loop_new (NULL, FALSE);
   bus = gst_element_get_bus (bin);
-  gst_bus_add_watch (bus, (GstBusFunc) eos_watch, loop);
+  bus_watch = gst_bus_add_watch (bus, (GstBusFunc) eos_watch, loop);
   gst_object_unref (bus);
 
   start_pipeline (bin, pad);
@@ -291,7 +292,7 @@ test_pipeline (const char *pipeline)
     GstStructure *s;
     GstCaps *muxcaps;
 
-    muxcaps = gst_pad_get_negotiated_caps (sinkpad);
+    muxcaps = gst_pad_get_current_caps (sinkpad);
     fail_unless (muxcaps != NULL);
     s = gst_caps_get_structure (muxcaps, 0);
     fail_unless (gst_structure_has_name (s, "application/ogg"));
@@ -305,6 +306,7 @@ test_pipeline (const char *pipeline)
 
   /* clean up */
   g_main_loop_unref (loop);
+  g_source_remove (bus_watch);
   gst_object_unref (pad);
   gst_object_unref (bin);
 }
@@ -348,7 +350,7 @@ GST_END_TEST;
 GST_START_TEST (test_theora)
 {
   test_pipeline
-      ("videotestsrc num-buffers=5 ! ffmpegcolorspace ! theoraenc ! oggmux");
+      ("videotestsrc num-buffers=5 ! videoconvert ! theoraenc ! oggmux");
 }
 
 GST_END_TEST;
@@ -358,7 +360,7 @@ GST_END_TEST;
 GST_START_TEST (test_theora_vorbis)
 {
   test_pipeline
-      ("videotestsrc num-buffers=10 ! ffmpegcolorspace ! theoraenc ! queue ! oggmux name=mux "
+      ("videotestsrc num-buffers=10 ! videoconvert ! theoraenc ! queue ! oggmux name=mux "
       "audiotestsrc num-buffers=2 ! audioconvert ! vorbisenc ! queue ! mux.");
 }
 
@@ -367,7 +369,7 @@ GST_END_TEST;
 GST_START_TEST (test_vorbis_theora)
 {
   test_pipeline
-      ("videotestsrc num-buffers=2 ! ffmpegcolorspace ! theoraenc ! queue ! oggmux name=mux "
+      ("videotestsrc num-buffers=2 ! videoconvert ! theoraenc ! queue ! oggmux name=mux "
       "audiotestsrc num-buffers=10 ! audioconvert ! vorbisenc ! queue ! mux.");
 }
 
