@@ -135,7 +135,7 @@ GST_START_TEST (test_granulepos_offset)
   GError *error = NULL;
 
   pipe_str = g_strdup_printf ("videotestsrc timestamp-offset=%" G_GUINT64_FORMAT
-      " num-buffers=10 ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
+      " num-buffers=10 ! video/x-raw,format=(string)I420,framerate=10/1"
       " ! theoraenc ! fakesink name=fs0", TIMESTAMP_OFFSET);
 
   bin = gst_parse_launch (pipe_str, &error);
@@ -228,7 +228,7 @@ GST_START_TEST (test_continuity)
   GError *error = NULL;
 
   pipe_str = g_strdup_printf ("videotestsrc num-buffers=10"
-      " ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
+      " ! video/x-raw,format=(string)I420,framerate=10/1"
       " ! theoraenc ! fakesink name=fs0");
 
   bin = gst_parse_launch (pipe_str, &error);
@@ -272,7 +272,6 @@ GST_START_TEST (test_continuity)
 
   {
     GstClockTime next_timestamp;
-    gint64 last_granulepos;
 
     /* first buffer should have timestamp of TIMESTAMP_OFFSET, granulepos to
      * match the timestamp of the end of the last sample in the output buffer.
@@ -281,7 +280,6 @@ GST_START_TEST (test_continuity)
      * take care to timestamp correctly based on the offset of the input data
      * however, so it does do sub-granulepos timestamping. */
     buffer = gst_buffer_straw_get_buffer (bin, pad);
-    last_granulepos = GST_BUFFER_OFFSET_END (buffer);
     check_buffer_timestamp (buffer, 0);
     /* plain division because I know the answer is exact */
     check_buffer_duration (buffer, GST_SECOND / 10);
@@ -310,10 +308,16 @@ GST_START_TEST (test_continuity)
 
 GST_END_TEST;
 
-static gboolean
-drop_second_data_buffer (GstPad * droppad, GstBuffer * buffer, gpointer unused)
+static GstProbeReturn
+drop_second_data_buffer (GstPad * droppad, GstProbeType probe_type,
+    gpointer probe_obj, gpointer unused)
 {
-  return !(GST_BUFFER_OFFSET (buffer) == 1);
+  GstBuffer *buffer = probe_obj;
+
+  if (GST_BUFFER_OFFSET (buffer) == 1)
+    return GST_PROBE_DROP;
+  else
+    return GST_PROBE_OK;
 }
 
 GST_START_TEST (test_discontinuity)
@@ -326,7 +330,7 @@ GST_START_TEST (test_discontinuity)
   guint drop_id;
 
   pipe_str = g_strdup_printf ("videotestsrc num-buffers=10"
-      " ! video/x-raw-yuv,format=(fourcc)I420,framerate=10/1"
+      " ! video/x-raw,format=(string)I420,framerate=10/1"
       " ! theoraenc ! fakesink name=fs0");
 
   bin = gst_parse_launch (pipe_str, &error);
@@ -357,8 +361,8 @@ GST_START_TEST (test_discontinuity)
     gst_object_unref (sink);
   }
 
-  drop_id = gst_pad_add_buffer_probe (droppad,
-      G_CALLBACK (drop_second_data_buffer), NULL);
+  drop_id = gst_pad_add_probe (droppad, GST_PROBE_TYPE_BUFFER,
+      drop_second_data_buffer, NULL, NULL);
   gst_buffer_straw_start_pipeline (bin, pad);
 
   /* header packets should have timestamp == NONE, granulepos 0 */
@@ -415,7 +419,7 @@ GST_START_TEST (test_discontinuity)
   }
 
   gst_buffer_straw_stop_pipeline (bin, pad);
-  gst_pad_remove_buffer_probe (droppad, drop_id);
+  gst_pad_remove_probe (droppad, drop_id);
 
   gst_object_unref (droppad);
   gst_object_unref (pad);
