@@ -45,7 +45,7 @@
 #ifdef HAVE_X
 #include <gdk/gdkx.h>
 #endif
-#include <gst/interfaces/xoverlay.h>
+#include <gst/interfaces/videooverlay.h>
 
 GST_DEBUG_CATEGORY_STATIC (seek_debug);
 #define GST_CAT_DEFAULT (seek_debug)
@@ -66,9 +66,7 @@ gtk_widget_get_allocation (GtkWidget * w, GtkAllocation * a)
 #endif
 
 /* configuration */
-
-//#define SOURCE "filesrc"
-#define SOURCE "gnomevfssrc"
+#define SOURCE "filesrc"
 
 #define ASINK "alsasink"
 //#define ASINK "osssink"
@@ -485,7 +483,7 @@ make_theora_pipeline (const gchar * location)
   src = gst_element_factory_make_or_warn (SOURCE, "src");
   demux = gst_element_factory_make_or_warn ("oggdemux", "demux");
   decoder = gst_element_factory_make_or_warn ("theoradec", "decoder");
-  convert = gst_element_factory_make_or_warn ("ffmpegcolorspace", "convert");
+  convert = gst_element_factory_make_or_warn ("videoconvert", "convert");
   videosink = gst_element_factory_make_or_warn (VSINK, "sink");
 
   g_object_set (G_OBJECT (src), "location", location, NULL);
@@ -567,8 +565,7 @@ make_vorbis_theora_pipeline (const gchar * location)
   video_bin = gst_bin_new ("v_decoder_bin");
   v_queue = gst_element_factory_make_or_warn ("queue", "v_queue");
   v_decoder = gst_element_factory_make_or_warn ("theoradec", "v_dec");
-  v_convert =
-      gst_element_factory_make_or_warn ("ffmpegcolorspace", "v_convert");
+  v_convert = gst_element_factory_make_or_warn ("videoconvert", "v_convert");
   v_scale = gst_element_factory_make_or_warn ("videoscale", "v_scale");
   videosink = gst_element_factory_make_or_warn (VSINK, "v_sink");
 
@@ -647,8 +644,7 @@ make_avi_msmpeg4v3_mp3_pipeline (const gchar * location)
   video_bin = gst_bin_new ("v_decoder_bin");
   v_queue = gst_element_factory_make_or_warn ("queue", "v_queue");
   v_decoder = gst_element_factory_make_or_warn ("ffdec_msmpeg4", "v_dec");
-  v_convert =
-      gst_element_factory_make_or_warn ("ffmpegcolorspace", "v_convert");
+  v_convert = gst_element_factory_make_or_warn ("videoconvert", "v_convert");
   videosink = gst_element_factory_make_or_warn (VSINK, "v_sink");
 
   gst_bin_add (GST_BIN (video_bin), v_queue);
@@ -828,7 +824,7 @@ make_mpeg_pipeline (const gchar * location)
   video_bin = gst_bin_new ("v_decoder_bin");
   v_decoder = gst_element_factory_make_or_warn ("mpeg2dec", "v_dec");
   v_queue = gst_element_factory_make_or_warn ("queue", "v_queue");
-  v_filter = gst_element_factory_make_or_warn ("ffmpegcolorspace", "v_filter");
+  v_filter = gst_element_factory_make_or_warn ("videoconvert", "v_filter");
   videosink = gst_element_factory_make_or_warn (VSINK, "v_sink");
 
   gst_bin_add (GST_BIN (video_bin), v_decoder);
@@ -906,7 +902,7 @@ make_mpegnt_pipeline (const gchar * location)
 
   video_bin = gst_bin_new ("v_decoder_bin");
   v_decoder = gst_element_factory_make_or_warn ("mpeg2dec", "v_dec");
-  v_filter = gst_element_factory_make_or_warn ("ffmpegcolorspace", "v_filter");
+  v_filter = gst_element_factory_make_or_warn ("videoconvert", "v_filter");
   videosink = gst_element_factory_make_or_warn (VSINK, "v_sink");
   gst_element_link_many (v_decoder, v_filter, videosink, NULL);
 
@@ -1079,7 +1075,7 @@ query_rates (void)
 
       format = seek_formats[i].format;
 
-      if (gst_pad_query_convert (pad, GST_FORMAT_TIME, GST_SECOND, &format,
+      if (gst_pad_query_convert (pad, GST_FORMAT_TIME, GST_SECOND, format,
               &value)) {
         g_print ("%s %13" G_GINT64_FORMAT " | ", seek_formats[i].name, value);
       } else {
@@ -1110,8 +1106,8 @@ query_positions_elems (void)
 
       format = seek_formats[i].format;
 
-      if (gst_element_query_position (element, &format, &position) &&
-          gst_element_query_duration (element, &format, &total)) {
+      if (gst_element_query_position (element, format, &position) &&
+          gst_element_query_duration (element, format, &total)) {
         g_print ("%s %13" G_GINT64_FORMAT " / %13" G_GINT64_FORMAT " | ",
             seek_formats[i].name, position, total);
       } else {
@@ -1142,8 +1138,8 @@ query_positions_pads (void)
 
       format = seek_formats[i].format;
 
-      if (gst_pad_query_position (pad, &format, &position) &&
-          gst_pad_query_duration (pad, &format, &total)) {
+      if (gst_pad_query_position (pad, format, &position) &&
+          gst_pad_query_duration (pad, format, &total)) {
         g_print ("%s %13" G_GINT64_FORMAT " / %13" G_GINT64_FORMAT " | ",
             seek_formats[i].name, position, total);
       } else {
@@ -1231,24 +1227,19 @@ update_fill (gpointer data)
 static gboolean
 update_scale (gpointer data)
 {
-  GstFormat format = GST_FORMAT_TIME;
-
-  //position = 0;
-  //duration = 0;
-
   if (elem_seek) {
     if (seekable_elements) {
       GstElement *element = GST_ELEMENT (seekable_elements->data);
 
-      gst_element_query_position (element, &format, &position);
-      gst_element_query_duration (element, &format, &duration);
+      gst_element_query_position (element, GST_FORMAT_TIME, &position);
+      gst_element_query_duration (element, GST_FORMAT_TIME, &duration);
     }
   } else {
     if (seekable_pads) {
       GstPad *pad = GST_PAD (seekable_pads->data);
 
-      gst_pad_query_position (pad, &format, &position);
-      gst_pad_query_duration (pad, &format, &duration);
+      gst_pad_query_position (pad, GST_FORMAT_TIME, &position);
+      gst_pad_query_duration (pad, GST_FORMAT_TIME, &duration);
     }
   }
 
@@ -1408,7 +1399,7 @@ set_update_fill (gboolean active)
   if (active) {
     if (fill_id == 0) {
       fill_id =
-          g_timeout_add (FILL_INTERVAL, (GtkFunction) update_fill, pipeline);
+          g_timeout_add (FILL_INTERVAL, (GSourceFunc) update_fill, pipeline);
     }
   } else {
     if (fill_id) {
@@ -1427,7 +1418,7 @@ set_update_scale (gboolean active)
   if (active) {
     if (update_id == 0) {
       update_id =
-          g_timeout_add (UPDATE_INTERVAL, (GtkFunction) update_scale, pipeline);
+          g_timeout_add (UPDATE_INTERVAL, (GSourceFunc) update_scale, pipeline);
     }
   } else {
     if (update_id) {
@@ -2044,12 +2035,18 @@ shot_cb (GtkButton * button, gpointer data)
     gint width, height;
     GdkPixbuf *pixbuf;
     GError *error = NULL;
+    gsize size;
+    guint8 *data;
 
     /* get the snapshot buffer format now. We set the caps on the appsink so
      * that it can only be an rgb buffer. The only thing we have not specified
      * on the caps is the height, which is dependant on the pixel-aspect-ratio
      * of the source material */
+#if 0
     caps = GST_BUFFER_CAPS (buffer);
+#endif
+    /* FIXME, get the caps on the buffer somehow */
+    caps = NULL;
     if (!caps) {
       g_warning ("could not get snapshot format\n");
       goto done;
@@ -2066,12 +2063,17 @@ shot_cb (GtkButton * button, gpointer data)
 
     /* create pixmap from buffer and save, gstreamer video buffers have a stride
      * that is rounded up to the nearest multiple of 4 */
-    pixbuf = gdk_pixbuf_new_from_data (GST_BUFFER_DATA (buffer),
+    data = gst_buffer_map (buffer, &size, NULL, GST_MAP_READ);
+    pixbuf = gdk_pixbuf_new_from_data (data,
         GDK_COLORSPACE_RGB, FALSE, 8, width, height,
         GST_ROUND_UP_4 (width * 3), NULL, NULL);
 
     /* save the pixbuf */
     gdk_pixbuf_save (pixbuf, "snapshot.png", "png", &error, NULL);
+    gst_buffer_unmap (buffer, data, size);
+
+    /* save the pixbuf */
+    gdk_pixbuf_save (pixbuf, "snapshot.png", "png", &error, NULL);
 
   done:
     gst_buffer_unref (buffer);
@@ -2112,6 +2114,10 @@ step_cb (GtkButton * button, gpointer data)
   event = gst_event_new_step (format, amount, rate, flush, FALSE);
 
   res = send_event (event);
+
+  if (!res) {
+    g_print ("Sending step event failed\n");
+  }
 }
 
 static void
@@ -2432,7 +2438,7 @@ msg_clock_lost (GstBus * bus, GstMessage * message, GstPipeline * data)
 
 static gulong embed_xid = 0;
 
-/* We set the xid here in response to the prepare-xwindow-id message via a
+/* We set the xid here in response to the prepare-window-handle message via a
  * bus sync handler because we don't know the actual videosink used from the
  * start (as we don't know the pipeline, or bin elements such as autovideosink
  * or gconfvideosink may be used which create the actual videosink only once
@@ -2440,26 +2446,28 @@ static gulong embed_xid = 0;
 static GstBusSyncReply
 bus_sync_handler (GstBus * bus, GstMessage * message, GstPipeline * data)
 {
-  if ((GST_MESSAGE_TYPE (message) == GST_MESSAGE_ELEMENT) &&
-      gst_structure_has_name (message->structure, "prepare-xwindow-id")) {
-    GstElement *element = GST_ELEMENT (GST_MESSAGE_SRC (message));
+  GstElement *element;
 
-    g_print ("got prepare-xwindow-id, setting XID %lu\n", embed_xid);
+  if (!gst_is_video_overlay_prepare_window_handle_message (message))
+    return GST_BUS_PASS;
 
-    if (g_object_class_find_property (G_OBJECT_GET_CLASS (element),
-            "force-aspect-ratio")) {
-      g_object_set (element, "force-aspect-ratio", TRUE, NULL);
-    }
+  element = GST_ELEMENT (GST_MESSAGE_SRC (message));
 
-    /* Should have been initialised from main thread before (can't use
-     * GDK_WINDOW_XID here with Gtk+ >= 2.18, because the sync handler will
-     * be called from a streaming thread and GDK_WINDOW_XID maps to more than
-     * a simple structure lookup with Gtk+ >= 2.18, where 'more' is stuff that
-     * shouldn't be done from a non-GUI thread without explicit locking).  */
-    g_assert (embed_xid != 0);
+  g_print ("got prepare-window-handle, setting XID %lu\n", embed_xid);
 
-    gst_x_overlay_set_window_handle (GST_X_OVERLAY (element), embed_xid);
+  if (g_object_class_find_property (G_OBJECT_GET_CLASS (element),
+          "force-aspect-ratio")) {
+    g_object_set (element, "force-aspect-ratio", TRUE, NULL);
   }
+
+  /* Should have been initialised from main thread before (can't use
+   * GDK_WINDOW_XID here with Gtk+ >= 2.18, because the sync handler will
+   * be called from a streaming thread and GDK_WINDOW_XID maps to more than
+   * a simple structure lookup with Gtk+ >= 2.18, where 'more' is stuff that
+   * shouldn't be done from a non-GUI thread without explicit locking).  */
+  g_assert (embed_xid != 0);
+
+  gst_video_overlay_set_window_handle (GST_VIDEO_OVERLAY (element), embed_xid);
   return GST_BUS_PASS;
 }
 #endif
@@ -2492,7 +2500,7 @@ realize_cb (GtkWidget * widget, gpointer data)
     /* This is here just for pedagogical purposes, GDK_WINDOW_XID will call it
      * as well */
     if (!gdk_window_ensure_native (window))
-      g_error ("Couldn't create native window needed for GstXOverlay!");
+      g_error ("Couldn't create native window needed for GstVideoOverlay!");
   }
 #endif
 
@@ -2535,7 +2543,7 @@ connect_bus_signals (GstElement * pipeline)
   GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
 
 #ifdef HAVE_X
-  /* handle prepare-xwindow-id element message synchronously */
+  /* handle prepare-window-handle element message synchronously */
   gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler,
       pipeline);
 #endif
@@ -2651,6 +2659,8 @@ read_joystick (GIOChannel * source, GIOCondition condition, gpointer user_data)
     g_print ("error reading joystick, read %u bytes of %u\n",
         (guint) bytes_read, (guint) sizeof (struct js_event));
     return TRUE;
+  } else if (result != G_IO_STATUS_NORMAL) {
+    g_print ("reading from joystick returned status %d", result);
   }
 
   switch (js->type & ~JS_EVENT_INIT) {
@@ -2841,8 +2851,6 @@ main (int argc, char **argv)
     shuttle_hscale = gtk_hscale_new (shuttle_adjustment);
     gtk_scale_set_digits (GTK_SCALE (shuttle_hscale), 2);
     gtk_scale_set_value_pos (GTK_SCALE (shuttle_hscale), GTK_POS_TOP);
-    gtk_range_set_update_policy (GTK_RANGE (shuttle_hscale),
-        GTK_UPDATE_CONTINUOUS);
     g_signal_connect (shuttle_hscale, "value_changed",
         G_CALLBACK (shuttle_value_changed), pipeline);
     g_signal_connect (shuttle_hscale, "format_value",
@@ -2861,7 +2869,6 @@ main (int argc, char **argv)
   gtk_scale_set_value_pos (GTK_SCALE (hscale), GTK_POS_RIGHT);
   gtk_range_set_show_fill_level (GTK_RANGE (hscale), TRUE);
   gtk_range_set_fill_level (GTK_RANGE (hscale), 100.0);
-  gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
   g_signal_connect (hscale, "button_press_event", G_CALLBACK (start_seek),
       pipeline);
