@@ -38,7 +38,7 @@ struct _App
   GMainLoop *loop;
 };
 
-App s_app;
+static App s_app;
 
 static gboolean
 bus_message (GstBus * bus, GstMessage * message, App * app)
@@ -94,14 +94,12 @@ have_subtitle (GstElement * appsink, App * app)
 
   if (buffer) {
     guint8 *data;
-    guint size;
-    GstFormat format;
+    gsize size;
     gint64 position;
     GstClock *clock;
     GstClockTime base_time, running_time;
 
-    format = GST_FORMAT_TIME;
-    gst_element_query_position (appsink, &format, &position);
+    gst_element_query_position (appsink, GST_FORMAT_TIME, &position);
 
     clock = gst_element_get_clock (appsink);
     base_time = gst_element_get_base_time (appsink);
@@ -114,10 +112,9 @@ have_subtitle (GstElement * appsink, App * app)
         ", running_time %" GST_TIME_FORMAT, GST_TIME_ARGS (position),
         GST_TIME_ARGS (running_time));
 
-    data = GST_BUFFER_DATA (buffer);
-    size = GST_BUFFER_SIZE (buffer);
-
+    data = gst_buffer_map (buffer, &size, NULL, GST_MAP_READ);
     gst_util_dump_mem (data, size);
+    gst_buffer_unmap (buffer, data, size);
   }
 }
 
@@ -126,6 +123,7 @@ main (int argc, char *argv[])
 {
   App *app = &s_app;
   GstBus *bus;
+  GstCaps *subcaps;
 
   gst_init (&argc, &argv);
 
@@ -146,6 +144,9 @@ main (int argc, char *argv[])
   g_object_set (G_OBJECT (app->textsink), "ts-offset", 0 * GST_SECOND, NULL);
   g_signal_connect (app->textsink, "new-buffer", G_CALLBACK (have_subtitle),
       app);
+  subcaps = gst_caps_from_string ("text/x-pango-markup; text/plain");
+  g_object_set (G_OBJECT (app->textsink), "caps", subcaps, NULL);
+  gst_caps_unref (subcaps);
 
   g_object_set (G_OBJECT (app->playbin), "text-sink", app->textsink, NULL);
 
