@@ -1,5 +1,5 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #ifndef __GST_AUDIO_ENUM_TYPES_H__
 #define __GST_AUDIO_ENUM_TYPES_H__
@@ -19,11 +19,9 @@ GType gst_ring_buffer_seg_state_get_type (void);
 #define GST_TYPE_RING_BUFFER_SEG_STATE (gst_ring_buffer_seg_state_get_type())
 GType gst_buffer_format_type_get_type (void);
 #define GST_TYPE_BUFFER_FORMAT_TYPE (gst_buffer_format_type_get_type())
-GType gst_buffer_format_get_type (void);
-#define GST_TYPE_BUFFER_FORMAT (gst_buffer_format_get_type())
 G_END_DECLS
 
 #endif /* __GST_AUDIO_ENUM_TYPES_H__ */
 
-/* Generated data ends here */
+
 
